@@ -44,7 +44,7 @@
 	  (snd-warning (format #f "auto-saved version of ~S (~S) is newer"
 			       (short-file-name snd)
 			       temp-file)))
-      (do ((i 0 (1+ i)))
+      (do ((i 0 (+ 1 i)))
 	  ((= i (channels snd)))
 	(if (hook-empty? (edit-hook snd i))
 	    (add-hook! (edit-hook snd i) (upon-edit snd))))
@@ -54,18 +54,17 @@
     (let ((temp-file (auto-save-temp-name snd)))
       (if (file-exists? temp-file)
 	  (delete-file temp-file))
-      (clear-unsaved-edits snd)
-      #f))
+      (clear-unsaved-edits snd)))
   
   (define (auto-save-func)
     (if auto-saving
 	(begin
 	  (for-each (lambda (snd)
 		      (if (> (unsaved-edits snd) 0)
-			  (begin
-			    (report-in-minibuffer "auto-saving..." snd)
+			  (let ((save-name (auto-save-temp-name snd)))
+			    (report-in-minibuffer (string-append "auto-saving as " save-name "...") snd)
 			    (in (* 1000 3) (lambda () (report-in-minibuffer "" snd)))
-			    (save-sound-as (auto-save-temp-name snd) snd)
+			    (save-sound-as save-name snd)
 			    (clear-unsaved-edits snd))))
 		    (sounds))
 	  (in (* 1000 auto-save-interval) auto-save-func))))
