@@ -1,11 +1,12 @@
-#!/usr/bin/env guile \
--e main -s
+#!/usr/bin/guile -s
 !#
+
 ;;; bess1.scm -- some examples from clm-2/rt.lisp and clm-2/bess5.cl
 
 ;; Author: Michael Scholz <scholz-micha@gmx.de>
 ;; Created: Thu May 29 04:14:35 CEST 2003
 ;; Last: Sun Jun 15 03:50:21 CEST 2003
+;; changed slightly 14-Jun-06 Bill to match bess.scm, avoid a segfault in Guile, fix pitch problem in make-oscil.
 
 ;;; Commentary:
 
@@ -25,15 +26,24 @@
 
 ;;; Code:
 
-(use-modules (ice-9 optargs) (ice-9 format))
+;;; load sndlib and xmlib
+(let ((sndlib (dynamic-link "libsndlib.so")))
+  (if (not (dynamic-object? sndlib))
+      (set! sndlib (dynamic-link "sndlib.so")))
+  (if (not (dynamic-object? sndlib))
+      (error "can't find sndlib.so or libsndlib.so")
+      (dynamic-call (dynamic-func "Init_sndlib" sndlib) #f)))
 
-;; You may correct next two lines to fit your needs.
-(load-extension "libsndlib" "init_sndlib")
-(load-extension "libxm" "init_xm")
+(let ((libxm (dynamic-link "libxm.so")))
+  (if (not (dynamic-object? libxm))
+      (error "can't find libxm")
+      (dynamic-call (dynamic-func "Init_libxm" libxm) #f)))
 
-(define* (main #:rest args) (rt-motif))
+;(define* (main #:rest args) (rt-motif))
 ;;(define* (main #:rest args) (rt-motif #:srate 11025 #:bufsize 1024))
 
+(use-modules (ice-9 optargs) (ice-9 format))
+
 (define *clm-srate* 22050)
 (define *clm-data-format* mus-lshort)
 (define *clm-rt-bufsize* 128)
@@ -48,7 +58,7 @@
 (define sliderback "lightsteelblue")
 (define background "lightsteelblue1")
 
-(define definstrument define*)
+(if (not (defined? 'definstrument)) (define definstrument define*))
 ;(define (seconds->samples secs) (inexact->exact (round (* secs (mus-srate)))))
 
 ;; called by XtAppAddWorkProc
@@ -67,7 +77,7 @@
 			       (fm-index 1.0)
 			       (amp-env '(0 0 25 1 75 1 100 0)))
   "(make-rt-violin dur freq amp (fm-index 1.0) (amp-env '(0 0 25 1 75 1
-100 0))) real time simple violin (see snd-7/fm.html)"
+100 0))) real time simple violin (see snd-11/fm.html)"
   (let* ((frq-scl (hz->radians freq))
          (maxdev (* frq-scl fm-index))
          (index1 (* maxdev (/ 5.0 (log freq))))
@@ -98,15 +108,15 @@
 			(bufsize *clm-rt-bufsize*)
 			(data-format *clm-data-format*))
   (let ((vmode (list->array 1 '(0 12 2 4 14 4 5 5 0 7 7 11 11)))
-	(vpits (make-array 0 (1+ lim)))
-	(vbegs (make-array 0 (1+ lim)))
+	(vpits (make-array 0 (+ 1 lim)))
+	(vbegs (make-array 0 (+ 1 lim)))
 	(cellbeg 0)
 	(cellsiz 6)
 	(cellctr 0)
 	(func #f)
 	(len 0)
 	(dur 0.0))
-    (do ((i 0 (1+ i)))
+    (do ((i 0 (+ 1 i)))
 	((= i lim))
       (array-set! vpits (inexact->exact (floor (random 12.0))) i)
       (array-set! vbegs (+ 1 (inexact->exact (floor (random 3.0)))) i))
@@ -118,17 +128,20 @@
       (if (> len 1)
 	  (set! len (- len 1))
 	  (begin
-	    (set! dur (* ctempo (array-ref vbegs (1+ cellctr))))
-	    (set! cellctr (1+ cellctr))
+	    (set! dur (* ctempo (array-ref vbegs (+ 1 cellctr))))
+	    (set! cellctr (+ 1 cellctr))
 	    (if (> cellctr (+ cellsiz cellbeg))
 		(begin
-		  (if (> (random 1.0) 0.5) (set! cellbeg (1+ cellbeg)))
-		  (if (> (random 1.0) 0.5) (set! cellsiz (1+ cellsiz)))
+		  (if (> (random 1.0) 0.5) (set! cellbeg (+ 1 cellbeg)))
+		  (if (> (random 1.0) 0.5) (set! cellsiz (+ 1 cellsiz)))
 		  (set! cellctr cellbeg)))
 	    (set! func (make-rt-violin dur
-				       (* cfreq 16.351 16
-					  (expt 2 (/ (array-ref vmode (array-ref vpits cellctr))
-						     12.0)))
+				       (let ((freq (* cfreq 16.351 16
+						      (expt 2 (/ (array-ref vmode (array-ref vpits cellctr))
+								 12.0)))))
+					 (if (< (* 8 freq) *clm-srate*)
+					     freq
+					     (/ freq 4)))
 				       (* camp 0.3) #:fm-index cindex))
 	    (set! len (inexact->exact (ceiling (/ (seconds->samples dur) bufsize))))))
       func)))
@@ -142,11 +155,11 @@
 
 (define bell '(0 0 10 0.25 90 1.0 100 1.0))
 
-(define pits (make-array 0 (1+ lim)))
-(define octs (make-array 0 (1+ lim)))
-(define rhys (make-array 0 (1+ lim)))
-(define begs (make-array 0 (1+ lim)))
-(define amps (make-array 0 (1+ lim)))
+(define pits (make-array 0 (+ 1 lim)))
+(define octs (make-array 0 (+ 1 lim)))
+(define rhys (make-array 0 (+ 1 lim)))
+(define begs (make-array 0 (+ 1 lim)))
+(define amps (make-array 0 (+ 1 lim)))
 
 (define (tune x)
   (let* ((pit (modulo x 12))
@@ -183,7 +196,7 @@
 	(whichway 1)
 	(func #f)
 	(len 0))
-    (do ((i 0 (1+ i)))
+    (do ((i 0 (+ 1 i)))
 	((= i lim))
       (array-set! octs (inexact->exact (floor (+ 4 (* 2 (rbell (random 1.0)))))) i)
       (array-set! pits (array-ref mode (inexact->exact (floor (* 12 (random 1.0))))) i)
@@ -209,10 +222,10 @@
 			  (expt 2 (array-ref octs cellctr))))
 	    (set! ampl (* camp 10 (max 0.003 (* (array-ref amps cellctr) 0.01))))
 	    (set! ind (* cindex (random 1.0) 3.0))
-	    (set! cellctr (1+ cellctr))
+	    (set! cellctr (+ 1 cellctr))
 	    (if (> cellctr (+ cellsiz cellbeg))
 		(begin
-		  (set! cellbeg (1+ cellbeg))
+		  (set! cellbeg (+ 1 cellbeg))
 		  (if (> (random 1.0) 0.5) (set! cellsiz (+ cellsiz whichway)))
 		  (if (and (> cellsiz 10) (> (random 1.0) 0.99))
 		      (set! whichway -2)
@@ -529,4 +542,6 @@
       (XtRealizeWidget shell))
     (XtAppMainLoop app)))
 
+(rt-motif)
+
 ;; bess1.scm ends here
