@@ -1,6 +1,6 @@
 ;;; various envelope functions
 ;;;
-;;; envelope-interp (x env #:optional (base 1.0)) -> value of env at x (base controls connecting segment type)
+;;; envelope-interp (x env :optional (base 1.0)) -> value of env at x (base controls connecting segment type)
 ;;; window-envelope (beg end env) -> portion of env lying between x axis values beg and end
 ;;; map-envelopes (func env1 env2) maps func over the breakpoints in env1 and env2 returning a new envelope
 ;;;   multiply-envelopes (env1 env2) multiplies break-points of env1 and env2 returning a new envelope
@@ -8,13 +8,16 @@
 ;;; max-envelope (env) -> max y value in env, min-envelope
 ;;; integrate-envelope (env) -> area under env
 ;;; envelope-last-x (env) -> max x axis break point position
-;;; stretch-envelope env old-attack new-attack #:optional old-decay new-decay -> divseg-like envelope mangler
-;;; scale-envelope (env scaler #:optional offset) scales y axis values by 'scaler' and optionally adds 'offset'
+;;; stretch-envelope env old-attack new-attack :optional old-decay new-decay -> divseg-like envelope mangler
+;;; scale-envelope (env scaler :optional offset) scales y axis values by 'scaler' and optionally adds 'offset'
 ;;; reverse-envelope (env) reverses the breakpoints in 'env'
-;;; concatenate-envelopes (#:rest envs) concatenates its arguments into a new envelope
-;;; repeat-envelope env repeats #:optional (reflected #f) (normalized #f) repeats an envelope
+;;; concatenate-envelopes (:rest envs) concatenates its arguments into a new envelope
+;;; repeat-envelope env repeats :optional (reflected #f) (normalized #f) repeats an envelope
 ;;; power-env: generator for extended envelopes (each segment has its own base)
 ;;; envelope-exp: interpolate segments into envelope to give exponential curves
+;;; rms-envelope
+;;; normalize-envelope
+;;; simplify-envelope
 
 (use-modules (ice-9 format) (ice-9 optargs))
 
@@ -22,33 +25,29 @@
 
 ;;; -------- envelope-interp
 
-(define envelope-interp                      ;env is list of x y breakpoint pairs, interpolate at x returning y
-  (lambda args                          ;  (x env #:optional (base 1.0)
-    "(envelope-interp x env #:optional (base 1.0)) -> value of env at x; base controls connecting segment 
+(define* (envelope-interp x env :optional base)   ;env is list of x y breakpoint pairs, interpolate at x returning y
+  "(envelope-interp x env :optional (base 1.0)) -> value of env at x; base controls connecting segment 
 type: (envelope-interp .3 '(0 0 .5 1 1 0) -> .6"
-    (let ((x (car args))
-	  (env (cadr args))
-	  (base (if (null? (cddr args)) #f (caddr args))))
-      (cond ((null? env) 0.0)		;no data -- return 0.0
-	    ((or (<= x (car env))	;we're sitting on x val (or if < we blew it)
-		 (null? (cddr env)))	;or we're at the end of the list
-	     (cadr env))		;so return current y value
-	    ((> (caddr env) x)		;x <= next env x axis value
-	     (if (or (= (cadr env) (cadddr env))
-		     (and base (= base 0.0)))
-		 (cadr env)		;y1=y0, so just return y0 (avoid endless calculations below)
-		 (if (or (not base) (= base 1.0))
-		     (+ (cadr env)	;y0+(x-x0)*(y1-y0)/(x1-x0)
-			(* (- x (car env))
-			   (/ (- (cadddr env) (cadr env))
-			      (- (caddr env) (car env)))))
-		     (+ (cadr env)
-			(* (/ (- (cadddr env) (cadr env))
-			      (- base 1.0))
-			   (- (expt base (/ (- x (car env))
-					    (- (caddr env) (car env))))
-			      1.0))))))
-	    (else (envelope-interp x (cddr env))))))) ;go on looking for x segment
+  (cond ((null? env) 0.0)		;no data -- return 0.0
+	((or (<= x (car env))	        ;we're sitting on x val (or if < we blew it)
+	     (null? (cddr env)))	;or we're at the end of the list
+	 (cadr env))		        ;so return current y value
+	((> (caddr env) x)		;x <= next env x axis value
+	 (if (or (= (cadr env) (cadddr env))
+		 (and base (= base 0.0)))
+	     (cadr env)		;y1=y0, so just return y0 (avoid endless calculations below)
+	     (if (or (not base) (= base 1.0))
+		 (+ (cadr env)	;y0+(x-x0)*(y1-y0)/(x1-x0)
+		    (* (- x (car env))
+		       (/ (- (cadddr env) (cadr env))
+			  (- (caddr env) (car env)))))
+		 (+ (cadr env) ; this does not exactly match xramp-channel
+		    (* (/ (- (cadddr env) (cadr env))
+			  (- base 1.0))
+		       (- (expt base (/ (- x (car env))
+					(- (caddr env) (car env))))
+			  1.0))))))
+	(else (envelope-interp x (cddr env) base)))) ;go on looking for x segment
 
 
 ;;; -------- window-envelope (a kinda brute-force translation from the CL version in env.lisp)
@@ -107,6 +106,7 @@ end: (window-envelope 1.0 3.0 '(0.0 0.0 5.0 1.0)) -> '(1.0 0.2 3.0 0.6)"
 				  ((member (car lst) nlst) (rem-dup (cdr lst) nlst))
 				  (else (rem-dup (cdr lst) (cons (car lst) nlst)))))))
 		  (rem-dup lst '())))))
+
       (if (null? e1)
 	  (at0 e2)
 	  (if (null? e2)
@@ -116,7 +116,7 @@ end: (window-envelope 1.0 3.0 '(0.0 0.0 5.0 1.0)) -> '(1.0 0.2 3.0 0.6)"
 		    (newe '()))
 		(set! xs (sort! (remove-duplicates xs) <))
 		(let ((len (length xs)))
-		  (do ((i 0 (1+ i)))
+		  (do ((i 0 (+ 1 i)))
 		      ((= i len))
 		    (let ((x (list-ref xs i)))
 		      (set! newe (append newe (list x (op (envelope-interp x ee1) (envelope-interp x ee2)))))))
@@ -181,7 +181,7 @@ envelope: (multiply-envelopes '(0 0 2 .5) '(0 0 1 2 2 1)) -> '(0 0 0.5 0.5 1.0 0
 
 (define (stretch-envelope . args)
 
-  "(stretch-envelope env old-attack new-attack #:optional old-decay new-decay) takes 'env' and 
+  "(stretch-envelope env old-attack new-attack :optional old-decay new-decay) takes 'env' and 
 returns a new envelope based on it but with the attack and optionally decay portions stretched 
 or squeezed; 'old-attack' is the original x axis attack end point, 'new-attack' is where that 
 section should end in the new envelope.  Similarly for 'old-decay' and 'new-decay'.  This mimics 
@@ -259,8 +259,8 @@ divseg in early versions of CLM and its antecedents in Sambox and Mus10 (linen).
     
 ;;; -------- scale-envelope
 
-(define* (scale-envelope e scl #:optional (offset 0))
-  "(scale-envelope env scaler #:optional (offset 0)) scales y axis values by 'scaler' and optionally adds 'offset'"
+(define* (scale-envelope e scl :optional (offset 0))
+  "(scale-envelope env scaler :optional (offset 0)) scales y axis values by 'scaler' and optionally adds 'offset'"
   (if (null? e)
       '()
       (append (list (car e) (+ offset (* scl (cadr e))))
@@ -288,8 +288,8 @@ divseg in early versions of CLM and its antecedents in Sambox and Mus10 (linen).
 
 ;;; -------- concatenate-envelopes
 
-(define* (concatenate-envelopes #:rest envs)
-  "(concatenate-envelopes #:rest envs) concatenates its arguments into a new envelope"
+(define (concatenate-envelopes . envs)
+  "(concatenate-envelopes :rest envs) concatenates its arguments into a new envelope"
   (define (cat-1 e newe xoff x0)
     (if (null? e)
 	newe
@@ -314,14 +314,14 @@ divseg in early versions of CLM and its antecedents in Sambox and Mus10 (linen).
     (reverse ne)))
 
 
-(define* (repeat-envelope ur-env repeats #:optional (reflected #f) (normalized #f))
-    "(repeat-envelope env repeats #:optional (reflected #f) (normalized #f)) repeats 'env' 'repeats' 
+(define* (repeat-envelope ur-env repeats :optional (reflected #f) (normalized #f))
+    "(repeat-envelope env repeats :optional (reflected #f) (normalized #f)) repeats 'env' 'repeats' 
 times.  (repeat-envelope '(0 0 100 1) 2) -> (0 0 100 1 101 0 201 1). 
 If the final y value is different from the first y value, a quick ramp is 
 inserted between repeats. 'normalized' causes the new envelope's x axis 
 to have the same extent as the original's. 'reflected' causes every other 
 repetition to be in reverse."
-  (let* ((times (if reflected (inexact->exact (floor (/ repeats 2))) repeats))
+  (let* ((times (if reflected (floor (/ repeats 2)) repeats))
 	 (e (if reflected
 		(let* ((lastx (list-ref ur-env (- (length ur-env) 2)))
 		       (rev-env (cddr (reverse ur-env)))
@@ -338,14 +338,14 @@ repetition to be in reverse."
 	 (first-y-is-last-y (= first-y (list-ref e (- (length e) 1))))
 	 (new-env (list first-y x))
 	 (len (length e)))
-    (do ((i 0 (1+ i)))
+    (do ((i 0 (+ 1 i)))
 	((= i times))
       (do ((j 2 (+ j 2)))
 	  ((>= j len))
 	(set! x (+ x (- (list-ref e j) (list-ref e (- j 2)))))
 	(set! new-env (cons x new-env))
 	(set! new-env (cons (list-ref e (+ j 1)) new-env)))
-      (if (and (< i (1- times)) (not first-y-is-last-y))
+      (if (and (< i (- times 1)) (not first-y-is-last-y))
 	  (begin
 	    (set! x (+ x (/ x-max 100.0)))
 	    (set! new-env (cons x new-env))
@@ -361,31 +361,31 @@ repetition to be in reverse."
 
 
 ;;; -------- power-env 
+;;;
+;;; (this could also be done using multi-expt-env (based on env-any) in generators.scm)
 
-(if (not (provided? 'def-clm-struct))
-    (load "ws.scm"))
+(if (not (provided? 'snd-ws.scm)) (load-from-path "ws.scm"))
 
-(def-clm-struct penv envs total-envs current-env current-pass)
+(def-clm-struct penv (envs #f :type clm-vector) (total-envs 0 :type int) (current-env 0 :type int) (current-pass 0 :type int))
 
 (define (power-env pe)
-  ;; this doesn't yet work within the Snd run macro
   (let* ((val (env (vector-ref (penv-envs pe) (penv-current-env pe)))))
-    (set! (penv-current-pass pe) (1- (penv-current-pass pe)))
+    (set! (penv-current-pass pe) (- (penv-current-pass pe) 1))
     (if (= (penv-current-pass pe) 0)
-      (if (< (penv-current-env pe) (1- (penv-total-envs pe)))
+      (if (< (penv-current-env pe) (- (penv-total-envs pe) 1))
 	  (begin
-	    (set! (penv-current-env pe) (1+ (penv-current-env pe)))
-	    (set! (penv-current-pass pe) (mus-length (vector-ref (penv-envs pe) (penv-current-env pe)))))))
+	    (set! (penv-current-env pe) (+ 1 (penv-current-env pe)))
+	    (set! (penv-current-pass pe) (- (length (vector-ref (penv-envs pe) (penv-current-env pe))) 1)))))
     val))
 
-(define* (make-power-env envelope #:key (scaler 1.0) (offset 0.0) duration)
-  (let* ((len (1- (inexact->exact (floor (/ (length envelope) 3)))))
+(define* (make-power-env envelope :key (scaler 1.0) (offset 0.0) duration)
+  (let* ((len (- (floor (/ (length envelope) 3)) 1))
 	 (pe (make-penv :envs (make-vector len)
 			:total-envs len
 			:current-env 0
 			:current-pass 0))
 	 (xext (- (list-ref envelope (- (length envelope) 3)) (car envelope))))
-    (do ((i 0 (1+ i))
+    (do ((i 0 (+ 1 i))
 	 (j 0 (+ j 3)))
 	((= i len))
       (let ((x0 (list-ref envelope j))
@@ -396,18 +396,18 @@ repetition to be in reverse."
 	(vector-set! (penv-envs pe) i (make-env (list 0.0 y0 1.0 y1) 
 						:base base :scaler scaler :offset offset 
 						:duration (* duration (/ (- x1 x0) xext))))))
-    (set! (penv-current-pass pe) (mus-length (vector-ref (penv-envs pe) 0)))
+    (set! (penv-current-pass pe) (- (length (vector-ref (penv-envs pe) 0)) 1))
     pe))
 
-(define* (power-env-channel pe #:optional (beg 0) dur snd chn edpos (edname "power-env-channel"))
+(define* (power-env-channel pe :optional (beg 0) dur snd chn edpos (edname "power-env-channel"))
   ;; split into successive calls on env-channel
   (let ((curbeg beg)) ; sample number
     (as-one-edit
      (lambda ()
-       (do ((i 0 (1+ i)))
+       (do ((i 0 (+ 1 i)))
 	   ((= i (penv-total-envs pe)))
 	 (let* ((e (vector-ref (penv-envs pe) i))
-		(len (1+ (mus-length e))))
+		(len (length e)))
 	   (env-channel e curbeg len snd chn edpos)
 	   (set! curbeg (+ curbeg len)))))
      edname)))
@@ -415,8 +415,9 @@ repetition to be in reverse."
 
 ;;; here's a simpler version that takes the breakpoint list, rather than the power-env structure:
 
-(define* (powenv-channel envelope #:optional (beg 0) dur snd chn edpos)
-  ;; envelope with a separate base for each segment: (powenv-channel '(0 0 .325  1 1 32.0 2 0 32.0))
+(define* (powenv-channel envelope :optional (beg 0) dur snd chn edpos)
+  "(powenv-channel envelope :optional (beg 0) dur snd chn edpos) returns an envelope with a separate base for \
+each segment: (powenv-channel '(0 0 .325  1 1 32.0 2 0 32.0))"
   (let* ((curbeg beg)
 	 (fulldur (or dur (frames snd chn edpos)))
 	 (len (length envelope))
@@ -436,7 +437,7 @@ repetition to be in reverse."
 	     (set! y0 y1)
 	     (set! x1 (list-ref envelope i))
 	     (set! y1 (list-ref envelope (+ i 1)))
-	     (let* ((curdur (inexact->exact (round (* fulldur (/ (- x1 x0) xrange))))))
+	     (let* ((curdur (round (* fulldur (/ (- x1 x0) xrange)))))
 	       (xramp-channel y0 y1 base curbeg curdur snd chn edpos)
 	       (set! curbeg (+ curbeg curdur)))
 	     (set! base (list-ref envelope (+ i 2)))))))))
@@ -453,7 +454,8 @@ repetition to be in reverse."
 ;;; power applies to whole envelope,
 ;;; xgrid is how fine a solution to sample our new envelope with.
 
-(define* (envelope-exp e #:optional (power 1.0) (xgrid 100))
+(define* (envelope-exp e :optional (power 1.0) (xgrid 100))
+  "(envelope-exp e :optional (power 1.0) (xgrid 100)) approximates an exponential curve connecting the breakpoints"
   (let* ((mn (min-envelope e))
 	 (largest-diff (exact->inexact (- (max-envelope e) mn)))
 	 (x-min (car e))
@@ -476,31 +478,120 @@ repetition to be in reverse."
 
 ;;; rms-envelope
 
-(define* (rms-envelope file #:key (beg 0.0) (dur #f) (rfreq 30.0) (db #f))
+(define* (rms-envelope file :key (beg 0.0) (dur #f) (rfreq 30.0) (db #f))
+  "(rms-envelope file :key (beg 0.0) (dur #f) (rfreq 30.0) (db #f)) returns an envelope of RMS values in 'file'"
   ;; based on rmsenv.ins by Bret Battey
   (let* ((e '())
 	 (incr (/ 1.0 rfreq))
-	 (fsr (mus-sound-srate file))
-	 (incrsamps (inexact->exact (round (* incr fsr))))
-	 (start (inexact->exact (round (* beg fsr))))
-	 (reader (make-sample-reader start file))
+	 (fsr (srate file))
+	 (incrsamps (round (* incr fsr)))
+	 (start (round (* beg fsr)))
+	 (reader (make-sampler start file))
 	 (end (if dur (min (inexact->exact (+ start (round (* fsr dur))))
 			   (mus-sound-frames file))
 		  (mus-sound-frames file)))
-	 (rms (make-average incrsamps)))
+	 (rms (make-moving-average incrsamps))) ; this could use make-moving-rms from dsp.scm
     (do ((i 0 (+ i incrsamps)))
 	((>= i end) 
 	 (reverse e))
       (let ((rms-val 0.0))
-	(do ((j 0 (1+ j)))
+	(do ((j 0 (+ 1 j)))
 	    ((= j incrsamps))
 	  (let ((val (reader)))
-	    (set! rms-val (average rms (* val val)))))
+	    (set! rms-val (moving-average rms (* val val)))))
 	(set! e (cons (exact->inexact (/ i fsr)) e))
 	(set! rms-val (sqrt rms-val))
 	(if db 
 	    (if (< rms-val .00001)
 		(set! e (cons -100.0 e))
-		(set! e (cons (* 20.0 (/ (log rms-val) (log 10.0))))))
+		(set! e (cons (* 20.0 (/ (log rms-val) (log 10.0))) e)))
 	    (set! e (cons rms-val e)))))))
 
+
+(define* (normalize-envelope env :optional (new-max 1.0))
+  (define (abs-max-envelope-1 e mx)
+    (if (null? e)
+	mx
+	(abs-max-envelope-1 (cddr e) (max mx (abs (cadr e))))))
+  (let ((peak (abs-max-envelope-1 (cddr env) (abs (cadr env)))))
+    (scale-envelope env (/ new-max peak))))
+
+
+;;; simplify-envelope
+;;;
+;;; this is not very good...
+
+(define* (simplify-envelope env :optional (ygrid 10) (xgrid 100))
+
+  ;; grid = how fine a fluctuation we will allow.
+  ;; the smaller the grid, the less likely a given bump will get through
+  ;; original x and y values are not changed, just sometimes omitted.
+
+  (define (point-on-line? px py qx qy tx ty)
+
+    ;; is point tx ty on line defined by px py and qx qy --
+    ;; #f if no, :before if on ray from p, :after if on ray from q, :within if between p and q
+    ;; (these are looking at the "line" as a fat vector drawn on a grid)
+    ;; taken from "Graphics Gems" by Glassner, code by A Paeth
+
+    (if (or (= py qy ty) 
+	    (= px qx tx))
+	:within
+	(if (< (abs (- (* (- qy py) (- tx px))
+		       (* (- ty py) (- qx px))))
+	       (max (abs (- qx px))
+		    (abs (- qy py))))
+	    (if (or (and (< qx px) (< px tx))
+		    (and (< qy py) (< py ty)))
+		:before
+		(if (or (and (< tx px) (< px qx))
+			(and (< ty py) (< py qy)))
+		    :before
+		    (if (or (and (< px qx) (< qx tx))
+			    (and (< py qy) (< qy ty)))
+			:after
+			(if (or (and (< tx qx) (< qx px))
+				(and (< ty qy) (< qy py)))
+			    :after
+			    :within))))
+	    #f)))
+
+  (if (and env
+	   (> (length env) 4))
+      (let* ((new-env (list (cadr env) (car env)))
+	     (ymax (max-envelope env))
+	     (ymin (min-envelope env))
+	     (xmax (list-ref env (- (length env) 2)))
+	     (xmin (car env)))
+	(if (= ymin ymax)
+	    (list xmin ymin xmax ymax)
+	  (let* ((y-scl (/ ygrid (- ymax ymin)))
+		 (x-scl (/ (or xgrid ygrid) (- xmax xmin)))
+		 (px #f) (py #f)
+		 (qx #f) (qy #f) 
+		 (tx #f) (ty #f) 
+		 (qtx #f) (qty #f))
+	    (do ((i 0 (+ i 2)))
+		((>= i (length env)))
+	      (let ((ttx (list-ref env i))
+		    (tty (list-ref env (+ i 1))))
+		(set! tx (round (* ttx x-scl)))
+		(set! ty (round (* tty y-scl)))
+		(if px
+		    (if (not (point-on-line? px py qx qy tx ty))
+			(begin
+			  (set! new-env (cons qtx new-env))
+			  (set! new-env (cons qty new-env))
+			  (set! px qx)
+			  (set! py qy)))
+		    (begin
+		      (set! px qx)
+		      (set! py qy)))
+		(set! qx tx)
+		(set! qy ty)
+		(set! qtx ttx)
+		(set! qty tty)))
+	    (set! new-env (cons qtx new-env))
+	    (set! new-env (cons qty new-env))
+	    (reverse new-env))))
+      env))
