@@ -12,10 +12,8 @@
 (use-modules (ice-9 optargs))
 (provide 'snd-fmv.scm)
 
-(define pi 3.141592653589793)
-
 (define make-fm-violin 
-  (lambda* (frequency amplitude #:key
+  (lambda* (frequency amplitude :key
 	    (fm-index 1.0)
 	    (amp-env #f)
 	    (periodic-vibrato-rate 5.0) 
@@ -39,9 +37,9 @@
 	    (fm2-index #f) 
 	    (fm3-index #f)
 	    (base 1.0)
-	    #:allow-other-keys)
+	    :allow-other-keys)
 
-"(make-fm-violin frequency amplitude #:key 
+"(make-fm-violin frequency amplitude :key 
   (fm-index 1.0) (amp-env #f) (periodic-vibrato-rate 5.0) 
   (random-vibrato-rate 16.0) (periodic-vibrato-amplitude 0.0025) 
   (random-vibrato-amplitude 0.005) (noise-amount 0.0) 
@@ -60,10 +58,10 @@ fm-violin takes the value returned by make-fm-violin and returns a new sample ea
   (define (test-v beg dur freq amp)
     (let ((v (make-fm-violin freq amp 
 	      :amp-env (let ((e (make-env :envelope '(0 0 1 1 2 0) 
-					  :scaler amp :end dur)))
+					  :scaler amp :length dur)))
 			 (lambda () (env e)))))
 	  (data (channel->vct beg dur)))
-      (do ((i 0 (1+ i))) ((= i dur))
+      (do ((i 0 (+ 1 i))) ((= i dur))
 	(vct-set! data i (+ (vct-ref data i) (v))))
       (vct->channel data beg dur))))"
 
@@ -123,17 +121,17 @@ fm-violin takes the value returned by make-fm-violin and returns a new sample ea
 				     (* (indf3) (oscil fmosc3 (+ (* fm3-rat vib) fuzz)))))
 			      0.0))))))))))
 
-#!
+#|
 (define test-v 
   (lambda (beg dur freq amp amp-env)
     (let ((v (make-fm-violin 
 	      freq amp 
 	      :amp-env (let ((e (make-env :envelope (or amp-env '(0 0 1 1 2 0)) 
 					  :scaler amp 
-					  :end dur)))
+					  :length dur)))
 			 (lambda () (env e)))))
 	  (data (channel->vct beg dur)))
-      (do ((i 0 (1+ i)))
+      (do ((i 0 (+ 1 i)))
 	  ((= i dur))
 	(vct-set! data i (+ (vct-ref data i)
 			    (v))))
@@ -148,20 +146,20 @@ fm-violin takes the value returned by make-fm-violin and returns a new sample ea
 	      freq amp 
 	      :amp-env (let ((e (make-env :envelope (or amp-env '(0 0 1 1 2 0)) 
 					  :scaler amp 
-					  :end dur)))
+					  :length dur)))
 			 (lambda () (env e)))
 	      :fm1-env (let ((osc (make-oscil 100.0)))
 			 (lambda () (oscil osc)))))
 	  (data (channel->vct beg dur)))
-      (do ((i 0 (1+ i)))
+      (do ((i 0 (+ 1 i)))
 	  ((= i dur))
 	(vct-set! data i (+ (vct-ref data i)
 			    (v))))
       (vct->channel data beg dur))))
-!#
+|#
 
-(define* (fm-violin-ins startime dur freq amp #:key (degree #f) (reverb-amount 0.0) (distance 1.0) #:allow-other-keys #:rest args)
-  "(fm-violin-ins startime dur freq amp #:key (degree #f) (reverb-amount 0.0) (distance 1.0) #:allow-other-keys #:rest args) 
+(define* (fm-violin-ins startime dur freq amp :key (degree #f) (reverb-amount 0.0) (distance 1.0) :allow-other-keys :rest args)
+  "(fm-violin-ins startime dur freq amp :key (degree #f) (reverb-amount 0.0) (distance 1.0) :allow-other-keys :rest args) 
 calls the fm-violin with the given args and mixes the results into the current sound"
     (let* ((beg (floor (* startime (srate))))
 	   (len (floor (* dur (srate))))
@@ -169,7 +167,7 @@ calls the fm-violin with the given args and mixes the results into the current s
 	   (loc (make-locsig :channels (channels) :degree (or degree (random 90.0)) :reverb reverb-amount :distance distance))
 	   (out-data (make-vct len))
 	   (v (apply make-fm-violin freq amp args)))
-      (do ((i 0 (1+ i)))
+      (do ((i 0 (+ 1 i)))
 	  ((= i len))
 	(vct-set! out-data i (v)))
       (if (= (channels) 2)
