@@ -9,10 +9,10 @@
 (provide 'snd-gtk-popup.scm)
 
 (if (not (provided? 'xg))
-    (let ((hxm (dlopen "xm.so")))
+    (let ((hxm (dlopen "xg.so")))
       (if (string? hxm)
-	  (snd-error (format #f "gtk-popup.scm needs the xg module: ~A" hxm))
-	  (dlinit hxm "init_xm"))))
+	  (snd-error (format #f "gtk-popup.scm needs the xg module (either 'make xg' or build Snd with --with-static-xg): ~A" hxm))
+	  (dlinit hxm "Init_libxg"))))
 
 (define (change-label w new-label)
   "(change-label widget new-label) changes widget's label to be new-label"
@@ -21,13 +21,6 @@
 	  (gtk_label_set_text (GTK_LABEL w) new-label)
 	  (gtk_label_set_text (GTK_LABEL (gtk_bin_get_child (GTK_BIN w))) new-label))))
 
-(define* (g_signal_connect obj name func #:optional data)
-  (g_signal_connect_closure_by_id (GPOINTER obj)
-				  (g_signal_lookup name (G_OBJECT_TYPE (GTK_OBJECT obj)))
-				  0
-				  (g_cclosure_new func data #f)
-				  #f))
-
 (define (make-popup-menu top-field-func entries)
   "(make-popup-menu top-field-func entries) creates a popup menu"
   (let ((menu (gtk_menu_new)))
@@ -53,7 +46,44 @@
     menu))
 
 
-;;; -------- selection popup
+(define (display-properties props)
+  ;; there's no way to tell Guile's format that enormous vectors should not be printed in full
+  ;; so we search for them here and handle them ourselves
+
+  (define (vector-print v)
+    (if (< (length v) 3)
+	(object->string v)
+	(let ((str (format #f "'#(~A" (vector-ref v 0))))
+	  (do ((i 1 (+ 1 i)))
+	      ((= i 3))
+	    (set! str (string-append str " " (object->string (vector-ref v i)))))
+	  (string-append str " ...)"))))
+
+  (let ((str ""))
+    (for-each
+     (lambda (pr)
+       (let ((property (car pr))
+	     (value (cdr pr)))
+	 (set! str (string-append str (string #\newline) "    " (object->string property) ": "))
+	 (if (vector? value)
+	     (set! str (string-append str (vector-print value)))
+	     (if (list? value)
+		 (let ((prev #f))
+		   (set! str (string-append str "'("))
+		   (for-each
+		    (lambda (v)
+		      (if prev 
+			  (set! str (string-append str " "))
+			  (set! prev #t))
+		      (if (vector? v)
+			  (set! str (string-append str (vector-print v)))
+			  (set! str (string-append str (object->string v)))))
+		    value)
+		   (set! str (string-append str ")")))
+		 (set! str (string-append str (object->string value)))))))
+     props)
+    str))
+
 
 (if (not popup-already-loaded)
     (let* ((selection-popup-menu 
@@ -63,6 +93,7 @@
 		  (stopping1 #f)
 		  (stop-widget #f)
 		  (stop-widget1 #f))
+	      ;; -------- selection popup
 	      (make-popup-menu 
 	       (lambda (w) ; top level settings
 		 #f)
@@ -131,13 +162,14 @@
 				    (delete-samples (+ len 1) (- (frames snd chn) len) snd chn))))))
 			 (let ((sndlist '()))
 			   (for-each (lambda (snd)
-				       (do ((i (1- (channels snd)) (1- i)))
+				       (do ((i (- (channels snd) 1) (- i 1)))
 					   ((< i 0))
 					 (if (selection-member? snd i)
 					     (set! sndlist (cons (list snd i) sndlist)))))
 				     (sounds))
 			   sndlist))))
 		(list "Save as"   every-menu (lambda (w data) (save-selection-dialog)))
+		(list "Selection->Mix" every-menu (lambda (w data) (selection->mix)))
 		(list "Copy->New" every-menu 
 		      (lambda (w data) 
 			(let ((new-file-name (snd-tempnam)))
@@ -162,8 +194,9 @@
 		(list "Reset controls" every-menu (lambda (w data) (reset-controls)))
 		(list "Unselect"       every-menu (lambda (w data) (set! (selection-member? #t) #f)))
 		(list "Reverse"        every-menu (lambda (w data) (reverse-selection)))
-		(list "Mix"            every-menu (lambda (w data) (mix-selection (cursor))))
-		(list "Invert"         every-menu (lambda (w data) (scale-selection-by -1)))))))
+		(list "Mix (at cursor)" every-menu (lambda (w data) (mix-selection (cursor))))
+;		(list "Invert"         every-menu (lambda (w data) (scale-selection-by -1)))
+		))))
 	   
 	   ;; -------- time domain popup
 	   
@@ -213,7 +246,7 @@
 			    (begin
 			      (change-label w "Stop")
 			      (set! stopping #t)
-			      (play 0 graph-popup-snd))))
+			      (play graph-popup-snd :start 0))))
 		      (lambda (wid)
 			(set! stop-widget wid)))
 		(list "Play channel" 
@@ -221,25 +254,25 @@
 		      (lambda (w data)
 			(set! stopping #t)
 			(change-label stop-widget "Stop")
-			(play 0 graph-popup-snd graph-popup-chn)))
+			(play graph-popup-snd :start 0 :channel graph-popup-chn)))
 		(list "Play from cursor"
 		      (lambda (w) (set! play-cursor-popup-menu w))
 		      (lambda (w data)
 			(set! stopping #t)
 			(change-label stop-widget "Stop")
-			(play (cursor graph-popup-snd graph-popup-chn) graph-popup-snd)))
+			(play graph-popup-snd :start (cursor graph-popup-snd graph-popup-chn))))
 		(list "Play previous"
 		      (lambda (w) (set! play-previous-popup-menu w))
 		      (lambda (w data)
 			(set! stopping #t)
 			(change-label stop-widget "Stop")
-			(play 0 graph-popup-snd graph-popup-chn #f #f (1- (edit-position)))))  ; play version before most-recent edit
+			(play graph-popup-snd :start 0 :channel graph-popup-chn :edit-position (- (edit-position) 1))))  ; play version before most-recent edit
 		(list "Play original"
 		      (lambda (w) (set! play-original-popup-menu w))
 		      (lambda (w data)
 			(set! stopping #t)
 			(change-label stop-widget "Stop")
-			(play 0 graph-popup-snd graph-popup-chn #f #f 0)))                     ; play unedited version
+			(play graph-popup-snd :start 0 :channel graph-popup-chn :edit-position 0)))                     ; play unedited version
 		(list "Undo"
 		      (lambda (w) (set! undo-popup-menu w))
 		      (lambda (w data)
@@ -307,24 +340,38 @@
 			(let ((snd graph-popup-snd))
 			  (info-dialog 
 			   (format #f "~A info" (file-name snd))
-			   (format #f "~A:~%  chans: ~D~%  srate: ~D~%  header: ~A~%  data format: ~A~%  length: ~1,3F~%  maxamp: ~A~%~A~A~A"
+			   (format #f "~A:~%  chans: ~D~%  srate: ~D~%  header: ~A~%  data format: ~A~%  length: ~1,3F~%  maxamp: ~A~%~A~A~A~A~A"
 				   (short-file-name snd)
-				   (chans snd)
+				   (channels snd)
 				   (srate snd)
 				   (mus-header-type-name (header-type snd))
 				   (mus-data-format-name (data-format snd))
 				   (exact->inexact (/ (frames snd graph-popup-chn) (srate snd)))
 				   (maxamp snd #t)
 				   (if (comment snd)
-				       (format #f "  comment: ~A~%" (comment snd))
+				       (format #f "  comment: \"~A\"~%" (comment snd))
 				       "")
 				   (let ((loops (mus-sound-loop-info (file-name snd))))
 				     (if (not (null? loops))
 					 (format #f "  loop: ~A~%" loops)
 					 ""))
 				   (if (= (header-type snd) mus-soundfont)
-				       (format #f "  sounds: ~:{~%     ~S start: ~A, loop: ~A ~A~}" (soundfont-info))
-				       ""))))))
+				       (format #f "  sounds: ~{~{~%     ~S start: ~A, loop: ~A ~A~}~}" (soundfont-info))
+				       "")
+				   (if (not (null? (sound-properties snd)))
+				       (format #f "  properties: ~A"
+					       (display-properties (sound-properties snd)))
+				       "")
+				   (let ((chan-str ""))
+				     (do ((i 0 (+ 1 i)))
+					 ((= i (channels snd)))
+				       (if (not (null? (channel-properties snd i)))
+					   (set! chan-str 
+						 (string-append chan-str
+								(format #f "~%  chan ~D properties: ~A" 
+									i (display-properties (channel-properties snd i)))))))
+				     chan-str)
+				   )))))
 		(list "Add mark"            every-menu 
 		      (lambda (w data)
 			(add-mark (cursor graph-popup-snd graph-popup-chn) graph-popup-snd graph-popup-chn)))
@@ -380,7 +427,10 @@
 	   (sizes (list 16 64 256 1024 4096 16384 65536 262144 1048576))
 	   (windows (list rectangular-window hann-window welch-window parzen-window bartlett-window hamming-window blackman2-window 
 			  blackman3-window blackman4-window exponential-window riemann-window kaiser-window cauchy-window 
-			  poisson-window gaussian-window tukey-window dolph-chebyshev-window hann-poisson-window connes-window))
+			  poisson-window gaussian-window tukey-window dolph-chebyshev-window hann-poisson-window connes-window
+			  samaraki-window ultraspherical-window bartlett-hann-window bohman-window flat-top-window
+			  blackman5-window blackman6-window blackman7-window blackman8-window blackman9-window blackman10-window 
+			  ))
 	   (types (list fourier-transform wavelet-transform autocorrelation cepstrum walsh-transform haar-transform))
 	   
 	   (fft-popup-menu #f)
@@ -463,7 +513,10 @@
 				      (lambda (w data) 
 					(set! (fft-window graph-popup-snd (choose-chan)) val))))
 			      (list "Rectangular" "Hann" "Welch" "Parzen" "Bartlett" "Hamming" "Blackman2" "Blackman3" "Blackman4"
-				    "Exponential" "Riemann" "Kaiser" "Cauchy" "Poisson" "Gaussian" "Tukey" "Dolph-Chebyshev" "Hann-Poisson" "Connes")
+				    "Exponential" "Riemann" "Kaiser" "Cauchy" "Poisson" "Gaussian" "Tukey" "Dolph-Chebyshev" "Hann-Poisson" 
+				    "Connes" "Samaraki" "Ultraspherical" "Bartlett-Hann" "Bohman" "Flat-top"
+				    "Blackman5" "Blackman6" "Blackman7" "Blackman8" "Blackman9" "Blackman10" 
+				    )
 			      windows)
 			 'fft-windows)))
 		(list "Transform type"
@@ -490,10 +543,8 @@
 				(list "daub4" "daub6" "daub8" "daub10" "daub12" "daub14" "daub16" "daub18" "daub20" "battle_lemarie" 
 				      "burt_adelson" "beylkin" "coif2" "coif4" "coif6" "sym2" "sym3" "sym4" "sym5" "sym6")))
 			 'wavelet-types)))
-		(list "Color" every-menu
-		      (lambda (w data) (color-dialog)))
-		(list "Orientation" every-menu
-		      (lambda (w data) (orientation-dialog))))))))
+		(list "Color/Orientation" every-menu
+		      (lambda (w data) (color-orientation-dialog))))))))
       (set! fft-types (reverse fft-types))
       (set! fft-windows (reverse fft-windows))
       (set! fft-norms (reverse fft-norms))
@@ -510,17 +561,18 @@
 			 (begin
 			   (set! graph-popup-snd snd)
 			   (set! graph-popup-chn chn)
-			   (if (= (channel-style snd) channels-combined)
-			       (call-with-current-continuation
+			   (if (and (= (channel-style snd) channels-combined)
+				    (> (channels snd) 1))
+			       (call-with-exit
 				(lambda (break)
-				  (do ((i 0 (1+ i)))
-				      ((= i (chans snd)))
+				  (do ((i 0 (+ 1 i)))
+				      ((= i (channels snd)))
 				    (let ((off (list-ref (axis-info snd i) 14)))
 				      (if (< y off)
 					  (begin
 					    (set! graph-popup-chn (- i 1))
 					    (break)))))
-				  (set! graph-popup-chn (- (chans snd) 1)))))
+				  (set! graph-popup-chn (- (channels snd) 1)))))
 			   (let ((fax (if (transform-graph? snd chn) (axis-info snd chn transform-graph) #f))
 				 (lax (if (lisp-graph? snd chn) (axis-info snd chn lisp-graph) #f)))
 			     (if (and fax
@@ -541,12 +593,12 @@
 					       (gtk_widget_set_sensitive child (not (= (fft-window graph-popup-snd graph-popup-chn) window))))
 					     fft-windows windows)
 				   (for-each (lambda (child type)
-					       (gtk_widget_set_sensitive child (not (= (transform-type graph-popup-snd graph-popup-chn) type))))
+					       (gtk_widget_set_sensitive child (not (equal? (transform-type graph-popup-snd graph-popup-chn) type))))
 					     fft-types types)
 				   (let ((ctr 0))
 				     (for-each (lambda (child)
 						 (gtk_widget_set_sensitive child (not (= (wavelet-type graph-popup-snd graph-popup-chn) ctr)))
-						 (set! ctr (1+ ctr)))
+						 (set! ctr (+ 1 ctr)))
 					       wavelet-types))
 				   (set! menu fft-popup-menu))
 				 (if (and lax
@@ -567,7 +619,7 @@
 					 (set! menu selection-popup-menu)
 					 (let* ((eds (edits graph-popup-snd graph-popup-chn))
 						(with-edit (if (> (car eds) 0) gtk_widget_show gtk_widget_hide))
-						(nchans (chans graph-popup-snd))
+						(nchans (channels graph-popup-snd))
 						(with-selection (if (selection?) gtk_widget_show gtk_widget_hide))
 						(with-marks (if (not (null? (marks graph-popup-snd graph-popup-chn))) gtk_widget_show gtk_widget_hide)))
 					   (set! menu graph-popup-menu)
@@ -608,7 +660,7 @@
 (define (edhist-save-edits)
   (let* ((old-val (assoc (cons edhist-snd edhist-chn) edhist-funcs))
 	 (cur-edits (edits edhist-snd edhist-chn))
-	 (new-func (edit-list->function edhist-snd edhist-chn (1+ (car cur-edits)) (apply + cur-edits))))
+	 (new-func (edit-list->function edhist-snd edhist-chn (+ 1 (car cur-edits)) (apply + cur-edits))))
     (if old-val
 	(set-cdr! old-val new-func)
 	;; perhaps this should save the previous function under the current file name?
@@ -627,7 +679,7 @@
     (let ((index (cadr (g_object_get_data (G_OBJECT w) "popup-index"))))
       (if (and (> index 0)
 	       (<= index (length edhist-funcs)))
-	  ((cdr (list-ref edhist-funcs (1- index))) edhist-snd edhist-chn))))
+	  ((cdr (list-ref edhist-funcs (- index 1))) edhist-snd edhist-chn))))
 	  
   (let ((wids (cdr edhist-widgets))
 	(funcs edhist-funcs)
@@ -651,13 +703,13 @@
 		 (change-label button label))
 	     (g_object_set_data (G_OBJECT button) "popup-index" (GPOINTER index))
 	     (gtk_widget_show button)
-	     (set! index (1+ index))))
+	     (set! index (+ 1 index))))
 	 edhist-funcs))
     (if (not (null? wids))
 	(for-each
 	 (lambda (w)
-	   (gtk_widget_hide w)))
-	 wids)
+	   (gtk_widget_hide w))
+	 wids))
     (gtk_menu_popup (GTK_MENU parent) #f #f #f #f 3 edhist-time)
     #f))
 
@@ -705,8 +757,8 @@ all saved edit lists."
 	    (lambda (w d) (edhist-help-edits)))))))
 
 (define (add-edhist-popup snd)
-  (let ((chns (chans snd)))
-    (do ((i 0 (1+ i)))
+  (let ((chns (channels snd)))
+    (do ((i 0 (+ 1 i)))
 	((= i chns))
       (let ((edhist (list-ref (channel-widgets snd i) 7)))
 	(if (not (member edhist edhist-channel-widgets))
@@ -733,9 +785,9 @@ all saved edit lists."
 
 (add-hook! after-open-hook add-edhist-popup)
 (add-hook! close-hook (lambda (snd)
-			(let ((chns (chans snd))
+			(let ((chns (channels snd))
 			      (name (short-file-name snd)))
-			  (do ((i 0 (1+ i)))
+			  (do ((i 0 (+ 1 i)))
 			      ((= i chns))
 			    (let* ((old-val (assoc (cons snd i) edhist-funcs)))
 			      (if old-val
