@@ -1,6 +1,6 @@
 ;;; examples of mark-related functions
 
-(use-modules (ice-9 format))
+(use-modules (ice-9 format) (ice-9 common-list))
 (provide 'snd-marks.scm)
 
 ;;; Contents:
@@ -27,11 +27,11 @@
 
 (define (mark-name->id name)
   "(mark-name->id name) is like find-mark but searches all currently accessible channels"
-  (call-with-current-continuation
+  (call-with-exit
    (lambda (return)
      (for-each
       (lambda (snd)
-	(do ((chn 0 (1+ chn)))
+	(do ((chn 0 (+ 1 chn)))
 	    ((= chn (channels snd)))
 	     (let ((mark (find-mark name snd chn)))
 	       (if (mark? mark) 
@@ -52,19 +52,19 @@
 ;;; -------- describe-mark shows mark history
 
 (define (describe-mark id)
-  "(describe-mark id) returns a description of the movements of mark id over the channel's edit history"
+  "(describe-mark mark) returns a description of the movements of mark over the channel's edit history"
   (let ((mark-setting (catch 'no-such-mark (lambda () (mark-home id)) (lambda args #f))))
     (if (not mark-setting)
         ;; not an active mark, so go scrounging for it
         ;;   we're looking at every edit of every channel
-	(set! mark-setting (call-with-current-continuation
+	(set! mark-setting (call-with-exit
 			    (lambda (return)
 			      (for-each
 			       (lambda (snd)
-				 (do ((chn 0 (1+ chn)))
+				 (do ((chn 0 (+ 1 chn)))
 				     ((= chn (channels snd)))
 				   (let* ((max-edits (apply + (edits snd chn))))
-				     (do ((ed 0 (1+ ed)))
+				     (do ((ed 0 (+ 1 ed)))
 					 ((> ed max-edits))
 				       (if (member id (marks snd chn ed))
 					   (return (list snd chn)))))))
@@ -75,8 +75,8 @@
                (chn (cadr mark-setting))
                (max-edits (apply + (edits snd chn)))
                (descr '())
-               (header (list 'mark id 'sound snd (short-file-name snd) 'channel chn)))
-          (do ((i max-edits (1- i)))
+               (header (list id sound: snd (short-file-name snd) 'channel: chn)))
+          (do ((i max-edits (- i 1)))
               ((< i 0) descr)
             (if (member id (marks snd chn i))
                 (set! descr (cons (mark-sample id i) descr))
@@ -91,7 +91,7 @@
 (define mark-sync-number 0)
 (define (start-sync) (set! mark-sync-number (+ (mark-sync-max) 1)))
 (define (stop-sync) (set! mark-sync-number 0))
-(define (click-to-sync id) (set! (mark-sync id) mark-sync-number) #f)
+(define (click-to-sync id) (set! (sync id) mark-sync-number) #f)
 ;(add-hook! mark-click-hook click-to-sync)
 
 
@@ -99,7 +99,7 @@
 
 (define syncup
   (lambda ids
-    "(syncup ids) pads the channels with zeros so that all the marks in ids list occur at the same time"
+    "(syncup marks) pads the channels with zeros so that all the given marks occur at the same time"
     (let* ((samps (map mark-sample ids))
 	   (max-samp (apply max samps)))
       (define (pad-to-sync lst-ids lst-samps)
@@ -121,36 +121,44 @@
 	 (m2-samp (mark-sample m2))
 	 (m1-home (mark-home m1))
 	 (m2-home (mark-home m2)))
+    (if (not (selection?))
+	(throw 'no-active-selection))
     (if (not (equal? m1-home m2-home))
-	(snd-print (format #f "mark ~A is in ~A[~A] but mark ~A is in ~A[~A]?" 
+	(snd-print (format #f "~A is in ~A[~A] but ~A is in ~A[~A]?" 
 			   m1 (car m1-home) (cadr m1-home)
 			   m2 (car m2-home) (cadr m2-home)))
 	(let* ((mark-samps (- m2-samp m1-samp))
 	       (selection-samps (selection-frames))
-	       (reg-data (region->vct))
-	       (reader (make-sample-reader m1-samp))
-	       (new-data (make-vct mark-samps))
-	       (gr (make-granulate :expansion (/ mark-samps selection-samps)))
+	       (reg-data (let ((data (make-vct selection-samps))
+			       (rd (make-sampler (selection-position))))
+			   (do ((i 0 (+ 1 i)))
+			       ((= i selection-samps))
+			     (vct-set! data i (rd)))
+			   data))
 	       (inctr 0))
-	  (vct-map! new-data
-		    (lambda ()
-		      (+ (next-sample reader)
-			 (granulate gr
-				    (lambda (dir)
-				      (if (>= inctr selection-samps)
-					  0.0
-					  (let ((val (vct-ref reg-data inctr)))
-					    (set! inctr (+ inctr dir))
-					    val)))))))
-	  (free-sample-reader reader)
-	  (vct->channel new-data m1-samp mark-samps (car m1-home) (cadr m1-home))))))
+	  (if (= mark-samps selection-samps)
+	      (map-channel (lambda (y) 
+			     (let ((val (+ y (vct-ref reg-data inctr))))
+			       (set! inctr (+ 1 inctr))
+			       val))
+			   m1-samp mark-samps (car m1-home) (cadr m1-home))
+	      (let* ((gr (make-granulate :expansion (/ mark-samps selection-samps))))
+		(map-channel (lambda (y)
+			       (+ y (granulate gr (lambda (dir)
+						    (if (or (>= inctr selection-samps)
+							    (< inctr 0))
+							0.0
+							(let ((val (vct-ref reg-data inctr)))
+							  (set! inctr (+ inctr dir))
+							  val))))))
+			     m1-samp mark-samps (car m1-home) (cadr m1-home))))))))
 
 
 ;;; -------- pad-marks inserts silence before each in a list of marks
 
 (define (pad-marks ids secs)
-  "(pad-marks ids secs) inserts secs seconds of silence before each mark in ids"
-  (let* ((silence-length (inexact->exact (floor (* secs (srate)))))
+  "(pad-marks marks secs) inserts secs seconds of silence before each mark"
+  (let* ((silence-length (floor (* secs (srate))))
 	 (silence-samps (make-vct silence-length)))
     (as-one-edit
      (lambda ()
@@ -174,7 +182,7 @@
 	      (channel (cadr (mark-home m)))
 	      (new-player (make-player sound channel)))
 	 (add-player new-player (mark-sample m))
-	 (set! chans (max chans (1+ channel)))
+	 (set! chans (max chans (+ 1 channel)))
 	 (set! rate (max rate (srate sound)))))
      (syncd-marks sync))
     (start-playing chans rate)))
@@ -207,11 +215,14 @@
 				  (find-another-mark (cdr ms)))))))))
       (if (and (mark? m1)
 	       (mark? m2))
-	  (play (mark-sample m1) 
-		(car (mark-home m1)) 
-		(cadr (mark-home m1)) 
-		#f 
-		(mark-sample m2))))))
+	  (let* ((pos1 (mark-sample m1))
+		 (pos2 (mark-sample m2))
+		 (beg (min pos1 pos2))
+		 (end (max pos1 pos2)))
+	    (play (car (mark-home m1)) 
+		  :channel (cadr (mark-home m1))
+		  :start beg 
+		  :end end))))))
 
 
 ;;; -------- report-mark-names causes mark names to be posted in the minibuffer as a sound is played
@@ -239,17 +250,20 @@
 		   (remove-local-hook! stop-playing-hook report-mark-names-stop-playing-hook))
 		 
 		 (add-hook! stop-playing-hook report-mark-names-stop-playing-hook)
-		 (add-hook! play-hook report-mark-names-play-hook)))))
+		 (add-hook! play-hook report-mark-names-play-hook)
+		 #f))))
 
 
 ;;; -------- eval-between-marks
 
 (define (eval-between-marks func)
   "(eval-between-marks func) evaluates func between the leftmost marks; func takes one arg, the original sample"
-  (define (find-if pred l) ; this is from guile/ice-9/common-list.scm but returns l not car l
+
+  (define (find-if pred l)
     (cond ((null? l) #f)
 	  ((pred (car l)) l)
 	  (else (find-if pred (cdr l)))))
+
   (if (procedure? func)
       ;; find leftmost two marks in selected chn
       (let ((chan (selected-channel))
@@ -265,25 +279,27 @@
 			   (len (- (mark-sample (cadr winl)) beg))
 			   (new-data (make-vct len))
 			   (old-data (channel->vct beg len snd chan)))
-		      (do ((k 0 (1+ k)))
+		      (do ((k 0 (+ 1 k)))
 			  ((= k len) (vct->channel new-data beg len snd chan))
 			(vct-set! new-data k (func (vct-ref old-data k))))))))))))
 
-;(bind-key (char->integer #\m) 0 (lambda () "eval between marks" (prompt-in-minibuffer "mark eval:" eval-between-marks)))
+;(bind-key #\m 0 (lambda () "eval between marks" (prompt-in-minibuffer "mark eval:" eval-between-marks)))
 
 
 ;;; -------- snap-marks
 
 (define (snap-marks)
   "snap-marks places marks at current selection boundaries"
-  (if (selection?)
-      (for-each 
-       (lambda (select)
-	 (let ((pos  (apply selection-position select))
-	       (len  (apply selection-frames select)))
-	   (apply add-mark pos select)
-	   (apply add-mark (+ pos len) select)))
-       (selection-members))))
+  (let ((ms (list)))
+    (if (selection?)
+	(for-each 
+	 (lambda (select)
+	   (let ((pos (apply selection-position select))
+		 (len (apply selection-frames select)))
+	     (set! ms (cons (apply add-mark pos select) ms))
+	     (set! ms (cons (apply add-mark (+ pos len) select) ms))))
+	(selection-members)))
+    (reverse ms)))
 
 
 ;;; -------- define-selection-via-marks
@@ -302,7 +318,7 @@
 	      (set! (selection-member? #t) #f)) ; clear entire current selection, if any
 	  (set! (selection-member? snd chn) #t)
 	  (set! (selection-position snd chn) beg)
-	  (set! (selection-frames snd chn) (1+ (- end beg)))))))
+	  (set! (selection-frames snd chn) (+ 1 (- end beg)))))))
 
 
 ;;; -------- snap-mark-to-beat
@@ -317,8 +333,8 @@
 			    (bps (/ (beats-per-minute snd chn) 60.0))
 			    (sr (srate snd))
 			    (beat (floor (/ (* samp bps) sr)))
-			    (lower (inexact->exact (floor (/ (* beat sr) bps))))
-			    (higher (inexact->exact (floor (/ (* (1+ beat) sr) bps)))))
+			    (lower (floor (/ (* beat sr) bps)))
+			    (higher (floor (/ (* (+ 1 beat) sr) bps))))
 		       (set! (mark-sample mrk)
 			     (if (< (- samp lower) (- higher samp))
 				 lower
@@ -328,7 +344,7 @@
 ;;;
 ;;; write out each section of a file between marks as a separate file
 
-(define* (mark-explode #:optional (htype mus-next) (dformat mus-bfloat))
+(define* (mark-explode :optional (htype mus-next) (dformat mus-bfloat))
   "(mark-explode :optional header-type data-format) splits a sound into a bunch of sounds based on mark placements"
   (let ((start 0)
 	(file-ctr 0)
@@ -338,15 +354,15 @@
        (let ((end (mark-sample mark)))
 	 (if (> end start)
 	     (let ((filename (format #f "mark-~D.snd" file-ctr)))
-	       (set! file-ctr (1+ file-ctr))
-	       (do ((i 0 (1+ i)))
-		   ((= i (chans snd)))
+	       (set! file-ctr (+ 1 file-ctr))
+	       (do ((i 0 (+ 1 i)))
+		   ((= i (channels snd)))
 		 (set! (selection-member? snd i) #t)
 		 (set! (selection-position snd i) start)
 		 (set! (selection-frames snd i) (- end start)))
 	       (save-selection filename :header-type htype :data-format dformat :srate (srate snd))
-	       (do ((i 0 (1+ i)))
-		   ((= i (chans snd)))
+	       (do ((i 0 (+ 1 i)))
+		   ((= i (channels snd)))
 		 (set! (selection-member? snd i) #f))))
 	 (set! start end)))
      (car (marks snd)))
@@ -355,26 +371,10 @@
 
 ;;; -------- mark property lists
 
-(define all-mark-properties '())
-
-(define mark-properties
-  (make-procedure-with-setter
-   (lambda (id)
-     (let ((data (assoc id all-mark-properties)))
-       (if data
-	   (cdr data)
-           '())))
-   (lambda (id new-val)
-     (let ((old-val (assoc id all-mark-properties)))
-       (if old-val
-	   (set-cdr! old-val new-val)
-	   (set! all-mark-properties (cons (cons id new-val) all-mark-properties)))
-       new-val))))
-     
 (define mark-property
   (make-procedure-with-setter
    (lambda (key id)
-     "(mark-property key id) returns the value associated with 'key' in the given mark's property list, or #f"
+     "(mark-property key mark) returns the value associated with 'key' in the given mark's property list, or #f"
      (if (mark? id)
 	 (let ((data (assoc key (mark-properties id))))
 	   (if data
@@ -390,20 +390,22 @@
 	   new-val)
 	 (throw 'no-such-mark (list "set! mark-property" id))))))
 
-(add-hook! close-hook
-	   (lambda (snd)
-	     (if (not (null? all-mark-properties))
-		 ;; prune out inactive mark properties
-		 (set! all-mark-properties (remove-if (lambda (val)
-						       (not (mark? (car val))))
-						     all-mark-properties)))
-	     #f))
-
 (define (save-mark-properties)
   "(save-mark-properties) sets up an after-save-state-hook function to save any mark-properties"
+
+  (define (open-appending filename)
+    (if (provided? 'snd-guile)
+	(open filename (logior O_RDWR O_APPEND))
+	(open-output-file filename "a")))
+
+  (define (close-appending fd)
+    (if (provided? 'snd-guile)
+	(close fd)
+	(close-output-port fd)))
+
   (add-hook! after-save-state-hook 
     (lambda (filename)
-      (let ((fd (open filename (logior O_RDWR O_APPEND))))
+      (let ((fd (open-appending filename)))
 	(format fd "~%~%;;; from remember-mark-properties in marks.scm~%")
 	(format fd "(if (not (defined? 'mark-property)) (load-from-path \"marks.scm\"))~%")
 	(for-each 
@@ -426,13 +428,13 @@
 	       chn-m))
 	    snd-m))
 	 (marks))
-	(close fd)))))
+	(close-appending fd)))))
 
 
 (define (mark-click-info n)
   "(mark-click-info n) is a mark-click-hook function that describes a mark and its properties"
   (help-dialog "Mark Help"
-	       (format #f "Mark ~D~A:~%  sample: ~D = ~,3F secs~A~A"
+	       (format #f "Mark ~A~A:~%  sample: ~D = ~,3F secs~A~A"
 		       n 
 		       (let ((name (mark-name n)))
 			 (if (> (string-length name) 0)
@@ -440,8 +442,8 @@
 			     ""))
 		       (mark-sample n)
 		       (/ (mark-sample n) (srate (car (mark-home n))))
-		       (if (not (= (mark-sync n) 0))
-			   (format #f "~%  sync: ~A" (mark-sync n))
+		       (if (not (= (sync n) 0))
+			   (format #f "~%  sync: ~A" (sync n))
 			   "")
 		       (let ((props (mark-properties n)))
 			 (if (and (list? props)
@@ -451,7 +453,7 @@
   #t)
 
 
-#!
+#|
 ;;; this code saves mark info in the sound file header, and reads it back in when the sound is later reopened
 
 (define (eval-header sndf)
@@ -462,7 +464,7 @@
 	      (lambda args #f))))
 
 (define (marks->string sndf)
-  (let ((str (format #f "(if (not (provided? 'snd-marks.scm)) (load \"marks.scm\"))~%(let ((m #f))~%"))
+  (let ((str (format #f "(if (not (provided? 'snd-marks.scm)) (load-from-path \"marks.scm\"))~%(let ((m #f))~%"))
 	(chan 0))
     (for-each
      (lambda (chan-marks)
@@ -471,16 +473,14 @@
 	  (set! str 
 		(string-append str 
 			       (format #f
-				       "  (set! m (add-mark ~A #f ~D))~%" 
+				       "  (set! m (add-mark ~A #f ~D ~A ~D))~%" 
 				       (mark-sample m)
-				       chan)))
-	  (if (and (string? (mark-name m))
-		   (> (string-length (mark-name m)) 0))
-	      (set! str 
-		    (string-append str 
-				   (format #f
-					   "  (set! (mark-name m) ~S)~%"
-					   (mark-name m)))))
+				       chan
+				       (if (and (string? (mark-name m))
+						(> (string-length (mark-name m)) 0))
+					   (format #f "~S" (mark-name m))
+					   #f)
+				       (sync m))))
 	  (if (not (null? (mark-properties m)))
 	      (set! str
 		    (string-append str 
@@ -488,10 +488,13 @@
 					   "  (set! (mark-properties m) '~A)~%"
 					   (mark-properties m))))))
 	  chan-marks)
-       (set! chan (1+ chan)))
+       (set! chan (+ 1 chan)))
      (marks sndf))
     (string-append str (format #f "  m)~%"))))
 		   
 (add-hook! output-comment-hook (lambda (str) (marks->string (selected-sound))))
 (add-hook! after-open-hook (lambda (snd) (eval-header snd)))
-!#
+|#
+
+
+
