@@ -1,4 +1,4 @@
-;;; maxf.scm -- CLM -> Snd/Guile translation of maxf.ins
+;;; maxf.scm -- CLM -> Snd/Scheme translation of maxf.ins
 
 ;; Translator/Author: Michael Scholz <scholz-micha@gmx.de>
 ;; Last: Tue Mar 25 04:32:23 CET 2003
@@ -35,10 +35,9 @@
 (use-modules (ice-9 format) (ice-9 optargs))
 (provide 'snd-maxf.scm)
 (if (not (provided? 'snd-ws.scm)) (load-from-path "ws.scm"))
-(if (not (provided? 'snd-jcrev.scm)) (load-from-path "jcrev.scm"))
+;;; (if (not (provided? 'snd-jcrev.scm)) (load-from-path "jcrev.scm"))
 
-(define pi 3.141592653589793)
-(define *locsig-type* mus-sinusoidal)
+(define *locsig-type* mus-interp-sinusoidal)
 
 (define (snd-msg frm . args)
   (let ((str (apply format (append (list #f frm) args))))
@@ -83,7 +82,7 @@
      (set! (mvm-pp2 b) (/ (* 2.0 pi ffreq) (mus-srate)))
      (set! (mvm-pp3 b) (* (mvm-pp2 b) famp))))
 
-(define* (maxfilter file beg #:key
+(define* (maxfilter file beg :key
 		    (att 1.0)
 		    (numf 1)
 		    (freqfactor 1.0)
@@ -92,7 +91,7 @@
 		    (degree (random 90.0))
 		    (distance 1.0)
 		    (reverb-amount 0.2))
-  "(maxfilter file beg #:key (att 1.0) (numf 1) (freqfactor 1.0)
+  "(maxfilter file beg :key (att 1.0) (numf 1) (freqfactor 1.0)
             (amplitude 1.0) (amp-env '(0 1 100 1))
             (degree (random 90.0)) (distance 1.0) (reverb-amount 0.2))
 
@@ -114,9 +113,9 @@ the desired phase.
    (numf  9)   9 filters
    (numf 12)  12 filters
    (numf 13)  13 filters"
-  (let* ((beg (inexact->exact (floor (* beg (mus-srate)))))
+  (let* ((beg (floor (* beg (mus-srate))))
 	 (dur (mus-sound-duration file))
-	 (end (+ beg (inexact->exact (floor (* dur (mus-srate))))))
+	 (end (+ beg (floor (* dur (mus-srate)))))
 	 (rdA (make-readin :file file :channel 0))
 	 (formfil (make-mvm))
 	 (ampf (make-env :envelope amp-env :scaler amplitude :duration dur))
@@ -126,8 +125,9 @@ the desired phase.
 	 (state-3 (make-array 0.0 13 3))
 	 (state-4 (make-array 0.0  4 3))
 	 (state-5 (make-array 0.0  2 3))
-	 (loc (make-locsig :degree degree :distance distance :channels (mus-channels *output*)
-			   :reverb reverb-amount :output *output* :revout *reverb*
+	 (loc (make-locsig :degree degree 
+			   :distance distance
+			   :reverb reverb-amount
 			   :type *locsig-type*)))
     (cond ((= numf 1)
 	   (snd-msg ";;;; State 0 (default): One filter~%")
@@ -309,11 +309,11 @@ the desired phase.
 	   (set! numf 1)))
     (run
      (lambda ()
-       (do ((i beg (1+ i)))
+       (do ((i beg (+ 1 i)))
 	   ((= i end))
 	 (let ((outvalA (* att (readin rdA)))
 	       (add-fl 0))
-	   (do ((j 0 (1+ j)))
+	   (do ((j 0 (+ 1 j)))
 	       ((= j numf))
 	     (cond ((= numf 1)
 		    (set-coeffs formfil (array-ref state-0 j 0)
@@ -346,7 +346,7 @@ the desired phase.
 ;;   (with-sound (:play 1 :statistics #t :channels 4 :output ofile :reverb jc-reverb
 ;; 			   :comment
 ;; 			   (format #f "maxfilter test, filters ~S, source ~A" number-ary ifile))
-;; 	      (do ((i 0 (1+ i))
+;; 	      (do ((i 0 (+ 1 i))
 ;; 		   (nary number-ary (cdr nary)))
 ;; 		  ((null? nary))
 ;; 		(maxfilter ifile i :numf (car nary) :degree (random 3454)))))
