@@ -1,21 +1,12 @@
 /* (simple, low level) midi support, work in progress... */
 /*   based on code of Scott Wilson and Craig Sapp */
+/* currently OSX, ALSA, SGI (both libraries), OSS */
 
-/*
- * int mus_midi_open_read(const char *name)
- * int mus_midi_open_write(const char *name)
- * int mus_midi_close(int line)
- * int mus_midi_read(int line, unsigned char *buffer, int bytes)
- * int mus_midi_write(int line, unsigned char *buffer, int bytes)
- * char *mus_midi_device_name(int sysdev)  [don't free result]
- * char *mus_midi_describe(void)           [free result]
- *
- *    not implemented (not sure it's needed): int mus_midi_flush(line)
- *
- * currently OSX, ALSA, SGI (both libraries), OSS
- */
+#include <mus-config.h>
 
-#include <config.h>
+#if USE_SND
+  #include "snd.h"
+#endif
 
 #include <stdio.h>
 #if HAVE_FCNTL_H
@@ -23,10 +14,10 @@
 #endif
 #include <errno.h>
 #include <stdlib.h>
-#if (defined(NEXT) || (defined(HAVE_LIBC_H) && (!defined(HAVE_UNISTD_H))))
+#if (defined(HAVE_LIBC_H) && (!defined(HAVE_UNISTD_H)))
   #include <libc.h>
 #else
-  #if (!(defined(_MSC_VER))) && (!(defined(MPW_C)))
+  #if (!(defined(_MSC_VER)))
     #include <unistd.h>
   #endif
 #endif
@@ -34,26 +25,8 @@
   #include <string.h>
 #endif
 
-#ifndef NO_SNDLIB
-  #include "sndlib.h"
-#else
-  int mus_midi_open_read(const char *name);
-  int mus_midi_open_write(const char *name);
-  int mus_midi_close(int line);
-  int mus_midi_read(int line, unsigned char *buffer, int bytes);
-  int mus_midi_write(int line, unsigned char *buffer, int bytes);
-  char *mus_midi_device_name(int sysdev);
-  char *mus_midi_describe(void);
-  #define MUS_AUDIO_SYSTEM(n) (((n) >> 16) & 0xffff)
-  #define MUS_AUDIO_DEVICE(n) ((n) & 0xffff)
-  #if HAVE_EXTENSION_LANGUAGE
-    void mus_midi_init(void);
-  #endif
-  #define CALLOC(a, b)  calloc((size_t)(a), (size_t)(b))
-  #define MALLOC(a)     malloc((size_t)(a))
-  #define FREE(a)       free(a)
-  #define REALLOC(a, b) realloc(a, (size_t)(b))
-#endif
+#include "_sndlib.h"
+
 
 
 /* ---------------- ALSA ---------------- */
@@ -73,7 +46,7 @@
 
 #ifdef SND_RAWMIDI_NONBLOCK
 
-#define MIDI_OK
+#define MIDI_OK 1
 #define DEV_BUFSIZE 1024  /* what is this for? */
 #define MIDI_READ 0     /* do we need MIDI_READ_WRITE? */
 #define MIDI_WRITE 1
@@ -84,16 +57,17 @@ static char **midi_names = NULL;
 static int *midi_directions = NULL;
 static int midis = 0;
 
+
 static int new_midi_line(const char *name, snd_rawmidi_t *line, snd_rawmidi_params_t *params, int input)
 {
   int i, loc = -1;
   if (midis == 0)
     {
       midis = 4;
-      midi_lines = (snd_rawmidi_t **)CALLOC(midis, sizeof(snd_rawmidi_t *));
-      midi_params = (snd_rawmidi_params_t **)CALLOC(midis, sizeof(snd_rawmidi_params_t *));
-      midi_names = (char **)CALLOC(midis, sizeof(char *));
-      midi_directions = (int *)CALLOC(midis, sizeof(int));
+      midi_lines = (snd_rawmidi_t **)calloc(midis, sizeof(snd_rawmidi_t *));
+      midi_params = (snd_rawmidi_params_t **)calloc(midis, sizeof(snd_rawmidi_params_t *));
+      midi_names = (char **)calloc(midis, sizeof(char *));
+      midi_directions = (int *)calloc(midis, sizeof(int));
       loc = 0;
     }
   else
@@ -108,19 +82,20 @@ static int new_midi_line(const char *name, snd_rawmidi_t *line, snd_rawmidi_para
 	{
 	  loc = midis;
 	  midis += 4;
-	  midi_lines = (snd_rawmidi_t **)REALLOC(midi_lines, midis * sizeof(snd_rawmidi_t *));
-	  midi_params = (snd_rawmidi_params_t **)REALLOC(midi_params, midis * sizeof(snd_rawmidi_params_t *));
-	  midi_names = (char **)REALLOC(midi_names, midis * sizeof(char *));
-	  midi_directions = (int *)REALLOC(midi_directions, midis * sizeof(int));
+	  midi_lines = (snd_rawmidi_t **)realloc(midi_lines, midis * sizeof(snd_rawmidi_t *));
+	  midi_params = (snd_rawmidi_params_t **)realloc(midi_params, midis * sizeof(snd_rawmidi_params_t *));
+	  midi_names = (char **)realloc(midi_names, midis * sizeof(char *));
+	  midi_directions = (int *)realloc(midi_directions, midis * sizeof(int));
 	}
     }
   midi_lines[loc] = line;
   midi_params[loc] = params;
-  midi_names[loc] = strdup(name);
+  midi_names[loc] = mus_strdup(name);
   midi_directions[loc] = input;
   return(loc);
 }
 
+
 static int midi_open(const char *name, int input)
 {
   int err; 
@@ -131,34 +106,27 @@ static int midi_open(const char *name, int input)
     err = snd_rawmidi_open(&line, NULL, name, SND_RAWMIDI_NONBLOCK);
   else err = snd_rawmidi_open(NULL, &line, name, SND_RAWMIDI_NONBLOCK);
   if (err)
-    {
-#ifdef NO_SNDLIB
-      fprintf(stderr, "can't open %s: %s\n", name, strerror(err));
-#else
-      return(mus_error(MUS_MIDI_OPEN_ERROR, "can't open %s: %s", name, strerror(err)));
-#endif
-      return(-1);
-    }
+    return(mus_error(MUS_MIDI_OPEN_ERROR, "can't open %s: %s", name, strerror(err)));
+
   snd_rawmidi_params_malloc(&params);
   err = snd_rawmidi_params_set_buffer_size(line, params, DEV_BUFSIZE);
   if (err)
     {
       snd_rawmidi_params_free(params);
       snd_rawmidi_close(line);
-#ifdef NO_SNDLIB
-      fprintf(stderr, "can't set %s buffer size to %d: %s\n", name, DEV_BUFSIZE, strerror(err));
-#else
       return(mus_error(MUS_MIDI_MISC_ERROR, "can't set %s buffer size to %d: %s", name, DEV_BUFSIZE, strerror(err)));
-#endif
-      return(-1);
     }
   return(new_midi_line(name, line, params, input));
 }
 
+
 /* name is apparently of the form "hw:%d,%d" card dev  or  "/dev/snd/midiC0D0"  */
+
 int mus_midi_open_read(const char *name)  {return(midi_open(name, MIDI_READ));}
+
 int mus_midi_open_write(const char *name) {return(midi_open(name, MIDI_WRITE));}
 
+
 int mus_midi_close(int line)
 {
   int err = 0;
@@ -172,50 +140,40 @@ int mus_midi_close(int line)
       midi_lines[line] = NULL;
     }
   if (err)
-    {
-#ifdef NO_SNDLIB
-      fprintf(stderr, "can't close %s: %s\n", midi_names[line], strerror(err));
-#else
-      return(mus_error(MUS_MIDI_CLOSE_ERROR, "can't close %s: %s", midi_names[line], strerror(err)));
-#endif
-      return(-1);
-    }
+    return(mus_error(MUS_MIDI_CLOSE_ERROR, "can't close %s: %s", midi_names[line], strerror(err)));
+
   return(0);
 }
 
+
 int mus_midi_read(int line, unsigned char *buffer, int bytes)
 {
   if ((line < 0) || (line >= midis)) return(-1);
   if (midi_directions[line] == MIDI_READ)
     return(snd_rawmidi_read(midi_lines[line], buffer, bytes));
-#ifdef NO_SNDLIB
-  fprintf(stderr, "can't read from output %s\n", midi_names[line]);
-#else
+
   return(mus_error(MUS_MIDI_READ_ERROR, "can't read from output %s", midi_names[line]));
-#endif
-  return(-1);
 }
 
+
 int mus_midi_write(int line, unsigned char *buffer, int bytes)
 {
   if ((line < 0) || (line >= midis)) return(-1);
   if (midi_directions[line] == MIDI_WRITE)
     return(snd_rawmidi_write(midi_lines[line], buffer, bytes));
-#ifdef NO_SNDLIB
-  fprintf(stderr, "can't write to input %s\n", midi_names[line]);
-#else
   return(mus_error(MUS_MIDI_WRITE_ERROR, "can't write to input %s", midi_names[line]));
-#endif
-  return(-1);
 }
 
+
 static char devname[64];
-char *mus_midi_device_name(int sysdev)
+
+const char *mus_midi_device_name(int sysdev)
 {
   sprintf(devname, "hw:%d,%d", MUS_AUDIO_SYSTEM(sysdev), MUS_AUDIO_DEVICE(sysdev));
-  return(devname);
+  return((const char *)devname);
 }
 
+
 /* snd_rawmidi_drop to clear or snd_rawmidi_drain|flush_output */
 
 char *mus_midi_describe(void) 
@@ -226,7 +184,7 @@ char *mus_midi_describe(void)
   char *buf = NULL;
   char one[256];
   snd_rawmidi_info_malloc(&info);
-  buf = (char *)CALLOC(1024, sizeof(char));
+  buf = (char *)calloc(1024, sizeof(char));
   for (i = 0; i < 8; i++)
     {
       /* presumably this should use the card|device from the info struct, not a blind loop */
@@ -255,11 +213,11 @@ char *mus_midi_describe(void)
 
 
 /* ---------------- SGI ---------------- */
-#ifdef SGI
+#ifdef MUS_SGI
 
 #include <dmedia/midi.h>
 
-#define MIDI_OK
+#define MIDI_OK 1
 #define MIDI_READ 0
 #define MIDI_WRITE 1
 
@@ -270,30 +228,28 @@ static int midis = 0;
 static int midi_initialized = 0;
 static int midi_ports = 0;
 
+
 static void mus_midi_initialize(void)
 {
   if (midi_initialized == 0)
     {
       midi_ports = mdInit();
       if (midi_ports == -1)
-#ifdef NO_SNDLIB
-	fprintf(stderr, "startmidi not called?");
-#else
-      mus_error(MUS_MIDI_INIT_ERROR, "startmidi not called?");
-#endif
+	mus_error(MUS_MIDI_INIT_ERROR, "startmidi not called?");
       midi_initialized = 1;
     }
 }
 
+
 static int new_midi_line(const char *name, MDport line, int input)
 {
   int i, loc = -1;
   if (midis == 0)
     {
       midis = 4;
-      midi_lines = (MDport *)CALLOC(midis, sizeof(MDport));
-      midi_names = (char **)CALLOC(midis, sizeof(char *));
-      midi_directions = (int *)CALLOC(midis, sizeof(int));
+      midi_lines = (MDport *)calloc(midis, sizeof(MDport));
+      midi_names = (char **)calloc(midis, sizeof(char *));
+      midi_directions = (int *)calloc(midis, sizeof(int));
       loc = 0;
     }
   else
@@ -308,17 +264,18 @@ static int new_midi_line(const char *name, MDport line, int input)
 	{
 	  loc = midis;
 	  midis += 4;
-	  midi_lines = (MDport *)REALLOC(midi_lines, midis * sizeof(MDport));
-	  midi_names = (char **)REALLOC(midi_names, midis * sizeof(char *));
-	  midi_directions = (int *)REALLOC(midi_directions, midis * sizeof(int));
+	  midi_lines = (MDport *)realloc(midi_lines, midis * sizeof(MDport));
+	  midi_names = (char **)realloc(midi_names, midis * sizeof(char *));
+	  midi_directions = (int *)realloc(midi_directions, midis * sizeof(int));
 	}
     }
   midi_lines[loc] = line;
-  midi_names[loc] = strdup(name);
+  midi_names[loc] = mus_strdup(name);
   midi_directions[loc] = input;
   return(loc);
 }
 
+
 int midi_open(const char *name, int input)
 {
   MDport md;
@@ -327,21 +284,18 @@ int midi_open(const char *name, int input)
     md = mdOpenInPort((char *)name);
   else md = mdOpenOutPort((char *)name);
   if (md == NULL)
-    {
-#ifdef NO_SNDLIB
-      fprintf(stderr, "can't open %s\n", name);
-      return(-1);
-#else
-      return(mus_error(MUS_MIDI_OPEN_ERROR, "can't open %s", name));
-#endif
-    }
+    return(mus_error(MUS_MIDI_OPEN_ERROR, "can't open %s", name));
+
   mdSetStampMode(md, MD_NOSTAMP);
   return(new_midi_line(name, md, input));
 }
 
+
 int mus_midi_open_read(const char *name)  {return(midi_open(name, MIDI_READ));}
+
 int mus_midi_open_write(const char *name) {return(midi_open(name, MIDI_WRITE));}
 
+
 int mus_midi_close(int line) 
 {
   if ((line < 0) || (line >= midis)) return(-1);
@@ -350,6 +304,7 @@ int mus_midi_close(int line)
   return(0);
 }
 
+
 int mus_midi_read(int line, unsigned char *buffer, int bytes)
 {
   MDevent mdv;
@@ -366,14 +321,10 @@ int mus_midi_read(int line, unsigned char *buffer, int bytes)
 	}
       return(bytes);
     }
-#ifdef NO_SNDLIB
-  fprintf(stderr, "can't read from output %s\n", midi_names[line]);
-  return(-1);
-#else
   return(mus_error(MUS_MIDI_READ_ERROR, "can't read from output %s", midi_names[line]));
-#endif
 }
 
+
 int mus_midi_write(int line, unsigned char *buffer, int bytes)
 {
   int i;
@@ -394,22 +345,19 @@ int mus_midi_write(int line, unsigned char *buffer, int bytes)
 	}
       return(bytes);
     }
-#ifdef NO_SNDLIB
-  fprintf(stderr, "can't write to input %s\n", midi_names[line]);
-  return(-1);
-#else
   return(mus_error(MUS_MIDI_WRITE_ERROR, "can't write to input %s", midi_names[line]));
-#endif
 }
 
-char *mus_midi_device_name(int sysdev)
+
+const char *mus_midi_device_name(int sysdev)
 {
   mus_midi_initialize();
   if (midi_ports > 0)
-    return(mdGetName(MUS_AUDIO_DEVICE(sysdev)));
+    return((const char *)mdGetName(MUS_AUDIO_DEVICE(sysdev)));
   return(NULL);
 }
 
+
 char *mus_midi_describe(void)
 {
   int i;
@@ -418,7 +366,7 @@ char *mus_midi_describe(void)
   mus_midi_initialize(); 
   if (midi_ports > 0)
     {
-      buf = (char *)CALLOC(midi_ports * 64, sizeof(char));
+      buf = (char *)calloc(midi_ports * 64, sizeof(char));
       for (i = 0; i < midi_ports; i++)
 	{
 	  sprintf(name, "%s\n", mdGetName(i));
@@ -432,25 +380,27 @@ char *mus_midi_describe(void)
 #endif
 
 
+
 /* ---------------- OSS ---------------- */
-#if HAVE_OSS
-#define MIDI_OK
+
+#if HAVE_OSS && (!HAVE_ALSA)
+#define MIDI_OK 1
 
 #include <sys/ioctl.h>
 
-#if (USR_LIB_OSS)
+#if (MUS_HAVE_USR_LIB_OSS)
   #include "/usr/lib/oss/include/sys/soundcard.h"
 #else
-  #if (USR_LOCAL_LIB_OSS)
+  #if (MUS_HAVE_USR_LOCAL_LIB_OSS)
     #include "/usr/local/lib/oss/include/sys/soundcard.h"
   #else
-    #if (OPT_OSS)
+    #if (MUS_HAVE_OPT_OSS)
       #include "/opt/oss/include/sys/soundcard.h"
     #else
-      #if (VAR_LIB_OSS)
+      #if (MUS_HAVE_VAR_LIB_OSS)
         #include "/var/lib/oss/include/sys/soundcard.h"
       #else
-        #if defined(HAVE_SYS_SOUNDCARD_H) || defined(LINUX)
+        #if defined(HAVE_SYS_SOUNDCARD_H) || defined(MUS_LINUX)
           #include <sys/soundcard.h>
         #else
           #if defined(HAVE_MACHINE_SOUNDCARD_H)
@@ -465,11 +415,17 @@ char *mus_midi_describe(void)
 #endif
 
 int mus_midi_open_read(const char *name) {return(open(name, O_RDONLY, O_NONBLOCK));}  /* name should be "/dev/sequencer" */
+
 int mus_midi_open_write(const char *name) {return(open(name, O_RDWR, O_NONBLOCK));}   /* O_WRONLY? */
+
 int mus_midi_close(int line) {return(close(line));}
+
 int mus_midi_read(int line, unsigned char *buffer, int bytes) {return(read(line, buffer, bytes));}
+
 int mus_midi_write(int line, unsigned char *buffer, int bytes) {return(write(line, buffer, bytes));}
-char *mus_midi_device_name(int sysdev) {return("/dev/sequencer");}
+
+const char *mus_midi_device_name(int sysdev) {return("/dev/sequencer");}
+
 
 char *mus_midi_describe(void)
 {
@@ -488,13 +444,13 @@ char *mus_midi_describe(void)
 	  /* not strdup for memcheck consistency */
 	  char *newstr;
 	  close(fd);
-	  newstr = (char *)CALLOC(16, sizeof(char));
+	  newstr = (char *)calloc(16, sizeof(char));
 	  strcpy(newstr, "no midi");
 	  return(newstr);
 	}
       else
 	{
-	  buf = (char *)CALLOC(1024, sizeof(char));
+	  buf = (char *)calloc(1024, sizeof(char));
 	  sprintf(buf, "%d midi device%s installed\n", numdevs, (numdevs == 1) ? "" : "s"); 
 	  for (i = 0; i < numdevs; i++)
 	    {
@@ -517,12 +473,13 @@ char *mus_midi_describe(void)
 
 /* ---------------- Mac OSX ---------------- */
 
-#ifdef MAC_OSX
-#define MIDI_OK
+#ifdef MUS_MAC_OSX
+#define MIDI_OK 1
 #include <CoreMIDI/MIDIServices.h>
 
 /* for now, just one in and one out */
 
+
 char *mus_midi_describe(void)
 {
   int i, n;
@@ -532,8 +489,8 @@ char *mus_midi_describe(void)
   char *buf;
   n = MIDIGetNumberOfDevices();
   if (n <= 0)
-    buf = (char *)CALLOC(192, sizeof(char));
-  else buf = (char *)CALLOC(n * 192, sizeof(char));
+    buf = (char *)calloc(192, sizeof(char));
+  else buf = (char *)calloc(n * 192, sizeof(char));
   if (n <= 0)
     sprintf(buf, "no midi");
   else
@@ -557,6 +514,7 @@ char *mus_midi_describe(void)
   return(buf);
 }
 
+
 #define BUFFER_SIZE 1024
 static MIDIClientRef us = NULL;
 static MIDIPortRef inp = NULL, outp = NULL;
@@ -567,12 +525,13 @@ static void init_midi(void)
 {
   if (buffer == NULL)
     {
-      buffer = (unsigned char *)CALLOC(BUFFER_SIZE, sizeof(unsigned char));
+      buffer = (unsigned char *)calloc(BUFFER_SIZE, sizeof(unsigned char));
       reader = 0;
       writer = 0;
     }
 }
 
+
 static void midi_read_callback(const MIDIPacketList *list, void *ref, void *con)
 {
   int i, j;
@@ -590,6 +549,7 @@ static void midi_read_callback(const MIDIPacketList *list, void *ref, void *con)
     }
 }
 
+
 int mus_midi_read(int line, unsigned char *buf, int bytes)
 {
   int i;
@@ -604,7 +564,9 @@ int mus_midi_read(int line, unsigned char *buf, int bytes)
   return(bytes);
 }
 
+
 static Byte bb[1024];
+
 int mus_midi_write(int line, unsigned char *buffer, int bytes)
 {
   MIDIPacketList *list = (MIDIPacketList *)bb;
@@ -617,6 +579,7 @@ int mus_midi_write(int line, unsigned char *buffer, int bytes)
   return(0);
 }
 
+
 int mus_midi_open_read(const char *name)
 {
   init_midi();
@@ -628,6 +591,7 @@ int mus_midi_open_read(const char *name)
   return(0);
 }
 
+
 int mus_midi_open_write(const char *name)
 {
   init_midi();
@@ -638,31 +602,35 @@ int mus_midi_open_write(const char *name)
   return(1);
 }
 
+
 int mus_midi_close(int line)
 {
   return(0);
 }
 
-char *mus_midi_device_name(int sysdev) {return("none");}
+
+const char *mus_midi_device_name(int sysdev) {return("none");}
 
 #endif
 
 
 
 /* ---------------- stubs ---------------- */
+
 #ifndef MIDI_OK
 int mus_midi_open_read(const char *name) {return(-1);}
 int mus_midi_open_write(const char *name) {return(-1);}
 int mus_midi_close(int line) {return(-1);}
 int mus_midi_read(int line, unsigned char *buffer, int bytes) {return(-1);}
 int mus_midi_write(int line, unsigned char *buffer, int bytes) {return(-1);}
-char *mus_midi_device_name(int sysdev) {return("none");} /* result should not be freed by caller */
+const char *mus_midi_device_name(int sysdev) {return("none");}
 char *mus_midi_describe(void) {return(NULL);}            /* result should be freed by caller (if not NULL) */
 #endif
 
 
 /* ---------------- XEN ---------------- */
-/* tie foregoing into Xen (Guile or Ruby) */
+
+/* tie foregoing into Xen */
 
 #if HAVE_EXTENSION_LANGUAGE
 
@@ -676,6 +644,7 @@ char *mus_midi_describe(void) {return(NULL);}            /* result should be fre
 #define S_mus_midi_device_name "mus-midi-device-name"
 #define S_mus_midi_describe    "mus-midi-describe"
 
+
 static XEN g_mus_midi_open_read(XEN name)
 {
   #define H_mus_midi_open_read "(" S_mus_midi_open_read " name): opens midi input port, returns int id"
@@ -683,6 +652,7 @@ static XEN g_mus_midi_open_read(XEN name)
   return(C_TO_XEN_INT(mus_midi_open_read(XEN_TO_C_STRING(name))));
 }
 
+
 static XEN g_mus_midi_open_write(XEN name)
 {
   #define H_mus_midi_open_write "(" S_mus_midi_open_write " name): opens midi output port, returns int id"
@@ -690,6 +660,7 @@ static XEN g_mus_midi_open_write(XEN name)
   return(C_TO_XEN_INT(mus_midi_open_write(XEN_TO_C_STRING(name))));
 }
 
+
 static XEN g_mus_midi_close(XEN line)
 {
   #define H_mus_midi_close "(" S_mus_midi_close " line): closes midi port"
@@ -697,6 +668,7 @@ static XEN g_mus_midi_close(XEN line)
   return(C_TO_XEN_INT(mus_midi_close(XEN_TO_C_INT(line))));
 }
 
+
 static XEN g_mus_midi_read(XEN line, XEN bytes) /* returns list of midi bytes */
 {
   #define H_mus_midi_read "(" S_mus_midi_read " line bytes): reads bytes from midi port, returns list"
@@ -706,19 +678,20 @@ static XEN g_mus_midi_read(XEN line, XEN bytes) /* returns list of midi bytes */
   XEN_ASSERT_TYPE(XEN_INTEGER_P(line), line, XEN_ARG_1, S_mus_midi_read, "an integer");
   XEN_ASSERT_TYPE(XEN_INTEGER_P(bytes), bytes, XEN_ARG_2, S_mus_midi_read, "an integer");
   len = XEN_TO_C_INT(bytes);
-  buf = (unsigned char *)CALLOC(len, sizeof(unsigned char));
+  buf = (unsigned char *)calloc(len, sizeof(unsigned char));
   err = mus_midi_read(XEN_TO_C_INT(line), buf, len);
   if (err == -1)
     {
-      FREE(buf);
+      free(buf);
       return(XEN_FALSE);
     }
   for (i = err; i >= 0; i--)
     lst = XEN_CONS(C_TO_XEN_INT((int)buf[i]), lst);
-  FREE(buf);
+  free(buf);
   return(lst);
 }
 
+
 static XEN g_mus_midi_write(XEN line, XEN buffer)
 {
   #define H_mus_midi_write "(" S_mus_midi_write " line byte-list): sends byte-list to midi port"
@@ -728,24 +701,26 @@ static XEN g_mus_midi_write(XEN line, XEN buffer)
   XEN_ASSERT_TYPE(XEN_INTEGER_P(line), line, XEN_ARG_1, S_mus_midi_write, "an integer");
   XEN_ASSERT_TYPE(XEN_LIST_P(buffer), buffer, XEN_ARG_2, S_mus_midi_write, "a list");
   len = XEN_LIST_LENGTH(buffer);
-  buf = (unsigned char *)CALLOC(len, sizeof(unsigned char));
+  buf = (unsigned char *)calloc(len, sizeof(unsigned char));
   for (i = 0, lst = XEN_COPY_ARG(buffer); i < len; i++, lst = XEN_CDR(lst))
     buf[i] = (unsigned char)(XEN_TO_C_INT(XEN_CAR(lst)));
   err = mus_midi_write(XEN_TO_C_INT(line), buf, len);
-  FREE(buf);
+  free(buf);
   if (err == -1)
     return(XEN_FALSE);
   return(C_TO_XEN_INT(err));
 }
 
+
 static XEN g_mus_midi_device_name(XEN dev)
 {
   /* sndlib style sys|dev packing, dev optional */
-  #define H_mus_midi_device_name "(" S_mus_midi_device_name " (sys-dev 0)): returns a name suitable for " S_mus_midi_open_read " and write"
+  #define H_mus_midi_device_name "(" S_mus_midi_device_name " :optional (sys-dev 0)): returns a name suitable for " S_mus_midi_open_read " and write"
   XEN_ASSERT_TYPE(XEN_INTEGER_IF_BOUND_P(dev), dev, XEN_ONLY_ARG, S_mus_midi_device_name, "and integer");
   return(C_TO_XEN_STRING(mus_midi_device_name(XEN_TO_C_INT_OR_ELSE(dev, 0))));
 }
 
+
 static XEN g_mus_midi_describe(void)
 {
   #define H_mus_midi_describe "(" S_mus_midi_describe "): returns a description of the midi hardware"
@@ -755,11 +730,12 @@ static XEN g_mus_midi_describe(void)
   if (str)
     {
       res = C_TO_XEN_STRING(str);
-      FREE(str);
+      free(str);
     }
   return(res);
 }
 
+
 #ifdef XEN_ARGIFY_1
   XEN_NARGIFY_1(g_mus_midi_open_read_w, g_mus_midi_open_read)
   XEN_NARGIFY_1(g_mus_midi_open_write_w, g_mus_midi_open_write)
