@@ -17,18 +17,17 @@
 (if (not (provided? 'xm))
     (let ((hxm (dlopen "xm.so")))
       (if (string? hxm)
-	  (snd-error (format #f "popup.scm needs the xm module: ~A" hxm))
-	  (dlinit hxm "init_xm"))))
+	  (snd-error (format #f "popup.scm needs the xm module (either 'make xm' or build Snd with --with-static-xm): ~A" hxm))
+	  (dlinit hxm "Init_libxm"))))
 
-(if (not (defined? 'for-each-child))
-    (define (for-each-child w func)
-      "(for-each-child widget func) applies func to widget and each of its children"
-      (func w)
-      (if (XtIsComposite w)
-	  (for-each 
-	   (lambda (n)
-	     (for-each-child n func))
-	   (cadr (XtGetValues w (list XmNchildren 0) 1))))))
+(define (for-each-child w func)
+  "(for-each-child widget func) applies func to widget and each of its children"
+  (func w)
+  (if (XtIsComposite w)
+      (for-each 
+       (lambda (n)
+	 (for-each-child n func))
+       (cadr (XtGetValues w (list XmNchildren 0) 1)))))
 
 (define (change-label w new-label)
   "(change-label widget new-label) changes widget's label to be new-label"
@@ -39,9 +38,7 @@
 (define (current-label w)
   "(current-label widget) returns widget's label"
   (let ((xmstr (cadr (XtGetValues w (list XmNlabelString 0)))))
-    (cadr (XmStringGetLtoR xmstr XmFONTLIST_DEFAULT_TAG))))
-
-
+    (XmStringUnparse xmstr #f XmCHARSET_TEXT XmCHARSET_TEXT #f 0 XmOUTPUT_ALL)))
 
 (define (make-popup-menu name parent top-args entries)
   "(make-popup-menu name parent top-args entries) creates a popup menu"
@@ -73,7 +70,7 @@
     (make-popup-menu 
      "selection-popup"
      (caddr (main-widgets))
-     (list XmNpopupEnabled #t
+     (list XmNpopupEnabled XmPOPUP_AUTOMATIC
 	   XmNbackground (highlight-color))
      (list
       (list "Selection" xmLabelWidgetClass      every-menu)
@@ -137,13 +134,14 @@
 			  (delete-samples (+ len 1) (- (frames snd chn) len) snd chn))))))
 	       (let ((sndlist '()))
 		 (for-each (lambda (snd)
-			     (do ((i (1- (channels snd)) (1- i)))
+			     (do ((i (- (channels snd) 1) (- i 1)))
 				 ((< i 0))
 			       (if (selection-member? snd i)
 				   (set! sndlist (cons (list snd i) sndlist)))))
 			   (sounds))
 		 sndlist))))
       (list "Save as"   xmPushButtonWidgetClass every-menu (lambda (w c i) (save-selection-dialog)))
+      (list "Selection->Mix"  xmPushButtonWidgetClass every-menu (lambda (w c i) (selection->mix)))
       (list "Copy->New" xmPushButtonWidgetClass every-menu 
 	    (lambda (w c i) 
 	      (let ((new-file-name (snd-tempnam)))
@@ -164,31 +162,25 @@
 		   (apply add-mark pos select)
 		   (apply add-mark (+ pos len) select)))
 	       (selection-members))))
-      (list "Info"      xmPushButtonWidgetClass every-menu 
+      (list "Selection Info"      xmPushButtonWidgetClass every-menu 
 	    (lambda (w c i)
 	      (let ((beg (selection-position))
 		    (len (selection-frames)))
 		(info-dialog 
 		 "Selection info"
-		 (format #f "start ~A, ~,3F~%end: ~A, ~,3F~%duration: ~A, ~,3F~%chans: ~D~%maxamp: ~,3F, rms: ~,3F"
+		 (format #f "start ~A, ~,3F~%end: ~A, ~,3F~%duration: ~A, ~,3F~%chans: ~D~%maxamp: ~,3F"
 			 beg (exact->inexact (/ beg (srate)))
 			 (+ beg len) (exact->inexact (/ (+ beg len) (srate)))
 			 len (exact->inexact (/ len (srate)))
 			 (selection-chans)
-			 (selection-maxamp)
-			 (let* ((reader (make-sample-reader beg))
-				(sum 0.0))
-			   (do ((i 0 (1+ i))) 
-			       ((= i len) 
-				(sqrt (/ sum len)))
-			     (let ((val (next-sample reader)))
-			       (set! sum (+ sum (* val val)))))))))))
+			 (selection-maxamp))))))
       (list "Apply controls" xmPushButtonWidgetClass every-menu (lambda (w c i) (apply-controls (selected-sound) 2))) ; 2=selection
       (list "Reset controls" xmPushButtonWidgetClass every-menu (lambda (w c i) (reset-controls)))
       (list "Unselect"       xmPushButtonWidgetClass every-menu (lambda (w c i) (set! (selection-member? #t) #f)))
       (list "Reverse"        xmPushButtonWidgetClass every-menu (lambda (w c i) (reverse-selection)))
-      (list "Mix"            xmPushButtonWidgetClass every-menu (lambda (w c i) (mix-selection (cursor))))
-      (list "Invert"         xmPushButtonWidgetClass every-menu (lambda (w c i) (scale-selection-by -1)))))))
+      (list "Mix (at cursor)"  xmPushButtonWidgetClass every-menu (lambda (w c i) (mix-selection (cursor))))
+;     (list "Invert"         xmPushButtonWidgetClass every-menu (lambda (w c i) (scale-selection-by -1)))
+      ))))
 
 
 ;;; -------- time domain popup
@@ -201,14 +193,16 @@
   (let ((every-menu (list XmNbackground (highlight-color)))
 	(stopping #f)
 	(stop-widget #f))
+
     (define (vector-print v)
-      (if (< (vector-length v) 3)
+      (if (< (length v) 3)
 	  (object->string v)
 	  (let ((str (format #f "'#(~A" (vector-ref v 0))))
-	    (do ((i 1 (1+ i)))
+	    (do ((i 1 (+ 1 i)))
 		((= i 3))
 	      (set! str (string-append str " " (object->string (vector-ref v i)))))
 	    (string-append str " ...)"))))
+
     (define (display-properties props)
       ;; there's no way to tell Guile's format that enormous vectors should not be printed in full
       ;; so we search for them here and handle them ourselves
@@ -248,7 +242,7 @@
     (make-popup-menu 
      "graph-popup"
      (caddr (main-widgets))
-     (list XmNpopupEnabled #t
+     (list XmNpopupEnabled XmPOPUP_AUTOMATIC
 	   XmNbackground (highlight-color))
      (list
 
@@ -264,29 +258,29 @@
 		  (begin
 		    (change-label w "Stop")
 		    (set! stopping #t)
-		    (play 0 graph-popup-snd))))
+		    (play graph-popup-snd 0))))
 	    (lambda (wid)
 	      (set! stop-widget wid)))
       (list "Play channel"       xmPushButtonWidgetClass every-menu 
 	    (lambda (w c i)
 	      (set! stopping #t)
 	      (change-label stop-widget "Stop")
-	      (play 0 graph-popup-snd graph-popup-chn)))
+	      (play graph-popup-snd 0 :channel graph-popup-chn)))
       (list "Play from cursor"   xmPushButtonWidgetClass every-menu 
 	    (lambda (w c i)
 	      (set! stopping #t)
 	      (change-label stop-widget "Stop")
-	      (play (cursor graph-popup-snd graph-popup-chn) graph-popup-snd)))
+	      (play graph-popup-snd :start (cursor graph-popup-snd graph-popup-chn))))
       (list "Play previous"      xmPushButtonWidgetClass every-menu
 	    (lambda (w c i)
 	      (set! stopping #t)
 	      (change-label stop-widget "Stop")
-	      (play 0 graph-popup-snd graph-popup-chn #f #f (1- (edit-position)))))  ; play version before most-recent edit
+	      (play graph-popup-snd 0 :channel graph-popup-chn :edit-position (- (edit-position) 1))))  ; play version before most-recent edit
       (list "Play original"      xmPushButtonWidgetClass every-menu
 	    (lambda (w c i)
 	      (set! stopping #t)
 	      (change-label stop-widget "Stop")
-	      (play 0 graph-popup-snd graph-popup-chn #f #f 0)))                     ; play unedited version
+	      (play graph-popup-snd 0 :channel graph-popup-chn :edit-position 0)))                     ; play unedited version
       (list "Undo"               xmPushButtonWidgetClass every-menu 
 	    (lambda (w c i)
 	      (undo 1 graph-popup-snd graph-popup-chn)))
@@ -335,9 +329,6 @@
       (list "Unselect"           xmPushButtonWidgetClass every-menu 
 	    (lambda (w c i) 
 	      (set! (selection-member? #t) #f)))
-      (list "Equalize panes"     xmPushButtonWidgetClass every-menu 
-	    (lambda (w c i)
-	      (equalize-panes)))
       (list "Apply controls"     xmPushButtonWidgetClass every-menu
 	    (lambda (w c i)
 	      (apply-controls)))
@@ -351,29 +342,29 @@
 		 (format #f "~A info" (file-name snd))
 		 (format #f "~A:~%  chans: ~D~%  srate: ~D~%  header: ~A~%  data format: ~A~%  length: ~1,3F~%  maxamp: ~A~%~A~A~A~A~A"
 			(short-file-name snd)
-			(chans snd)
+			(channels snd)
 			(srate snd)
 			(mus-header-type-name (header-type snd))
 			(mus-data-format-name (data-format snd))
 			(exact->inexact (/ (frames snd graph-popup-chn) (srate snd)))
 			(maxamp snd #t)
 			(if (comment snd)
-			    (format #f "  comment: ~A~%" (comment snd))
+			    (format #f "  comment: \"~A\"~%" (comment snd))
 			    "")
 			(let ((loops (mus-sound-loop-info (file-name snd))))
 			  (if (not (null? loops))
 			      (format #f "  loop: ~A~%" loops)
 			      ""))
 			(if (= (header-type snd) mus-soundfont)
-			    (format #f "  sounds: ~:{~%     ~S start: ~A, loop: ~A ~A~}" (soundfont-info))
+			    (format #f "  sounds: ~{~{~%     ~S start: ~A, loop: ~A ~A~}~}" (soundfont-info))
 			    "")
 			(if (not (null? (sound-properties snd)))
 			    (format #f "  properties: ~A"
 				    (display-properties (sound-properties snd)))
 			    "")
 			(let ((chan-str ""))
-			  (do ((i 0 (1+ i)))
-			      ((= i (chans snd)))
+			  (do ((i 0 (+ 1 i)))
+			      ((= i (channels snd)))
 			    (if (not (null? (channel-properties snd i)))
 				(set! chan-str 
 				      (string-append chan-str
@@ -427,7 +418,7 @@
      (lambda (w)
        (let ((name (XtName w)))
 	 (if (string=? name "Snd")
-	     (if (> (chans snd) 1)
+	     (if (> (channels snd) 1)
 		 (change-label w (format #f "~A[~D]" (short-file-name snd) chn))
 		 (change-label w (short-file-name snd)))
 	     (if (or (string=? name "Save")
@@ -436,9 +427,9 @@
 		     (string=? name "Play previous"))
 		 ((if (> (car eds) 0) XtManageChild XtUnmanageChild) w)
 		 (if (string=? name "Play channel")
-		     ((if (> (chans snd) 1) XtManageChild XtUnmanageChild) w)
+		     ((if (> (channels snd) 1) XtManageChild XtUnmanageChild) w)
 		     (if (string=? name "Equalize panes")
-			 ((if (or (> (chans snd) 1) 
+			 ((if (or (> (channels snd) 1) 
 				  (> (length (sounds)) 1))
 			      XtManageChild XtUnmanageChild) w)
 			 (if (string=? name "Redo")
@@ -493,7 +484,7 @@
   ;; used within graph if pointer is in the fft graph
   (let* ((every-menu (list XmNbackground (highlight-color)))
 	 (fft-popup (XmCreatePopupMenu (caddr (main-widgets)) "fft-popup"
-		       (append (list XmNpopupEnabled #t) every-menu))))
+		       (append (list XmNpopupEnabled XmPOPUP_AUTOMATIC) every-menu))))
 
     (define (choose-chan)
       (if (= (channel-style graph-popup-snd) channels-separate) graph-popup-chn #t))
@@ -560,7 +551,10 @@
 
     (let ((windows (list rectangular-window hann-window welch-window parzen-window bartlett-window hamming-window blackman2-window 
 			 blackman3-window blackman4-window exponential-window riemann-window kaiser-window cauchy-window 
-			 poisson-window gaussian-window tukey-window dolph-chebyshev-window hann-poisson-window connes-window)))
+			 poisson-window gaussian-window tukey-window dolph-chebyshev-window hann-poisson-window connes-window
+			 samaraki-window ultraspherical-window bartlett-hann-window bohman-window flat-top-window
+			 blackman5-window blackman6-window blackman7-window blackman8-window blackman9-window blackman10-window 
+			 )))
       (make-simple-popdown-menu
        "Window"
        (map (lambda (name val)
@@ -568,7 +562,10 @@
 		    (lambda (w c i) 
 		      (set! (fft-window graph-popup-snd (choose-chan)) val))))
 	    (list "Rectangular" "Hann" "Welch" "Parzen" "Bartlett" "Hamming" "Blackman2" "Blackman3" "Blackman4"
-		  "Exponential" "Riemann" "Kaiser" "Cauchy" "Poisson" "Gaussian" "Tukey" "Dolph-Chebyshev" "Hann-Poisson" "Connes")
+		  "Exponential" "Riemann" "Kaiser" "Cauchy" "Poisson" "Gaussian" "Tukey" "Dolph-Chebyshev" "Hann-Poisson" 
+		  "Connes" "Samaraki" "Ultraspherical" "Bartlett-Hann" "Bohman" "Flat-top"
+		  "Blackman5" "Blackman6" "Blackman7" "Blackman8" "Blackman9" "Blackman10" 
+		  )
 	    windows)
        fft-popup
        (lambda (lst)
@@ -591,7 +588,7 @@
        (lambda (lst)
 	 (for-each 
 	  (lambda (child type)
-	    (XtSetSensitive child (not (= (transform-type graph-popup-snd graph-popup-chn) type))))
+	    (XtSetSensitive child (not (equal? (transform-type graph-popup-snd graph-popup-chn) type))))
 	  lst types))
        every-menu))
 
@@ -614,11 +611,8 @@
 	  lst vals))
        every-menu))
 
-    (let ((color (XtCreateManagedWidget "Color" xmPushButtonWidgetClass fft-popup every-menu)))
-      (XtAddCallback color XmNactivateCallback (lambda (w c i) (color-dialog))))
-
-    (let ((orient (XtCreateManagedWidget "Orientation" xmPushButtonWidgetClass fft-popup every-menu)))
-      (XtAddCallback orient XmNactivateCallback (lambda (w c i) (orientation-dialog))))
+    (let ((color (XtCreateManagedWidget "Color/Orientation" xmPushButtonWidgetClass fft-popup every-menu)))
+      (XtAddCallback color XmNactivateCallback (lambda (w c i) (color-orientation-dialog))))
 
     fft-popup))
 
@@ -637,7 +631,7 @@
 (define (edhist-save-edits w c i)
   (let* ((old-val (assoc (cons edhist-snd edhist-chn) edhist-funcs))
 	 (cur-edits (edits edhist-snd edhist-chn))
-	 (new-func (edit-list->function edhist-snd edhist-chn (1+ (car cur-edits)) (apply + cur-edits))))
+	 (new-func (edit-list->function edhist-snd edhist-chn (+ 1 (car cur-edits)) (apply + cur-edits))))
     (if old-val
 	(set-cdr! old-val new-func)
 	;; perhaps this should save the previous function under the current file name?
@@ -656,7 +650,7 @@
     (let ((index (cadr (XtVaGetValues w (list XmNuserData 0)))))
       (if (and (> index 0)
 	       (<= index (length edhist-funcs)))
-	  ((cdr (list-ref edhist-funcs (1- index))) edhist-snd edhist-chn))))
+	  ((cdr (list-ref edhist-funcs (- index 1))) edhist-snd edhist-chn))))
 	  
   (let ((wids (cdr edhist-widgets))
 	(funcs edhist-funcs)
@@ -679,7 +673,7 @@
 		 (change-label button (format #f "~A[~A]" (short-file-name (car label)) (cdr label)))
 		 (change-label button label))
 	     (XtVaSetValues button (list XmNuserData index))
-	     (set! index (1+ index))))
+	     (set! index (+ 1 index))))
 	 edhist-funcs))
     (if (not (null? wids))
 	(for-each
@@ -702,7 +696,7 @@ all saved edit lists."
 (define edit-history-menu
   (let* ((every-menu (list XmNbackground (highlight-color)))
 	 (edhist-popup (XmCreatePopupMenu (caddr (main-widgets)) "edhist-popup"
-					  (append (list XmNpopupEnabled #t) every-menu))))
+					  (append (list XmNpopupEnabled XmPOPUP_AUTOMATIC) every-menu))))
     (XtCreateManagedWidget "Edits" xmLabelWidgetClass edhist-popup every-menu)
     (XtCreateManagedWidget "sep" xmSeparatorWidgetClass edhist-popup every-menu)
     (let ((edhist-save (XtCreateManagedWidget "Save" xmPushButtonWidgetClass edhist-popup every-menu)))
@@ -720,9 +714,9 @@ all saved edit lists."
     (let ((edhist-help (XtCreateManagedWidget "Help" xmPushButtonWidgetClass edhist-popup every-menu)))
       (XtAddCallback edhist-help XmNactivateCallback edhist-help-edits))
     (add-hook! close-hook (lambda (snd)
-			    (let ((chns (chans snd))
+			    (let ((chns (channels snd))
 				  (name (short-file-name snd)))
-			      (do ((i 0 (1+ i)))
+			      (do ((i 0 (+ 1 i)))
 				  ((= i chns))
 				(let* ((old-val (assoc (cons snd i) edhist-funcs)))
 				  (if old-val
@@ -745,94 +739,95 @@ all saved edit lists."
     (define (find-popup snd chn dats)
       (if (not (null? dats))
 	  (let ((cur (car dats)))
-	    (if (and (= (car cur) snd)
+	    (if (and (equal? (car cur) snd)
 		     (= (cadr cur) chn))
 		cur
 		(find-popup snd chn (cdr dats))))
 	  #f))
 
     (define (add-popup snd)
-      (do ((chn 0 (1+ chn)))
-	  ((= chn (chans snd)))
-	(if (not (find-popup snd chn popups))
-	    (let ((chn-grf (car (channel-widgets snd chn))))
-	      (set! popups (cons (list snd chn) popups))
-
-	      ;; edit history list spreadsheet support
-	      (XtAddCallback  (list-ref (channel-widgets snd chn) 7) XmNpopupHandlerCallback 
-		 (lambda (w data info)
-		   (let ((e (.event info)))
-		     (if (= ButtonPress (.type e))
-			 (begin
-			   (if edit-history-menu
-			       (for-each-child
-				edit-history-menu
-				(lambda (w)
-				  (let ((name (XtName w)))
-				    (if (string=? name "Clear")
-					(XtSetSensitive w (not (null? edhist-funcs)))
-					(if (string=? name "Save")
-					    (XtSetSensitive w (> (apply + (edits snd chn)) 0))
-					    (if (string=? name "Apply")
-						(XtSetSensitive w (not (null? edhist-funcs)))
-						(if (string=? name "Reapply")
-						    (XtSetSensitive w (not (eq? #f (assoc (cons snd chn) edhist-funcs))))))))))))
-			   (set! (.menuToPost info) (edit-history-popup-menu snd chn)))))))
-
-	      (XtAddCallback chn-grf XmNpopupHandlerCallback 
-		 (lambda (w data info)
-		   (let* ((e (.event info))
-			  (xe (- (.x_root e) (car (XtTranslateCoords w 0 0)))))
-		     (if (= ButtonPress (.type e))
-			 (begin
-			   ;; xe is where the mouse-click occurred in the graph window's (local) coordinates
-			   ;;   not the same as (.x e), which appears to be in the outer shell's coordinates
-			   (set! graph-popup-snd snd)
-			   (set! graph-popup-chn chn)
-			   (if (= (channel-style snd) channels-combined)
-			       (let ((ye (.y e))) ; y axis location of mouse-down
-				 (call-with-current-continuation
-				  (lambda (break)
-				    (do ((i 0 (1+ i)))
-					((= i (chans snd)))
-				      (let ((off (list-ref (axis-info snd i) 14)))
-					(if (< ye off)
-					    (begin
-					      (set! graph-popup-chn (- i 1))
-					      (break)))))
-				    (set! graph-popup-chn (- (chans snd) 1))))))
-			   
-			   (let ((fax (if (transform-graph? snd chn) (axis-info snd chn transform-graph) #f))
-				 (lax (if (lisp-graph? snd chn) (axis-info snd chn lisp-graph) #f)))
-			     (if (and fax
-				      (>= xe (list-ref fax 10))
-				      (<= xe (list-ref fax 12)))
-				 ;; in fft
-				 (begin
-				   (edit-fft-popup-menu snd chn)
-				   (set! (.menuToPost info) fft-popup-menu))
-				 
-				 (if (and lax
-					  (>= xe (list-ref lax 10))
-					  (<= xe (list-ref lax 12)))
-				     ;; in lisp
-				     ;;   nothing special implemented yet
-				     ;; (set! (.menuToPost info) graph-popup-menu)
-				     #f ; just a place-holder
-				     
-				     (if (and (selection?)
-					      (let* ((beg (/ (selection-position snd graph-popup-chn) (srate snd)))
-						     (end (/ (+ (selection-position snd graph-popup-chn) 
-								(selection-frames snd graph-popup-chn)) 
-							     (srate snd))))
-						(and (>= xe (x->position beg snd chn))
-						     (<= xe (x->position end snd chn)))))
-					 (set! (.menuToPost info) selection-popup-menu)
-					 
-					 (begin
-					   (edit-graph-popup-menu graph-popup-snd graph-popup-chn)
-					   (set! (.menuToPost info) graph-popup-menu)))))))))))))))
-
+      (do ((uchn 0 (+ 1 uchn)))
+	  ((= uchn (channels snd)))
+	(let ((chn uchn)) ; this is for s7
+	  (if (not (find-popup snd chn popups))
+	      (let ((chn-grf (car (channel-widgets snd chn))))
+		(set! popups (cons (list snd chn) popups))
+		
+		;; edit history list spreadsheet support
+		(XtAddCallback  (list-ref (channel-widgets snd chn) 7) XmNpopupHandlerCallback 
+		  (lambda (w data info)
+		    (let ((e (.event info)))
+		      (if (= ButtonPress (.type e))
+			  (begin
+			    (if edit-history-menu
+				(for-each-child
+				 edit-history-menu
+				 (lambda (w)
+				   (let ((name (XtName w)))
+				     (if (string=? name "Clear")
+					 (XtSetSensitive w (not (null? edhist-funcs)))
+					 (if (string=? name "Save")
+					     (XtSetSensitive w (> (apply + (edits snd chn)) 0))
+					     (if (string=? name "Apply")
+						 (XtSetSensitive w (not (null? edhist-funcs)))
+						 (if (string=? name "Reapply")
+						     (XtSetSensitive w (not (eq? #f (assoc (cons snd chn) edhist-funcs))))))))))))
+			    (set! (.menuToPost info) (edit-history-popup-menu snd chn)))))))
+		
+		(XtAddCallback chn-grf XmNpopupHandlerCallback 
+		  (lambda (w data info)
+		    (let* ((e (.event info))
+			   (xe (- (.x_root e) (car (XtTranslateCoords w 0 0)))))
+		      (if (= ButtonPress (.type e))
+			  (begin
+			    ;; xe is where the mouse-click occurred in the graph window's (local) coordinates
+			    ;;   not the same as (.x e), which appears to be in the outer shell's coordinates
+			    (set! graph-popup-snd snd)
+			    (set! graph-popup-chn chn)
+			    (if (and (= (channel-style snd) channels-combined)
+				     (> (channels snd) 1))
+				(let ((ye (.y e))) ; y axis location of mouse-down
+				  (call-with-exit
+				   (lambda (break)
+				     (do ((i 0 (+ 1 i)))
+					 ((= i (channels snd)))
+				       (let ((off (list-ref (axis-info snd i) 14)))
+					 (if (< ye off)
+					     (begin
+					       (set! graph-popup-chn (- i 1))
+					       (break)))))
+				     (set! graph-popup-chn (- (channels snd) 1))))))
+			    
+			    (let ((fax (if (transform-graph? snd chn) (axis-info snd chn transform-graph) #f))
+				  (lax (if (lisp-graph? snd chn) (axis-info snd chn lisp-graph) #f)))
+			      (if (and fax
+				       (>= xe (list-ref fax 10))
+				       (<= xe (list-ref fax 12)))
+				  ;; in fft
+				  (begin
+				    (edit-fft-popup-menu snd chn)
+				    (set! (.menuToPost info) fft-popup-menu))
+				  
+				  (if (and lax
+					   (>= xe (list-ref lax 10))
+					   (<= xe (list-ref lax 12)))
+				      ;; in lisp
+				      ;;   nothing special implemented yet
+				      ;; (set! (.menuToPost info) graph-popup-menu)
+				      #f ; just a place-holder
+				      
+				      (if (and (selection?)
+					       (let* ((beg (/ (selection-position snd graph-popup-chn) (srate snd)))
+						      (end (/ (+ (selection-position snd graph-popup-chn) 
+								 (selection-frames snd graph-popup-chn)) 
+							      (srate snd))))
+						 (and (>= xe (x->position beg snd chn))
+						      (<= xe (x->position end snd chn)))))
+					  (set! (.menuToPost info) selection-popup-menu)
+					  
+					  (begin
+					    (edit-graph-popup-menu graph-popup-snd graph-popup-chn)
+					    (set! (.menuToPost info) graph-popup-menu))))))))))))))))
     (add-hook! after-open-hook add-popup)
     (for-each add-popup (sounds))))
 
@@ -847,7 +842,7 @@ color name, an xm Pixel, a snd color, or a list of rgb values (as in Snd's make-
 		    (cmap (DefaultColormap dpy scr))
 		    (col (XColor)))
 	       (if (= (XAllocNamedColor dpy cmap new-color col col) 0)
-		   (snd-error "can't allocate ~S" new-color)
+		   (snd-error (format #f "can't allocate ~S" new-color))
 		   (.pixel col)))
 	     (if (color? new-color)
 		 new-color
@@ -907,7 +902,7 @@ color name, an xm Pixel, a snd color, or a list of rgb values (as in Snd's make-
 	   children)
 	  (let ((current-sounds (collector (sounds))))
 	    (if (< (length children) (length current-sounds)) ; only active if len (collector (sounds)) > 1
-		(do ((i (length children) (1+ i)))
+		(do ((i (length children) (+ 1 i)))
 		    ((= i (length current-sounds)))
 		  (let ((child (XtCreateManagedWidget "" xmPushButtonWidgetClass top-two args)))
 		    (XtAddCallback child XmNactivateCallback
@@ -935,14 +930,14 @@ color name, an xm Pixel, a snd color, or a list of rgb values (as in Snd's make-
 			 (list-ref (main-widgets) 4))))
 	 (every-menu (list XmNbackground (highlight-color)))
 	 (listener-popup (XmCreatePopupMenu listener "listener-popup"
-			   (append (list XmNpopupEnabled #t) every-menu))))
+			   (append (list XmNpopupEnabled XmPOPUP_AUTOMATIC) every-menu))))
 
     (define (edited snds)
       (remove-if (lambda (n) 
-		   (call-with-current-continuation
+		   (call-with-exit
 		    (lambda (return)
-		      (do ((i 0 (1+ i)))
-			  ((= i (chans n)) #t)
+		      (do ((i 0 (+ 1 i)))
+			  ((= i (channels n)) #t)
 			(if (not (= (car (edits n i)) 0))
 			    (return #f))))))
 		 snds))
@@ -956,7 +951,7 @@ color name, an xm Pixel, a snd color, or a list of rgb values (as in Snd's make-
     (XtCreateManagedWidget "sep" xmSeparatorWidgetClass listener-popup every-menu)
 
     (let ((listener-popup-menu
-	   (list (make-popdown-entry "Play" listener-popup (lambda (snd) (play 0 snd)) every-menu identity #t)
+	   (list (make-popdown-entry "Play" listener-popup (lambda (snd) (play snd 0)) every-menu identity #t)
 
 		 (let ((help-widget (XtCreateManagedWidget "Help" xmPushButtonWidgetClass listener-popup every-menu)))
 		   (XtAddCallback help-widget XmNactivateCallback
@@ -965,19 +960,19 @@ color name, an xm Pixel, a snd color, or a list of rgb values (as in Snd's make-
 			      (help (and selected (snd-help selected))))
 			 (if help (help-dialog selected help)))))
 		   help-widget)
+		 
+		 (let ((clear-widget (XtCreateManagedWidget "Clear listener" xmPushButtonWidgetClass listener-popup every-menu)))
+		   (XtAddCallback clear-widget XmNactivateCallback (lambda (w c i) (clear-listener)))
+		   clear-widget)
 
 		 (let ((open-widget (XtCreateManagedWidget "Open" xmPushButtonWidgetClass listener-popup every-menu)))
 		   (XtAddCallback open-widget XmNactivateCallback (lambda (w c i) (open-file-dialog)))
 		   open-widget)
 		 
-		 (make-popdown-entry "Close" listener-popup close-sound every-menu identity #t)
+		 (make-popdown-entry "Close" listener-popup close-sound every-menu #f #t)
 		 (make-popdown-entry "Save" listener-popup save-sound every-menu edited #t)
 		 (make-popdown-entry "Revert" listener-popup revert-sound every-menu edited #t)
 
-		 (let ((panes-widget (XtCreateManagedWidget "Equalize panes" xmPushButtonWidgetClass listener-popup every-menu)))
-		   (XtAddCallback panes-widget XmNactivateCallback (lambda (w c i) (equalize-panes)))
-		   panes-widget)
-
 		 (make-popdown-entry "Focus" listener-popup 
 				     (lambda (us)
 				       (let* ((pane (car (sound-widgets us)))
@@ -1007,7 +1002,9 @@ color name, an xm Pixel, a snd color, or a list of rgb values (as in Snd's make-
 		       (let ((top-one (list-ref n 1))
 			     (top-two (list-ref n 2))
 			     (top-two-cascade (list-ref n 3))
-			     (len (length ((list-ref n 4) (sounds)))))
+			     (len (if (list-ref n 4)
+				      (length ((list-ref n 4) (sounds)))
+				      0)))
 			 (XtUnmanageChild top-two)
 			 (XtUnmanageChild top-two-cascade)
 			 (if top-one (XtUnmanageChild top-one))
