@@ -2,15 +2,15 @@
 
 (use-modules (ice-9 optargs))
 (provide 'snd-pvoc.scm)
-(if (not (provided? 'snd-snd7.scm)) (load "snd7.scm"))
+(if (not (provided? 'snd-snd7.scm)) (load-from-path "snd7.scm"))
 
-(define* (make-pvocoder fftsize overlap interp #:optional analyze edit synthesize)
-  "(make-pvocoder fftsize overlap interp #:optional analyze edit synthesize) makes a new (Scheme-based, not CLM) phase-vocoder generator"
+(define* (make-pvocoder fftsize overlap interp :optional analyze edit synthesize)
+  "(make-pvocoder fftsize overlap interp :optional analyze edit synthesize) makes a new (Scheme-based, not CLM) phase-vocoder generator"
 
   (let* ((N (or fftsize 512))
-	 (N2 (inexact->exact (floor (/ N 2))))
+	 (N2 (floor (/ N 2)))
 	 (hop (or overlap 4))
-	 (D (inexact->exact (floor (/ N hop)))))
+	 (D (floor (/ N hop))))
 
     ;; basic: fftsize overlap
     ;;  everything else via closures (interp in particular)
@@ -74,8 +74,7 @@
   (define (pvoc-edit pv) (list-ref pv 14))
   (define (pvoc-synthesize pv) (list-ref pv 15))
 
-  (let* ((pi 3.141592653589793)
-	 (pi2 (* 2 pi)))
+  (let* ((pi2 (* 2 pi)))
 
     (if (>= (pvoc-output pv) (pvoc-interp pv))
 	;; get next block of amp/phase info
@@ -98,7 +97,7 @@
 		    (let ((indat (pvoc-in-data pv)))
 		      ;; extra loop here since I find the optimized case confusing (we could dispense with the data move)
 		      (vct-move! indat 0 D)
-		      (do ((i (- N D) (1+ i)))
+		      (do ((i (- N D) (+ 1 i)))
 			  ((= i N))
 			(vct-set! indat i (input)))))
 		(let ((buf (modulo filptr N)))
@@ -108,10 +107,10 @@
 			(vct-add! amps (pvoc-in-data pv))
 			(vct-multiply! amps (pvoc-window pv)))
 		      (begin
-			(do ((k 0 (1+ k)))
+			(do ((k 0 (+ 1 k)))
 			    ((= k N))
 			  (vct-set! amps buf (* (vct-ref (pvoc-window pv) k) (vct-ref (pvoc-in-data pv) k)))
-			  (set! buf (1+ buf))
+			  (set! buf (+ 1 buf))
 			  (if (= buf N) (set! buf 0))))))
 		(set-pvoc-filptr pv (+ filptr D))
 		(mus-fft amps freqs N 1)
@@ -121,10 +120,10 @@
 	      ((pvoc-edit pv) pv)
 	      (begin
 		;; if no editing func:
-		(do ((k 0 (1+ k))
+		(do ((k 0 (+ 1 k))
 		     (pscl (/ 1.0 D))
 		     (kscl (/ pi2 N)))
-		    ((= k (inexact->exact (floor (/ N 2)))))
+		    ((= k (floor (/ N 2))))
 		  (let ((phasediff (- (vct-ref freqs k) (vct-ref (pvoc-lastphase pv) k))))
 		    (vct-set! (pvoc-lastphase pv) k (vct-ref freqs k))
 		    (if (> phasediff pi) (do () ((<= phasediff pi)) (set! phasediff (- phasediff pi2))))
@@ -138,7 +137,7 @@
 	    (vct-scale! freqs scl)
 	    )))
 
-    (set-pvoc-output pv (1+ (pvoc-output pv)))
+    (set-pvoc-output pv (+ 1 (pvoc-output pv)))
 
     (if (pvoc-synthesize pv)
 	((pvoc-synthesize pv) pv)
@@ -153,11 +152,11 @@
 
 ;;;   (let* ((ind (open-sound "oboe.snd"))
 ;;;	     (pv (make-pvocoder 256 4 64))
-;;;	     (rd (make-sample-reader 0)))
-;;;	(map-channel (lambda (y) (pvocoder pv rd))))
+;;;	     (rd (make-sampler 0)))
+;;;	(map-channel (lambda (y) (pvocoder pv (lambda () (rd)))))
 
 
-#!
+#|
 ;;; ---------------- same thing using phase-vocoder gen
 
 (define test-pv-1
@@ -168,11 +167,11 @@
 				  #f ;no change to edits
 				  #f ;no change to synthesis
 				  ))
-	  (reader (make-sample-reader 0)))
+	  (reader (make-sampler 0)))
       (map-channel (lambda (val)
 		     (phase-vocoder pv (lambda (dir) 
 					 (next-sample reader)))))
-      (free-sample-reader reader))))
+      (free-sampler reader))))
 
 (define test-pv-2
   (lambda (freq)
@@ -182,28 +181,28 @@
 				  #f
 				  #f ; no change to synthesis
 				  ))
-	  (reader (make-sample-reader 0)))
+	  (reader (make-sampler 0)))
       (map-channel (lambda (val)
 		     (phase-vocoder pv (lambda (dir) 
 					 (next-sample reader)))))
-      (free-sample-reader reader))))
+      (free-sampler reader))))
 
 (define test-pv-3
   (lambda (time)
     (let* ((pv (make-phase-vocoder #f
-				   512 4 (inexact->exact (floor (* 128 time))) 1.0
+				   512 4 (floor (* 128 time)) 1.0
 				   #f ;no change to analysis
 				   #f ;no change to edits
 				   #f ;no change to synthesis
 				   ))
-	   (reader (make-sample-reader 0))
-	   (len (inexact->exact (floor (* time (frames)))))
+	   (reader (make-sampler 0))
+	   (len (floor (* time (frames))))
 	   (data (make-vct len))
 	   )
       (vct-map! data
 		(lambda ()
 		  (phase-vocoder pv (lambda (dir) (next-sample reader)))))
-      (free-sample-reader reader)
+      (free-sampler reader)
       (vct->channel data 0 len))))
 
 (define test-pv-4
@@ -213,26 +212,26 @@
 				  #f ;no change to analysis
 				  (lambda (v)
 				    (let ((N (mus-length v)))
-				      (do ((i 0 (1+ i)))
+				      (do ((i 0 (+ 1 i)))
 					  ((= i N))
 					(if (< (vct-ref (phase-vocoder-amp-increments v) i) gate)
 					    (vct-set! (phase-vocoder-amp-increments v) i 0.0)))
 				      #t))
 				  #f ;no change to synthesis
 				  ))
-	  (reader (make-sample-reader 0))
+	  (reader (make-sampler 0))
 	  )
       (map-channel (lambda (val)
 		     (phase-vocoder pv (lambda (dir) 
 					 (next-sample reader)))))
-      (free-sample-reader reader))))
-!#
+      (free-sampler reader))))
+|#
 
 
 ;;; -------- another version of the phase vocoder --------
 
 (define pvoc
-  (lambda* (#:key
+  (lambda* (:key
 	   (fftsize 512) (overlap 4) (time 1.0)
 	   (pitch 1.0) (gate 0.0) (hoffset 0.0)
 	   (snd 0) (chn 0))
@@ -242,16 +241,15 @@
   'gate' specifies a resynthesis gate in dB (partials with amplitudes lower than
   the gate value will not be synthesized), 'hoffset is a pitch offset in Hz."
 
-    (let* ((pi 3.141592653589793)
-	   (len (frames))
+    (let* ((len (frames))
 	   (filptr 0)           ; index into the file
 	   (pi2 (* 2 pi))       ; handy constant
 	   (sr (srate))
 	   (N fftsize)          ; fft size
-	   (N2 (inexact->exact (floor (/ N 2))))
-	   (Nw fftsize) ;; window size -- currently restricted to the fftsize
-	   (D (inexact->exact (floor (/ fftsize overlap)))) ; decimation factor (how often do we take an fft)
-	   (interp (* (inexact->exact (floor (/ fftsize overlap))) time)) ; interpolation factor how often do we synthesize
+	   (N2 (floor (/ N 2)))
+	   ;; (Nw fftsize) ;; window size -- currently restricted to the fftsize
+	   (D (floor (/ fftsize overlap))) ; decimation factor (how often do we take an fft)
+	   (interp (* (floor (/ fftsize overlap)) time)) ; interpolation factor how often do we synthesize
 	   ;; take a resynthesis gate specificed in dB, convert to linear amplitude
 	   (syngate (if (= 0.0 gate) 0.0 (expt 10 (/ (- (abs gate)) 20))))
 	   (poffset (hz->radians hoffset))
@@ -267,39 +265,37 @@
 	   (fundamental (/ pi2 N))
 	   (output interp)      ; count of samples that have been output
 	   (resynth-oscils (make-vector N2))  ; synthesis oscillators
-	   (nextpct 10.0)       ; how often to print out the percentage complete message
-	   (outlen (inexact->exact (floor (* time len))))
+	   ;; (nextpct 10.0)       ; how often to print out the percentage complete message
+	   (outlen (floor (* time len)))
 	   (out-data (make-vct (max len outlen)))
 	   (in-data (channel->vct 0 (* N 2) snd chn))
 	   (in-data-beg 0))
       ;; setup oscillators
-      (do ((i 0 (1+ i)))
+      (do ((i 0 (+ 1 i)))
 	  ((= i N2))
 	(vector-set! resynth-oscils i (make-oscil :frequency 0)))
       (vct-scale! window (/ 2.0 (* 0.54 fftsize))) ;den = hamming window integrated
-      (call-with-current-continuation
+      (call-with-exit
        (lambda (break)
-	 (do ((i 0 (1+ i)))
+	 (do ((i 0 (+ 1 i)))
 	     ((>= i outlen))
-	   ;; begin the master run loop
 	   (if (>= output interp) ;; if all the samples have been output then do the next frame
 	       (let ((buffix (modulo filptr N)))
 					; buffix is the index into the input buffer
 					; it wraps around circularly as time increases in the input
-		 (if (c-g?) (break "interrupted"))
 		 (set! output 0)       ; reset the output sample counter
 		 ;; save the old amplitudes and frequencies
 		 (vct-fill! lastamp 0.0)
 		 (vct-fill! lastfreq 0.0)
 		 (vct-add! lastamp fdr)
 		 (vct-add! lastfreq fdi)
-		 (do ((k 0 (1+ k)))
+		 (do ((k 0 (+ 1 k)))
 		     ((= k N))
 		   ;; apply the window and then stuff into the input array
 		   (vct-set! fdr buffix (* (vct-ref window k) (vct-ref in-data (- filptr in-data-beg))))
-		   (set! filptr (1+ filptr))
+		   (set! filptr (+ 1 filptr))
 		   ;; increment the buffer index with wrap around
-		   (set! buffix (1+ buffix))
+		   (set! buffix (+ 1 buffix))
 		   (if (>= buffix N) (set! buffix 0)))
 		 ;; rewind the file for the next hop
 		 (set! filptr (- filptr (- N D)))
@@ -312,7 +308,7 @@
 		 ;; compute the fft
 		 (mus-fft fdr fdi N 1)
 		 ;; now convert into magnitude and interpolated frequency
-		 (do ((k 0 (1+ k)))
+		 (do ((k 0 (+ 1 k)))
 		     ((= k N2))
 		   (let* ((a (vct-ref fdr k))
 			  (b (vct-ref fdi k))
@@ -348,7 +344,7 @@
 	   (vct-add! lastamp ampinc)
 	   (vct-add! lastfreq freqinc)
 	   (vct-set! out-data i (oscil-bank lastamp resynth-oscils lastfreq))
-	   (set! output (1+ output)))
+	   (set! output (+ 1 output)))
 	 (vct->channel out-data 0 (max len outlen)))))))
 
 
