@@ -1,29 +1,12 @@
 # rgb.rb -- Guile -> Ruby translation
 
-# Translator/Author: Michael Scholz <scholz-micha@gmx.de>
-# Last: Fri Mar 14 01:29:47 CET 2003
-# Version: $Revision: 1.3 $
+# Translator/Author: Michael Scholz <mi-scholz@users.sourceforge.net>
+# Created: Sun Mar 09 03:51:38 CET 2003
+# Changed: Fri Jul 06 01:52:54 CEST 2007
 
 # X11 color names converted to Snd (Ruby) colors
 # tan -> tawny 24-Aug-01
 
-# e.g. set_colormap(Color_map_jet)
-Color_map_black_and_white = 0
-Color_map_gray = 1
-Color_map_hot = 2
-Color_map_cool = 3
-Color_map_bone = 4
-Color_map_copper = 5
-Color_map_pink = 6
-Color_map_jet = 7
-Color_map_prism = 8
-Color_map_autumn = 9
-Color_map_winter = 10
-Color_map_spring = 11
-Color_map_summer = 12
-Color_map_rainbow = 13
-Color_map_flag = 14
-
 # rgb.scm
 Snow = make_color(1.00, 0.98, 0.98)
 Ghost_white = make_color(0.97, 0.97, 1.00)
