@@ -61,19 +61,23 @@
 		  our-srate our-chans our-short our-dac-buffer-size-in-bytes))
 	(data (make-sound-data our-chans our-dac-buffer-size-in-shorts))
 	(vobj (make-vct our-dac-buffer-size-in-shorts)))
-    (do ()
-	((c-g?))
-      (mus-audio-read in-port data our-dac-buffer-size-in-shorts)
-      ;; now process the sound...
-      (func data)
-      (mus-audio-write out-port data our-dac-buffer-size-in-shorts))
+    (catch #t ; try to make sure we close the audio ports upon an error
+	   (lambda ()
+	     (do ()
+		 ((c-g?))
+	       (mus-audio-read in-port data our-dac-buffer-size-in-shorts)
+	       ;; now process the sound...
+	       (func data)
+	       (mus-audio-write out-port data our-dac-buffer-size-in-shorts)))
+	   (lambda args
+	     (display (format #f ";in-out error: ~A" args))))
     (mus-audio-close in-port)
     (mus-audio-close out-port)))
 
 ;;; now some "func"s for in-out above
 (define (amplify amp)
   (lambda (data)
-    (do ((i 0 (1+ i)))
+    (do ((i 0 (+ 1 i)))
 	((= i our-dac-buffer-size-in-shorts))
       (sound-data-set! data our-chan i (* (sound-data-ref data our-chan i) amp)))))
 
@@ -82,7 +86,7 @@
 (define (lp)
   (let ((val 0.0))
     (lambda (data)
-      (do ((i 0 (1+ i)))
+      (do ((i 0 (+ 1 i)))
 	  ((= i our-dac-buffer-size-in-shorts))
 	(let ((curval (sound-data-ref data our-chan i)))
 	  (sound-data-set! data our-chan i (* .5 (+ curval val)))
@@ -91,10 +95,10 @@
 ;;; (in-out (lp) 1 0)
 
 (define (hp frq)
-  (let ((flt (make-formant .99 frq))
+  (let ((flt (make-formant frq .99))
 	(vobj (make-vct our-dac-buffer-size-in-shorts)))
     (lambda (data)
-      (do ((i 0 (1+ i)))
+      (do ((i 0 (+ 1 i)))
 	  ((= i our-dac-buffer-size-in-shorts))
 	(let ((curval (sound-data-ref data our-chan i)))
 	  (sound-data-set! data our-chan i (formant flt curval))))
