@@ -19,12 +19,13 @@
 (provide 'snd-singer.scm)
 (if (not (provided? 'snd-ws.scm)) (load-from-path "ws.scm"))
 
-(definstrument (singer beg amp-1 data)
+(use-modules (ice-9 optargs) (ice-9 format))
+
+(definstrument (singer beg amp data)
   ;; data is a list of lists very similar to the sequence of synthesize calls in Perry's original implementation.
   ;;    Each imbedded list has the form: dur shape glot pitch glotamp noiseamps vibramt.
   ;;    See below for examples.
   (let* ((setup (car data))
-	 (amp amp-1)
 	 (durs (map car data))
 	 (dur (apply + durs))
 	 (begs (let ((bg beg))
@@ -33,12 +34,9 @@
 				(set! bg (+ bg x))
 				bg)
 			      durs))))
-	 (beg-samps (map
-		     (lambda (x)
-		       (inexact->exact (round (* x (mus-srate)))))
-		     begs))
+	 (beg-samps (map seconds->samples begs))
 	 (change-times (let* ((len (length beg-samps))
-			      (nbegs (append beg-samps (list (list-ref beg-samps (1- len))))))
+			      (nbegs (append beg-samps (list (list-ref beg-samps (- len 1))))))
 			 (list->vct nbegs)))
 	 (shps (map cadr data))
 	 (glts (map caddr data))
@@ -73,15 +71,15 @@
 		 init))
 	 (noiseamps (let* ((len (length data))
 			   (v (make-vct len)))
-		      (do ((i 0 (1+ i)))
+		      (do ((i 0 (+ 1 i)))
 			  ((= i len))
 			(vct-set! v i (exact->inexact (list-ref (list-ref data i) 5))))
 		      v))
-	 (frq-env (make-env :envelope pfun :duration dur))
-	 (vib-env (make-env :envelope vfun :duration dur))
-	 (vib-osc (make-oscil :frequency 6.0))
-	 (glot-env (make-env :envelope gfun :duration dur))
-	 (noise-env (make-env :envelope nfun :duration dur))
+	 (frq-env (make-env pfun :duration dur))
+	 (vib-env (make-env vfun :duration dur))
+	 (vib-osc (make-oscil 6.0))
+	 (glot-env (make-env gfun :duration dur))
+	 (noise-env (make-env nfun :duration dur))
 	 (ran-vib (make-rand-interp :frequency 10 :amplitude .02))
 
 	 (tractlength 9)		;length of vocal tract
@@ -93,30 +91,30 @@
 	 (glot-datai (make-vct (* 2 (length glts))))
 	 (glot-datar (make-vct (* 2 (length glts)))))
 
-    (do ((k 0 (1+ k))
+    (do ((k 0 (+ 1 k))
 	 (i 0 (+ i tractlength+8)))
 	((= k (length shps)))
       (let ((shp (cdr (list-ref shps k))))
-	(do ((j i (1+ j))
-	     (m 0 (1+ m)))
+	(do ((j i (+ 1 j))
+	     (m 0 (+ 1 m)))
 	    ((= m (length shp)))
 	  (vct-set! shape-data j (list-ref shp m)))))
 	     
-    (do ((k 0 (1+ k))
+    (do ((k 0 (+ 1 k))
 	 (i 0 (+ i 2)))
 	((= k (length glts)))
       (let ((glt (list-ref glts k)))
 	(vct-set! glot-datai i 0.0)
-	(vct-set! glot-datai (1+ i) (car glt))
+	(vct-set! glot-datai (+ 1 i) (car glt))
 	(vct-set! glot-datar i (cadr glt))
-	(vct-set! glot-datar (1+ i) (caddr glt))))
+	(vct-set! glot-datar (+ 1 i) (caddr glt))))
     (let* ((table-size 1000)		; size of glottis wave-table
 	   (noseposition 3)
 	   (noselength 6)
 	   (noselength-1 (- noselength 1))
 	   (noselength-2 (- noselength 2))
 	   (nose-ring-time 1000)	; naso pharynx response decay time
-	   (two-pi (* 2 3.14159))
+	   (two-pi (* 2 pi))
 	   (one-over-two-pi  0.159154943)
 	   (two-pi-over-table-size (/ two-pi table-size))
 	   (table-size-over-sampling-rate (/ table-size (mus-srate)))
@@ -126,9 +124,9 @@
 	   (tong-hump-gain (- 1.0 tong-hump-pole))
 	   (tong-tip-pole 0.998)
 	   (tong-tip-gain (- 1.0 tong-tip-pole))
-	   (glot-table (make-vct (1+ table-size)))
-	   (glot-table2 (make-vct (1+ table-size)))
-	   (gn-table (make-vct (1+ table-size)))
+	   (glot-table (make-vct (+ 1 table-size)))
+	   (glot-table2 (make-vct (+ 1 table-size)))
+	   (gn-table (make-vct (+ 1 table-size)))
 	   (gn-gain 0.0)
 	   (gn-out 0.0)
 	   (gn-del (make-vct 4))
@@ -198,8 +196,8 @@
 	   (new-tract 1)
 	   (first-tract 1)
 	   (offset -1)
-	   (bg (inexact->exact (floor (* (mus-srate) beg))))
-	   (nd (inexact->exact (vct-ref change-times (1- (vct-length change-times)))))
+	   (bg (seconds->samples beg))
+	   (nd (inexact->exact (vct-ref change-times (- (length change-times) 1))))
 	   (next-offset bg)
 	   (last-sfd -1)
 	   (last-gfd -1))
@@ -211,20 +209,20 @@
       (vct-set! nose-coeffs 4 0.24)
       (vct-set! nose-coeffs 5 0.3571)
 
-      (do ((i 0 (1+ i))) ((= i 8)) (vct-set! radii i 1.0))
+      (do ((i 0 (+ 1 i))) ((= i 8)) (vct-set! radii i 1.0))
       (vct-set! radii 8 0.7)
       (vct-set! radii 9 -0.5)
-      (do ((i 0 (1+ i))) ((= i 8)) (vct-set! target-radii i 1.0))
+      (do ((i 0 (+ 1 i))) ((= i 8)) (vct-set! target-radii i 1.0))
       (vct-set! target-radii 8 0.7)
       (vct-set! target-radii 9 -0.5)
 
-      (do ((i 0 (1+ i))) ((= i tractlength+8)) (vct-set! radii-poles i dpole))
+      (do ((i 0 (+ 1 i))) ((= i tractlength+8)) (vct-set! radii-poles i dpole))
       (vct-set! radii-poles 2 tong-hump-pole)
       (vct-set! radii-poles 3 tong-hump-pole)
       (vct-set! radii-poles 4 tong-hump-pole)
       (vct-set! radii-poles 5 tong-tip-pole)
 
-      (do ((i 0 (1+ i))) ((= i tractlength+8)) (vct-set! radii-pole-gains i dgain))
+      (do ((i 0 (+ 1 i))) ((= i tractlength+8)) (vct-set! radii-pole-gains i dgain))
       (vct-set! radii-pole-gains 2 tong-hump-gain)
       (vct-set! radii-pole-gains 3 tong-hump-gain)
       (vct-set! radii-pole-gains 4 tong-hump-gain)
@@ -233,18 +231,18 @@
       (ws-interrupt?)
       (run
        (lambda ()
-	 (do ((i bg (1+ i)))
+	 (do ((i bg (+ 1 i)))
 	     ((= i nd))
 	   (if (= i next-offset)
 	       (begin
 		 ;; time to check for new tract shapes, glottal pulse shapes etc.
-		 (set! offset (1+ offset))
+		 (set! offset (+ 1 offset))
 		 (set! fnoiseamp (vct-ref noiseamps offset))
 		 (if (= last-sfd -1)
 		     (set! last-sfd 0)
 		     (let ((new-sfd (+ last-sfd 8 tractlength)))
-		       (do ((j last-sfd (1+ j))
-			    (k new-sfd (1+ k)))
+		       (do ((j last-sfd (+ 1 j))
+			    (k new-sfd (+ 1 k)))
 			   ((= j new-sfd))
 			 (if (> (abs (- (vct-ref shape-data j) (vct-ref shape-data k))) .001)
 			     (set! new-tract 1)))
@@ -254,20 +252,21 @@
 		     (let ((new-gfd (+ last-gfd 2)))
 		       (set! last-gfd new-gfd)))
 		 (set! next-offset (inexact->exact (vct-ref change-times (+ offset 1))))))
+
 	   (if (not (= new-tract 0))
 	       (begin
-		 (do ((j last-sfd (1+ j))
-		      (k 0 (1+ k)))
+		 (do ((j last-sfd (+ 1 j))
+		      (k 0 (+ 1 k)))
 		     ((= k tractlength+8))
 		   (vct-set! target-radii k (vct-ref shape-data j)))
 		 (if (= first-tract 1)
 		     (begin
-		       (do ((k 0 (1+ k)))
+		       (do ((k 0 (+ 1 k)))
 			   ((= k tractlength+8))
 			 (vct-set! radii k (vct-ref target-radii k)))))
 		 (set! change-radii 0)
 		 (set! initial-noise-position (vct-ref radii tractlength+1))
-		 (do ((j 0 (1+ j)))
+		 (do ((j 0 (+ 1 j)))
 		     ((= j tractlength+8))
 		   (if (> (abs (- (vct-ref target-radii j) (vct-ref radii j))) 0.001)
 		       (set! change-radii 1)))))
@@ -276,18 +275,18 @@
 	       (begin
 		 (if (= new-tract 0)
 		     (begin
-		       (do ((j 0 (1+ j)))
+		       (do ((j 0 (+ 1 j)))
 			   ((= j tractlength+8))
 			 (vct-set! radii j (+ (* (vct-ref radii j) (vct-ref radii-poles j))
 					      (* (vct-ref target-radii j) (vct-ref radii-pole-gains j)))))))
 		 ;; set tract shape
 		 (vct-set! temp-arr 0 1.0)
-		 (do ((j 1 (1+ j)))
+		 (do ((j 1 (+ 1 j)))
 		     ((= j tractlength))
 		   (vct-set! temp-arr j (* (vct-ref radii (- j 1)) (vct-ref radii (- j 1))))
 		   (if (= (vct-ref temp-arr j) 0.0) 
 		       (vct-set! temp-arr j 1e-10)))
-		 (do ((j 1 (1+ j)))
+		 (do ((j 1 (+ 1 j)))
 		     ((= j tractlength))
 		   (vct-set! coeffs j (/ (- (vct-ref temp-arr (- j 1)) (vct-ref temp-arr j))
 					 (+ (vct-ref temp-arr (- j 1)) (vct-ref temp-arr j)))))
@@ -340,7 +339,7 @@
 	       (begin
 		 (if (= first-glot 0)
 		     (begin
-		       (do ((i 0 (1+ i)))
+		       (do ((i 0 (+ 1 i)))
 			   ((> i table-size))
 			 (vct-set! glot-table2 i (vct-ref glot-table i)))))
 		 (let* ((harms (inexact->exact (vct-ref glot-datai (+ last-gfd 1))))
@@ -362,7 +361,7 @@
 			 (vct-set! cosines 1 (* (+ (- (sin a2)) (* (- (cos a2) (cos b2)) temp)) temp1 one-over-two-pi))))
 		   (vct-set! sines 1 (+ (vct-ref sines 1) (* (+ 0.75 (- (cos a2)) (* (cos (* 2 a2)) 0.25)) one-over-two-pi)))
 		   (vct-set! cosines 1 (+ (vct-ref cosines 1) (- (* (- (sin a2) (* (sin (* 2 a2)) 0.25)) one-over-two-pi) (* a 0.5))))
-		   (do ((k 2 (1+ k))
+		   (do ((k 2 (+ 1 k))
 			(ka2 (* 2 a2) (+ ka2 a2))
 			(ka1 a2 (+ ka1 a2))
 			(ka3 (* 3 a2) (+ ka3 a2)))
@@ -380,11 +379,11 @@
 		     (vct-set! cosines k (+ (vct-ref cosines k) (- (/ (sin ka2) k) (/ (* (sin ka1) 0.5) (- k 1)) (/ (* (sin ka3) 0.5) (+ k 1)))))
 		     (vct-set! cosines k (* (vct-ref cosines k) one-over-two-pi)))
 		   (vct-fill! glot-table 0.0)
-		   (do ((j 0 (1+ j))
+		   (do ((j 0 (+ 1 j))
 			(x 0.0 (+ x two-pi-over-table-size)))
 		       ((> j table-size))
 					;(vct-set! glot-table j 0.0)
-		     (do ((k 1 (1+ k)))
+		     (do ((k 1 (+ 1 k)))
 			 ((> k harms))
 		       (vct-set! glot-table j (+ (vct-ref glot-table j) (+ (* (vct-ref cosines k) (cos (* k x)))
 									   (* (vct-ref sines k) (sin (* k x)))))))))
@@ -392,7 +391,7 @@
 		 (set! delta (/ 1.0 (- next-offset i)))
 		 (if (not (= first-glot 0))
 		     (begin
-		       (do ((i 0 (1+ i)))
+		       (do ((i 0 (+ 1 i)))
 			   ((> i table-size))
 			 (vct-set! glot-table2 i (vct-ref glot-table i)))
 		       (set! first-glot 0)))
@@ -430,8 +429,8 @@
 					 (* (vct-ref gn-coeffs 2) (vct-ref gn-del 2))
 					 (* (vct-ref gn-coeffs 1) (vct-ref gn-del 1))
 					 (* (vct-ref gn-coeffs 0) (vct-ref gn-del 0))))
-			 (do ((j 3 (1- j))
-			      (k 2 (1- k)))
+			 (do ((j 3 (- j 1))
+			      (k 2 (- k 1)))
 			     ((< j 1))
 			   (vct-set! gn-del j (vct-ref gn-del k)))
 			 (vct-set! gn-del 0 gn-out)))
@@ -444,7 +443,7 @@
 	     (vct-set! lt 0 (+ (vct-ref dline1 2) (vct-ref dline2 2)))
 	     (vct-set! dline2 1 (+ (vct-ref dline2 2) (* (vct-ref coeffs 1) (- glotsamp (vct-ref dline2 2)))))
 	     (set! temp (+ glotsamp (- (vct-ref dline2 1) (vct-ref dline2 2))))
-	     (do ((j 2 (1+ j)))
+	     (do ((j 2 (+ 1 j)))
 		 ((= j noseposition))
 	       (vct-set! dline2 j (+ (vct-ref dline2 (+ j 1)) (* (vct-ref coeffs j) (- (vct-ref dline1 (- j 1)) (vct-ref dline2 (+ j 1))))))
 	       (set! temp1 temp)
@@ -463,7 +462,7 @@
 		     (set! nose-last-plus-refl (- nose-reftemp minussamp)))
 		   (begin
 		     (if (not (= velum-pos 0.0))
-			 (set! time-nose-closed 0) 
+			 (set! time-nose-closed 0)
 			 (set! time-nose-closed (+ time-nose-closed)))
 		     ;; nasal tick
 		     (let* ((nose-t1 0.0)
@@ -475,7 +474,7 @@
 		       (set! nose-reftemp (* (vct-ref nose-coeffs 1) (- plus-in (vct-ref nose2 2))))
 		       (vct-set! nose2 1 (+ (vct-ref nose2 2) nose-reftemp))
 		       (set! nose-temp (+ plus-in nose-reftemp))
-		       (do ((j 2 (1+ j)))
+		       (do ((j 2 (+ 1 j)))
 			   ((= j noselength-1))
 			 (set! nose-reftemp (* (vct-ref nose-coeffs j) (- (vct-ref nose1 (- j 1)) (vct-ref nose2 (+ j 1)))))
 			 (vct-set! nose2 j (+ (vct-ref nose2 (+ j 1)) nose-reftemp))
@@ -495,7 +494,7 @@
 	     (set! temp1 temp)
 	     (set! temp nose-last-plus-refl)
 	     (vct-set! dline1 (- j 1) temp1)
-	     (do ((j (+ noseposition 1) (1+ j)))
+	     (do ((j (+ noseposition 1) (+ 1 j)))
 		 ((= j tractlength-1))
 	       (vct-set! dline2 j (+ (vct-ref dline2 (+ j 1)) (* (vct-ref coeffs j) (- (vct-ref dline1 (- j 1)) (vct-ref dline2 (+ j 1))))))
 	       (set! temp1 temp)
@@ -509,13 +508,13 @@
 	     (if (not (= noise-gain 0.0))
 		 (begin
 		   (set! noise-input (- 1.0 (random 2.0))) ;a guess
-		   (do ((j 3 (1- j))
-			(k 2 (1- k)))
+		   (do ((j 3 (- j 1))
+			(k 2 (- k 1)))
 		       ((< j 1))
 		     (vct-set! outz j (vct-ref outz k)))
 		   (vct-set! outz 0 noise-output)
 		   (set! noise-output (- noise-input inz2))
-		   (do ((i 0 (1+ i)))
+		   (do ((i 0 (+ 1 i)))
 		       ((= i 4))
 		     (set! noise-output (- noise-output (* (vct-ref noise-c i) (vct-ref outz i)))))
 		   (set! inz2 inz1)
@@ -526,9 +525,10 @@
 	   (outa i (* amp (+ last-lip-out nose-last-output (vct-ref lt 1))) *output*)
 	   ))))))
 
-#!
-(singer 0 .1 (list (list .4 ehh.shp test.glt 523.0 .8 0.0 .01) (list .6 oo.shp test.glt 523.0 .7 .1 .01)))
+#|
+(with-sound () (singer 0 .1 (list (list .4 ehh.shp test.glt 523.0 .8 0.0 .01) (list .6 oo.shp test.glt 523.0 .7 .1 .01))))
 
+(with-sound ()
 (singer 0 .1 (list (list .05 ehh.shp test.glt 523.0 0.8 0.0 .01) 
 		   (list .15 ehh.shp test.glt 523.0 0.8 0.0 .01) 
 		   (list .05 kkk.shp test.glt 523.0 0.0 0.0 .01) 
@@ -543,8 +543,8 @@
 		   (list .15 ehh.shp test.glt 523.0 0.8 0.0 .01) 
 		   (list .05 mmm.shp test.glt 523.0 0.8 0.0 .01) 
 		   (list .15 mmm.shp test.glt 523.0 0.8 0.0 .01) 			      
-		   (list .10 mmm.shp test.glt 523.0 0.0 0.0 .01) ))
-!#
+		   (list .10 mmm.shp test.glt 523.0 0.0 0.0 .01) )))
+|#
 
 (define test.glt (list 10 .65 .65))
 (define loud.glt (list 13 .6 .6))
@@ -626,9 +626,3 @@
 (define none.shp (list 8 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 0.0 0.0 1.0 0.0 0.0 0.0 0.0 0.0 0.0))
 (define sss.shp (list 8 0.928177 1.3588235 1.3588235 0.679558 0.61764705 0.63529414 0.31764707 0.65294117 0.7 -0.103125 7.0 0.105292 1500.0 0.916452 4943.75 0.97222227 0.0))
 (define zzz.shp (list 8 0.928177 1.3588235 1.3588235 0.679558 0.61764705 0.63529414 0.31764707 0.65294117 0.7 -0.103125 7.0 0.016 1500.0 0.9257112 4943.75 0.925926 0.0))
-
-
-
-
-
-
