@@ -5,10 +5,6 @@
   #define HAVE_DECL_ISNAN 1
 #endif
 
-#ifndef STATFS_ARGS
-  #define STATFS_ARGS 2
-#endif
-
 #if HAVE_RUBY
   #undef _
 #endif
@@ -19,72 +15,51 @@
   #define gettext_noop(String) String
   #define N_(String) gettext_noop (String)
 #else
-  #define _(String) (String)
+#define _(String) ((char *)(String))
   #define N_(String) String
   #define textdomain(Domain)
   #define bindtextdomain(Package, Directory)
 #endif
 
-#ifdef PRId64
-/* this is needed because guile's libguile.h->tags.h->inttypes.h picks up
-   a version of PRId64 that doesn't work with gettext as advertised.
-*/
-  #undef PRId64
+#if (!HAVE_FAM)
+  #define FAMRequest int
+  #define FAMEvent int
+  #define FAMConnection void*
 #endif
-#if (defined(SIZEOF_OFF_T) && (SIZEOF_OFF_T > 4)) || (defined(_FILE_OFFSET_BITS) && (_FILE_OFFSET_BITS == 64))
-  #define PRId64 "%lld"
+
+#define NO_SUCH_ENVELOPE XEN_ERROR_TYPE("no-such-envelope")
+#define NO_SUCH_SAMPLE   XEN_ERROR_TYPE("no-such-sample")
+#define NO_SUCH_EDIT     XEN_ERROR_TYPE("no-such-edit")
+#define CANNOT_SAVE      XEN_ERROR_TYPE("cannot-save")
+#define CANT_UPDATE_FILE XEN_ERROR_TYPE("cant-update-file")
+
+#if (SIZEOF_INT == SIZEOF_VOID_P)
+  typedef int pointer_or_int_t;
 #else
-  #define PRId64 "%d"
-#endif
-
-#define NO_SUCH_MIX          XEN_ERROR_TYPE("no-such-mix")
-#define NO_SUCH_TRACK        XEN_ERROR_TYPE("no-such-track")
-#define NO_SUCH_REGION       XEN_ERROR_TYPE("no-such-region")
-#define NO_SUCH_ENVELOPE     XEN_ERROR_TYPE("no-such-envelope")
-#define NO_SUCH_SAMPLE       XEN_ERROR_TYPE("no-such-sample")
-#define NO_SUCH_EDIT         XEN_ERROR_TYPE("no-such-edit")
-#define CANNOT_SAVE          XEN_ERROR_TYPE("cannot-save")
-#define CANNOT_PRINT         XEN_ERROR_TYPE("cannot-print")
-#define NO_SUCH_AXIS_INFO    XEN_ERROR_TYPE("no-such-axis")
-#define NO_SUCH_WIDGET       XEN_ERROR_TYPE("no-such-widget")
-#define NO_SUCH_AXIS_CONTEXT XEN_ERROR_TYPE("no-such-graphics-context")
-#define NO_SUCH_PLAYER       XEN_ERROR_TYPE("no-such-player")
-#define NO_SUCH_DIRECTION    XEN_ERROR_TYPE("no-such-direction")
-#define NO_SUCH_KEY          XEN_ERROR_TYPE("no-such-key")
-#define NO_SUCH_PLUGIN       XEN_ERROR_TYPE("no-such-plugin")
-#define PLUGIN_ERROR         XEN_ERROR_TYPE("plugin-error")
-#define NO_SUCH_MARK         XEN_ERROR_TYPE("no-such-mark")
-#define NO_SUCH_MENU         XEN_ERROR_TYPE("no-such-menu")
-#define CANNOT_PARSE         XEN_ERROR_TYPE("cannot-parse")
-#define NO_ACTIVE_SELECTION  XEN_ERROR_TYPE("no-active-selection")
-#define BAD_ARITY            XEN_ERROR_TYPE("bad-arity")
-#define NO_SUCH_COLOR        XEN_ERROR_TYPE("no-such-color")
-#define NO_SUCH_SOUND        XEN_ERROR_TYPE("no-such-sound")
-#define SND_GSL_ERROR        XEN_ERROR_TYPE("gsl-error")
-#define NO_SUCH_COLORMAP     XEN_ERROR_TYPE("no-such-colormap")
-
-#ifndef Float
-  #define Float float
-#endif
-
-#define SND_INIT_FILE_ENVIRONMENT_NAME "SND_INIT_FILE"
-#ifndef WINDOZE
-  #define INIT_FILE_NAME "~/.snd"
-#else
-  #define INIT_FILE_NAME "snd-init"
+  typedef long long int pointer_or_int_t;
 #endif
 
 #ifndef STRFTIME_FORMAT
-#define STRFTIME_FORMAT "%a %d-%b-%Y %H:%M %Z"
+  #define STRFTIME_FORMAT "%a %d-%b-%Y %H:%M %Z"
+#endif
+
+#if HAVE_STRCASECMP
+  #define STRCMP(a, b) strcasecmp(a, b)
+  #define STRNCMP(a, b, c) strncasecmp(a, b, c)
+#else
+  #define STRCMP(a, b) strcmp(a, b)
+  #define STRNCMP(a, b, c) strncmp(a, b, c)
 #endif
 
 #define XOR(a, b) ((a) ^ (b))
 /* can't get used to this operator -- in the good old days, ^ meant exponentiation */
 
-#define UNPACK_SOUND(a) (a >> 16)
-#define UNPACK_CHANNEL(a) (a & 0xff)
+#define UNPACK_SOUND(a) ((int)a >> 16)
+#define UNPACK_CHANNEL(a) ((int)a & 0xff)
 #define PACK_SOUND_AND_CHANNEL(a, b) ((a << 16) | b)
 
+#define POINT_BUFFER_SIZE 8192
+
 #define FILE_BUFFER_SIZE 8192
 /* this shouldn't be too much larger than the average un-peak-env'd time domain window size:
  *   as a read crosses the in-core boundary, it jumps ahead to the next windowful (everything
@@ -95,80 +70,118 @@
  */
 #define MAX_BUFFER_SIZE 65536
 #define MIX_FILE_BUFFER_SIZE 2048
-#define TIME_STR_SIZE 64
 #define PRINT_BUFFER_SIZE 512
-#define LABEL_BUFFER_SIZE 64
+#define LABEL_BUFFER_SIZE 128
 #define REPORTING_SIZE (MAX_BUFFER_SIZE * 8)
+
 /* progress bar (hourglass icon) is displayed if more than this many samples are being processed */
-#define NUM_GLASSES 15
+#if HAVE_XPM || USE_GTK
+  #define NUM_HOURGLASSES 15
+#else
+  #define NUM_HOURGLASSES 1
+#endif
 #define NUM_BOMBS 15
 
-#define AMP_ENV_CUTOFF 50000
+#define MARK_PLAY_ARROW_SIZE 10
 
-typedef enum {FILE_SAVE_AS, EDIT_SAVE_AS} save_dialog_t;
+#define PEAK_ENV_CUTOFF 50000
+#define MIN_INIT 1000000.0
+#define MAX_INIT -1000000.0
 
 #define DEFAULT_OUTPUT_CHANS 1
-#define DEFAULT_OUTPUT_SRATE 22050
-#define DEFAULT_OUTPUT_TYPE MUS_NEXT
-#define DEFAULT_OUTPUT_FORMAT MUS_BSHORT
+#define DEFAULT_OUTPUT_SRATE 44100
+#define DEFAULT_OUTPUT_HEADER_TYPE MUS_NEXT
+
+#if MUS_LITTLE_ENDIAN
+  #define DEFAULT_OUTPUT_DATA_FORMAT MUS_LFLOAT
+#else
+  #define DEFAULT_OUTPUT_DATA_FORMAT MUS_BFLOAT
+#endif
+/* mus-next is probably best here since intermediate/temp files can be any length (> 2^32 bytes)
+ *   and the next header-specified size, although 32 bits, is explicitly "advisory".
+ */
 
 #define NO_COMPLETER -1
 #define NO_SELECTION -1
 #define NO_END_SPECIFIED -1
 #define NO_MIX_TAG -1
 #define INVALID_MIX_ID -1
-#define INVALID_MIX_CHANNEL -2
 #define SAVE_ALL_CHANS -1
-#define INVALID_TRACK_ID -1
 #define ANY_MIX_ID -2
-#define ANY_TRACK_ID -2
 #define NO_COLOR -1
+#define NOT_A_GC_LOC -1
+#define NOT_A_SOUND -1
 
-typedef enum {NOT_IN_BACKGROUND, IN_BACKGROUND} play_process_t;
-typedef enum {WITHOUT_VIRTUAL_CHANNELS, WITH_VIRTUAL_CHANNELS} virtual_channels_t;
-typedef enum {WITH_FW_BUTTONS, WITH_ARROWS} fw_button_t;
-typedef enum {WITHOUT_HOOK, WITH_HOOK} with_hook_t;
-typedef enum {WITHOUT_WORD_WRAP, WITH_WORD_WRAP} with_word_wrap_t;
+#define MIX_FILE_NO_MIX -1
+#define MIX_FILE_NO_FILE -2
+#define MIX_FILE_NO_SP -3
+#define MIX_FILE_NO_TEMP_FILE -4
 
 #ifndef POPUP_BUTTON
   #define POPUP_BUTTON 3
 #endif
 
+typedef enum {SOUND_SAVE_AS, SELECTION_SAVE_AS, REGION_SAVE_AS} save_dialog_t;
+typedef enum {NOT_IN_BACKGROUND, IN_BACKGROUND} play_process_t;
+typedef enum {WITHOUT_VIRTUAL_CHANNELS, WITH_VIRTUAL_CHANNELS} virtual_channels_t;
+typedef enum {WITH_FW_BUTTONS, WITH_ARROWS} fw_button_t;
+typedef enum {WITHOUT_HOOK, WITH_HOOK} with_hook_t;
+typedef enum {WITHOUT_WORD_WRAP, WITH_WORD_WRAP} with_word_wrap_t;
+typedef enum {DRAG_ENTER, DRAG_LEAVE, DRAG_MOTION} drag_style_t;
 typedef enum {ENVED_AMPLITUDE, ENVED_SPECTRUM, ENVED_SRATE} enved_target_t;
 typedef enum {ENVELOPE_LINEAR, ENVELOPE_EXPONENTIAL} env_type_t;
-typedef enum {GRAPH_LINES, GRAPH_DOTS, GRAPH_FILLED, GRAPH_DOTS_AND_LINES, GRAPH_LOLLIPOPS} graph_style_t;
-#if defined(__GNUC__) && (!(defined(__cplusplus)))
-  #define GRAPH_STYLE_OK(Grf) ({ graph_style_t _snd_0_h_0 = Grf; ((_snd_0_h_0 >= GRAPH_LINES) && (_snd_0_h_0 <= GRAPH_LOLLIPOPS)); })
-#else
-  #define GRAPH_STYLE_OK(Grf) ((Grf >= GRAPH_LINES) && (Grf <= GRAPH_LOLLIPOPS))
-#endif
- /* using lollipop rather than the suggested popsicle because 
-  *   popsicle is a (capitalized) trade-name in the USA and I want lowercase,
-  *   lollipop is about 200 years older as an English word (1724 vs 1923),
-  *   and I think lollipop is more accurate -- in my experience lollipops have
-  *     circular candies whereas popsicles have a sort of squared-off bottom.
-  */
-
+typedef enum {GRAPH_LINES, GRAPH_DOTS, GRAPH_FILLED, GRAPH_DOTS_AND_LINES, GRAPH_LOLLIPOPS, NUM_GRAPH_STYLES} graph_style_t;
 typedef enum {APPLY_TO_SOUND, APPLY_TO_CHANNEL, APPLY_TO_SELECTION} snd_apply_t;
 typedef enum {GRAPH_ONCE, GRAPH_AS_SONOGRAM, GRAPH_AS_SPECTROGRAM, GRAPH_AS_WAVOGRAM} graph_type_t;
-typedef enum {ZOOM_FOCUS_LEFT, ZOOM_FOCUS_RIGHT, ZOOM_FOCUS_ACTIVE, ZOOM_FOCUS_MIDDLE} zoom_focus_t;
-typedef enum {DONT_LOCK_MIXES, LOCK_MIXES} lock_mix_t;
-typedef enum {DONT_DELETE_ME, DELETE_ME, ALREADY_DELETED, MULTICHANNEL_DELETION} file_delete_t;
+typedef enum {ZOOM_FOCUS_LEFT, ZOOM_FOCUS_RIGHT, ZOOM_FOCUS_ACTIVE, ZOOM_FOCUS_MIDDLE, ZOOM_FOCUS_PROC} zoom_focus_t;
+typedef enum {DONT_DELETE_ME, DELETE_ME, ALREADY_DELETED, MULTICHANNEL_DELETION, MULTICHANNEL_DELETION_IF_FILE} file_delete_t;
 typedef enum {SND_REOPEN_CLOSED_FILE, SND_OPEN_CHANNEL, SND_COPY_READER, SND_INSERT_FILE, SND_CHANGE_FILE, SND_OVERRIDE_FILE, SND_MIX_FILE} open_reason_t;
 typedef enum {CURSOR_CROSS, CURSOR_LINE, CURSOR_PROC} cursor_style_t;
-typedef enum {SHOW_NO_AXES, SHOW_ALL_AXES, SHOW_X_AXIS, SHOW_ALL_AXES_UNLABELLED, SHOW_X_AXIS_UNLABELLED} show_axes_t;
-typedef enum {DONT_NORMALIZE, NORMALIZE_BY_CHANNEL, NORMALIZE_BY_SOUND, NORMALIZE_GLOBALLY} fft_normalize_t;
-typedef enum {NO_PROBLEM, BLIND_LEAP, GIVE_UP, HUNKER_DOWN} disk_space_t;
+typedef enum {SHOW_NO_AXES, SHOW_ALL_AXES, SHOW_X_AXIS, SHOW_ALL_AXES_UNLABELLED, SHOW_X_AXIS_UNLABELLED, SHOW_BARE_X_AXIS, NUM_SHOW_AXES} show_axes_t;
+typedef enum {DONT_NORMALIZE, NORMALIZE_BY_CHANNEL, NORMALIZE_BY_SOUND, NORMALIZE_GLOBALLY, NUM_TRANSFORM_NORMALIZATIONS} fft_normalize_t;
+typedef enum {NO_DISK_SPACE, NOT_ENOUGH_DISK_SPACE, DISK_SPACE_OK} disk_space_t;
 typedef enum {CHAN_GC, CHAN_IGC, CHAN_SELGC, CHAN_CGC, CHAN_MGC, CHAN_MXGC, CHAN_TMPGC} chan_gc_t;
-typedef enum {CURRENT_FILE_VIEWER, PREVIOUS_FILE_VIEWER, REGION_VIEWER} file_viewer_t;
-typedef enum {COLOR_DIALOG, ORIENTATION_DIALOG, ENVED_DIALOG, ERROR_DIALOG, YES_OR_NO_DIALOG, TRANSFORM_DIALOG,
-	      FILE_OPEN_DIALOG, FILE_SAVE_AS_DIALOG, VIEW_FILES_DIALOG, RAW_DATA_DIALOG, NEW_FILE_DIALOG,
-	      FILE_MIX_DIALOG, EDIT_HEADER_DIALOG, FIND_DIALOG, HELP_DIALOG, COMPLETION_DIALOG, MIX_DIALOG,
-	      PRINT_DIALOG, RECORDER_DIALOG, REGION_DIALOG, POST_IT_DIALOG, TRACK_DIALOG} snd_dialog_t;
-#define NUM_DIALOGS 22
+typedef enum {NOT_A_VIEWER, FILE_VIEWER, REGION_VIEWER} file_viewer_t; /* 1 and 2, according to docs for mouse-enter-label-hook */
+typedef enum {COLOR_ORIENTATION_DIALOG, UNUSED_DIALOG_1, ENVED_DIALOG, UNUSED_DIALOG_2, UNUSED_DIALOG_3, TRANSFORM_DIALOG,
+	      FILE_OPEN_DIALOG, SOUND_SAVE_AS_DIALOG, VIEW_FILES_DIALOG, RAW_DATA_DIALOG, NEW_FILE_DIALOG,
+	      FILE_MIX_DIALOG, EDIT_HEADER_DIALOG, FIND_DIALOG, HELP_DIALOG, UNUSED_DIALOG_4, MIX_DIALOG,
+	      PRINT_DIALOG, RECORDER_DIALOG, REGION_DIALOG, POST_IT_DIALOG, UNUSED_DIALOG_5, SELECTION_SAVE_AS_DIALOG,
+              FILE_INSERT_DIALOG, REGION_SAVE_AS_DIALOG, PREFERENCES_DIALOG, NUM_DIALOGS} snd_dialog_t;
 typedef enum {SOUND_IDLE, SOUND_NORMAL, SOUND_WRAPPER, SOUND_REGION, SOUND_READER} sound_inuse_t;
-typedef enum {NOT_FILING, INPUT_FILING, REGION_FILING, CHANNEL_FILING, TEMP_FILING, CHANGE_FILING, INSERT_FILING, MACRO_FILING} sp_filing_t;
+typedef enum {NOT_FILING, INPUT_FILING, SELECTION_FILING, CHANNEL_FILING, TEMP_FILING, CHANGE_FILING, 
+	      INSERT_FILING, MACRO_FILING, DOIT_SELECTION_FILING, DOIT_CHANNEL_FILING, SAVE_EDITS_FILING} sp_filing_t;
+typedef enum {IO_NO_ERROR, IO_SAVE_HOOK_CANCELLATION, IO_BAD_CHANNEL, IO_CANT_REOPEN_FILE, IO_TOO_MANY_OPEN_FILES, 
+	      IO_UNKNOWN_SNDLIB_ERROR, IO_NO_MEMORY, IO_CANT_OPEN_FILE, IO_NO_FILENAME, IO_BAD_DATA_FORMAT, IO_BAD_HEADER_TYPE,
+	      IO_SNDLIB_UNINITIALIZED, IO_NOT_A_SOUND_FILE, IO_FILE_CLOSED, IO_WRITE_ERROR, IO_INTERRUPTED, IO_CANT_CLOSE_FILE, 
+	      IO_BAD_HEADER, IO_DISK_FULL, IO_WRITE_PROTECTED, IO_CANT_READ_SELECTION_FILE, IO_NEED_WRITE_CONFIRMATION, 
+	      IO_NO_CHANGES, IO_EDIT_HOOK_CANCELLATION, IO_CANT_CREATE_FILE, IO_ERROR_NUM} io_error_t;
+typedef enum {INSERTION_EDIT, DELETION_EDIT, CHANGE_EDIT, INITIALIZE_EDIT, SCALED_EDIT, ZERO_EDIT, RAMP_EDIT, 
+	      PTREE_EDIT, EXTEND_EDIT, MIX_EDIT, CHANGE_MIX_EDIT,
+              NUM_EDIT_TYPES} edit_t;
+typedef enum {SAMPLER, REGION_READER, MIX_READER} reader_t;
+typedef enum {MIX_FOLLOWS_SYNC, MIX_SETS_SYNC_LOCALLY} mix_sync_t;
+typedef enum {CHANNEL_FREED, CHANNEL_INACTIVE, CHANNEL_INITIALIZED, CHANNEL_HAS_EDIT_LIST, CHANNEL_HAS_AXES} channel_state_t;
 
+enum {BLACK_AND_WHITE_COLORMAP, GRAY_COLORMAP, HOT_COLORMAP, COOL_COLORMAP, BONE_COLORMAP, COPPER_COLORMAP, PINK_COLORMAP, JET_COLORMAP, PRISM_COLORMAP,
+      AUTUMN_COLORMAP, WINTER_COLORMAP, SPRING_COLORMAP, SUMMER_COLORMAP, RAINBOW_COLORMAP, FLAG_COLORMAP, PHASES_COLORMAP, NUM_BUILTIN_COLORMAPS};
+
+
+#define SERIOUS_IO_ERROR(Err) ((Err != IO_NO_ERROR) && \
+                               (Err != IO_EDIT_HOOK_CANCELLATION) && \
+                               (Err != IO_SAVE_HOOK_CANCELLATION) && \
+                               (Err != IO_INTERRUPTED) && \
+                               (Err != IO_NEED_WRITE_CONFIRMATION) && \
+                               (Err != IO_NO_CHANGES))
+
+typedef enum {SS_ANY_WATCHER, SS_FILE_OPEN_WATCHER, SS_MUS_ERROR_WATCHER, SS_SND_ERROR_WATCHER} ss_watcher_t;
+typedef enum {SS_FILE_OPENED, SS_FILE_CLOSED, SS_MUS_ERROR, SS_SND_ERROR} ss_watcher_reason_t;
+typedef enum {SP_ANY_WATCHER, SP_READ_ONLY_WATCHER, SP_REVERT_WATCHER} sp_watcher_t;
+typedef enum {SP_READ_ONLY_CHANGED, SP_IS_CLOSING, SP_REVERTED} sp_watcher_reason_t;
+typedef enum {SELECTION_INACTIVE, SELECTION_ACTIVE, SELECTION_CHANGED, SELECTION_IN_DOUBT} selection_watcher_reason_t;
+typedef enum {WITH_READABLE_HEADERS, WITH_WRITABLE_HEADERS, WITH_BUILTIN_HEADERS} header_choice_t;
+enum {SORT_A_TO_Z, SORT_Z_TO_A, SORT_NEW_TO_OLD, SORT_OLD_TO_NEW, SORT_SMALL_TO_BIG, SORT_BIG_TO_SMALL, SORT_XEN};
+
+typedef enum {FILE_READ_WRITE, FILE_READ_ONLY} read_only_t;
 #define FORCE_REFFT true
 #define DONT_FORCE_REFFT false
 #define FORCE_REDISPLAY true
@@ -177,35 +190,59 @@ typedef enum {NOT_FILING, INPUT_FILING, REGION_FILING, CHANNEL_FILING, TEMP_FILI
 #define WITHOUT_EVENTS false
 #define NOT_IN_AS_ONE_EDIT false
 #define IN_AS_ONE_EDIT true
+#define FILE_SELECTED true
+#define FILE_NOT_SELECTED false
+#define EXIT_FORCED true
+#define EXIT_NOT_FORCED false
 
 #if HAVE_RUBY
   #define TO_PROC_NAME(Str) xen_scheme_procedure_to_ruby(Str)
   #define TO_VAR_NAME(Str) xen_scheme_constant_to_ruby(Str)
   #define PROC_OPEN "("
   #define PROC_SEP ", "
-  #define VECTOR_OPEN "["
-  #define VECTOR_CLOSE "]"
+  #define LIST_OPEN "["
+  #define LIST_CLOSE "]"
   #define BPAREN ""
   #define EPAREN ""
-#else
+  #define PROC_FALSE "false"
+  #define PROC_TRUE "true"
+  #define PROC_QUOTE ""
+#endif
+#if HAVE_FORTH
   #define TO_PROC_NAME(Str) Str
   #define TO_VAR_NAME(Str) Str
   #define PROC_OPEN " "
   #define PROC_SEP " "
-  #define VECTOR_OPEN "'#("
-  #define VECTOR_CLOSE ")"
+  #define LIST_OPEN " '( "
+  #define LIST_CLOSE " ) "
+  #define BPAREN ""
+  #define EPAREN " drop"
+  #define PROC_FALSE "#f"
+  #define PROC_TRUE  "#t"
+  #define PROC_QUOTE ""
+#endif
+#if HAVE_SCHEME || (!HAVE_EXTENSION_LANGUAGE)
+  #define TO_PROC_NAME(Str) Str
+  #define TO_VAR_NAME(Str) Str
+  #define PROC_OPEN " "
+  #define PROC_SEP " "
+  #define LIST_OPEN "(list "
+  #define LIST_CLOSE ")"
   #define BPAREN "("
   #define EPAREN ")"
+  #define PROC_FALSE "#f"
+  #define PROC_TRUE  "#t"
+  #define PROC_QUOTE "'"
 #endif
 
 #define MAX_MAIN_MENUS 64
 #define NO_REGIONS -2
 #define INVALID_REGION -1
+#define NO_LIST -1
 
 typedef enum {READ_FORWARD, READ_BACKWARD} read_direction_t;
-typedef enum {DONT_FOLLOW, FOLLOW_ALWAYS, FOLLOW_ONCE} tracking_cursor_t;
+typedef enum {DONT_TRACK, ALWAYS_TRACK, TRACK_ONCE} tracking_cursor_t;
 typedef enum {DONT_UPDATE_DISPLAY, UPDATE_DISPLAY} cut_selection_regraph_t;
-typedef enum {NOT_FROM_ENVED, FROM_ENVED} enved_progress_t;
 typedef enum {SEARCH_OK, SEARCH_FAILED} search_result_t;
 typedef enum {IGNORE_CACHE, REMOVE_FROM_CACHE} cache_remove_t;
 typedef enum {FFT_UNCHANGED, FFT_CHANGED, FFT_CHANGE_LOCKED} fft_change_t;
@@ -213,35 +250,31 @@ typedef enum {WITHOUT_GRAPH, WITH_GRAPH, WITHOUT_INITIAL_GRAPH_HOOK} channel_gra
 typedef enum {NOT_PRINTING, PRINTING} printing_t;
 typedef enum {NO_X_AXIS, WITH_X_AXIS} with_x_axis_t;
 typedef enum {NO_GRID, WITH_GRID} with_grid_t;
-typedef enum {WITH_LINEAR_AXES, WITH_LOG_X_AXIS, WITH_LOG_Y_AXIS, WITH_LOG_AXES} log_axis_t;
+typedef enum {WITH_LINEAR_AXES, WITH_LOG_X_AXIS, WITH_LOG_Y_AXIS} log_axis_t;
 
-#if defined(__GNUC__) && (!(defined(__cplusplus)))
-  #define IS_PLAYER(snd) ({ snd_info *_snd_0_h_1 = snd; ((_snd_0_h_1) && (_snd_0_h_1->index < 0)); })
-#else
-  #define IS_PLAYER(snd) ((snd) && (snd->index < 0))
-#endif
-
-#define PLAYER(snd) (-(snd->index))
-typedef enum {NO_PLAYERS, PLAYERS_OK} sp_sound_t;
+#define GET_NEW_SYNC -1
+#define GET_ORIGINAL_SYNC -2
 
 #define OVER_SELECTION true
 #define OVER_SOUND false
 
 #define AT_CURRENT_EDIT_POSITION -1
 
-typedef enum {X_AXIS_IN_SECONDS, X_AXIS_IN_SAMPLES, X_AXIS_AS_PERCENTAGE, X_AXIS_IN_BEATS} x_axis_style_t;
-typedef enum {SPEED_CONTROL_AS_FLOAT, SPEED_CONTROL_AS_RATIO, SPEED_CONTROL_AS_SEMITONE} speed_style_t;
+typedef enum {X_AXIS_IN_SECONDS, X_AXIS_IN_SAMPLES, X_AXIS_AS_PERCENTAGE, X_AXIS_IN_BEATS, 
+	      X_AXIS_IN_MEASURES, X_AXIS_AS_CLOCK, NUM_X_AXIS_STYLES} x_axis_style_t;
+typedef enum {SPEED_CONTROL_AS_FLOAT, SPEED_CONTROL_AS_RATIO, SPEED_CONTROL_AS_SEMITONE, NUM_SPEED_CONTROL_STYLES} speed_style_t;
 typedef enum {CURSOR_IN_VIEW, CURSOR_ON_LEFT, CURSOR_ON_RIGHT, CURSOR_IN_MIDDLE, KEYBOARD_NO_ACTION} kbd_cursor_t;
-typedef enum {CHANNELS_SEPARATE, CHANNELS_COMBINED, CHANNELS_SUPERIMPOSED} channel_style_t;
+typedef enum {CHANNELS_SEPARATE, CHANNELS_COMBINED, CHANNELS_SUPERIMPOSED, NOT_A_CHANNEL_STYLE} channel_style_t;
+#define NUM_CHANNEL_STYLES 3
 typedef enum {FD_CLOSED, FD_OPEN} fd_open_t;
 typedef enum {PRINT_SND, PRINT_ENV} print_choice_t;
 typedef enum {SND_DATA_NO_DATA, SND_DATA_FILE, SND_DATA_BUFFER} snd_data_file_t;
 typedef enum {SOUNDS_VERTICAL, SOUNDS_HORIZONTAL, SOUNDS_IN_NOTEBOOK, SOUNDS_IN_SEPARATE_WINDOWS} sound_style_t;
-enum {FOURIER, WAVELET, WALSH, AUTOCORRELATION, CEPSTRUM, HAAR}; /* not typedef'd -- grows as new ones are added */
-#define NUM_FFT_WINDOWS 19
+enum {FOURIER, WAVELET, WALSH, AUTOCORRELATION, CEPSTRUM, HAAR, NUM_BUILTIN_TRANSFORM_TYPES}; /* not typedef'd -- grows as new ones are added */
 #define NUM_WAVELETS 48
 
-typedef enum {FCP_X_ANGLE, FCP_X_SCALE, FCP_Y_ANGLE, FCP_Y_SCALE, FCP_Z_ANGLE, FCP_Z_SCALE, FCP_CUTOFF, FCP_START, FCP_BETA, FCP_BEATS} fcp_t;
+typedef enum {FCP_X_ANGLE, FCP_X_SCALE, FCP_Y_ANGLE, FCP_Y_SCALE, FCP_Z_ANGLE, FCP_Z_SCALE, 
+	      FCP_SPECTRUM_END, FCP_SPECTRUM_START, FCP_ALPHA, FCP_BETA, FCP_BEATS} fcp_t;
 typedef enum {TIME_AXIS_INFO, TRANSFORM_AXIS_INFO, LISP_AXIS_INFO} axis_info_t;
 typedef enum {COLOR_POSITION, COLOR_ZOOM} slider_choice_t;
 typedef enum {MINI_OFF, MINI_CURSOR, MINI_FIND, MINI_PROMPT, MINI_REPORT, MINI_USER} minibuffer_choice_t;
@@ -249,8 +282,11 @@ typedef enum {MINI_OFF, MINI_CURSOR, MINI_FIND, MINI_PROMPT, MINI_REPORT, MINI_U
 typedef enum {PLAY_COMPLETE, PLAY_CLOSE, PLAY_BUTTON_UNSET, PLAY_STOP_CALLED, PLAY_C_G, PLAY_NO_CHANNEL,
 	      PLAY_ERROR, PLAY_APPLY, PLAY_EDIT, PLAY_C_T} play_stop_t;
 
-#define DEFAULT_MIN_DB -60.0
-#define NO_LIST -1
+typedef enum {NO_REQUESTOR, FROM_UPDATE, FROM_VIEW_FILES, FROM_DRAG_AND_DROP, FROM_OPEN_DIALOG,
+	      FROM_KEYBOARD, FROM_STARTUP, FROM_REGION_EDIT, FROM_NEW_FILE_DIALOG, FROM_OPEN_SOUND, 
+	      FROM_OPEN_RAW_SOUND, FROM_VIEW_SOUND, FROM_NEW_SOUND, FROM_RAW_DATA_DIALOG, FROM_MIX_DIALOG,
+              FROM_INSERT_DIALOG, FROM_VIEW_FILES_MIX_DIALOG, FROM_VIEW_FILES_INSERT_DIALOG, 
+	      FROM_OPEN_RECENT_MENU, FROM_OPEN_DIALOG_POPUP} open_requestor_t;
 
 #define DEFAULT_AMP_CONTROL 1.0
 #define DEFAULT_CONTRAST_CONTROL 0.0
@@ -263,6 +299,7 @@ typedef enum {PLAY_COMPLETE, PLAY_CLOSE, PLAY_BUTTON_UNSET, PLAY_STOP_CALLED, PL
 #define DEFAULT_FILTER_CONTROL_P false
 #define DEFAULT_REVERB_CONTROL_P false
 
+
 #define filter_control_in_dB(ss) ss->Filter_Control_In_Db
 #define in_set_filter_control_in_dB(ss, val) ss->Filter_Control_In_Db = val
 #define DEFAULT_FILTER_CONTROL_IN_DB false
@@ -363,22 +400,17 @@ typedef enum {PLAY_COMPLETE, PLAY_CLOSE, PLAY_BUTTON_UNSET, PLAY_STOP_CALLED, PL
 #define in_set_filter_control_order(ss, val) ss->Filter_Control_Order = val
 #define DEFAULT_FILTER_CONTROL_ORDER 20
 
-#define tempo_control_min(ss) ss->Tempo_Control_Min
-#define in_set_tempo_control_min(ss, val) ss->Tempo_Control_Min = val
-#define DEFAULT_TEMPO_CONTROL_MIN 0.0
-
-#define tempo_control_max(ss) ss->Tempo_Control_Max
-#define in_set_tempo_control_max(ss, val) ss->Tempo_Control_Max = val
-#define DEFAULT_TEMPO_CONTROL_MAX 8.0
-
 #define in_show_controls(ss) ss->Show_Controls
 #define in_set_show_controls(ss, val) ss->Show_Controls = val
 #define DEFAULT_SHOW_CONTROLS false
 
-#define cursor_follows_play(ss) ss->Cursor_Follows_Play
-#define in_set_cursor_follows_play(ss, val) ss->Cursor_Follows_Play = val
-#define DEFAULT_CURSOR_FOLLOWS_PLAY DONT_FOLLOW
+#define with_tracking_cursor(ss) ss->With_Tracking_Cursor
+#define in_set_with_tracking_cursor(ss, val) ss->With_Tracking_Cursor = val
+#define DEFAULT_WITH_TRACKING_CURSOR DONT_TRACK
 
+#define just_sounds(ss) ss->Just_Sounds
+#define set_just_sounds(val) ss->Just_Sounds = val
+#define DEFAULT_JUST_SOUNDS false
 
 #define DEFAULT_SYNC 0
 #define DEFAULT_INIT_WINDOW_X -1
@@ -386,8 +418,8 @@ typedef enum {PLAY_COMPLETE, PLAY_CLOSE, PLAY_BUTTON_UNSET, PLAY_STOP_CALLED, PL
 #define DEFAULT_INIT_WINDOW_WIDTH 0
 #define DEFAULT_INIT_WINDOW_HEIGHT 0
 
-#define default_output_type(ss) ss->Default_Output_Type
-#define set_default_output_type(a) ss->Default_Output_Type = a
+#define default_output_header_type(ss) ss->Default_Output_Header_Type
+#define set_default_output_header_type(a) ss->Default_Output_Header_Type = a
 
 #define default_output_chans(ss) ss->Default_Output_Chans
 #define set_default_output_chans(a) ss->Default_Output_Chans = a
@@ -395,8 +427,8 @@ typedef enum {PLAY_COMPLETE, PLAY_CLOSE, PLAY_BUTTON_UNSET, PLAY_STOP_CALLED, PL
 #define default_output_srate(ss) ss->Default_Output_Srate
 #define set_default_output_srate(a) ss->Default_Output_Srate = a
 
-#define default_output_format(ss) ss->Default_Output_Format
-#define set_default_output_format(a) ss->Default_Output_Format = a
+#define default_output_data_format(ss) ss->Default_Output_Data_Format
+#define set_default_output_data_format(a) ss->Default_Output_Data_Format = a
 
 #define dac_size(ss) ss->Dac_Size
 #define set_dac_size(a) ss->Dac_Size = a
@@ -408,7 +440,7 @@ typedef enum {PLAY_COMPLETE, PLAY_CLOSE, PLAY_BUTTON_UNSET, PLAY_STOP_CALLED, PL
      #define DEFAULT_DAC_SIZE 256
   #endif
 #else
-  #if MAC_OSX
+  #if MUS_MAC_OSX
     #define DEFAULT_DAC_SIZE 64
   #else
     #define DEFAULT_DAC_SIZE 1024
@@ -459,6 +491,10 @@ typedef enum {PLAY_COMPLETE, PLAY_CLOSE, PLAY_BUTTON_UNSET, PLAY_STOP_CALLED, PL
 #define in_set_color_scale(a) ss->Color_Scale = a
 #define DEFAULT_COLOR_SCALE 1.0
 
+#define fft_window_alpha(ss) ss->Fft_Window_Alpha
+#define in_set_fft_window_alpha(a) ss->Fft_Window_Alpha = a
+#define DEFAULT_FFT_WINDOW_ALPHA 0.0
+
 #define fft_window_beta(ss) ss->Fft_Window_Beta
 #define in_set_fft_window_beta(a) ss->Fft_Window_Beta = a
 #define DEFAULT_FFT_WINDOW_BETA 0.0
@@ -489,11 +525,13 @@ typedef enum {PLAY_COMPLETE, PLAY_CLOSE, PLAY_BUTTON_UNSET, PLAY_STOP_CALLED, PL
 
 #define optimization(ss) ss->Optimization
 #define set_optimization(a) ss->Optimization = a
-#define DEFAULT_OPTIMIZATION 0
+#define DEFAULT_OPTIMIZATION 6
 
 #define dot_size(ss) ss->Dot_Size
 #define in_set_dot_size(a) ss->Dot_Size = a
 #define DEFAULT_DOT_SIZE 1
+#define MIN_DOT_SIZE 0
+#define MAX_DOT_SIZE 100
 
 #define grid_density(ss) ss->Grid_Density
 #define in_set_grid_density(a) ss->Grid_Density = a
@@ -511,13 +549,13 @@ typedef enum {PLAY_COMPLETE, PLAY_CLOSE, PLAY_BUTTON_UNSET, PLAY_STOP_CALLED, PL
 #define set_ask_before_overwrite(a) ss->Ask_Before_Overwrite = a
 #define DEFAULT_ASK_BEFORE_OVERWRITE false
 
-#define spectro_cutoff(ss) ss->Spectro_Cutoff
-#define in_set_spectro_cutoff(a) ss->Spectro_Cutoff = a
-#define DEFAULT_SPECTRO_CUTOFF 1.0
+#define spectrum_end(ss) ss->Spectrum_End
+#define in_set_spectrum_end(a) ss->Spectrum_End = a
+#define DEFAULT_SPECTRUM_END 1.0
 
-#define spectro_start(ss) ss->Spectro_Start
-#define in_set_spectro_start(a) ss->Spectro_Start = a
-#define DEFAULT_SPECTRO_START 0.0
+#define spectrum_start(ss) ss->Spectrum_Start
+#define in_set_spectrum_start(a) ss->Spectrum_Start = a
+#define DEFAULT_SPECTRUM_START 0.0
 
 #define spectro_x_angle(ss) ss->Spectro_X_Angle
 #define in_set_spectro_x_angle(a) ss->Spectro_X_Angle = a
@@ -652,14 +690,14 @@ typedef enum {PLAY_COMPLETE, PLAY_CLOSE, PLAY_BUTTON_UNSET, PLAY_STOP_CALLED, PL
 
 #define temp_dir(ss) ss->Temp_Dir
 #define set_temp_dir(a) ss->Temp_Dir = a
-#ifndef DEFAULT_TEMP_DIR
-  #define DEFAULT_TEMP_DIR NULL
+#ifndef MUS_DEFAULT_TEMP_DIR
+  #define MUS_DEFAULT_TEMP_DIR NULL
 #endif
 
 #define save_dir(ss) ss->Save_Dir
 #define set_save_dir(a) ss->Save_Dir = a
-#ifndef DEFAULT_SAVE_DIR
-  #define DEFAULT_SAVE_DIR NULL
+#ifndef MUS_DEFAULT_SAVE_DIR
+  #define MUS_DEFAULT_SAVE_DIR NULL
 #endif
 
 #define ladspa_dir(ss) ss->Ladspa_Dir
@@ -668,17 +706,8 @@ typedef enum {PLAY_COMPLETE, PLAY_CLOSE, PLAY_BUTTON_UNSET, PLAY_STOP_CALLED, PL
   #define DEFAULT_LADSPA_DIR NULL
 #endif
 
-#define vu_font(ss) ss->Vu_Font
-#define set_vu_font(a) ss->Vu_Font = a
-#define DEFAULT_VU_FONT NULL
-
-#define vu_font_size(ss) ss->Vu_Font_Size
-#define set_vu_font_size(a) ss->Vu_Font_Size = a
-#define DEFAULT_VU_FONT_SIZE 1.0
-
-#define vu_size(ss) ss->Vu_Size
-#define set_vu_size(a) ss->Vu_Size = a
-#define DEFAULT_VU_SIZE 1.0
+#define open_file_dialog_directory(ss) ss->Open_File_Dialog_Directory
+#define set_open_file_dialog_directory(a) ss->Open_File_Dialog_Directory = a
 
 #define wavelet_type(ss) ss->Wavelet_Type
 #define in_set_wavelet_type(a) ss->Wavelet_Type = a
@@ -716,6 +745,10 @@ typedef enum {PLAY_COMPLETE, PLAY_CLOSE, PLAY_BUTTON_UNSET, PLAY_STOP_CALLED, PL
 #define set_with_background_processes(a) ss->With_Background_Processes = a
 #define DEFAULT_WITH_BACKGROUND_PROCESSES true
 
+#define with_file_monitor(ss) ss->With_File_Monitor
+#define set_with_file_monitor(a) ss->With_File_Monitor = a
+#define DEFAULT_WITH_FILE_MONITOR true
+
 #define wavo_hop(ss) ss->Wavo_Hop
 #define in_set_wavo_hop(a) ss->Wavo_Hop = a
 #define DEFAULT_WAVO_HOP 3
@@ -732,9 +765,14 @@ typedef enum {PLAY_COMPLETE, PLAY_CLOSE, PLAY_BUTTON_UNSET, PLAY_STOP_CALLED, PL
 #define in_set_beats_per_minute(a) ss->Beats_Per_Minute = a
 #define DEFAULT_BEATS_PER_MINUTE 60.0
 
+#define beats_per_measure(ss) ss->Beats_Per_Measure
+#define in_set_beats_per_measure(a) ss->Beats_Per_Measure = a
+#define DEFAULT_BEATS_PER_MEASURE 4
+
 #define zero_pad(ss) ss->Zero_Pad
 #define in_set_zero_pad(a) ss->Zero_Pad = a
 #define DEFAULT_ZERO_PAD 0
+#define MAX_ZERO_PAD 1000
 
 #define show_transform_peaks(ss) ss->Show_Transform_Peaks
 #define in_set_show_transform_peaks(a) ss->Show_Transform_Peaks = a
@@ -784,37 +822,45 @@ typedef enum {PLAY_COMPLETE, PLAY_CLOSE, PLAY_BUTTON_UNSET, PLAY_STOP_CALLED, PL
 #define in_set_fft_log_frequency(a) ss->Fft_Log_Frequency = a
 #define DEFAULT_FFT_LOG_FREQUENCY false
 
+#define fft_with_phases(ss) ss->Fft_With_Phases
+#define in_set_fft_with_phases(a) ss->Fft_With_Phases = a
+#define DEFAULT_FFT_WITH_PHASES false
+
 #define cursor_style(ss) ss->Cursor_Style
 #define in_set_cursor_style(a) ss->Cursor_Style = a
 #define DEFAULT_CURSOR_STYLE CURSOR_CROSS
 
+#define tracking_cursor_style(ss) ss->Tracking_Cursor_Style
+#define in_set_tracking_cursor_style(a) ss->Tracking_Cursor_Style = a
+#define DEFAULT_TRACKING_CURSOR_STYLE CURSOR_CROSS
+
 #define cursor_size(ss) ss->Cursor_Size
 #define in_set_cursor_size(a) ss->Cursor_Size = a
 #define DEFAULT_CURSOR_SIZE 15
 
 #define channel_style(ss) ss->Channel_Style
 #define in_set_channel_style(a) ss->Channel_Style = a
-#define DEFAULT_CHANNEL_STYLE CHANNELS_SEPARATE
+#define DEFAULT_CHANNEL_STYLE CHANNELS_COMBINED
 
 #define sound_style(ss) ss->Sound_Style
 #define set_sound_style(a) ss->Sound_Style = a
 #define DEFAULT_SOUND_STYLE SOUNDS_VERTICAL
 
 #define listener_prompt(ss) ss->Listener_Prompt
-#define set_listener_prompt(a) ss->Listener_Prompt = a
+#define in_set_listener_prompt(a) ss->Listener_Prompt = a
 #define DEFAULT_LISTENER_PROMPT ">"
 
 #define print_length(ss) ss->Print_Length
 #define set_print_length(a) ss->Print_Length = a
 #define DEFAULT_PRINT_LENGTH 12
 
-#define previous_files_sort(ss) ss->Previous_Files_Sort
-#define set_previous_files_sort(a) ss->Previous_Files_Sort = a
-#define DEFAULT_PREVIOUS_FILES_SORT 0
+#define view_files_sort(ss) ss->View_Files_Sort
+#define set_view_files_sort(a) ss->View_Files_Sort = a
+#define DEFAULT_VIEW_FILES_SORT SORT_A_TO_Z
 
 #define enved_clip_p(ss) ss->enved->clip_p
 #define in_set_enved_clip_p(a) ss->enved->clip_p = a
-#define DEFAULT_ENVED_CLIP_P false
+#define DEFAULT_ENVED_CLIP_P true
 
 #define enved_wave_p(ss) ss->Enved_Wave_p
 #define in_set_enved_wave_p(a) ss->Enved_Wave_p = a
@@ -854,9 +900,9 @@ typedef enum {PLAY_COMPLETE, PLAY_CLOSE, PLAY_BUTTON_UNSET, PLAY_STOP_CALLED, PL
 
 #define in_graph_cursor(ss) ss->Graph_Cursor
 
-#define data_clipped(ss) ss->Data_Clipped
-#define set_data_clipped(a) ss->Data_Clipped = a
-#define DEFAULT_DATA_CLIPPED false
+#define clipping(ss) ss->Clipping
+#define set_clipping(a) ss->Clipping = a
+#define DEFAULT_CLIPPING false
 
 #define html_dir(ss) ss->HTML_Dir
 #define set_html_dir_1(a) ss->HTML_Dir = a
@@ -864,7 +910,7 @@ typedef enum {PLAY_COMPLETE, PLAY_CLOSE, PLAY_BUTTON_UNSET, PLAY_STOP_CALLED, PL
 
 #define html_program(ss) ss->HTML_Program
 #define set_html_program(a) ss->HTML_Program = a
-#define DEFAULT_HTML_PROGRAM "mozilla"
+#define DEFAULT_HTML_PROGRAM "firefox"
 
 #define graphs_horizontal(ss) ss->Graphs_Horizontal
 #define in_set_graphs_horizontal(a) ss->Graphs_Horizontal = a
@@ -886,4 +932,8 @@ typedef enum {PLAY_COMPLETE, PLAY_CLOSE, PLAY_BUTTON_UNSET, PLAY_STOP_CALLED, PL
 #define set_mark_tag_height(a) ss->Mark_Tag_Height = a
 #define DEFAULT_MARK_TAG_HEIGHT 4
 
+#define min_dB(ss) ss->Min_dB
+#define set_min_dB(a) ss->Min_dB = a
+#define DEFAULT_MIN_DB -60.0
+
 #endif
