@@ -1,46 +1,46 @@
 #include "snd.h"
 
-/* -------------------------------- DATA STRUCTURES -------------------------------- 
- *
- * axis_info: axis data
- * file_info: header data describing sound data in file
- * fft_info: data relating to one fft display
- * chan_info: state info for one channel
- * snd_info: state info for one sound
- * snd_state: overall state of program
- */
 
 chan_info *make_chan_info(chan_info *cip, int chan, snd_info *sound)
 {
   chan_info *cp; /* may be re-used */
   if (!cip)
     {
-      cp = (chan_info *)CALLOC(1, sizeof(chan_info)); 
-      cp->cgx = (chan_context *)CALLOC(1, sizeof(chan_context));
-      (cp->cgx)->ax = (axis_context *)CALLOC(1, sizeof(axis_context));
-      cp->have_mixes = false;
+      cp = (chan_info *)calloc(1, sizeof(chan_info)); 
+      cp->cgx = (chan_context *)calloc(1, sizeof(chan_context));
+      cp->cgx->ax = (axis_context *)calloc(1, sizeof(axis_context));
+#if USE_GTK && USE_CAIRO
+      cp->cgx->progress_pct = -1.0;
+#endif
       cp->last_sonogram = NULL;
+      cp->last_wavogram = NULL;
       cp->temp_sonogram = NULL;
 #if HAVE_GL
       cp->gl_fft_list = NO_LIST;
+      cp->gl_wavo_list = NO_LIST;
 #endif
       cp->edit_hook = XEN_FALSE;
+      cp->edit_hook_loc = NOT_A_GC_LOC;
       cp->after_edit_hook = XEN_FALSE;
+      cp->after_edit_hook_loc = NOT_A_GC_LOC;
       cp->undo_hook = XEN_FALSE;
-      cp->properties = XEN_FALSE; /* will be a vector of 1 element (permanently protected) if it's ever used */
+      cp->undo_hook_loc = NOT_A_GC_LOC;
+      cp->properties = XEN_FALSE; /* will be a vector of 1 element if it's ever used */
+      cp->properties_loc = NOT_A_GC_LOC;
+      cp->active = CHANNEL_INACTIVE;
     }
   else cp = cip;
   cp->tcgx = NULL;
   cp->chan = chan;
   cp->sound = sound;
-  cp->sound_ctr = -1;
+  cp->sound_ctr = NOT_A_SOUND;
   cp->edit_ctr = -1;
   cp->sound_size = 0;
   cp->ptrees = NULL;
   cp->ptree_size = 0;
   cp->ptree_ctr = -1;
   cp->edit_size = 0;
-  cp->tracks = NULL;
+  cp->tracking = false;
   cp->cursor_on = false;
   cp->cursor_visible = false;
   cp->fft_cursor_visible = false;
@@ -48,9 +48,10 @@ chan_info *make_chan_info(chan_info *cip, int chan, snd_info *sound)
   cp->selection_visible = false;
   cp->editable = true; /* must be unset (region display, variable display, etc) */
   cp->cursor_style = cursor_style(ss);
+  cp->tracking_cursor_style = tracking_cursor_style(ss);
   cp->cursor_size = cursor_size(ss);
   cp->cursor_proc = XEN_UNDEFINED;
-  cp->cursor_proc_loc = -1;
+  cp->cursor_proc_loc = NOT_A_GC_LOC;
   cp->squelch_update = false;
   cp->show_y_zero = show_y_zero(ss);
   cp->show_grid = show_grid(ss);
@@ -64,9 +65,10 @@ chan_info *make_chan_info(chan_info *cip, int chan, snd_info *sound)
   cp->verbose_cursor = verbose_cursor(ss);
   cp->fft_log_frequency = fft_log_frequency(ss);
   cp->fft_log_magnitude = fft_log_magnitude(ss);
-  cp->min_dB = ss->min_dB;
+  cp->fft_with_phases = fft_with_phases(ss);
+  cp->min_dB = min_dB(ss);
   cp->lin_dB = ss->lin_dB;
-  cp->in_as_one_edit = false;
+  cp->in_as_one_edit = 0;
   cp->wavelet_type = wavelet_type(ss);
   cp->spectro_x_angle = spectro_x_angle(ss);
   cp->spectro_y_angle = spectro_y_angle(ss);
@@ -74,9 +76,10 @@ chan_info *make_chan_info(chan_info *cip, int chan, snd_info *sound)
   cp->spectro_x_scale = spectro_x_scale(ss);
   cp->spectro_y_scale = spectro_y_scale(ss);
   cp->spectro_z_scale = spectro_z_scale(ss);
-  cp->spectro_cutoff = spectro_cutoff(ss);
-  cp->spectro_start = spectro_start(ss);
+  cp->spectrum_end = spectrum_end(ss);
+  cp->spectrum_start = spectrum_start(ss);
   cp->spectro_hop = spectro_hop(ss);
+  cp->fft_window_alpha = fft_window_alpha(ss);
   cp->fft_window_beta = fft_window_beta(ss);
   cp->transform_size = transform_size(ss);
   cp->transform_graph_type = transform_graph_type(ss);
@@ -92,19 +95,17 @@ chan_info *make_chan_info(chan_info *cip, int chan, snd_info *sound)
   cp->grid_density = grid_density(ss);
   cp->x_axis_style = x_axis_style(ss);
   cp->beats_per_minute = beats_per_minute(ss);
+  cp->beats_per_measure = beats_per_measure(ss);
   cp->show_axes = show_axes(ss);
   cp->graph_time_p = true; /* the default state (button is set when we start) */
   cp->graph_transform_p = false;
   cp->printing = NOT_PRINTING;
   cp->waiting_to_make_graph = false;
   cp->new_peaks = false;
-  cp->amp_envs = NULL;
   cp->sonogram_data = NULL;
   cp->lisp_info = NULL;
   cp->amp_control = NULL;
   cp->hookable = WITH_HOOK;
-  cp->gzy = 1.0;
-  cp->gsy = 1.0;
   cp->cx = 0;
   cp->cy = 0;
   cp->fft_cx = 0;
@@ -112,17 +113,23 @@ chan_info *make_chan_info(chan_info *cip, int chan, snd_info *sound)
   cp->last_search_result = SEARCH_OK;
   if (cp->last_sonogram) 
     {
-      FREE(cp->last_sonogram); 
+      free(cp->last_sonogram); 
       cp->last_sonogram = NULL;
     }
-  cp->active = true;
+  if (cp->last_wavogram) 
+    {
+      free(cp->last_wavogram); 
+      cp->last_wavogram = NULL;
+    }
+  cp->active = CHANNEL_INITIALIZED;
   return(cp);
 }
 
+
 static chan_info *free_chan_info(chan_info *cp)
 {
   /* this does not free the associated widgets -- they are merely unmanaged */
-  cp->active = false;
+  cp->active = CHANNEL_INACTIVE;
   /* need an indication right away that this channel is being deleted -- during free_snd_info (close-sound),
    *   an error may occur (an edit list temp file might have vanished for example), and normally Snd
    *   attempts to post an error message in the sound's minibuffer.  To force this out, we have to
@@ -139,16 +146,10 @@ static chan_info *free_chan_info(chan_info *cp)
   cp->graph_transform_p = false;
   cp->printing = NOT_PRINTING;
   cp->graph_time_p = true;
-  release_dangling_readers(cp, -1);
-  if (cp->samples) {FREE(cp->samples); cp->samples = NULL;}
-  if (cp->cursors) {FREE(cp->cursors); cp->cursors = NULL;}
-  if (cp->tracks) free_track_info_list(cp); /* needs to precede free_edit_list which clobbers cp->edit_size */
   if (cp->edits) free_edit_list(cp);
   if (cp->sounds) free_sound_list(cp);
   if (cp->ptrees) free_ptree_list(cp);
-  if (cp->enved_spectra) free_enved_spectra(cp);
-  free_mark_list(cp, -1);
-  free_mixes(cp);
+  free_channel_mixes(cp);
   cp->sound = NULL;  /* a backpointer */
   cp->cursor_on = false;
   cp->cursor_visible = false;
@@ -158,14 +159,14 @@ static chan_info *free_chan_info(chan_info *cp)
   if (cp->amp_control)
     {
       /* not sure this is the right thing */
-      FREE(cp->amp_control);
+      free(cp->amp_control);
       cp->amp_control = NULL;
     }
   if (XEN_PROCEDURE_P(cp->cursor_proc))
     {
       snd_unprotect_at(cp->cursor_proc_loc);
       cp->cursor_proc = XEN_UNDEFINED;
-      cp->cursor_proc_loc = -1;
+      cp->cursor_proc_loc = NOT_A_GC_LOC;
     }
   if (XEN_VECTOR_P(cp->properties)) /* using vector as node for GC */
     XEN_VECTOR_SET(cp->properties, 0, XEN_EMPTY_LIST);
@@ -176,50 +177,83 @@ static chan_info *free_chan_info(chan_info *cp)
       /* special case -- background fft process never got a chance to run */
       if (cp->temp_sonogram == cp->last_sonogram) cp->last_sonogram = NULL;
       free_sonogram_fft_state(cp->temp_sonogram);
-      FREE(cp->temp_sonogram); 
+      free(cp->temp_sonogram); 
       cp->temp_sonogram = NULL;
     } 
   if (cp->last_sonogram) 
     {
       free_sonogram_fft_state(cp->last_sonogram);
-      FREE(cp->last_sonogram); 
+      free(cp->last_sonogram); 
       cp->last_sonogram = NULL;
     }
-  if (cp->lisp_info) cp->lisp_info = free_lisp_info(cp);
+  if (cp->last_wavogram) 
+    {
+      free(cp->last_wavogram); 
+      cp->last_wavogram = NULL;
+    }
+  if (cp->lisp_info) 
+    {
+      free_lisp_info(cp);
+      cp->lisp_info = NULL;
+    }
   cp->graph_lisp_p = false;
   cp->selection_transform_size = 0;
-  cp->edit_hook_checked = false;
+
+  if (cp->as_one_edit_positions)
+    {
+      free(cp->as_one_edit_positions);
+      cp->as_one_edit_positions = NULL;
+      cp->as_one_edit_positions_size = 0;
+    }
+
   if (XEN_HOOK_P(cp->edit_hook))
     {
       XEN_CLEAR_HOOK(cp->edit_hook);
-      snd_unprotect(cp->edit_hook);
+      snd_unprotect_at(cp->edit_hook_loc);
       cp->edit_hook = XEN_FALSE;
+      cp->edit_hook_loc = NOT_A_GC_LOC;
     }
   if (XEN_HOOK_P(cp->after_edit_hook))
     {
       XEN_CLEAR_HOOK(cp->after_edit_hook);
-      snd_unprotect(cp->after_edit_hook);
+      snd_unprotect_at(cp->after_edit_hook_loc);
       cp->after_edit_hook = XEN_FALSE;
+      cp->after_edit_hook_loc = NOT_A_GC_LOC;
     }
   if (XEN_HOOK_P(cp->undo_hook))
     {
       XEN_CLEAR_HOOK(cp->undo_hook);
-      snd_unprotect(cp->undo_hook);
+      snd_unprotect_at(cp->undo_hook_loc);
       cp->undo_hook = XEN_FALSE;
+      cp->undo_hook_loc = NOT_A_GC_LOC;
     }
   return(cp);  /* pointer is left for possible future re-use */
 }
 
+
 snd_info *make_basic_snd_info(int chans)
 {
   snd_info *sp = NULL;
-  sp = (snd_info *)CALLOC(1, sizeof(snd_info));
-  sp->chans = (chan_info **)CALLOC(chans, sizeof(chan_info *));
+  sp = (snd_info *)calloc(1, sizeof(snd_info));
+  sp->chans = (chan_info **)calloc(chans, sizeof(chan_info *));
   sp->allocated_chans = chans;
-  sp->properties = XEN_FALSE; /* will be a vector of 1 element (permanently protected) if it's ever used */
+  sp->properties = XEN_FALSE; /* will be a vector of 1 element if it's ever used */
+  sp->properties_loc = NOT_A_GC_LOC;
+#if HAVE_PTHREADS
+  /* multichannel cases can all try to change the "*" in the file name */
+  sp->starred_name_lock = (mus_lock_t *)malloc(sizeof(mus_lock_t));
+  MUS_LOCK_INIT(sp->starred_name_lock);
+  /* similarly the stop and bomb icons */
+  sp->stop_sign_lock = (mus_lock_t *)malloc(sizeof(mus_lock_t));
+  MUS_LOCK_INIT(sp->stop_sign_lock);
+  /* edit history list of multichannel file (after explicit open) */
+  sp->edit_history_lock = (mus_lock_t *)malloc(sizeof(mus_lock_t));
+  MUS_LOCK_INIT(sp->edit_history_lock);
+#endif
   return(sp);
 }
 
+
 void initialize_control_panel(snd_info *sp)
 {
   sp->expand_control = DEFAULT_EXPAND_CONTROL;
@@ -273,13 +307,14 @@ void initialize_control_panel(snd_info *sp)
   sp->filter_control_in_dB = filter_control_in_dB(ss);
   sp->filter_control_in_hz = filter_control_in_hz(ss);
   if (sp->filter_control_in_hz)
-    sp->filter_control_xmax = (Float)(SND_SRATE(sp) / 2);
+    sp->filter_control_xmax = (mus_float_t)(SND_SRATE(sp) / 2);
   else sp->filter_control_xmax = 1.0;
   sp->filter_control_changed = false;
   sp->saved_controls = NULL;
 }
 
-snd_info *make_snd_info(snd_info *sip, const char *filename, file_info *hdr, int snd_slot, bool read_only)
+
+snd_info *make_snd_info(snd_info *sip, const char *filename, file_info *hdr, int snd_slot, read_only_t read_only)
 {
   snd_info *sp = NULL;
   int chans, i;
@@ -289,32 +324,45 @@ snd_info *make_snd_info(snd_info *sip, const char *filename, file_info *hdr, int
   if (!sip)
     {
       sp = make_basic_snd_info(chans);
-      sp->sgx = (snd_context *)CALLOC(1, sizeof(snd_context));
+      sp->sgx = (snd_context *)calloc(1, sizeof(snd_context));
     }
   else 
     {
       sp = sip;
       if (sp->allocated_chans < chans) 
 	{
-	  sp->chans = (chan_info **)REALLOC(sp->chans, chans * sizeof(chan_info *));
+	  sp->chans = (chan_info **)realloc(sp->chans, chans * sizeof(chan_info *));
 	  for (i = sp->allocated_chans; i < chans; i++) sp->chans[i] = NULL;
 	  sp->allocated_chans = chans;
 	}
     }
-  sp->read_only = read_only;  /* need to be sure this is set before any hooks run */
+  sp->user_read_only = read_only;  /* need to be sure this is set before any hooks run */
+  sp->bomb_in_progress = false;
+  sp->watchers = NULL;
+  sp->watchers_size = 0;
   sp->index = snd_slot;
   sp->nchans = chans;
   sp->hdr = hdr;
   sp->inuse = SOUND_NORMAL;
-  sp->filename = copy_string(filename);
-  sp->short_filename = filename_without_home_directory(sp->filename); /* a pointer into filename, not a new string */
+  sp->filename = mus_strdup(filename);
+  sp->short_filename = filename_without_directory(sp->filename); /* a pointer into filename, not a new string */
   sp->sync = DEFAULT_SYNC;
   sp->previous_sync = sp->sync;
   initialize_control_panel(sp);
-  sp->searching = 0;
+  sp->search_count = 0;
+  sp->selectpos = -1;
+
   if (chans > 1)
-    sp->channel_style = channel_style(ss);
+    {
+      if (ss->update_sound_channel_style != NOT_A_CHANNEL_STYLE)
+	{
+	  sp->channel_style = ss->update_sound_channel_style;
+	  ss->update_sound_channel_style = NOT_A_CHANNEL_STYLE;
+	}
+      else sp->channel_style = channel_style(ss);
+    }
   else sp->channel_style = CHANNELS_SEPARATE;
+
   sp->loading = false;
   sp->marking = 0;
   sp->filing = NOT_FILING;
@@ -326,17 +374,31 @@ snd_info *make_snd_info(snd_info *sip, const char *filename, file_info *hdr, int
   sp->lacp = NULL;
   sp->search_tree = NULL;
   sp->search_proc = XEN_UNDEFINED;
+  sp->search_proc_loc = NOT_A_GC_LOC;
   sp->prompt_callback = XEN_UNDEFINED;
+  sp->prompt_callback_loc = NOT_A_GC_LOC;
   sp->delete_me = NULL;
   sp->name_string = NULL;
   sp->active = true;
-  sp->cursor_follows_play = cursor_follows_play(ss);
+  sp->with_tracking_cursor = with_tracking_cursor(ss);
   return(sp);
 }
 
+
 void free_snd_info(snd_info *sp)
 {
   int i;
+  if (sp->watchers)
+    {
+      int i;
+      call_sp_watchers(sp, SP_ANY_WATCHER, SP_IS_CLOSING);
+      for (i = 0; i < sp->watchers_size; i++)
+	if (sp->watchers[i])
+	  free(sp->watchers[i]); /* normally watcher will do this, I assume */
+      free(sp->watchers);
+      sp->watchers = NULL;
+      sp->watchers_size = 0;
+    }
   if (sp->sgx)
     {
       /* make sure trough colors are ok upon reuse */
@@ -349,10 +411,11 @@ void free_snd_info(snd_info *sp)
     }
   /* leave most for reuse as in free_chan_info */
   sp->active = false;
+  sp->selectpos = -1;
   if (sp->sgx)
     {
       env_editor *edp;
-      edp = (env_editor *)(sp->sgx->flt);
+      edp = sp->sgx->flt;
       if (edp)
 	{
 	  edp->edited = false;
@@ -371,55 +434,52 @@ void free_snd_info(snd_info *sp)
   sp->inuse = SOUND_IDLE;
   sp->playing_mark = NULL;
   sp->playing = 0;
-  sp->searching = 0;
+  sp->search_count = 0;
   sp->loading = false;
+  sp->bomb_in_progress = false;
   sp->marking = 0;
   sp->filing = NOT_FILING;
   sp->applying = false;
   sp->channel_style = CHANNELS_SEPARATE;
-  sp->read_only = false;
+  sp->user_read_only = FILE_READ_WRITE;
+  sp->file_read_only = FILE_READ_WRITE;
   sp->need_update = false;
+  sp->file_unreadable = false;
   sp->minibuffer_on = MINI_OFF;
-  if (sp->search_expr) 
-    {
-      FREE(sp->search_expr); 
-      sp->search_expr = NULL;
-    }
-  if (sp->search_tree)
-    free_ptree(sp->search_tree);
-  sp->search_tree = NULL;
-  if (XEN_PROCEDURE_P(sp->search_proc))
-    snd_unprotect(sp->search_proc);
-  sp->search_proc = XEN_UNDEFINED;
+  clear_sound_search_procedure(sp, true);
   if (XEN_PROCEDURE_P(sp->prompt_callback))
-    snd_unprotect(sp->prompt_callback);
+    {
+      snd_unprotect_at(sp->prompt_callback_loc);
+      sp->prompt_callback_loc = NOT_A_GC_LOC;
+    }
   sp->prompt_callback = XEN_UNDEFINED;
   if (XEN_VECTOR_P(sp->properties)) /* using vector as node for GC */
     XEN_VECTOR_SET(sp->properties, 0, XEN_EMPTY_LIST);
   sp->selected_channel = NO_SELECTION;
   sp->short_filename = NULL;                      /* was a pointer into filename */
-  if (sp->filename) FREE(sp->filename);
+  if (sp->filename) free(sp->filename);
   sp->filename = NULL;
   if (sp->filter_control_envelope) sp->filter_control_envelope = free_env(sp->filter_control_envelope);
   if (sp->saved_controls) free_controls(sp);
   sp->delete_me = NULL;
-  if (sp->name_string) FREE(sp->name_string);
+  if (sp->name_string) free(sp->name_string);
   sp->name_string = NULL;
   sp->lacp = NULL;
-  if (sp->hdr) sp->hdr = free_file_info(sp->hdr);
+  sp->hdr = free_file_info(sp->hdr);
   if (sp->edited_region) clear_region_backpointer(sp);
   clear_mini_strings(sp);
   clear_filter_strings(sp);
   clear_players();
-  reflect_mix_or_track_change(ANY_MIX_ID, ANY_TRACK_ID, false);
+  reflect_mix_change(ANY_MIX_ID);
 }
 
+
 snd_info *completely_free_snd_info(snd_info *sp)
 {
   int i;
   chan_info *cp;
   free_snd_info(sp);
-  if (sp->sgx) FREE(sp->sgx);
+  if (sp->sgx) free(sp->sgx);
   for (i = 0; i < sp->allocated_chans; i++) 
     {
       cp = sp->chans[i];
@@ -427,51 +487,34 @@ snd_info *completely_free_snd_info(snd_info *sp)
 	{
 	  if (cp->cgx) 
 	    {
-	      if ((cp->cgx)->ax) 
-		FREE((cp->cgx)->ax);
-	      FREE(cp->cgx);
+	      if (cp->cgx->ax) 
+		free(cp->cgx->ax);
+	      free(cp->cgx);
 	    }
 	  if (XEN_VECTOR_P(cp->properties))
-	    snd_unprotect(cp->properties);
-	  FREE(cp);
+	    {
+	      snd_unprotect_at(cp->properties_loc);
+	      cp->properties_loc = NOT_A_GC_LOC;
+	    }
+	  cp->active = CHANNEL_FREED;
+	  free(cp);
+	  /* it's possible to have dangling readers (snd_fd) with pointers to this channel (see note in snd-edits.c under unlist_reader) */
 	}
     }
-  FREE(sp->chans);
+  free(sp->chans);
   if (XEN_VECTOR_P(sp->properties))
-    snd_unprotect(sp->properties);
-  FREE(sp);
+    {
+      snd_unprotect_at(sp->properties_loc);
+      sp->properties_loc = NOT_A_GC_LOC;
+    }
+  free(sp);
   return(NULL);
 }
 
-bool map_over_chans(bool (*func)(chan_info *, void *), void *userptr)
-{
-  /* argument to func is chan_info pointer+void pointer of user spec, return non-zero = abort map, skips inactive sounds */
-  /* currently used only in cases where only normal sounds are expected */
-  int i, j;
-  bool val = false;
-  for (i = 0; i < ss->max_sounds; i++)
-    {
-      snd_info *sp;
-      chan_info *cp;
-      sp = ss->sounds[i];
-      if ((sp) && (sp->inuse == SOUND_NORMAL))
-	for (j = 0; j < sp->nchans; j++)
-	  if ((cp = ((chan_info *)(sp->chans[j]))))
-	    {
-	      if (cp)
-		val = (*func)(cp, userptr);
-	      if (val) return(val);
-	    }
-      }
-  return(val);
-}
 
-void for_each_chan_1(void (*func)(chan_info *, void *), void *userptr)
+void for_each_chan_with_int(void (*func)(chan_info *ncp, int val), int value)
 {
   int i, j;
-#if DEBUGGING
-  if (ss->max_sounds > 2000) abort();
-#endif
   for (i = 0; i < ss->max_sounds; i++)
     {
       snd_info *sp;
@@ -480,16 +523,62 @@ void for_each_chan_1(void (*func)(chan_info *, void *), void *userptr)
       if ((sp) && ((sp->inuse == SOUND_NORMAL) || (sp->inuse == SOUND_WRAPPER)))
 	for (j = 0; j < sp->nchans; j++)
 	  if ((cp = ((chan_info *)(sp->chans[j]))))
-	    (*func)(cp, userptr);
+	    (*func)(cp, value);
     }
 }
 
-void for_each_chan(void (*func)(chan_info *))
+
+void for_each_chan_with_mus_long_t(void (*func)(chan_info *ncp, mus_long_t val), mus_long_t value)
+{
+  int i, j;
+  for (i = 0; i < ss->max_sounds; i++)
+    {
+      snd_info *sp;
+      chan_info *cp;
+      sp = ss->sounds[i];
+      if ((sp) && ((sp->inuse == SOUND_NORMAL) || (sp->inuse == SOUND_WRAPPER)))
+	for (j = 0; j < sp->nchans; j++)
+	  if ((cp = ((chan_info *)(sp->chans[j]))))
+	    (*func)(cp, value);
+    }
+}
+
+
+void for_each_chan_with_bool(void (*func)(chan_info *ncp, bool val), bool value)
+{
+  int i, j;
+  for (i = 0; i < ss->max_sounds; i++)
+    {
+      snd_info *sp;
+      chan_info *cp;
+      sp = ss->sounds[i];
+      if ((sp) && ((sp->inuse == SOUND_NORMAL) || (sp->inuse == SOUND_WRAPPER)))
+	for (j = 0; j < sp->nchans; j++)
+	  if ((cp = ((chan_info *)(sp->chans[j]))))
+	    (*func)(cp, value);
+    }
+}
+
+
+void for_each_chan_with_float(void (*func)(chan_info *ncp, mus_float_t val), mus_float_t value)
+{
+  int i, j;
+  for (i = 0; i < ss->max_sounds; i++)
+    {
+      snd_info *sp;
+      chan_info *cp;
+      sp = ss->sounds[i];
+      if ((sp) && ((sp->inuse == SOUND_NORMAL) || (sp->inuse == SOUND_WRAPPER)))
+	for (j = 0; j < sp->nchans; j++)
+	  if ((cp = ((chan_info *)(sp->chans[j]))))
+	    (*func)(cp, value);
+    }
+}
+
+
+void for_each_chan(void (*func)(chan_info *ncp))
 {
   int i, j;
-#if DEBUGGING
-  if (ss->max_sounds > 2000) abort();
-#endif
   for (i = 0; i < ss->max_sounds; i++)
     {
       snd_info *sp;
@@ -502,7 +591,8 @@ void for_each_chan(void (*func)(chan_info *))
     }
 }
 
-void for_each_normal_chan_1(void (*func)(chan_info *, void *), void *userptr)
+
+void for_each_normal_chan_with_void(void (*func)(chan_info *ncp, void *ptr), void *userptr)
 {
   int i, j;
   for (i = 0; i < ss->max_sounds; i++)
@@ -517,7 +607,40 @@ void for_each_normal_chan_1(void (*func)(chan_info *, void *), void *userptr)
     }
 }
 
-void for_each_normal_chan(void (*func)(chan_info *))
+
+void for_each_normal_chan_with_int(void (*func)(chan_info *ncp, int val), int value)
+{
+  int i, j;
+  for (i = 0; i < ss->max_sounds; i++)
+    {
+      snd_info *sp;
+      chan_info *cp;
+      sp = ss->sounds[i];
+      if ((sp) && (sp->inuse == SOUND_NORMAL))
+	for (j = 0; j < sp->nchans; j++)
+	  if ((cp = ((chan_info *)(sp->chans[j]))))
+	    (*func)(cp, value);
+    }
+}
+
+
+void for_each_normal_chan_with_refint(void (*func)(chan_info *ncp, int *val), int *value)
+{
+  int i, j;
+  for (i = 0; i < ss->max_sounds; i++)
+    {
+      snd_info *sp;
+      chan_info *cp;
+      sp = ss->sounds[i];
+      if ((sp) && (sp->inuse == SOUND_NORMAL))
+	for (j = 0; j < sp->nchans; j++)
+	  if ((cp = ((chan_info *)(sp->chans[j]))))
+	    (*func)(cp, value);
+    }
+}
+
+
+void for_each_normal_chan(void (*func)(chan_info *ncp))
 {
   int i, j;
   for (i = 0; i < ss->max_sounds; i++)
@@ -532,22 +655,33 @@ void for_each_normal_chan(void (*func)(chan_info *))
     }
 }
 
-bool map_over_sound_chans(snd_info *sp, bool (*func)(chan_info *, void *), void *userptr)
+
+void for_each_sound_chan_with_int(snd_info *sp, void (*func)(chan_info *ncp, int val1), int value)
+{
+  int j;
+   chan_info *cp;
+  for (j = 0; j < sp->nchans; j++)
+    if ((cp = sp->chans[j]))
+      (*func)(cp, value);
+}
+
+
+bool map_over_sound_chans(snd_info *sp, bool (*func)(chan_info *ncp))
 {
-  /* argument to func is chan_info pointer+void pointer of user spec, return non-zero = abort map, skips inactive sounds */
   int j;
   bool val = false;
   chan_info *cp;
   for (j = 0; j < sp->nchans; j++)
     if ((cp = sp->chans[j]))
       {
-	val = (*func)(cp, userptr);
+	val = (*func)(cp);
 	if (val) return(val);
       }
   return(val);
 }
 
-void for_each_sound_chan(snd_info *sp, void (*func)(chan_info *))
+
+void for_each_sound_chan(snd_info *sp, void (*func)(chan_info *ncp))
 {
   int j;
   chan_info *cp;
@@ -556,25 +690,21 @@ void for_each_sound_chan(snd_info *sp, void (*func)(chan_info *))
       (*func)(cp);
 }
 
-bool map_over_sounds(bool (*func)(snd_info *, void *), void *userptr)
+
+void for_each_sound(void (*func)(snd_info *usp))
 {
-  /* argument to func is snd_info pointer, return true = abort map, skips inactive sounds */
   int i;
-  bool val = false;
   for (i = 0; i < ss->max_sounds; i++)
     {
       snd_info *sp;
       sp = ss->sounds[i];
       if ((sp) && (sp->inuse == SOUND_NORMAL))
-	{
-	  val = (*func)(sp, userptr);
-	  if (val) return(val);
-	}
+	(*func)(sp);
     }
-  return(val);
 }
 
-void for_each_sound(void (*func)(snd_info *, void *), void *userptr)
+
+void for_each_sound_with_void(void (*func)(snd_info *usp, void *ptr), void *userptr)
 {
   int i;
   for (i = 0; i < ss->max_sounds; i++)
@@ -586,11 +716,11 @@ void for_each_sound(void (*func)(snd_info *, void *), void *userptr)
     }
 }
 
-bool map_over_separate_chans(bool (*func)(chan_info *, void *), void *userptr)
+
+void for_each_separate_chan(void (*func)(chan_info *ncp))
 {
   /* used only to lock/unlock panes */
   int i;
-  bool val =false;
   for (i = 0; i < ss->max_sounds; i++)
     {
       snd_info *sp;
@@ -598,14 +728,13 @@ bool map_over_separate_chans(bool (*func)(chan_info *, void *), void *userptr)
       if ((sp) && (sp->inuse == SOUND_NORMAL))
 	{
 	  if (sp->channel_style != CHANNELS_SEPARATE)
-	    val = (*func)(sp->chans[0], userptr);
-	  else val = map_over_sound_chans(sp, func, userptr);
-	  if (val) return(val);
+	    (*func)(sp->chans[0]);
+	  else for_each_sound_chan(sp, func);
 	}
     }
-  return(val);
 }
 
+
 bool snd_ok(snd_info *sp) 
 {
   return((sp) && 
@@ -613,6 +742,7 @@ bool snd_ok(snd_info *sp)
 	 (sp->active));
 }
 
+
 int active_channels(virtual_channels_t count_virtual_channels)
 {
   int chans = 0, i;
@@ -632,6 +762,7 @@ int active_channels(virtual_channels_t count_virtual_channels)
   return(chans);
 }
 
+
 #define SOUNDS_ALLOC_SIZE 4
 
 int find_free_sound_slot(int desired_chans)
@@ -665,11 +796,12 @@ int find_free_sound_slot(int desired_chans)
     }
   j = ss->max_sounds;
   ss->max_sounds += SOUNDS_ALLOC_SIZE;
-  ss->sounds = (snd_info **)REALLOC(ss->sounds, ss->max_sounds * sizeof(snd_info *));
+  ss->sounds = (snd_info **)realloc(ss->sounds, ss->max_sounds * sizeof(snd_info *));
   for (i = j; i < ss->max_sounds; i++) ss->sounds[i] = NULL;
   return(j);
 }
 
+
 int find_free_sound_slot_for_channel_display(void)
 {
   int i, j;
@@ -677,11 +809,12 @@ int find_free_sound_slot_for_channel_display(void)
     if (ss->sounds[i] == NULL) return(i);
   j = ss->max_sounds;
   ss->max_sounds += SOUNDS_ALLOC_SIZE;
-  ss->sounds = (snd_info **)REALLOC(ss->sounds, ss->max_sounds * sizeof(snd_info *));
+  ss->sounds = (snd_info **)realloc(ss->sounds, ss->max_sounds * sizeof(snd_info *));
   for (i = j; i < ss->max_sounds; i++) ss->sounds[i] = NULL;
   return(j);
 }
 
+
 static snd_info *any_active_sound(void)
 {
   int i;
@@ -695,14 +828,18 @@ static snd_info *any_active_sound(void)
   return(NULL);
 }
 
+
 snd_info *selected_sound(void)
 {
-  if (ss->selected_sound != NO_SELECTION)
+  if ((ss->selected_sound != NO_SELECTION) &&
+      (snd_ok(ss->sounds[ss->selected_sound])))
     return(ss->sounds[ss->selected_sound]);
+  ss->selected_sound = NO_SELECTION;
   return(NULL);
 }
 
-snd_info *any_selected_sound (void)
+
+snd_info *any_selected_sound(void)
 {
   snd_info *sp;
   sp = selected_sound();
@@ -710,13 +847,19 @@ snd_info *any_selected_sound (void)
   return(sp);
 }
 
+
 chan_info *any_selected_channel(snd_info *sp)
 {
+  int chan = 0;
   if (sp->selected_channel != NO_SELECTION) 
-    return(sp->chans[sp->selected_channel]);
-  return(sp->chans[0]);
+    chan = sp->selected_channel;
+  if ((sp->chans[chan]) && 
+      (sp->chans[chan]->active > CHANNEL_INITIALIZED))
+    return(sp->chans[chan]);
+  return(NULL);
 }
 
+
 chan_info *selected_channel(void)
 {
   if (ss->selected_sound != NO_SELECTION)
@@ -729,40 +872,46 @@ chan_info *selected_channel(void)
   return(NULL);
 }
 
+
 static XEN select_sound_hook;
 static XEN select_channel_hook;
+static int current_selectpos = 0;
 
 static void select_sound(snd_info *sp)
 {
   if ((sp == NULL) || (sp->inuse != SOUND_NORMAL)) return;
+
   if (XEN_HOOKED(select_sound_hook))
     run_hook(select_sound_hook,
-	     XEN_LIST_1(C_TO_XEN_INT(sp->index)),
+	     XEN_LIST_1(C_INT_TO_XEN_SOUND(sp->index)),
 	     S_select_sound_hook);
+
   if (ss->selected_sound != sp->index)
     {
       reflect_sound_selection(sp);
       ss->selected_sound = sp->index;
-      highlight_selected_sound();
-      reflect_undo_or_redo_in_menu(any_selected_channel(sp));
       new_active_channel_alert();
     }
+  sp->selectpos = current_selectpos++;
+  run_watchers();
 }
 
+
 chan_info *color_selected_channel(snd_info *sp)
 {
   chan_info *ncp;
   ncp = sp->chans[sp->selected_channel];
   recolor_graph(ncp, true);
   (ncp->cgx)->selected = true;
-  if ((ss->sgx)->data_color != (ss->sgx)->selected_data_color) 
-    update_graph(ncp);
+  update_graph(ncp); /* update any indication of change in selected channel (channel id for example) */
   return(ncp);
 }
 
+
 void select_channel(snd_info *sp, int chan)
 {
   chan_info *cp, *ncp;
+
   if ((sp == NULL) || (sp->inuse != SOUND_NORMAL)) return;
   cp = selected_channel();
   if (cp != sp->chans[chan])
@@ -772,21 +921,21 @@ void select_channel(snd_info *sp, int chan)
       if (cp) 
 	{
 	  recolor_graph(cp, false);
-	  (cp->cgx)->selected = false;
-	  if (sp != cp->sound) (cp->sound)->selected_channel = NO_SELECTION;
+	  cp->cgx->selected = false;
+	  if (sp != cp->sound) cp->sound->selected_channel = NO_SELECTION;
 	  update_graph(cp);
 	}
       if (XEN_HOOKED(select_channel_hook))
 	run_hook(select_channel_hook,
-		 XEN_LIST_2(C_TO_XEN_INT(sp->index),
+		 XEN_LIST_2(C_INT_TO_XEN_SOUND(sp->index),
 			    C_TO_XEN_INT(chan)),
 		 S_select_channel_hook);
       ncp = color_selected_channel(sp);
-      reflect_undo_or_redo_in_menu(ncp);
       goto_graph(ncp);
     }
 }
 
+
 chan_info *current_channel(void)
 {
   snd_info *sp = NULL;
@@ -797,35 +946,50 @@ chan_info *current_channel(void)
   return(NULL);
 }
 
+
 sync_info *free_sync_info(sync_info *si)
 {
   if (si)
     {
-      if (si->begs) FREE(si->begs);
+      if (si->begs) free(si->begs);
       si->begs = NULL;
-      if (si->cps) FREE(si->cps);
+      if (si->cps) free(si->cps);
       si->cps = NULL;
-      FREE(si);
+      free(si);
     }
   return(NULL);
 }
 
+
+static int syncd_channels(int sync)
+{
+  if (sync != 0)
+    {
+      int i, chans = 0;
+      snd_info *sp;
+      for (i = 0; i < ss->max_sounds; i++)
+	{
+	  sp = ss->sounds[i];
+	  if ((sp) && (sp->inuse == SOUND_NORMAL) && (sp->sync == sync)) 
+	    chans += sp->nchans;
+	}
+      return(chans);
+    }
+  return(0);
+}
+
+
 sync_info *snd_sync(int sync)
 {
-  int i, j, k, chans = 0;
+  int i, j, k, chans;
   snd_info *sp;
-  for (i = 0; i < ss->max_sounds; i++)
-    {
-      sp = ss->sounds[i];
-      if ((sp) && (sp->inuse == SOUND_NORMAL) && (sp->sync == sync)) 
-	chans += sp->nchans;
-    }
+  chans = syncd_channels(sync);
   if (chans > 0)
     {
       sync_info *si;
-      si = (sync_info *)CALLOC(1, sizeof(sync_info));
-      si->begs = (off_t *)CALLOC(chans, sizeof(off_t));
-      si->cps = (chan_info **)CALLOC(chans, sizeof(chan_info *));
+      si = (sync_info *)calloc(1, sizeof(sync_info));
+      si->begs = (mus_long_t *)calloc(chans, sizeof(mus_long_t));
+      si->cps = (chan_info **)calloc(chans, sizeof(chan_info *));
       si->chans = chans;
       j = 0;
       for (i = 0; i < ss->max_sounds; i++)
@@ -840,18 +1004,20 @@ sync_info *snd_sync(int sync)
   return(NULL);
 }
 
-sync_info *make_simple_sync(chan_info *cp, off_t beg)
+
+sync_info *make_simple_sync(chan_info *cp, mus_long_t beg)
 {
   sync_info *si;
-  si = (sync_info *)CALLOC(1, sizeof(sync_info));
+  si = (sync_info *)calloc(1, sizeof(sync_info));
   si->chans = 1;
-  si->cps = (chan_info **)CALLOC(1, sizeof(chan_info *));
+  si->cps = (chan_info **)calloc(1, sizeof(chan_info *));
   si->cps[0] = cp;
-  si->begs = (off_t *)CALLOC(1, sizeof(off_t));
+  si->begs = (mus_long_t *)calloc(1, sizeof(mus_long_t));
   si->begs[0] = beg;
   return(si);
 }
 
+
 sync_info *sync_to_chan(chan_info *cp)
 {
   snd_info *sp;
@@ -861,20 +1027,21 @@ sync_info *sync_to_chan(chan_info *cp)
   return(make_simple_sync(cp, 0));
 }
 
+
 snd_info *find_sound(const char *name, int nth)
 {
   char *sname;
   int i, which = 0;
   if (name == NULL) return(NULL);
-  sname = filename_without_home_directory(name);
+  sname = filename_without_directory(name);
   for (i = 0; i < ss->max_sounds; i++)
     {
       snd_info *sp;
       sp = ss->sounds[i];
       if ((sp) && (sp->inuse == SOUND_NORMAL))
-	if ((strcmp(name, sp->short_filename) == 0) || 
-	    (strcmp(name, sp->filename) == 0) ||
-	    ((sname != NULL) && (strcmp(sname, sp->short_filename) == 0)))
+	if ((mus_strcmp(name, sp->short_filename)) || 
+	    (mus_strcmp(name, sp->filename)) ||
+	    (mus_strcmp(sname, sp->short_filename)))
 	  {
 	    if (which == nth) return(sp);
 	    which++;
@@ -883,40 +1050,40 @@ snd_info *find_sound(const char *name, int nth)
   return(NULL);
 }
 
+
 static char *display_maxamps(const char *filename, int chans)
 {
   char *ampstr;
   int i, len;
   mus_sample_t *vals;
-  off_t *times;
+  mus_long_t *times;
   len = chans * 32;
-  ampstr = (char *)CALLOC(len, sizeof(char));
-  vals = (mus_sample_t *)CALLOC(chans, sizeof(mus_sample_t));
-  times = (off_t *)CALLOC(chans, sizeof(off_t));
-  sprintf(ampstr, _("\nmax amp: "));
+  ampstr = (char *)calloc(len, sizeof(char));
+  vals = (mus_sample_t *)calloc(chans, sizeof(mus_sample_t));
+  times = (mus_long_t *)calloc(chans, sizeof(mus_long_t));
+  mus_snprintf(ampstr, len, "%s", _("\nmax amp: "));
   mus_sound_maxamps(filename, chans, vals, times);
   for (i = 0; i < chans; i++)
     {
-      ampstr = snd_strcat(ampstr, prettyf(MUS_SAMPLE_TO_FLOAT(vals[i]), 3), &len);
-      ampstr = snd_strcat(ampstr, " ", &len);
+      ampstr = mus_strcat(ampstr, prettyf(MUS_SAMPLE_TO_DOUBLE(vals[i]), 3), &len);
+      ampstr = mus_strcat(ampstr, " ", &len);
     }
-  FREE(vals);
-  FREE(times);
+  free(vals);
+  free(times);
   return(ampstr);
 }
 
-static char timestr[TIME_STR_SIZE];
-#define INFO_BUFFER_SIZE 1024
 
 void display_info(snd_info *sp)
 {
+  #define INFO_BUFFER_SIZE 1024
   if (sp)
     {
       file_info *hdr;
       hdr = sp->hdr;
       if (hdr)
 	{
-	  char *comment = NULL, *ampstr = NULL;
+	  char *comment = NULL, *ampstr = NULL, *quoted_comment = NULL;
 	  char *buffer = NULL;
 	  comment = hdr->comment;
 	  while ((comment) && (*comment) && 
@@ -925,52 +1092,41 @@ void display_info(snd_info *sp)
 		  ((*comment) == ' ') || 
 		  ((*comment) == '\xd')))
 	    comment++;
+	  if ((comment) && (*comment))
+	    quoted_comment = mus_format("\"%s\"", comment);
 	  if (mus_sound_maxamp_exists(sp->filename))
 	    ampstr = display_maxamps(sp->filename, sp->nchans);
-	  buffer = (char *)CALLOC(INFO_BUFFER_SIZE, sizeof(char));
-#if HAVE_STRFTIME
-	  strftime(timestr, TIME_STR_SIZE, STRFTIME_FORMAT, localtime(&(sp->write_date)));
-#endif
+	  buffer = (char *)calloc(INFO_BUFFER_SIZE, sizeof(char));
 	  mus_snprintf(buffer, INFO_BUFFER_SIZE, 
-		       _("srate: %d\nchans: %d\nlength: %.3f (" PRId64 " %s)\ntype: %s\nformat: %s\nwritten: %s%s%s%s\n"),
+		       _("srate: %d\nchans: %d\nlength: %.3f (" MUS_LD " %s)\ntype: %s\nformat: %s\nwritten: %s%s%s%s\n"),
 		       hdr->srate,
 		       hdr->chans,
-		       (Float)((double)(hdr->samples) / (Float)(hdr->chans * hdr->srate)),
-		       (off_t)((hdr->samples) / (hdr->chans)),
+		       (mus_float_t)((double)(hdr->samples) / (mus_float_t)(hdr->chans * hdr->srate)),
+		       (mus_long_t)((hdr->samples) / (hdr->chans)),
 		       (hdr->chans == 1) ? _("samples") : _("frames"),
 		       mus_header_type_name(hdr->type),
 		       mus_data_format_name(hdr->format),
-		       timestr,
+		       snd_strftime(STRFTIME_FORMAT, sp->write_date),
 		       (ampstr) ? ampstr : "",
-		       (comment) ? _("\ncomment: ") : "",
-		       (comment) ? comment : "");
+		       (quoted_comment) ? _("\ncomment: ") : "",
+		       (quoted_comment) ? quoted_comment : "");
 	  post_it(sp->short_filename, buffer);
-	  if (ampstr) FREE(ampstr);
-	  FREE(buffer);
+	  if (ampstr) free(ampstr);
+	  if (quoted_comment) free(quoted_comment);
+	  free(buffer);
 	}
     }
 }
 
-#if DEBUGGING && HAVE_GUILE
-static XEN g_display_info(void)
-{
-  display_info(selected_sound());
-  return(XEN_FALSE);
-}
-#endif
 
 void g_init_data(void)
 {
   #define H_select_sound_hook S_select_sound_hook " (snd): called whenever a sound is selected."
 
-  #define H_select_channel_hook S_select_channel_hook "(snd chn): called whenever a channel is selected. \
+  #define H_select_channel_hook S_select_channel_hook " (snd chn): called whenever a channel is selected. \
 Its arguments are the sound index and the channel number."
 
-  XEN_DEFINE_HOOK(select_sound_hook, S_select_sound_hook, 1, H_select_sound_hook);       /* arg = sound index */
-  XEN_DEFINE_HOOK(select_channel_hook, S_select_channel_hook, 2, H_select_channel_hook); /* args = sound index, channel */
-
-#if DEBUGGING && HAVE_GUILE
-  XEN_DEFINE_PROCEDURE("popup-display-info", g_display_info, 0, 0, 0, "internal test function");
-#endif
+  select_sound_hook = XEN_DEFINE_HOOK(S_select_sound_hook, 1, H_select_sound_hook);       /* arg = sound index */
+  select_channel_hook = XEN_DEFINE_HOOK(S_select_channel_hook, 2, H_select_channel_hook); /* args = sound index, channel */
 }
 
