@@ -1,12 +1,19 @@
 #include "snd.h"
 
-/* 
-   SOMEDAY: highlight bracketed help text in red fg
-     can this be done with tags? -- currently using a list of strings -- would need to change to a text widget 
+/* ---------------- HELP MONOLOG ---------------- */
+
+/* I tried gtkhtml (and XmHTML in 1999), and in both cases decided not to use them.  Both work fine,
+ *   but is it that much of a difference to have an html widget in a dialog as opposed to mozilla
+ *   running remotely?  And it would require fixing up all the help texts.  
  */
 
 static GtkWidget *help_dialog = NULL;
-static void dismiss_help_dialog(GtkWidget *w, gpointer context) {gtk_widget_hide(help_dialog);}
+
+static void dismiss_help_dialog(GtkWidget *w, gpointer context) 
+{
+  gtk_widget_hide(help_dialog);
+}
+
 
 static gint delete_help_dialog(GtkWidget *w, GdkEvent *event, gpointer context) 
 {
@@ -15,14 +22,14 @@ static gint delete_help_dialog(GtkWidget *w, GdkEvent *event, gpointer context)
 }
 
 
-/* ---------------- HELP MONOLOG ---------------- */
-
 #define HELP_ROWS 12
-#define HELP_COLUMNS 56
-/* these set the initial size of the (non XmHTML) help dialog text area */
+#define HELP_COLUMNS 72
+/* these set the initial size of the help dialog text area */
 
-static GtkWidget *help_text = NULL, *related_items = NULL;
+static GtkWidget *help_text = NULL;
+slist *related_items = NULL;
 static char *original_help_text = NULL;
+static const char **help_urls = NULL;
 
 /* gtk_text_buffer_insert_with_tags_by_name (buffer, &iter, "some text in red", -1, "red_foreground", NULL); */
 
@@ -31,45 +38,42 @@ static void add_help_text(GtkWidget *text, const char *message)
   sg_text_insert(text, (char *)message);
 }
 
+
 int help_text_width(const char *txt, int start, int end)
 {
+#if 0
   char *buf;
   int len;
-  buf = (char *)CALLOC(end - start + 2, sizeof(char));
+  buf = (char *)calloc(end - start + 2, sizeof(char));
   strncpy(buf, txt, end - start);
-  len = sg_text_width(buf, ss->sgx->listener_fnt);
-  FREE(buf);
+  len = sg_text_width(buf, LISTENER_FONT(ss));
+  free(buf);
   if (len > 0) return(len);
+#endif
   return((end - start) * 8);
 }
 
-static int old_help_text_width = 0;
+
 static with_word_wrap_t outer_with_wrap = WITHOUT_WORD_WRAP;
+static int old_help_text_width = 0;
 
 static gboolean help_expose_callback(GtkWidget *w, GdkEventExpose *ev, gpointer data)
 {
   int curwid;
-  curwid = widget_width(help_text);
+  curwid = widget_width(help_dialog); /* was help_text, but I'm getting shivering in expose events */
   if (old_help_text_width == 0)
     old_help_text_width = curwid;
   else
     {
-      if ((outer_with_wrap == WITH_WORD_WRAP) && (abs(curwid - old_help_text_width) > 10))
+      if ((outer_with_wrap == WITH_WORD_WRAP) && 
+	  (abs(curwid - old_help_text_width) > 50))
 	{
-	  char *cur_help_str = NULL;
-	  char *new_help_str = NULL;
-	  int end;
-	  GtkTextIter s, e;
-	  GtkTextBuffer *buf;
-	  buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(help_text));
-	  end = gtk_text_buffer_get_char_count(buf);
-	  gtk_text_buffer_get_iter_at_offset(buf, &s, 0);
-	  gtk_text_buffer_get_iter_at_offset(buf, &e, end);
-	  cur_help_str = gtk_text_buffer_get_text(buf, &s, &e, true);
+	  char *cur_help_str = NULL, *new_help_str = NULL;
+	  cur_help_str = sg_get_text(help_text, 0, -1);
 	  new_help_str = word_wrap(original_help_text, curwid);
 	  gtk_text_buffer_set_text(gtk_text_view_get_buffer(GTK_TEXT_VIEW(help_text)), "", 0);
 	  sg_text_insert(help_text, new_help_str);
-	  if (new_help_str) FREE(new_help_str);
+	  if (new_help_str) free(new_help_str);
 	  if (cur_help_str) g_free(cur_help_str);
 	  old_help_text_width = curwid;
 	}
@@ -77,10 +81,11 @@ static gboolean help_expose_callback(GtkWidget *w, GdkEventExpose *ev, gpointer
   return(false);
 }
 
-static bool new_help(const char *pattern)
+
+static bool new_help(const char *pattern, bool complain)
 {
-  char *url;
-  char **xrefs;
+  const char *url;
+  const char **xrefs;
   url = snd_url(pattern);
   if (url)
     {
@@ -94,17 +99,19 @@ static bool new_help(const char *pattern)
 	  xrefs = help_name_to_xrefs(pattern);
 	  snd_help_with_xrefs(pattern, XEN_TO_C_STRING(xstr), WITH_WORD_WRAP, xrefs, NULL);
 	  snd_unprotect_at(gc_loc);
-	  if (xrefs) FREE(xrefs);
+	  if (xrefs) free(xrefs);
 	  return(true);
 	}
+      url_to_html_viewer(url);
+      return(true);
     }
-  if (!(snd_topic_help(pattern)))
+  if ((!(snd_topic_help(pattern))) && (complain))
     {
       xrefs = help_name_to_xrefs(pattern);
       if (xrefs)
 	{
 	  snd_help_with_xrefs(pattern, "(no help found)", WITH_WORD_WRAP, xrefs, NULL);
-	  FREE(xrefs);
+	  free(xrefs);
 	  return(true);
 	}
       else snd_help_with_xrefs(pattern, "(no help found)", WITH_WORD_WRAP, NULL, NULL);
@@ -112,6 +119,7 @@ static bool new_help(const char *pattern)
   return(false);
 }
 
+
 static char **help_history = NULL;
 static int help_history_size = 0;
 static int help_history_pos = 0;
@@ -122,8 +130,8 @@ static void add_pattern_to_help_history(const char *pattern)
   if (!help_needed) return;
   if (help_history_size == 0)
     {
-      help_history_size = 8;
-      help_history = (char **)CALLOC(help_history_size, sizeof(char *));
+      help_history_size = 16; /* not 8! -- need room for cycle below */
+      help_history = (char **)calloc(help_history_size, sizeof(char *));
     }
   else
     {
@@ -132,17 +140,18 @@ static void add_pattern_to_help_history(const char *pattern)
 	  int i;
 	  for (i = 0; i < 8; i++) 
 	    {
-	      if (help_history[i]) FREE(help_history[i]);
+	      if (help_history[i]) free(help_history[i]);
 	      help_history[i] = help_history[i + 8];
 	      help_history[i + 8] = NULL;
 	    }
 	  help_history_pos = 8;
 	}
     }
-  if (help_history[help_history_pos]) FREE(help_history[help_history_pos]);
-  help_history[help_history_pos++] = copy_string(pattern);
+  if (help_history[help_history_pos]) free(help_history[help_history_pos]);
+  help_history[help_history_pos++] = mus_strdup(pattern);
 }
 
+
 static void help_next_callback(GtkWidget *w, gpointer context)
 {
   if ((help_history_pos < help_history_size) && 
@@ -150,11 +159,12 @@ static void help_next_callback(GtkWidget *w, gpointer context)
     {
       help_needed = false;
       help_history_pos++;
-      new_help(help_history[help_history_pos - 1]);
+      new_help(help_history[help_history_pos - 1], true);
       help_needed = true;
     }
 }
 
+
 static void help_previous_callback(GtkWidget *w, gpointer context)
 {
   if ((help_history_pos > 1) &&
@@ -162,15 +172,16 @@ static void help_previous_callback(GtkWidget *w, gpointer context)
     {
       help_needed = false;
       help_history_pos--;
-      new_help(help_history[help_history_pos - 1]);
+      new_help(help_history[help_history_pos - 1], true);
       help_needed = true;
     }
 }
 
+
 static char *find_highlighted_text(const char *value)
 {
   int i, len, start = -1;
-  len = snd_strlen(value);
+  len = mus_strlen(value);
   for (i = 0; i < len; i++)
     if (value[i] == '{')
       start = i + 1;
@@ -184,7 +195,7 @@ static char *find_highlighted_text(const char *value)
 	      {
 		int k;
 		char *topic;
-		topic = (char *)CALLOC(end - start + 1, sizeof(char));
+		topic = (char *)calloc(end - start + 1, sizeof(char));
 		for (i = start, k = 0; i < end; i++, k++)
 		  topic[k] = value[i];
 		return(topic);
@@ -194,82 +205,69 @@ static char *find_highlighted_text(const char *value)
   return(NULL);
 }
 
-static void help_browse_callback(GtkTreeSelection *selection, gpointer *gp)
+
+static void help_browse_callback(const char *name, int row, void *data)
 {
-  GtkTreeIter iter;
-  gchar *value = NULL;
-  GtkTreeModel *model;
-  if (!(gtk_tree_selection_get_selected(selection, &model, &iter))) return;
-  gtk_tree_model_get(model, &iter, 0, &value, -1);
-  if (value)
+  char *topic = NULL;
+  if ((help_urls) && (help_urls[row]))
+    url_to_html_viewer(help_urls[row]);
+  else
     {
-      char *topic = NULL;
-      topic = find_highlighted_text(value);
+      topic = find_highlighted_text(name);
       if (topic)
 	{
 	  name_to_html_viewer(topic);
-	  FREE(topic);
+	  free(topic);
 	}
       else
 	{
-	  if (value)
-	    new_help(value);
+	  if (name)
+	    new_help(name, true);
 	}
     }
-  g_free(value);
 }
 
-static void double_callback(GtkTreeView *tree_view, GtkTreePath *path, GtkTreeViewColumn *column, gpointer data)
-{
-  GtkTreeModel *model = gtk_tree_view_get_model(tree_view);
-  GtkTreeIter iter;
-  gchar *value;
-  gtk_tree_model_get_iter(model, &iter, path);
-  gtk_tree_model_get(model, &iter, 0, &value, -1);
-  if (value)
-    {
-      gchar *topic;
-      topic = find_highlighted_text(value);
-      if (topic)
-	{
-	  name_to_html_viewer(topic);
-	  FREE(topic);
-	}
-      else
-	{
-	  if (value)
-	    name_to_html_viewer(value);
-	}
-      g_free(value);
-    }
-}
 
 static gboolean text_release_callback(GtkTreeSelection *selection, gpointer *gp)
 {
   /* this needs to be bool return false -- otherwise, apparently, the mouse-drag->selection never gets turned off! */
   GtkTextIter start, end;
   #define HELP_BUFFER gtk_text_view_get_buffer(GTK_TEXT_VIEW(help_text))
+
   if (gtk_text_buffer_get_selection_bounds(HELP_BUFFER, &start, &end))
     {
       char *txt;
       txt = gtk_text_buffer_get_text(HELP_BUFFER, &start, &end, true);
+      /* only change help text display if it's one word in the selection, and we can find new help */
       if (txt)
 	{
-	  new_help(txt);
+	  int i, len;
+	  bool one_word = true;
+	  len = mus_strlen(txt);
+	  for (i = 0; i < len; i++)
+	    if (isspace(txt[i]))
+	      {
+		one_word = false;
+		break;
+	      }
+	  if (one_word) new_help(txt, false);
 	  g_free(txt);
 	}
     }
+
   return(false);
 }
 
+
 static void search_activated(GtkWidget *w, gpointer context)
 { 
   char *str = NULL;
   str = (char *)gtk_entry_get_text(GTK_ENTRY(w)); /* no free, I think */
-  if (new_help(str))
+  if (new_help(str, true))
     gtk_entry_set_text(GTK_ENTRY(w), "");
 }
 
+
 static GtkWidget *help_next_button = NULL, *help_previous_button = NULL;
 
 static void create_help_monolog(void)
@@ -285,98 +283,105 @@ static void create_help_monolog(void)
   gtk_window_resize(GTK_WINDOW(help_dialog), HELP_COLUMNS * 9, HELP_ROWS * 40);
   gtk_widget_realize(help_dialog);
 
-  ok_button = gtk_button_new_with_label(_("Dismiss"));
+  ok_button = gtk_button_new_from_stock(GTK_STOCK_QUIT);
   gtk_widget_set_name(ok_button, "quit_button");
-  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(help_dialog)->action_area), ok_button, false, true, 20);
+  gtk_box_pack_start(GTK_BOX(DIALOG_ACTION_AREA(help_dialog)), ok_button, false, true, 20);
   SG_SIGNAL_CONNECT(ok_button, "clicked", dismiss_help_dialog, NULL);
   gtk_widget_show(ok_button);
+  set_stock_button_label(ok_button, _("Go Away"));
 
-  help_previous_button = gtk_button_new_with_label(_("Previous"));
+  help_previous_button = gtk_button_new_from_stock(GTK_STOCK_GO_BACK);
   gtk_widget_set_name(help_previous_button, "reset_button");
-  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(help_dialog)->action_area), help_previous_button, true, true, 10);
-  help_next_button = gtk_button_new_with_label(_("Next"));
-  gtk_widget_set_name(help_next_button, "doit_button");
-  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(help_dialog)->action_area), help_next_button, true, true, 10);
-  SG_SIGNAL_CONNECT(help_next_button, "clicked", help_next_callback, NULL);
+  gtk_box_pack_start(GTK_BOX(DIALOG_ACTION_AREA(help_dialog)), help_previous_button, true, true, 10);
   SG_SIGNAL_CONNECT(help_previous_button, "clicked", help_previous_callback, NULL);
-  gtk_widget_show(help_next_button);
   gtk_widget_show(help_previous_button);
 
+  help_next_button = gtk_button_new_from_stock(GTK_STOCK_GO_FORWARD);
+  gtk_widget_set_name(help_next_button, "doit_button");
+  gtk_box_pack_start(GTK_BOX(DIALOG_ACTION_AREA(help_dialog)), help_next_button, true, true, 10);
+  SG_SIGNAL_CONNECT(help_next_button, "clicked", help_next_callback, NULL);
+  gtk_widget_show(help_next_button);
+
   frame = gtk_frame_new(NULL);
-  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(help_dialog)->vbox), frame, true, true, 10); 
+  gtk_box_pack_start(GTK_BOX(DIALOG_CONTENT_AREA(help_dialog)), frame, true, true, 10); 
   gtk_widget_show(frame);
 
-  help_text = make_scrolled_text(frame, false, NULL, NULL);
+  help_text = make_scrolled_text(frame, false, 0, false); /* last arg ignored */
   gtk_widget_add_events(help_text, GDK_BUTTON_RELEASE);
   gtk_text_view_set_left_margin(GTK_TEXT_VIEW(help_text), 10);
   SG_SIGNAL_CONNECT(help_text, "button_release_event", text_release_callback, NULL);
 
-  related_items = sg_make_list(_("related topics"), 
-			       GTK_DIALOG(help_dialog)->vbox, 
-			       BOX_PACK, NULL, 0, NULL,
-			       GTK_SIGNAL_FUNC(help_browse_callback), 0, 0, 0, 0);
-  gtk_widget_add_events(related_items, GDK_ALL_EVENTS_MASK);
-  SG_SIGNAL_CONNECT(related_items, "row_activated", double_callback, NULL);
+  related_items = slist_new_with_title(_("related topics"), DIALOG_CONTENT_AREA(help_dialog), NULL, 0, BOX_PACK);
+  related_items->select_callback = help_browse_callback;
 
   hbox = gtk_hbox_new(false, 0);
-  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(help_dialog)->vbox), hbox, false, false, 10); 
+  gtk_box_pack_start(GTK_BOX(DIALOG_CONTENT_AREA(help_dialog)), hbox, false, false, 10); 
   gtk_widget_show(hbox);
 
   label = gtk_label_new(_("help topic:"));
   gtk_box_pack_start(GTK_BOX(hbox), label, false, false, 0); 
   gtk_widget_show(label);
 
-  search = snd_entry_new(hbox, true);
+  search = snd_entry_new(hbox, WITH_WHITE_BACKGROUND);
   SG_SIGNAL_CONNECT(search, "activate", search_activated, NULL);
   gtk_widget_show(help_dialog);
   SG_SIGNAL_CONNECT(GTK_OBJECT(help_dialog), "expose_event", help_expose_callback, NULL);
   set_dialog_widget(HELP_DIALOG, help_dialog);
-
-  /* gtk_text_buffer_create_tag(gtk_text_view_get_buffer(GTK_TEXT_VIEW(help_text)), "red_foreground", "foreground", "red", NULL); */
 }
 
+
 GtkWidget *snd_help(const char *subject, const char *helpstr, with_word_wrap_t with_wrap)
 {
   /* place help string in scrollable help window */
   /* if window is already active, add this help at the top and reposition */
+
   outer_with_wrap = with_wrap;
-  if (!(help_dialog)) create_help_monolog(); else raise_dialog(help_dialog);
+  if (!(help_dialog)) 
+    create_help_monolog(); 
+  else raise_dialog(help_dialog);
+
   gtk_window_set_title(GTK_WINDOW(help_dialog), subject);
   original_help_text = (char *)helpstr;
   gtk_text_buffer_set_text(gtk_text_view_get_buffer(GTK_TEXT_VIEW(help_text)), "", 0);
+
   if (with_wrap == WITH_WORD_WRAP)
     {
       char *new_help = NULL;
       new_help = word_wrap(helpstr, (int)(widget_width(help_text) * 1.3));
       add_help_text(help_text, new_help);
-      if (new_help) FREE(new_help);
+      if (new_help) free(new_help);
     }
   else add_help_text(help_text, helpstr);
-  gtk_list_store_clear(GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(related_items))));
+
   if (help_needed) add_pattern_to_help_history(subject);
+  slist_clear(related_items); /* this can clobber "subject"! */
   gtk_widget_set_sensitive(help_next_button, (help_history_pos < help_history_size) && (help_history[help_history_pos]));
   gtk_widget_set_sensitive(help_previous_button, (help_history_pos > 1));
   return(help_dialog);
 }
 
-GtkWidget *snd_help_with_xrefs(const char *subject, const char *helpstr, with_word_wrap_t with_wrap, char **xrefs, char **urls)
+
+GtkWidget *snd_help_with_xrefs(const char *subject, const char *helpstr, with_word_wrap_t with_wrap, const char **xrefs, const char **urls)
 {
   GtkWidget *w;
+  help_urls = urls;
   w = snd_help(subject, helpstr, with_wrap);
   if (xrefs)
     {
       int i;
       for (i = 0; (xrefs[i]); i++)
-	sg_list_append(related_items, xrefs[i]);
+	slist_append(related_items, xrefs[i]);
     }
   return(w);
 }
 
-void snd_help_append(char *text)
+
+void snd_help_append(const char *text)
 {
   if (help_text) sg_text_insert(help_text, text);
 }
 
+
 void snd_help_back_to_top(void)
 {
   if (help_text)
@@ -388,3 +393,10 @@ void snd_help_back_to_top(void)
       gtk_text_view_scroll_to_iter(GTK_TEXT_VIEW(help_text), &pos, 0.0, true, 0.0, 0.0);
     }
 }
+
+
+bool help_dialog_is_active(void)
+{
+  return((help_dialog) &&
+	 (GTK_WIDGET_VISIBLE(help_dialog)));
+}
