@@ -8,13 +8,14 @@
   ;; taken loosely from glxvisuals.c
   "(gl-info) prints out GL-related info"
   (define (class-of n)
-    (cond ((= n StaticGray) "static-gray")
-	  ((= n GrayScale) "gray-scale")
-	  ((= n StaticColor) "static-color")
-	  ((= n PseudoColor) "pseudo-color")
-	  ((= n TrueColor) "true-color")
-	  ((= n DirectColor) "direct-color")
-	  (#t "??")))
+    (and (number? n)
+	 (cond ((= n StaticGray) "static-gray")
+	       ((= n GrayScale) "gray-scale")
+	       ((= n StaticColor) "static-color")
+	       ((= n PseudoColor) "pseudo-color")
+	       ((= n TrueColor) "true-color")
+	       ((= n DirectColor) "direct-color")
+	       (#t "??"))))
   (let* ((cx (snd-glx-context))
 	 (dpy (XtDisplay (cadr (main-widgets))))
 	 (version (glXQueryVersion dpy 0 0)))
@@ -113,7 +114,7 @@
       (let ((rl-data (sound-data->vct input-data 0 (vector-ref data slice))))
 	(snd-spectrum rl-data blackman2-window bins #t 0.0 #t #f)
 	(redraw-graph))
-      (set! slice (1+ slice))
+      (set! slice (+ 1 slice))
       (if (>= slice slices)
 	  (set! slice 0))
       #f)
@@ -128,7 +129,7 @@
 	  (begin
 	    (XtRemoveWorkProc input-proc)
 	    (set! input-proc 0)))
-      (do ((i 0 (1+ i)))
+      (do ((i 0 (+ 1 i)))
 	  ((= i slices))
 	(vct-scale! (vector-ref data i) 0.0)))
     
@@ -159,21 +160,21 @@
     ;; turn display with orientation dialog
     ;;  for example: x-angle 290, y angle: 60
     
-    (lambda* (start #:optional scl pc-spectrum fft-size)
+    (lambda* (start :optional scl pc-spectrum fft-size)
 	     (if start
 		 (begin
 		   (set! cutoff pc-spectrum)
 		   (set! scaler scl)
 		   (set! bins fft-size)
 		   (set! input-data (make-sound-data 1 (* bins 2)))
-		   (do ((i 0 (1+ i)))
+		   (do ((i 0 (+ 1 i)))
 		       ((= i slices))
 		     (vector-set! data i (make-vct bins)))
 		   (start-it))
 		 (stop-it)))))
 
   
-(define* (start-waterfall #:optional (scl 1.0) (pc-spectrum 0.2) (fft-size 512))
+(define* (start-waterfall :optional (scl 1.0) (pc-spectrum 0.2) (fft-size 512))
   "(start-waterfall (scl 1.0) (pc-spectrum 0.2) (fft-size 512)) starts a 'waterfall' spectrum display of the incoming audio data"
   (waterfall #t scl pc-spectrum fft-size))
 
@@ -267,7 +268,7 @@
   (display (format #f "GL_LIGHT_MODEL_TWO_SIDE: ~A~%" (glGetBooleanv GL_LIGHT_MODEL_TWO_SIDE)))
 
   (let ((nlights (car (glGetIntegerv GL_MAX_LIGHTS))))
-    (do ((i 0 (1+ i)))
+    (do ((i 0 (+ 1 i)))
 	((= i nlights))
       (if (car (glGetBooleanv (+ GL_LIGHT0  i)))
 	  (begin
@@ -450,7 +451,7 @@
 	  (glNewList gl-list GL_COMPILE)
 	    (glBegin GL_LINES)
 	    (apply glColor3f (color->list (data-color)))
-	    (do ((i 0 (1+ i)))
+	    (do ((i 0 (+ 1 i)))
 		((= i 256))
 	      (glVertex3f (/ i 256.0) 0.0 0.0)
 	      (glVertex3f (/ i 256.0) (vct-ref rl i) (vct-ref im i)))
