@@ -1,67 +1,49 @@
 #include "snd.h"
 
-static GtkWidget *completion_dialog = NULL;
-static bool first_time = true;
-static GtkWidget *listener_text = NULL, *completion_list = NULL;
-static int printout_end;
+/* where to post the completions list? 
+ *   I'd like a drop-down scrolled list as in Motif, but I can't figure out any way to do it
+ */
+
+static GtkWidget *listener_text = NULL;
+
+static GtkWidget *completion_pane = NULL;
+static bool completion_list_active = false;
+
+static int printout_end = 0;
+
 #define LISTENER_BUFFER gtk_text_view_get_buffer(GTK_TEXT_VIEW(listener_text))
 
-static void list_completions_callback(GtkTreeSelection *selection, gpointer *gp)
+#if 0
+static slist *completion_list = NULL;
+
+static void list_completions_callback(const char *name, int row, void *data)
 {
-  GtkTreeIter iter;
-  gchar *value = NULL;
-  GtkTreeModel *model;
   int beg, end, i, j, old_len, new_len;
   char *old_text;
-  if (first_time)
-    {
-      GtkTreeSelection *tree;
-      first_time = false;
-      tree = gtk_tree_view_get_selection(GTK_TREE_VIEW(completion_list));
-      gtk_tree_selection_unselect_all(tree);
-      return;
-    }
-  if (!(gtk_tree_selection_get_selected(selection, &model, &iter))) return;
-  gtk_tree_model_get(model, &iter, 0, &value, -1);
   beg = printout_end + 1;
   end = gtk_text_buffer_get_char_count(LISTENER_BUFFER);
   old_text = sg_get_text(listener_text, beg, end);
-  old_len = snd_strlen(old_text);
-  new_len = snd_strlen(value);
+  old_len = mus_strlen(old_text);
+  new_len = mus_strlen(name);
   for (i = old_len - 1, j = new_len - 1; j >= 0; j--)
     {
-      if (old_text[i] != value[j])
+      if (old_text[i] != name[j])
 	{
 	  i = old_len - 1;
-	  if (old_text[i] == value[j]) i--;
+	  if (old_text[i] == name[j]) i--;
 	}
       else i--;
     }
-  append_listener_text(0, (char *)(value - 1 + old_len - i));
+  completion_list_active = false;
+  append_listener_text(0, (char *)(name - 1 + old_len - i));
   if (old_text) g_free(old_text);
-  gtk_widget_hide(completion_dialog);
-  if (value) g_free(value);
-}
-
-static void dismiss_completion_callback(GtkWidget *w, gpointer context)
-{
-  gtk_widget_hide(completion_dialog);
-}
-
-static void help_completion_callback(GtkWidget *w, gpointer context)
-{
-  completion_dialog_help();
-}
-
-static gint delete_completion_dialog(GtkWidget *w, GdkEvent *event, gpointer context)
-{
-  gtk_widget_hide(completion_dialog);
-  return(true);
+  gtk_widget_hide(completion_pane);
 }
+#endif
 
+#if 0
 static void start_completion_dialog(int num_items, char **items)
 {
-  GtkTreeSelection *tree;
   if (!completion_dialog)
     {
       GtkWidget *help_button, *dismiss_button;
@@ -73,74 +55,98 @@ static void start_completion_dialog(int num_items, char **items)
       gtk_widget_realize(completion_dialog);
       gtk_window_resize(GTK_WINDOW(completion_dialog), 260, 200);
 
-      help_button = gtk_button_new_with_label(_("Help"));
+      help_button = gtk_button_new_from_stock(GTK_STOCK_HELP);
       gtk_widget_set_name(help_button, "help_button");
-      dismiss_button = gtk_button_new_with_label(_("Dismiss"));
+
+      dismiss_button = gtk_button_new_from_stock(GTK_STOCK_QUIT);
       gtk_widget_set_name(dismiss_button, "quit_button");
-      gtk_box_pack_start(GTK_BOX(GTK_DIALOG(completion_dialog)->action_area), dismiss_button, false, true, 10);
-      gtk_box_pack_end(GTK_BOX(GTK_DIALOG(completion_dialog)->action_area), help_button, false, true, 10);
+      set_stock_button_label(dismiss_button, _("Go Away"));
+
+      gtk_box_pack_start(GTK_BOX(DIALOG_ACTION_AREA(completion_dialog)), dismiss_button, false, true, 10);
+      gtk_box_pack_end(GTK_BOX(DIALOG_ACTION_AREA(completion_dialog)), help_button, false, true, 10);
       SG_SIGNAL_CONNECT(dismiss_button, "clicked", dismiss_completion_callback, NULL);
       SG_SIGNAL_CONNECT(help_button, "clicked", help_completion_callback, NULL);
       gtk_widget_show(dismiss_button);
       gtk_widget_show(help_button);
   
-      first_time = true;
-      completion_list = sg_make_list(_("Completions"), 
-				     GTK_DIALOG(completion_dialog)->vbox,
-				     CONTAINER_ADD,
-				     NULL,
-				     num_items, items,
-				     GTK_SIGNAL_FUNC(list_completions_callback),
-				     0, 0, 0, 0);
-
+      completion_list = slist_new(DIALOG_CONTENT_AREA(completion_dialog), items, num_items, CONTAINER_ADD);
+      completion_list->select_callback = list_completions_callback;
       set_dialog_widget(COMPLETION_DIALOG, completion_dialog);
     }
   else 
     {
       int i;
-      GtkTreeIter iter;
-      GtkListStore *model;
-      model = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(completion_list)));
-      gtk_list_store_clear(model);
+      slist_clear(completion_list);
       for (i = 0; i < num_items; i++) 
-	{
-	  gtk_list_store_append(model, &iter);
-	  gtk_list_store_set(model, &iter, 0, items[i], -1);
-	}
+	slist_append(completion_list, items[i]);
       raise_dialog(completion_dialog);
     }
-  tree = gtk_tree_view_get_selection(GTK_TREE_VIEW(completion_list));
-  gtk_tree_selection_unselect_all(tree);
   gtk_widget_show(completion_dialog);
 }
+#endif
 
 
-void save_listener_text(FILE *fp)
+int save_listener_text(FILE *fp)
 {
+  /* return -1 if fwrite error */
   if (listener_text)
     {
       char *str = NULL;
       str = sg_get_text(listener_text, 0, -1);
       if (str)
 	{
-	  fwrite((void *)str, sizeof(char), snd_strlen(str), fp);
+	  size_t bytes;
+	  bytes = fwrite((void *)str, sizeof(char), mus_strlen(str), fp);
 	  g_free(str);
+	  if (bytes == 0) return(-1);
 	}
     }
+  return(0);
 }
 
-void append_listener_text(int end, char *msg)
+
+void append_listener_text(int end, const char *msg)
 {
   /* "end" arg needed in Motif */
-  if ((listener_print_p(msg)) && (listener_text))
+  if ((listener_print_p(msg)) && 
+      (listener_text))
     {
       int chars;
       chars = gtk_text_buffer_get_char_count(LISTENER_BUFFER);
       if (chars > 0) sg_set_cursor(listener_text, chars + 1);
-      sg_text_insert(listener_text, msg);
+      sg_text_insert(listener_text, (char *)msg);
     }
 }
 
+
+static GtkTextTag *prompt_not_editable = NULL;
+
+static void append_listener_prompt()
+{
+  /* append cr + prompt with not-editable tag */
+  if (listener_text)
+    {
+      int chars;
+      GtkTextIter pos;
+      chars = gtk_text_buffer_get_char_count(LISTENER_BUFFER);
+      if (chars > 0) sg_set_cursor(listener_text, chars + 1);
+      gtk_text_buffer_get_end_iter(LISTENER_BUFFER, &pos);
+      gtk_text_buffer_insert_with_tags(LISTENER_BUFFER, &pos, listener_prompt_with_cr(), ss->listener_prompt_length + 1, prompt_not_editable, NULL);
+    }
+}
+
+
+static void sg_text_delete(GtkWidget *w, int start, int end)
+{
+  GtkTextIter s, e;
+  GtkTextBuffer *buf;
+  buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(w));
+  gtk_text_buffer_get_iter_at_offset(buf, &s, start);
+  gtk_text_buffer_get_iter_at_offset(buf, &e, end); 
+  gtk_text_buffer_delete(buf, &s, &e);
+}
+
+
 static void listener_completion(int end)
 {
   int beg;
@@ -160,75 +166,78 @@ static void listener_completion(int end)
 	  g_free(old_text);
 	  return;
 	}
-      if (strcmp(old_text, new_text) == 0) 
+      if (mus_strcmp(old_text, new_text))
 	matches = get_completion_matches();
       sg_text_delete(listener_text, beg, end);
       append_listener_text(0, new_text);
       goto_window(listener_text);
       if (new_text) 
 	{
-	  FREE(new_text); 
+	  free(new_text); 
 	  new_text = NULL;
 	}
+#if 0
       if (matches > 1)
 	{
 	  clear_possible_completions();
 	  set_save_completions(true);
 	  if (file_text) 
-	    new_text = filename_completer(file_text); 
-	  else new_text = command_completer(old_text);
+	    new_text = filename_completer(NULL, file_text, NULL); 
+	  else new_text = expression_completer(NULL, old_text, NULL);
 	  if (new_text) 
 	    {
-	      FREE(new_text); 
+	      free(new_text); 
 	      new_text = NULL;
 	    }
-	  display_completions();
+
+	  /* display_completions(); */
+	  completion_list_active = true;
 	  set_save_completions(false);
-	  if (file_text) FREE(file_text);
 	}
+#endif
+      if (file_text) free(file_text);
       if (old_text) g_free(old_text);
     }
 }
 
-void snd_completion_help(int matches, char **pbuffer) 
-{
-  if (matches > 0)
-    start_completion_dialog(matches, pbuffer);
-}
 
 
-/* ---------------- command widget replacement ---------------- */
 
-void listener_append(char *msg)
+/* ---------------- listener widget ---------------- */
+
+void listener_append(const char *msg)
 {
+  append_listener_text(0, msg); /* do this in any case to make sure print-hook gets to run (listener_print_p in append_listener_text) */
   if (listener_text)
     {
-      if ((ss->sgx)->graph_is_active)
-	(ss->sgx)->graph_is_active = false;
-      append_listener_text(0, msg);
+      if (ss->sgx->graph_is_active)
+	ss->sgx->graph_is_active = false;
       printout_end = gtk_text_buffer_get_char_count(LISTENER_BUFFER) - 1;
     }
 }
 
-void listener_append_and_prompt(char *msg)
+
+void listener_append_and_prompt(const char *msg)
 {
+  if (msg)
+    append_listener_text(0, msg);
+  append_listener_prompt();
   if (listener_text)
     {
       int cmd_eot;
-      if (msg)
-	append_listener_text(0, msg);
-      append_listener_text(0, listener_prompt_with_cr());
       cmd_eot = gtk_text_buffer_get_char_count(LISTENER_BUFFER);
       printout_end = cmd_eot - 1;
     }
 }
 
-static void command_return_callback(void)
+
+static void listener_return_callback(void)
 {
   if (!(ss->error_lock))
-    command_return(listener_text, printout_end);
+    listener_return(listener_text, printout_end);
 }
 
+
 static void back_to_start(void)
 {
   char *full_str = NULL, *prompt;
@@ -240,8 +249,7 @@ static void back_to_start(void)
     {
       int i;
       for (i = start_of_text; i >= 0; i--)
-	if ((full_str[i] == prompt[0]) && 
-	    ((i == 0) || (full_str[i - 1] == '\n')))
+	if (is_prompt(full_str, i))
 	  {
 	    start_of_text = i + 1;
 	    break;
@@ -251,6 +259,7 @@ static void back_to_start(void)
   if (full_str) g_free(full_str);
 }
 
+
 void listener_delete_text(int new_end)
 {
   int old_end;
@@ -259,6 +268,7 @@ void listener_delete_text(int new_end)
     sg_text_delete(listener_text, new_end, old_end);
 }
 
+
 static void clear_back_to_prompt(GtkWidget *w)
 {
   int beg, end;
@@ -269,6 +279,25 @@ static void clear_back_to_prompt(GtkWidget *w)
   sg_text_delete(w, beg, end);
 }
 
+
+#if HAVE_GTK_TEXT_BUFFER_SELECT_RANGE && HAVE_GTK_WIDGET_GET_CLIPBOARD
+static void ctrl_k(GtkWidget *w)
+{
+  GtkTextIter beg, end;
+  GtkTextBuffer *buf;
+  buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(w));
+  gtk_text_buffer_get_iter_at_mark(buf, &beg, gtk_text_buffer_get_mark(buf, "insert"));
+  end = beg;
+  gtk_text_iter_forward_to_end(&end);
+  if (!gtk_text_iter_equal(&beg, &end))
+    {
+      gtk_text_buffer_select_range(buf, &beg, &end);
+      gtk_text_buffer_cut_clipboard(buf, gtk_widget_get_clipboard(w, GDK_SELECTION_CLIPBOARD), true);
+    }
+}
+#endif
+
+
 static void sg_text_replace(GtkWidget *w, int beg, int end, char *text)
 {
   GtkTextIter pos, endpos, pos1;
@@ -281,6 +310,7 @@ static void sg_text_replace(GtkWidget *w, int beg, int end, char *text)
   gtk_text_buffer_insert(buf, &pos1, text, strlen(text));
 }
 
+
 static void text_transpose(GtkWidget *w)
 {
   int curpos;
@@ -298,6 +328,7 @@ static void text_transpose(GtkWidget *w)
     }
 }
 
+
 static void word_upper(GtkWidget *w, int cap, int up)
 {
   int curpos, endpos;
@@ -356,6 +387,7 @@ static void listener_help(void)
     }
 }
 
+
 static GtkTextTag *flash_tag = NULL;
 static int flashes = 0;
 static int paren_pos = -1;
@@ -368,10 +400,11 @@ static void add_inverse(int pos)
   buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(listener_text));
   gtk_text_buffer_get_iter_at_offset(buf, &start, pos);
   gtk_text_buffer_get_iter_at_offset(buf, &end, pos + 1);
-  if (!flash_tag) flash_tag = gtk_text_buffer_create_tag (buf, "red_background", "background", "red", NULL);
+  if (!flash_tag) flash_tag = gtk_text_buffer_create_tag(buf, "red_background", "background", "red", NULL);
   gtk_text_buffer_apply_tag(buf, flash_tag, &start, &end);
 }
 
+
 static void remove_inverse(int pos)
 {
   GtkTextIter start, end;
@@ -383,6 +416,7 @@ static void remove_inverse(int pos)
   gtk_text_buffer_remove_tag(buf, flash_tag, &start, &end);
 }
 
+
 static gint flash_unbalanced_paren(gpointer data)
 {
   flashes--;
@@ -397,6 +431,7 @@ static gint flash_unbalanced_paren(gpointer data)
   return(0);
 }
 
+
 bool highlight_unbalanced_paren(void)
 {
   /* if cursor is positioned at close paren, try to find reason for unbalanced expr and highlight it */
@@ -411,7 +446,7 @@ bool highlight_unbalanced_paren(void)
 	  ((str[pos - 2] != '\\') || (str[pos - 3] != '#')))
 	{
 	  int parens;
-	  parens = find_matching_paren(str, 2, pos - 1, listener_prompt(ss), &paren_pos);
+	  parens = find_matching_paren(str, 2, pos - 1, &paren_pos);
 	  if (parens == 0)
 	    {
 	      add_inverse(paren_pos);
@@ -436,10 +471,11 @@ static void add_underline(int pos)
   buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(listener_text));
   gtk_text_buffer_get_iter_at_offset(buf, &start, pos);
   gtk_text_buffer_get_iter_at_offset(buf, &end, pos + 1);
-  if (!tag) tag = gtk_text_buffer_create_tag(buf, "underline", "underline", PANGO_UNDERLINE_SINGLE, NULL);
+  if (!tag) tag = gtk_text_buffer_create_tag(buf, "underline", "underline", PANGO_UNDERLINE_DOUBLE, NULL);
   gtk_text_buffer_apply_tag(buf, tag, &start, &end);
 }
 
+
 static void remove_underline(int pos)
 {
   GtkTextIter start, end;
@@ -447,11 +483,12 @@ static void remove_underline(int pos)
   buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(listener_text));
   gtk_text_buffer_get_iter_at_offset(buf, &start, pos);
   gtk_text_buffer_get_iter_at_offset(buf, &end, pos + 1);
-  if (!tag) tag = gtk_text_buffer_create_tag(buf, "underline", "underline", PANGO_UNDERLINE_SINGLE, NULL);
+  if (!tag) tag = gtk_text_buffer_create_tag(buf, "underline", "underline", PANGO_UNDERLINE_DOUBLE, NULL);
   gtk_text_buffer_remove_tag(buf, tag, &start, &end);
 }
 
-static gboolean listener_key_release(GtkWidget *w, GdkEventKey *event, gpointer data)
+
+static void check_parens(void)
 {
   int current_position, parens, pos;
   if (old_pos != -1)
@@ -466,7 +503,7 @@ static gboolean listener_key_release(GtkWidget *w, GdkEventKey *event, gpointer
       fstr = sg_get_text(listener_text, 0, current_position);
       if (fstr[current_position - 1] == ')')
 	{
-	  parens = find_matching_paren(fstr, 1, current_position - 1, listener_prompt(ss), &pos);
+	  parens = find_matching_paren(fstr, 1, current_position - 1, &pos);
 	  if (parens == 0)
 	    {
 	      add_underline(pos);
@@ -475,250 +512,298 @@ static gboolean listener_key_release(GtkWidget *w, GdkEventKey *event, gpointer
 	}
       g_free(fstr);
     }
+}
+
+
+static gboolean listener_key_release(GtkWidget *w, GdkEventKey *event, gpointer data)
+{
+  check_parens();
   return(false);
 }
 
+
 static gboolean listener_key_press(GtkWidget *w, GdkEventKey *event, gpointer data)
 {
-  if ((ss->sgx)->graph_is_active) 
+  if ((completion_pane) &&
+      (completion_list_active))
+    {
+      gtk_widget_hide(completion_pane);
+      completion_list_active = false;
+    }
+
+  if (ss->sgx->graph_is_active) 
     {
       chan_info *cp;
       cp = current_channel();
       graph_key_press(channel_graph(cp), event, (gpointer)cp); 
-      return(false);
+      return(true); /* don't repeat the keystroke */
     }
+
+#if HAVE_S7
+  /* TODO: set ss->listener_char = char */
+#endif
+
+
+  if (EVENT_KEYVAL(event) == GDK_Tab)
+    {
+      listener_completion(gtk_text_buffer_get_char_count(LISTENER_BUFFER));
+      return(true);
+    }
+
+  if (EVENT_KEYVAL(event) == GDK_Return)
+    listener_return_callback();
   else
     {
-      if (event->keyval == GDK_Tab)
+      if (((EVENT_KEYVAL(event) == snd_K_g) || (EVENT_KEYVAL(event) == snd_K_G)) && 
+	  (EVENT_STATE(event) & snd_ControlMask))
 	{
-	  listener_completion(gtk_text_buffer_get_char_count(LISTENER_BUFFER));
-	  return(true);
+	  if (EVENT_STATE(event) & snd_MetaMask)
+	    clear_listener();
+	  else control_g(any_selected_sound());
 	}
       else
 	{
-	  if (event->keyval == GDK_Return)
-	    command_return_callback();
+	  if (((EVENT_KEYVAL(event) == snd_K_a) || (EVENT_KEYVAL(event) == snd_K_A)) && 
+	      (EVENT_STATE(event) & snd_ControlMask))
+	    {
+	      back_to_start();
+	    }
 	  else
 	    {
-	      if (((event->keyval == snd_K_g) || (event->keyval == snd_K_G)) && 
-		  (event->state & snd_ControlMask))
+	      if (EVENT_KEYVAL(event) == GDK_BackSpace)
 		{
-		  if (event->state & snd_MetaMask)
-		    clear_listener();
-		  else control_g(any_selected_sound());
+		  int current_position;
+		  char *fstr;
+		  current_position = sg_cursor_position(listener_text);
+		  if (current_position > 1)
+		    {
+		      fstr = sg_get_text(listener_text, current_position - 2, current_position);
+		      if ((current_position != (printout_end - 2)) && 
+			  (!(mus_strcmp(fstr, listener_prompt_with_cr()))))
+			{
+			  g_free(fstr);
+			  return(false);
+			}
+		      g_free(fstr);
+		    }
 		}
 	      else
 		{
-		  if (((event->keyval == snd_K_a) || (event->keyval == snd_K_A)) && 
-		      (event->state & snd_ControlMask))
+		  if ((EVENT_KEYVAL(event) == snd_K_greater) && (EVENT_STATE(event) & snd_MetaMask))
 		    {
-		      back_to_start();
+		      int end;
+		      end = gtk_text_buffer_get_char_count(LISTENER_BUFFER);
+		      sg_set_cursor(listener_text, end + 1);
 		    }
 		  else
 		    {
-		      if (event->keyval == GDK_BackSpace)
+		      if ((EVENT_KEYVAL(event) == snd_K_less) && (EVENT_STATE(event) & snd_MetaMask))
 			{
-			  int current_position;
-			  char *fstr;
-			  current_position = sg_cursor_position(listener_text);
-			  if (current_position > 1)
-			    {
-			      fstr = sg_get_text(listener_text, current_position - 2, current_position);
-			      if ((current_position != (printout_end - 2)) && 
-				  (strcmp(fstr, listener_prompt_with_cr()) != 0))
-				{
-				  g_free(fstr);
-				  return(false);
-				}
-			      g_free(fstr);
-			    }
+			  sg_set_cursor(listener_text, 2);
 			}
-		      else
+		      else 
 			{
-			  if ((event->keyval == snd_K_greater) && (event->state & snd_MetaMask))
+			  if (((EVENT_KEYVAL(event) == snd_K_p) || (EVENT_KEYVAL(event) == snd_K_P)) && (EVENT_STATE(event) & snd_MetaMask))
 			    {
-			      int end;
-			      end = gtk_text_buffer_get_char_count(LISTENER_BUFFER);
-			      sg_set_cursor(listener_text, end + 1);
+			      clear_back_to_prompt(listener_text);
+			      restore_listener_string(true);
 			    }
-			  else
+			  else 
 			    {
-			      if ((event->keyval == snd_K_less) && (event->state & snd_MetaMask))
+			      if (((EVENT_KEYVAL(event) == snd_K_n) || (EVENT_KEYVAL(event) == snd_K_N)) && (EVENT_STATE(event) & snd_MetaMask))
 				{
-				  sg_set_cursor(listener_text, 2);
+				  clear_back_to_prompt(listener_text);
+				  restore_listener_string(false);
 				}
 			      else 
 				{
-				  if (((event->keyval == snd_K_p) || (event->keyval == snd_K_P)) && (event->state & snd_MetaMask))
+				  if ((EVENT_KEYVAL(event) == GDK_question) && (EVENT_STATE(event) & snd_ControlMask))
 				    {
-				      clear_back_to_prompt(listener_text);
-				      restore_listener_string(true);
+				      listener_help();
 				    }
-				  else 
+				  else
 				    {
-				      if (((event->keyval == snd_K_n) || (event->keyval == snd_K_N)) && (event->state & snd_MetaMask))
+				      if (((EVENT_KEYVAL(event) == snd_K_c) || (EVENT_KEYVAL(event) == snd_K_C)) && (EVENT_STATE(event) & snd_MetaMask))
 					{
-					  clear_back_to_prompt(listener_text);
-					  restore_listener_string(false);
-					}
-				      else 
+					  /* M-c (as opposed to M-C) is trapped somewhere else */
+					  word_upper(listener_text, true, false);
+					} 
+				      else
 					{
-					  if ((event->keyval == GDK_question) && (event->state & snd_ControlMask))
+					  if (((EVENT_KEYVAL(event) == snd_K_l) || (EVENT_KEYVAL(event) == snd_K_L)) && (EVENT_STATE(event) & snd_MetaMask))
 					    {
-					      listener_help();
+					      word_upper(listener_text, false, false);
 					    }
 					  else
 					    {
-					      if (((event->keyval == snd_K_c) || (event->keyval == snd_K_C)) && (event->state & snd_MetaMask))
+					      if (((EVENT_KEYVAL(event) == snd_K_u) || (EVENT_KEYVAL(event) == snd_K_U)) && (EVENT_STATE(event) & snd_MetaMask))
 						{
-						  /* M-c (as opposed to M-C) is trapped somewhere else */
-						  word_upper(listener_text, true, false);
-						} 
+						  word_upper(listener_text, false, true);
+						}
 					      else
 						{
-						  if (((event->keyval == snd_K_l) || (event->keyval == snd_K_L)) && (event->state & snd_MetaMask))
+						  if (((EVENT_KEYVAL(event) == snd_K_t) || (EVENT_KEYVAL(event) == snd_K_T)) && (EVENT_STATE(event) & snd_ControlMask))
 						    {
-						      word_upper(listener_text, false, false);
+						      text_transpose(listener_text);
 						    }
 						  else
 						    {
-						      if (((event->keyval == snd_K_u) || (event->keyval == snd_K_U)) && (event->state & snd_MetaMask))
+						      if ((EVENT_KEYVAL(event) == snd_K_underscore) && (EVENT_STATE(event) & snd_ControlMask))
 							{
-							  word_upper(listener_text, false, true);
+							  backup_listener_to_previous_expression();
 							}
 						      else
 							{
-							  if (((event->keyval == snd_K_t) || (event->keyval == snd_K_T)) && (event->state & snd_ControlMask))
+#if HAVE_GTK_TEXT_BUFFER_SELECT_RANGE && HAVE_GTK_WIDGET_GET_CLIPBOARD
+							  if ((EVENT_KEYVAL(event) == snd_K_k) && (EVENT_STATE(event) & snd_ControlMask))
 							    {
-							      text_transpose(listener_text);
+							      /* select to line end, copy to clipboard, delete */
+							      ctrl_k(listener_text);
 							    }
 							  else
-							    {
-							      if ((event->keyval == snd_K_underscore) && (event->state & snd_ControlMask))
-								{
-								  backup_listener_to_previous_command();
-								}
-							      else
-								return(false);
-							    }}}}}}}}}}}}}}}
+#endif
+							    return(false);
+							}}}}}}}}}}}}}}
   g_signal_stop_emission(GTK_OBJECT(w), g_signal_lookup("key_press_event", G_OBJECT_TYPE(GTK_OBJECT(w))), 0);
   return(false);
 }
 
+
 static XEN listener_click_hook; 
 
 static gboolean listener_button_press(GtkWidget *w, GdkEventButton *ev, gpointer data)
 {
-  (ss->sgx)->graph_is_active = false;
-  /* this code from gedit/src/gedit-view.c.
-   *    click in listener can place cursor (and autoscroll window) at any random place!
-   *    so we have to explicitly find the correct place and put the cursor there.
-   */
-  {
-    gint x, y;
-    GtkTextIter iter;
-    GtkTextIter start, end;
-    GtkTextView *view = (GtkTextView *)w;
-    gtk_text_view_window_to_buffer_coords(view, GTK_TEXT_WINDOW_TEXT, (gint)(ev->x), (gint)(ev->y), &x, &y);
-    gtk_text_view_get_iter_at_location(view, &iter, x, y);
-    if (!(gtk_text_buffer_get_selection_bounds(gtk_text_view_get_buffer(view), &start, &end) &&          
-	  gtk_text_iter_in_range(&iter, &start, &end)))
-      gtk_text_buffer_place_cursor(gtk_text_view_get_buffer(view), &iter);
-    if (XEN_HOOKED(listener_click_hook))
-      run_hook(listener_click_hook,
-	       XEN_LIST_1(C_TO_XEN_INT((int)gtk_text_iter_get_offset(&iter))),
-	       S_listener_click_hook);
-  }
-  goto_listener();
+  ss->sgx->graph_is_active = false;
   return(false);
 }
 
+
+static gboolean listener_button_release(GtkWidget *w, GdkEventButton *ev, gpointer data)
+{
+  check_parens();
+  return(false);
+}
+
+
 static XEN mouse_enter_listener_hook;
 static XEN mouse_leave_listener_hook;
 static XEN mouse_enter_text_hook;
 static XEN mouse_leave_text_hook;
 
+#if 0
+static bool cursor_blinks(GtkWidget *w)
+{
+  GtkSettings *settings;
+  gboolean blink = false;
+  settings = gtk_widget_get_settings(w);
+  g_object_get(settings, "gtk-cursor-blink", &blink, NULL);
+  return((bool)blink);
+}
+#endif
+
+
+static bool cursor_set_blinks(GtkWidget *w, bool blinks)
+{
+  GtkSettings *settings;
+  settings = gtk_widget_get_settings(w);
+  g_object_set(settings, "gtk-cursor-blink", (gboolean)blinks, NULL);
+  return(blinks);
+}
+
+
 static gboolean listener_focus_callback(GtkWidget *w, GdkEventCrossing *ev, gpointer unknown)
 {
   if (XEN_HOOKED(mouse_enter_listener_hook))
     run_hook(mouse_enter_listener_hook,
 	     XEN_LIST_1(XEN_WRAP_WIDGET(listener_text)),
 	     S_mouse_enter_listener_hook);
+  cursor_set_blinks(w, true);
   return(false);
 }
 
+
 static gboolean listener_unfocus_callback(GtkWidget *w, GdkEventCrossing *ev, gpointer unknown)
 {
   if (XEN_HOOKED(mouse_leave_listener_hook))
     run_hook(mouse_leave_listener_hook,
 	     XEN_LIST_1(XEN_WRAP_WIDGET(listener_text)),
 	     S_mouse_leave_listener_hook);
-  gtk_window_set_focus(GTK_WINDOW(gtk_widget_get_toplevel(w)), NULL);
+  cursor_set_blinks(w, false);
   return(false);
 }
 
+
 static gboolean mouse_enter_text_callback(GtkWidget *w, GdkEventCrossing *ev, gpointer unknown)
 {
-  gtk_widget_modify_base(w, GTK_STATE_NORMAL, ss->sgx->white);
+  widget_modify_base(w, GTK_STATE_NORMAL, ss->sgx->white);
   if (XEN_HOOKED(mouse_enter_text_hook))
     run_hook(mouse_enter_text_hook,
 	     XEN_LIST_1(XEN_WRAP_WIDGET(w)),
 	     S_mouse_enter_text_hook);
+  cursor_set_blinks(w, true);
   return(false);
 }
 
+
 static gboolean mouse_leave_text_callback(GtkWidget *w, GdkEventCrossing *ev, gpointer unknown)
 {
-  gtk_widget_modify_base(w, GTK_STATE_NORMAL, ss->sgx->basic_color);
+  widget_modify_base(w, GTK_STATE_NORMAL, ss->sgx->basic_color);
   if (XEN_HOOKED(mouse_leave_text_hook))
     run_hook(mouse_leave_text_hook,
 	     XEN_LIST_1(XEN_WRAP_WIDGET(w)),
 	     S_mouse_leave_text_hook);
-  gtk_window_set_focus(GTK_WINDOW(gtk_widget_get_toplevel(w)), NULL);
-  /* apparently this widget hangs onto "focus" when the mouse leaves it,
-   *  but that means the cursor keeps blinking, and that causes a noticeable
-   *  amount of useless traffic to the X server, which we want to minimize
-   *  during playing.
-   */
+  cursor_set_blinks(w, false);
   return(false);
 }
 
+
 void connect_mouse_to_text(GtkWidget *text)
 {
   SG_SIGNAL_CONNECT(text, "enter_notify_event", mouse_enter_text_callback, NULL);
   SG_SIGNAL_CONNECT(text, "leave_notify_event", mouse_leave_text_callback, NULL);
 }
 
-GtkWidget *snd_entry_new(GtkWidget *container, bool with_white_background)
+
+GtkWidget *snd_entry_new(GtkWidget *container, snd_entry_bg_t with_white_background)
 {
   GtkWidget *text;
+  GtkSettings *settings;
   text = gtk_entry_new();
   gtk_editable_set_editable(GTK_EDITABLE(text), true);
+  settings = gtk_widget_get_settings(text);
+  g_object_set(settings, "gtk-entry-select-on-focus", false, NULL);
   gtk_box_pack_start(GTK_BOX(container), text, true, true, 2);
   gtk_widget_show(text);
-  if (with_white_background) gtk_widget_modify_bg(text, GTK_STATE_NORMAL, ss->sgx->white);
+  if (with_white_background == WITH_WHITE_BACKGROUND) 
+    {
+      widget_modify_bg(text, GTK_STATE_NORMAL, ss->sgx->white);
+      widget_modify_base(text, GTK_STATE_SELECTED, ss->sgx->white); 
+    }
   connect_mouse_to_text(text);
   return(text);
 }
 
 
-static void make_command_widget(int height)
+static void make_listener_widget(int height)
 {
   if (!listener_text)
     {
       GtkWidget *frame;
+
       frame = gtk_frame_new(NULL);
       gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_ETCHED_IN);
       gtk_widget_show(frame);
+
       if (sound_style(ss) != SOUNDS_IN_SEPARATE_WINDOWS)
-	gtk_paned_add2(GTK_PANED(SOUND_PANE(ss)), frame);
+	gtk_paned_pack2(GTK_PANED(SOUND_PANE(ss)), frame, false, true); /* add2 but resize=false */
       else gtk_container_add(GTK_CONTAINER(MAIN_PANE(ss)), frame);
-      listener_text = make_scrolled_text(frame, true, NULL, NULL);
+
+      listener_text = make_scrolled_text(frame, true, 0, false); 
       gtk_widget_set_name(listener_text, "listener_text");
 
       {
 	/* sigh... activate Emacs key bindings to some extent */
-	/*   these appear to be set in gtk+-2.1.1/gtk/gtkrc.key.emacs */
+	/*   more are handled by the gtkrc mechanism (taken from gtk/gtrc.key.emacs) */
 	GtkBindingSet *set;
 	set = gtk_binding_set_by_class(GTK_TEXT_VIEW_GET_CLASS(GTK_TEXT_VIEW(listener_text)));
 
@@ -807,12 +892,14 @@ static void make_command_widget(int height)
 				     G_TYPE_ENUM, GTK_DELETE_WORD_ENDS,
 				     G_TYPE_INT, 1);
 
-	/* C-k delete to end of line */
+	/* C-k delete to end of line -- see explicit handling above */
 	gtk_binding_entry_remove(set, GDK_k, GDK_CONTROL_MASK);
+#if (!(HAVE_GTK_TEXT_BUFFER_SELECT_RANGE && HAVE_GTK_WIDGET_GET_CLIPBOARD))
 	gtk_binding_entry_add_signal(set, GDK_k, GDK_CONTROL_MASK,
 				     "delete_from_cursor", 2,
 				     G_TYPE_ENUM, GTK_DELETE_PARAGRAPH_ENDS,
 				     G_TYPE_INT, 1);
+#endif
 
 	/* M-delete delete to start of line */
 	gtk_binding_entry_remove(set, GDK_Delete, GDK_MOD1_MASK);
@@ -835,48 +922,84 @@ static void make_command_widget(int height)
       SG_SIGNAL_CONNECT(listener_text, "key_press_event", listener_key_press, NULL);
       SG_SIGNAL_CONNECT(listener_text, "key_release_event", listener_key_release, NULL);
       SG_SIGNAL_CONNECT(listener_text, "button_press_event", listener_button_press, NULL);
+      SG_SIGNAL_CONNECT(listener_text, "button_release_event", listener_button_release, NULL);
       SG_SIGNAL_CONNECT(listener_text, "enter_notify_event", listener_focus_callback, NULL);
       SG_SIGNAL_CONNECT(listener_text, "leave_notify_event", listener_unfocus_callback, NULL);
       ss->sgx->listener_pane = listener_text;
-      sg_text_insert(listener_text, listener_prompt(ss));
+
+      if (!prompt_not_editable) 
+	prompt_not_editable = gtk_text_buffer_create_tag(LISTENER_BUFFER, "prompt_not_editable", 
+							 "editable", false, 
+							 "weight", PANGO_WEIGHT_BOLD,
+							 NULL);
+      {
+	GtkTextIter pos;
+	gtk_text_buffer_get_end_iter(LISTENER_BUFFER, &pos);
+	gtk_text_buffer_insert_with_tags(LISTENER_BUFFER, &pos, listener_prompt(ss), ss->listener_prompt_length, prompt_not_editable, NULL);
+      }
+
+      gtk_widget_show(listener_text);
+
+      /* an attempt to add the completions list here as the end of an hbox failed */
+
     }
 }
 
+
 void goto_listener(void) 
 {
   goto_window(listener_text);
 }
 
-void color_listener(GdkColor *pix)
+
+void color_listener(color_info *pix)
 {
-  (ss->sgx)->listener_color = pix;
+  ss->sgx->listener_color = pix;
   if (listener_text) 
-    gtk_widget_modify_base(listener_text, GTK_STATE_NORMAL, (ss->sgx)->listener_color);
+    widget_modify_base(listener_text, GTK_STATE_NORMAL, ss->sgx->listener_color);
 }
 
-void color_listener_text(GdkColor *pix)
+
+void color_listener_text(color_info *pix)
 {
-  (ss->sgx)->listener_text_color = pix;
+  ss->sgx->listener_text_color = pix;
   if (listener_text) 
-    gtk_widget_modify_text(listener_text, GTK_STATE_NORMAL, (ss->sgx)->listener_text_color);
+#if USE_CAIRO
+    gtk_widget_modify_text(listener_text, GTK_STATE_NORMAL, rgb_to_gdk_color(ss->sgx->listener_text_color));
+#else
+    gtk_widget_modify_text(listener_text, GTK_STATE_NORMAL, ss->sgx->listener_text_color);
+#endif
 }
 
+
 void handle_listener(bool open)
 {
-  if (open)
+  if ((open) && (!listener_text))
+    make_listener_widget(100);
+
+  if ((SOUND_PANE(ss)) && /* might be run -separate with no sound open */
+      (sound_style(ss) != SOUNDS_IN_SEPARATE_WINDOWS))
     {
-      if (!listener_text)
-	make_command_widget(100);
-      else gtk_widget_show(listener_text);
-      set_view_listener_label(_("Hide listener"));
-    }
-  else
-    {
-      set_view_listener_label(_("Show listener"));
-      gtk_widget_hide(listener_text);
+      int hgt;
+      hgt = widget_height(SOUND_PANE(ss));
+      if (open)
+	{
+	  if (hgt > 100) /* we can get here before the sound window has opened, but with one pending.
+			  *   the position is in terms of current size, which is useless in this case.
+			  */
+	    gtk_paned_set_position(GTK_PANED(SOUND_PANE(ss)), (gint)(hgt * .75));
+	}
+      else gtk_paned_set_position(GTK_PANED(SOUND_PANE(ss)), hgt);
     }
 }
 
+
+bool listener_exists(void)
+{
+  return((bool)listener_text);
+}
+
+
 int listener_height(void) 
 {
   if ((listener_text) && (GTK_WIDGET_VISIBLE(listener_text)))
@@ -884,6 +1007,7 @@ int listener_height(void)
   else return(0);
 }
 
+
 int listener_width(void) 
 {
   if ((listener_text) && (GTK_WIDGET_VISIBLE(listener_text)))
@@ -891,9 +1015,10 @@ int listener_width(void)
   else return(0);
 }
 
-static XEN g_listener_selected_text(void)
+
+static XEN g_listener_selection(void)
 {
-  #define H_listener_selection "(" S_listener_selection "): currently selected text in listener or #f"
+  #define H_listener_selection "(" S_listener_selection "): currently selected text in listener or " PROC_FALSE
   XEN res = XEN_FALSE;
   if (listener_text)
     {
@@ -912,26 +1037,30 @@ static XEN g_listener_selected_text(void)
   return(res);
 }
 
+
 void set_listener_text_font(void)
 {
   if (listener_text)
     gtk_widget_modify_font(GTK_WIDGET(listener_text), LISTENER_FONT(ss));
 }
 
+
 static XEN g_reset_listener_cursor(void)
 {
   #define H_reset_listener_cursor "(" S_reset_listener_cursor "): reset listener cursor to the default pointer"
   if (listener_text)
-    gdk_window_set_cursor(listener_text->window, (ss->sgx)->arrow_cursor);
+    gdk_window_set_cursor(WIDGET_TO_WINDOW(listener_text), ss->sgx->arrow_cursor);
   return(XEN_FALSE);
 }
 
+
 void clear_listener(void)
 {
   if (listener_text)
     sg_text_delete(listener_text, 1, sg_cursor_position(listener_text));
 }
 
+
 static XEN g_goto_listener_end(void)
 {
   #define H_goto_listener_end "(" S_goto_listener_end "): move cursor and scroll to bottom of listener pane"
@@ -945,60 +1074,72 @@ static XEN g_goto_listener_end(void)
   return(XEN_FALSE);
 }
 
+
 #ifdef XEN_ARGIFY_1
-XEN_NARGIFY_0(g_listener_selected_text_w, g_listener_selected_text)
+XEN_NARGIFY_0(g_listener_selection_w, g_listener_selection)
 XEN_NARGIFY_0(g_reset_listener_cursor_w, g_reset_listener_cursor)
 XEN_NARGIFY_0(g_goto_listener_end_w, g_goto_listener_end)
 #else
-#define g_listener_selected_text_w g_listener_selected_text
+#define g_listener_selection_w g_listener_selection
 #define g_reset_listener_cursor_w g_reset_listener_cursor
 #define g_goto_listener_end_w g_goto_listener_end
 #endif
 
 void g_init_gxlistener(void)
 {
-#if HAVE_GUILE
+#if HAVE_SCHEME
   #define H_mouse_enter_listener_hook S_mouse_enter_listener_hook " (listener): called when the mouse \
 enters the lisp listener pane:\n\
   (add-hook! " S_mouse_enter_listener_hook "\n\
     (lambda (widget)\n\
       (" S_focus_widget " widget)))"
-#else
+#endif
+#if HAVE_RUBY
   #define H_mouse_enter_listener_hook S_mouse_enter_listener_hook " (listener): called when the mouse \
 enters the lisp listener pane:\n\
   $mouse_enter_listener_hook.add-hook!(\"enter\") do |widget|\n\
     focus_widget(widget)\n\
   end"
+#endif
+#if HAVE_FORTH
+  #define H_mouse_enter_listener_hook S_mouse_enter_listener_hook " (listener): called when the mouse \
+enters the lisp listener pane:\n\
+" S_mouse_enter_listener_hook " lambda: <{ wid }> wid " S_focus_widget " ; add-hook!"
 #endif
 
   #define H_mouse_leave_listener_hook S_mouse_leave_listener_hook " (listener): called when the mouse \
 leaves the lisp listener pane"
 
-  XEN_DEFINE_HOOK(mouse_enter_listener_hook, S_mouse_enter_listener_hook, 1, H_mouse_enter_listener_hook);    /* arg = listener_text widget */
-  XEN_DEFINE_HOOK(mouse_leave_listener_hook, S_mouse_leave_listener_hook, 1, H_mouse_leave_listener_hook);    /* arg = listener_text widget */
+  mouse_enter_listener_hook = XEN_DEFINE_HOOK(S_mouse_enter_listener_hook, 1, H_mouse_enter_listener_hook);    /* arg = listener_text widget */
+  mouse_leave_listener_hook = XEN_DEFINE_HOOK(S_mouse_leave_listener_hook, 1, H_mouse_leave_listener_hook);    /* arg = listener_text widget */
 
-#if HAVE_GUILE
+#if HAVE_SCHEME
   #define H_mouse_enter_text_hook S_mouse_enter_text_hook " (widget): called when the mouse enters a text widget:\n\
 (add-hook! " S_mouse_enter_text_hook "\n\
   (lambda (w)\n\
     (" S_focus_widget " w)))"
-#else
+#endif
+#if HAVE_RUBY
   #define H_mouse_enter_text_hook S_mouse_enter_text_hook " (widget): called when the mouse enters a text widget:\n\
 $mouse_enter_text_hook.add_hook!(\"enter\") do |w|\n\
     focus_widget(w)\n\
   end"
+#endif
+#if HAVE_FORTH
+  #define H_mouse_enter_text_hook S_mouse_enter_text_hook " (widget): called when the mouse enters a text widget:\n\
+" S_mouse_enter_text_hook " lambda: <{ wid }> wid " S_focus_widget " ; add-hook!"
 #endif
 
   #define H_mouse_leave_text_hook S_mouse_leave_text_hook " (widget): called when the mouse leaves a text widget"
 
-  XEN_DEFINE_HOOK(mouse_enter_text_hook, S_mouse_enter_text_hook, 1, H_mouse_enter_text_hook);    /* arg = text widget */
-  XEN_DEFINE_HOOK(mouse_leave_text_hook, S_mouse_leave_text_hook, 1, H_mouse_leave_text_hook);    /* arg = text widget */
+  mouse_enter_text_hook = XEN_DEFINE_HOOK(S_mouse_enter_text_hook, 1, H_mouse_enter_text_hook);    /* arg = text widget */
+  mouse_leave_text_hook = XEN_DEFINE_HOOK(S_mouse_leave_text_hook, 1, H_mouse_leave_text_hook);    /* arg = text widget */
 
-  XEN_DEFINE_PROCEDURE(S_listener_selection, g_listener_selected_text_w, 0, 0, 0, H_listener_selection);
+  XEN_DEFINE_PROCEDURE(S_listener_selection, g_listener_selection_w,       0, 0, 0, H_listener_selection);
   XEN_DEFINE_PROCEDURE(S_reset_listener_cursor, g_reset_listener_cursor_w, 0, 0, 0, H_reset_listener_cursor);
-  XEN_DEFINE_PROCEDURE(S_goto_listener_end, g_goto_listener_end_w, 0, 0, 0, H_goto_listener_end);
+  XEN_DEFINE_PROCEDURE(S_goto_listener_end, g_goto_listener_end_w,         0, 0, 0, H_goto_listener_end);
 
   #define H_listener_click_hook S_listener_click_hook " (pos): called when listener clicked; pos is text pos of click in listener"
-  XEN_DEFINE_HOOK(listener_click_hook,    S_listener_click_hook, 1,    H_listener_click_hook);    /* arg = pos */
+  listener_click_hook = XEN_DEFINE_HOOK(S_listener_click_hook, 1,    H_listener_click_hook);    /* arg = pos */
 }
 
