@@ -2,14 +2,34 @@
 
 /* -------- region browser -------- */
 
+typedef struct {
+  GtkWidget *rw, *nm, *pl;
+  int pos;
+} regrow;
+
 static GtkWidget *region_dialog = NULL, *region_list, *region_grf;
 static regrow **region_rows = NULL;
 static int region_rows_size = 0;
 static snd_info *rsp = NULL;
 static int current_region = -1;
 static GtkWidget *srate_text, *length_text, *chans_text, *maxamp_text;
+static GtkWidget *save_as_button = NULL, *insert_button = NULL, *mix_button = NULL;
 static regrow *region_row(int n);
 
+
+static void set_current_region(int rg)
+{
+  bool reg_ok = false;
+  current_region = rg;
+  reflect_region_in_save_as_dialog();
+  if (rg >= 0)
+    reg_ok = region_ok(region_list_position_to_id(rg));
+  if (save_as_button) gtk_widget_set_sensitive(save_as_button, reg_ok);
+  if (mix_button) gtk_widget_set_sensitive(mix_button, reg_ok);
+  if (insert_button) gtk_widget_set_sensitive(insert_button, reg_ok);
+}
+
+
 void reflect_regions_in_region_browser(void)
 {
   if (rsp)
@@ -18,10 +38,11 @@ void reflect_regions_in_region_browser(void)
       rsp->active = true;
       if (rsp->chans)
 	for (i = 0; i < rsp->nchans; i++)
-	  rsp->chans[i]->active = true;
+	  rsp->chans[i]->active = CHANNEL_HAS_AXES;
     }
 }
 
+
 void reflect_no_regions_in_region_browser(void)
 {
   if (rsp)
@@ -30,10 +51,11 @@ void reflect_no_regions_in_region_browser(void)
       rsp->active = false;
       if (rsp->chans)
 	for (i = 0; i < rsp->nchans; i++)
-	  rsp->chans[i]->active = false;
+	  rsp->chans[i]->active = CHANNEL_INACTIVE;
     }
 }
 
+
 static void region_update_graph(chan_info *cp)
 {
   if (current_region == -1) return;
@@ -43,6 +65,7 @@ static void region_update_graph(chan_info *cp)
   rsp->nchans = 1;
 }
 
+
 void reflect_region_graph_style(void)
 {
   if (current_region == -1) return;
@@ -58,37 +81,40 @@ void reflect_region_graph_style(void)
     }
 }
 
+
 static void unhighlight_region(void)
 {
   if (current_region != -1)
     {
       regrow *oldr;
       oldr = region_row(current_region);
-      gtk_widget_modify_bg(oldr->nm, GTK_STATE_NORMAL, ss->sgx->basic_color);
-      gtk_widget_modify_base(oldr->nm, GTK_STATE_NORMAL, ss->sgx->basic_color);
-      gtk_widget_modify_bg(oldr->rw, GTK_STATE_NORMAL, ss->sgx->basic_color);
-      gtk_widget_modify_base(oldr->rw, GTK_STATE_NORMAL, ss->sgx->basic_color);
+      widget_modify_bg(oldr->nm, GTK_STATE_NORMAL, ss->sgx->highlight_color);
+      widget_modify_base(oldr->nm, GTK_STATE_NORMAL, ss->sgx->highlight_color);
+      widget_modify_bg(oldr->rw, GTK_STATE_NORMAL, ss->sgx->highlight_color);
+      widget_modify_base(oldr->rw, GTK_STATE_NORMAL, ss->sgx->highlight_color);
     }
 }
 
+
 static void highlight_region(void)
 {
   if (current_region != -1)
     {
       regrow *oldr;
       oldr = region_row(current_region);
-      gtk_widget_modify_bg(oldr->nm, GTK_STATE_NORMAL, ss->sgx->zoom_color);
-      gtk_widget_modify_base(oldr->nm, GTK_STATE_NORMAL, ss->sgx->zoom_color);
-      gtk_widget_modify_bg(oldr->rw, GTK_STATE_NORMAL, ss->sgx->zoom_color);
-      gtk_widget_modify_base(oldr->rw, GTK_STATE_NORMAL, ss->sgx->zoom_color);
+      widget_modify_bg(oldr->nm, GTK_STATE_NORMAL, ss->sgx->zoom_color);
+      widget_modify_base(oldr->nm, GTK_STATE_NORMAL, ss->sgx->zoom_color);
+      widget_modify_bg(oldr->rw, GTK_STATE_NORMAL, ss->sgx->zoom_color);
+      widget_modify_base(oldr->rw, GTK_STATE_NORMAL, ss->sgx->zoom_color);
     }
 }
 
+
 static void make_region_labels(file_info *hdr)
 {
   char *str;
   if (hdr == NULL) return;
-  str = (char *)CALLOC(PRINT_BUFFER_SIZE, sizeof(char));
+  str = (char *)calloc(PRINT_BUFFER_SIZE, sizeof(char));
   mus_snprintf(str, PRINT_BUFFER_SIZE, _("srate: %d"), hdr->srate);
   set_label(srate_text, str);
   mus_snprintf(str, PRINT_BUFFER_SIZE, _("chans: %d"), hdr->chans);
@@ -97,9 +123,10 @@ static void make_region_labels(file_info *hdr)
   set_label(length_text, str);
   mus_snprintf(str, PRINT_BUFFER_SIZE, _("maxamp: %.3f"), region_maxamp(region_list_position_to_id(current_region)));
   set_label(maxamp_text, str);
-  FREE(str);
+  free(str);
 }
 
+
 void update_region_browser(bool grf_too)
 {
   int i, len;
@@ -124,7 +151,7 @@ void update_region_browser(bool grf_too)
     {
       chan_info *cp;
       unhighlight_region();
-      current_region = 0;
+      set_current_region(0);
       highlight_region();
       goto_window(region_rows[0]->nm);
       cp = rsp->chans[0];
@@ -141,35 +168,41 @@ void update_region_browser(bool grf_too)
     }
 }
 
+
 static gint region_browser_delete_callback(GtkWidget *w, GdkEvent *event, gpointer context)
 {
   gtk_widget_hide(region_dialog);
   return(true);
 }
 
+
 static void region_ok_callback(GtkWidget *w, gpointer context)
 {
   gtk_widget_hide(region_dialog);
 }
 
+
 bool region_browser_is_active(void)
 {
   return((region_dialog) && 
 	 (GTK_WIDGET_VISIBLE(region_dialog)));
 }
 
+
 static gboolean region_resize_callback(GtkWidget *w, GdkEventConfigure *ev, gpointer data)
 {
   region_update_graph((chan_info *)data);
   return(false);
 }
 
+
 static gboolean region_expose_callback(GtkWidget *w, GdkEventExpose *ev, gpointer data)
 {
   region_update_graph((chan_info *)data);
   return(false);
 }
 
+
 void delete_region_and_update_browser(int pos)
 {
   int act;
@@ -180,27 +213,52 @@ void delete_region_and_update_browser(int pos)
     {
       if (act != NO_REGIONS)
 	{
-	  current_region = 0;
+	  set_current_region(0);
 	  highlight_region();
 	  goto_window(region_rows[0]->nm);
 	}
-      else 
-	current_region = -1;
+      else set_current_region(-1);
       update_region_browser(true);
     }
 }
 
-static void region_delete_callback(GtkWidget *w, gpointer context)
+
+static void region_unlist_callback(GtkWidget *w, gpointer context)
 {
   if (current_region != -1)
     delete_region_and_update_browser(current_region);
 }
 
+
+static void region_insert_callback(GtkWidget *w, gpointer context)
+{
+  if ((current_region != -1) &&
+      (selected_channel()))
+    paste_region(region_list_position_to_id(current_region), selected_channel());
+}
+
+
+static void region_mix_callback(GtkWidget *w, gpointer context)
+{
+  if ((current_region != -1) &&
+      (selected_channel()))
+    add_region(region_list_position_to_id(current_region), selected_channel());
+}
+
+
+static void region_save_callback(GtkWidget *w, gpointer context)
+{
+  if (current_region != -1)
+    make_region_save_as_dialog(true);
+}
+
+
 static void region_help_callback(GtkWidget *w, gpointer context)
 {
   region_dialog_help();
 }
 
+
 static gboolean region_up_arrow_callback(GtkWidget *w, GdkEventButton *ev, gpointer data)
 {
   chan_info *cp;
@@ -217,6 +275,7 @@ static gboolean region_up_arrow_callback(GtkWidget *w, GdkEventButton *ev, gpoin
   return(false);
 }
 
+
 static gboolean region_down_arrow_callback(GtkWidget *w, GdkEventButton *ev, gpointer data)
 {
   chan_info *cp;
@@ -233,13 +292,14 @@ static gboolean region_down_arrow_callback(GtkWidget *w, GdkEventButton *ev, gpo
   return(false);
 }
 
+
 static void region_focus_callback(GtkWidget *w, gpointer context) /* button clicked callback */
 {
   chan_info *cp;
   regrow *r = (regrow *)context;
   unhighlight_region();
   if (region_list_position_to_id(r->pos) == INVALID_REGION) return; /* needed by auto-tester */
-  current_region = r->pos;
+  set_current_region(r->pos);
   cp = rsp->chans[0];
   cp->sound = rsp;
   cp->chan  = 0;
@@ -251,6 +311,7 @@ static void region_focus_callback(GtkWidget *w, gpointer context) /* button clic
   region_update_graph(cp);
 }
 
+
 void reflect_play_region_stop(int n)
 {
   if (region_rows)
@@ -261,84 +322,213 @@ void reflect_play_region_stop(int n)
     }
 }
 
+
 static void region_play_callback(GtkWidget *w, gpointer context)
 {
   regrow *r = (regrow *)context;
-  if (GTK_TOGGLE_BUTTON(r->pl)->active)
+  if (TOGGLE_BUTTON_ACTIVE(r->pl))
     play_region(region_list_position_to_id(r->pos), IN_BACKGROUND);
   else stop_playing_region(region_list_position_to_id(r->pos), PLAY_BUTTON_UNSET);
 }
 
+
 static void region_print_callback(GtkWidget *w, gpointer context)
 {
   if (current_region != -1)
     region_print(eps_file(ss), "region", rsp->chans[0]);
 }
 
+
 static void region_edit_callback(GtkWidget *w, gpointer context)
 {
   if (current_region != -1) 
     region_edit(current_region);
 }
 
+
 static gboolean region_labels_mouse_enter(GtkWidget *w, GdkEventCrossing *ev, gpointer data)
 {
   g_signal_stop_emission(GTK_OBJECT(w), g_signal_lookup("enter_notify_event", G_OBJECT_TYPE(GTK_OBJECT(w))), 0);
   return(false);
 }
 
-static GtkWidget *print_button, *edit_button;
-static GtkWidget *dismiss_button, *help_button, *delete_button;
+
+static void reflect_file_in_region_browser(ss_watcher_reason_t reason, void *ignore)
+{
+  if (region_dialog)
+    {
+      bool file_p;
+      file_p = (bool)(any_selected_sound());
+      set_sensitive(mix_button, file_p);
+      set_sensitive(insert_button, file_p);
+    }
+}
+
+
+char *regrow_get_label(void *ur)
+{
+  regrow *r = (regrow *)ur;
+  return((char *)gtk_label_get_text(GTK_LABEL(BIN_CHILD(r->nm))));
+}
+
+
+int regrow_get_pos(void *ur)
+{
+  regrow *r = (regrow *)ur;
+  return(r->pos);
+}
+
+
+static gboolean regrow_mouse_enter_label(GtkWidget *w, GdkEventCrossing *ev, gpointer gp)
+{
+  mouse_enter_label((void *)gp, REGION_VIEWER);
+  return(false);
+}
+
+
+static gboolean regrow_mouse_leave_label(GtkWidget *w, GdkEventCrossing *ev, gpointer gp)
+{
+  mouse_leave_label((void *)gp, REGION_VIEWER);
+  return(false);
+}
+
+
+static regrow *make_regrow(GtkWidget *ww, GCallback play_callback, GCallback name_callback)
+{
+  regrow *r;
+  r = (regrow *)calloc(1, sizeof(regrow));
+
+  /* assume "ww" is a vbox widget in this case */
+  r->rw = gtk_hbox_new(false, 0);
+  gtk_box_pack_start(GTK_BOX(ww), r->rw, false, false, 0);
+  widget_modify_bg(r->rw, GTK_STATE_NORMAL, ss->sgx->highlight_color);
+  widget_modify_base(r->rw, GTK_STATE_NORMAL, ss->sgx->highlight_color);
+  gtk_widget_show(r->rw);
+
+  r->pl = gtk_check_button_new();
+  gtk_box_pack_start(GTK_BOX(r->rw), r->pl, false, false, 2);
+  SG_SIGNAL_CONNECT(r->pl, "toggled", play_callback, r);
+  gtk_widget_show(r->pl);
+
+  r->nm = gtk_button_new_with_label("");
+  sg_left_justify_button(r->nm);
+  gtk_box_pack_start(GTK_BOX(r->rw), r->nm, true, true, 2);
+  SG_SIGNAL_CONNECT(r->nm, "clicked", name_callback, r);
+  SG_SIGNAL_CONNECT(r->nm, "enter_notify_event", regrow_mouse_enter_label, r);
+  SG_SIGNAL_CONNECT(r->nm, "leave_notify_event", regrow_mouse_leave_label, r);
+  set_user_data(G_OBJECT(r->nm), (gpointer)r);
+  widget_modify_bg(r->nm, GTK_STATE_NORMAL, ss->sgx->highlight_color);
+  widget_modify_base(r->nm, GTK_STATE_NORMAL, ss->sgx->highlight_color);
+  gtk_widget_show(r->nm);
+
+  return(r);
+}
+
+
+static GtkWidget *print_button, *edit_button, *unlist_button;
+static GtkWidget *dismiss_button, *help_button;
 
 static void make_region_dialog(void)
 {
   int i, id;
   regrow *r;
   chan_info *cp;
-  ww_info *wwl;
   GtkWidget *infobox, *labels, *labbox;
+  GtkWidget *sep1, *cww, *toppane, *tophbox, *plw, *formw;
+
   region_dialog = snd_gtk_dialog_new();
   SG_SIGNAL_CONNECT(region_dialog, "delete_event", region_browser_delete_callback, NULL);
   gtk_window_set_title(GTK_WINDOW(region_dialog), _("Regions"));
   sg_make_resizable(region_dialog);
   gtk_container_set_border_width(GTK_CONTAINER(region_dialog), 10);
-  gtk_window_resize(GTK_WINDOW(region_dialog), 400, 400);
+  gtk_window_resize(GTK_WINDOW(region_dialog), 400, 500);
   gtk_widget_realize(region_dialog);
 
-  help_button = gtk_button_new_with_label(_("Help"));
+  help_button = gtk_button_new_from_stock(GTK_STOCK_HELP);
   gtk_widget_set_name(help_button, "help_button");
-  dismiss_button = gtk_button_new_with_label(_("Dismiss"));
+
+  dismiss_button = gtk_button_new_from_stock(GTK_STOCK_QUIT);
   gtk_widget_set_name(dismiss_button, "quit_button");
-  delete_button = gtk_button_new_with_label(_("Delete"));
-  gtk_widget_set_name(delete_button, "doit_button");
+  set_stock_button_label(dismiss_button, _("Go Away"));
 
-  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(region_dialog)->action_area), dismiss_button, true, true, 4);
-  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(region_dialog)->action_area), delete_button, true, true, 4);
-  gtk_box_pack_end(GTK_BOX(GTK_DIALOG(region_dialog)->action_area), help_button, true, true, 4);
+  insert_button = sg_button_new_from_stock_with_label(_("Insert"), GTK_STOCK_PASTE);
+  gtk_widget_set_name(insert_button, "doit_button");
 
-  SG_SIGNAL_CONNECT(delete_button, "clicked", region_delete_callback, NULL);
+  mix_button = sg_button_new_from_stock_with_label(_("Mix"), GTK_STOCK_ADD);
+  gtk_widget_set_name(mix_button, "doit_again_button");
+
+  save_as_button = gtk_button_new_from_stock(GTK_STOCK_SAVE_AS);
+  gtk_widget_set_name(save_as_button, "reset_button");
+
+  gtk_box_pack_start(GTK_BOX(DIALOG_ACTION_AREA(region_dialog)), dismiss_button, true, true, 4);
+  gtk_box_pack_start(GTK_BOX(DIALOG_ACTION_AREA(region_dialog)), insert_button, true, true, 4);
+  gtk_box_pack_start(GTK_BOX(DIALOG_ACTION_AREA(region_dialog)), mix_button, true, true, 4);
+  gtk_box_pack_start(GTK_BOX(DIALOG_ACTION_AREA(region_dialog)), save_as_button, true, true, 4);
+  gtk_box_pack_end(GTK_BOX(DIALOG_ACTION_AREA(region_dialog)), help_button, true, true, 4);
+
+  SG_SIGNAL_CONNECT(insert_button, "clicked", region_insert_callback, NULL);
+  SG_SIGNAL_CONNECT(mix_button, "clicked", region_mix_callback, NULL);
   SG_SIGNAL_CONNECT(help_button, "clicked", region_help_callback, NULL);
   SG_SIGNAL_CONNECT(dismiss_button, "clicked", region_ok_callback, NULL);
+  SG_SIGNAL_CONNECT(save_as_button, "clicked", region_save_callback, NULL);
 
-  gtk_widget_show(delete_button);
+  gtk_widget_show(insert_button);
+  gtk_widget_show(mix_button);
   gtk_widget_show(help_button);
   gtk_widget_show(dismiss_button);
+  gtk_widget_show(save_as_button);
+
+  region_grf = gtk_vpaned_new();
+  gtk_box_pack_start(GTK_BOX(DIALOG_CONTENT_AREA(region_dialog)), region_grf, true, true, 0);
+  gtk_widget_show(region_grf);
+
+  toppane = gtk_hbox_new(false, 0);
+  gtk_paned_add1(GTK_PANED(region_grf), toppane);
+  gtk_widget_show(toppane);
+
+  formw = gtk_vbox_new(false, 0);
+  gtk_box_pack_start(GTK_BOX(toppane), formw, true, true, 4);
+  gtk_widget_show(formw);
+
+  sep1 = gtk_vseparator_new(); /* not hsep -- damned thing insists on drawing a line */
+  gtk_box_pack_start(GTK_BOX(formw), sep1, false, false, 2);
+  gtk_widget_show(sep1);
+
+  tophbox = gtk_hbox_new(false, 0);
+  gtk_box_pack_start(GTK_BOX(formw), tophbox, false, false, 4);
+  gtk_widget_show(tophbox);
+
+  plw = gtk_label_new(_("play")); 
+  gtk_box_pack_start(GTK_BOX(tophbox), plw, false, false, 2);
+  gtk_widget_show(plw);
+
+  sep1 = gtk_vseparator_new();
+  gtk_box_pack_start(GTK_BOX(formw), sep1, false, false, 2);
+  gtk_widget_show(sep1);
+
+
+  region_list = gtk_vbox_new(false, 0);
+
+  cww = gtk_scrolled_window_new(NULL, NULL);
+  gtk_box_pack_start(GTK_BOX(formw), cww, true, true, 0);
+  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(cww), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(cww), region_list);
+
+  gtk_widget_show(region_list);
+  gtk_widget_show(cww);
 
-  wwl = make_title_row(GTK_DIALOG(region_dialog)->vbox, _("play"), NULL, DONT_PAD_TITLE, WITHOUT_SORT_BUTTON, WITH_PANED_WINDOW);
-  region_list = wwl->list;
 
   infobox = gtk_vbox_new(false, 0);
-  gtk_box_pack_start(GTK_BOX(wwl->toppane), infobox, false, false, 2);
+  gtk_box_pack_start(GTK_BOX(toppane), infobox, false, false, 2);
   gtk_widget_show(infobox);
   
-  region_rows = (regrow **)CALLOC(max_regions(ss), sizeof(regrow *));
+  region_rows = (regrow **)calloc(max_regions(ss), sizeof(regrow *));
   region_rows_size = max_regions(ss);
   for (i = 0; i < max_regions(ss); i++)
     {
       r = make_regrow(region_list, (void (*)())region_play_callback, (void (*)())region_focus_callback);
       region_rows[i] = r;
       r->pos = i;
-      r->parent = REGION_VIEWER;
     }
 
   update_region_browser(false);
@@ -352,13 +542,13 @@ static void make_region_dialog(void)
   labels = gtk_button_new();
   gtk_box_pack_start(GTK_BOX(infobox), labels, true, true, 2);
   gtk_widget_show(labels);
-  gtk_widget_modify_bg(labels, GTK_STATE_NORMAL, ss->sgx->highlight_color);
+  widget_modify_bg(labels, GTK_STATE_NORMAL, ss->sgx->highlight_color);
   SG_SIGNAL_CONNECT(labels, "enter_notify_event", region_labels_mouse_enter, NULL);
 
   labbox = gtk_vbox_new(true, 0);
   gtk_container_add(GTK_CONTAINER(labels), labbox);
   gtk_widget_show(labbox);
-  gtk_widget_modify_bg(labbox, GTK_STATE_NORMAL, ss->sgx->highlight_color);
+  widget_modify_bg(labbox, GTK_STATE_NORMAL, ss->sgx->highlight_color);
 
   srate_text = gtk_label_new(_("srate:"));
   sg_left_justify_label(srate_text);
@@ -384,26 +574,35 @@ static void make_region_dialog(void)
   SG_SIGNAL_CONNECT(edit_button, "clicked", region_edit_callback, NULL);
   gtk_box_pack_start(GTK_BOX(infobox), edit_button, true, true, 2);
   gtk_widget_show(edit_button);
-  gtk_widget_modify_bg(edit_button, GTK_STATE_NORMAL, ss->sgx->lighter_blue);
-  gtk_widget_modify_bg(edit_button, GTK_STATE_ACTIVE, ss->sgx->red);
+  widget_modify_bg(edit_button, GTK_STATE_NORMAL, ss->sgx->lighter_blue);
 
   print_button = gtk_button_new_with_label(_("print"));
   SG_SIGNAL_CONNECT(print_button, "clicked", region_print_callback, NULL);
   gtk_box_pack_start(GTK_BOX(infobox), print_button, true, true, 2);
   gtk_widget_show(print_button);
-  gtk_widget_modify_bg(print_button, GTK_STATE_NORMAL, ss->sgx->lighter_blue);
-  gtk_widget_modify_bg(print_button, GTK_STATE_ACTIVE, ss->sgx->red);
+  widget_modify_bg(print_button, GTK_STATE_NORMAL, ss->sgx->lighter_blue);
+
+  unlist_button = gtk_button_new_with_label(_("unlist"));
+  SG_SIGNAL_CONNECT(unlist_button, "clicked", region_unlist_callback, NULL);
+  gtk_box_pack_start(GTK_BOX(infobox), unlist_button, true, true, 2);
+  gtk_widget_show(unlist_button);
+  widget_modify_bg(unlist_button, GTK_STATE_NORMAL, ss->sgx->lighter_blue);
+
+  /* these no longer work right
+  widget_modify_bg(edit_button, GTK_STATE_ACTIVE, ss->sgx->red);
+  widget_modify_bg(print_button, GTK_STATE_ACTIVE, ss->sgx->red);
+  widget_modify_bg(unlist_button, GTK_STATE_ACTIVE, ss->sgx->red);
+  */
 
-  region_grf = wwl->panes;
   gtk_widget_show(region_dialog);
 
   id = region_list_position_to_id(0);
   rsp = make_simple_channel_display(region_srate(id), region_len(id), WITH_ARROWS, region_graph_style(ss), region_grf, WITHOUT_EVENTS);
   rsp->inuse = SOUND_REGION;
-  current_region = 0;
+  set_current_region(0);
   cp = rsp->chans[0];
 
-  gtk_paned_set_position(GTK_PANED(region_grf), 150);
+  gtk_paned_set_position(GTK_PANED(region_grf), 220);
   SG_SIGNAL_CONNECT(channel_graph(cp), "expose_event", region_resize_callback, cp);
   SG_SIGNAL_CONNECT(channel_graph(cp), "configure_event", region_expose_callback, cp);
 
@@ -417,11 +616,11 @@ static void make_region_dialog(void)
   make_region_labels(rsp->hdr);
   highlight_region();
   region_update_graph(cp);
-  FREE(wwl); 
-  wwl = NULL;
+  add_ss_watcher(SS_FILE_OPEN_WATCHER, reflect_file_in_region_browser, NULL);
   set_dialog_widget(REGION_DIALOG, region_dialog);
 }
 
+
 void view_region_callback(GtkWidget *w, gpointer context)
 {
   /* put up scrollable dialog describing/playing/editing the region list */
@@ -431,28 +630,31 @@ void view_region_callback(GtkWidget *w, gpointer context)
     {
       update_region_browser(true);
       raise_dialog(region_dialog);
-      current_region = 0;
+      set_current_region(0);
     }
 }
 
+
 bool region_dialog_is_active(void)
 {
   return((region_dialog != NULL) && 
 	 (GTK_WIDGET_VISIBLE(region_dialog)));
 }
 
+
 void allocate_region_rows(int n)
 {
   if ((region_dialog) && 
       (n > region_rows_size))
     {
       int i;
-      region_rows = (regrow **)REALLOC(region_rows, n * sizeof(regrow *));
+      region_rows = (regrow **)realloc(region_rows, n * sizeof(regrow *));
       for (i = region_rows_size; i < n; i++) region_rows[i] = NULL;
       region_rows_size = n;
     }
 }
 
+
 static regrow *region_row(int n)
 {
   if (n < region_rows_size)
@@ -463,13 +665,19 @@ static regrow *region_row(int n)
 	  r = make_regrow(region_list, (void (*)())region_play_callback, (void (*)())region_focus_callback);
 	  region_rows[n] = r;
 	  r->pos = n;
-	  r->parent = REGION_VIEWER;
 	}
       return(region_rows[n]);
     }
   return(NULL);
 }
 
+
+int region_dialog_region(void)
+{
+  return(region_list_position_to_id(current_region));
+}
+
+
 static XEN g_view_regions_dialog(void) 
 {
   #define H_view_regions_dialog "(" S_view_regions_dialog "): start the region dialog"
@@ -478,12 +686,14 @@ static XEN g_view_regions_dialog(void)
   return(XEN_WRAP_WIDGET(region_dialog));
 }
 
+
 #ifdef XEN_ARGIFY_1
 XEN_NARGIFY_0(g_view_regions_dialog_w, g_view_regions_dialog)
 #else
 #define g_view_regions_dialog_w g_view_regions_dialog
 #endif
 
+
 void g_init_gxregion(void)
 {
   XEN_DEFINE_PROCEDURE(S_view_regions_dialog, g_view_regions_dialog_w, 0, 0, 0,  H_view_regions_dialog);
