@@ -2,6 +2,7 @@
 
 /* create Postscript version of graph */
 
+
 static char *pbuf = NULL;
 static int bbx = 0, bby = 0, bx0 = 0, by0 = 0;
 static int ps_fd;
@@ -10,56 +11,59 @@ static char *nbuf = NULL;
 static int nbuf_ctr = 0;
 #define NBUF_SIZE 8192
 
+
 static void ps_flush(int fd)
 {
   if (nbuf_ctr > 0)
     {
-      write(fd, nbuf, nbuf_ctr);
+      ssize_t bytes;
+      bytes = write(fd, nbuf, nbuf_ctr);
+      if (bytes == 0) fprintf(stderr, "ps_flush write error");
       nbuf_ctr = 0;
+      memset((void *)nbuf, 0, NBUF_SIZE);
     }
 }
 
-static void ps_write(char *buf)
+
+static void ps_write(const char *buf)
 {
   /* sending tiny buffers over the net is a total loss -- grab a bunch at a time */
+  int i, len;
   if (!nbuf)
     {
-      nbuf = (char *)CALLOC(NBUF_SIZE, sizeof(char));
+      nbuf = (char *)calloc(NBUF_SIZE, sizeof(char));
       nbuf_ctr = 0;
     }
-  while (*buf)
+  len = mus_strlen(buf);
+  for (i = 0; i < len; i++)
     {
-      nbuf[nbuf_ctr] = (*buf);
-      buf++;
-      nbuf_ctr++;
+      nbuf[nbuf_ctr++] = buf[i];
       if (nbuf_ctr == NBUF_SIZE) ps_flush(ps_fd);
     }
+  memset((void *)buf, 0, PRINT_BUFFER_SIZE);
 }
 
+
 static char *previous_locale = NULL;
 
-static int start_ps_graph(char *output, char *title) 
+static int start_ps_graph(const char *output, const char *title) 
 { 
-  time_t ts;
   ps_fd = CREAT(output, 0666);
   if (ps_fd == -1) return(-1);
-  if (!pbuf) pbuf = (char *)CALLOC(PRINT_BUFFER_SIZE, sizeof(char));
+  if (!pbuf) pbuf = (char *)calloc(PRINT_BUFFER_SIZE, sizeof(char));
   bbx = 0;
   bby = 0;
 
 #if HAVE_SETLOCALE
-  previous_locale = copy_string(setlocale(LC_NUMERIC, "C")); /* must use decimal point in floats since PostScript assumes that format */
+  previous_locale = mus_strdup(setlocale(LC_NUMERIC, "C")); /* must use decimal point in floats since PostScript assumes that format */
 #endif
 
   mus_snprintf(pbuf, PRINT_BUFFER_SIZE, 
-	       "%%!PS-Adobe-2.0 EPSF-2.0\n%%%%Title: %s\n%%%%Creator: Snd: %s\n%%%%CreationDate: ", 
-	       title, SND_DATE);
+	       "%%!PS-Adobe-2.0 EPSF-2.0\n%%%%Title: %s\n%%%%Creator: Snd: %s\n%%%%CreationDate: %s", 
+	       title, 
+	       SND_DATE,
+	       snd_local_time());
   ps_write(pbuf);
-#if HAVE_STRFTIME
-  time(&ts);
-  strftime(pbuf, PRINT_BUFFER_SIZE, STRFTIME_FORMAT, localtime(&ts));
-  ps_write(pbuf);
-#endif
   mus_snprintf(pbuf, PRINT_BUFFER_SIZE, 
 	       "\n%%%%BoundingBox:(atend)\n%%%%EndComments\n%%%%EndProlog\n%%%%Page: 1 1\n");
   ps_write(pbuf);
@@ -74,6 +78,7 @@ static int start_ps_graph(char *output, char *title)
   return(0);
 }
 
+
 static void ps_graph(chan_info *cp, int x0, int y0)
 {
   cp->printing = PRINTING;
@@ -83,6 +88,7 @@ static void ps_graph(chan_info *cp, int x0, int y0)
   cp->printing = NOT_PRINTING;
 }
 
+
 static void end_ps_graph(void)
 {
   mus_snprintf(pbuf, PRINT_BUFFER_SIZE, 
@@ -99,17 +105,18 @@ static void end_ps_graph(void)
 #if HAVE_SETLOCALE
       setlocale(LC_NUMERIC, previous_locale);
 #endif
-      FREE(previous_locale);
+      free(previous_locale);
       previous_locale = NULL;
     }
   if (nbuf)
     {
-      FREE(nbuf);
+      free(nbuf);
       nbuf = NULL;
       nbuf_ctr = 0;
     }
 }
 
+
 /* the x and y values in the "points" are relative to grf_x/y:
  *
  *  x: ap->x_axis_x0 + (val - ap->x0) * ap->x_scale
@@ -123,41 +130,47 @@ static int reflect_y(axis_info *ap, int y)
   return(ap->height - y);
 }
 
-static Float *xpts = NULL;
-static Float *ypts = NULL;
-static Float *ypts1 = NULL;
+
+static mus_float_t *xpts = NULL;
+static mus_float_t *ypts = NULL;
+static mus_float_t *ypts1 = NULL;
 
 void ps_allocate_grf_points(void)
 {
-  if (!xpts) xpts = (Float *)CALLOC(POINT_BUFFER_SIZE, sizeof(Float));
-  if (!ypts) ypts = (Float *)CALLOC(POINT_BUFFER_SIZE, sizeof(Float));
-  if (!ypts1) ypts1 = (Float *)CALLOC(POINT_BUFFER_SIZE, sizeof(Float));
+  if (!xpts) xpts = (mus_float_t *)calloc(POINT_BUFFER_SIZE, sizeof(mus_float_t));
+  if (!ypts) ypts = (mus_float_t *)calloc(POINT_BUFFER_SIZE, sizeof(mus_float_t));
+  if (!ypts1) ypts1 = (mus_float_t *)calloc(POINT_BUFFER_SIZE, sizeof(mus_float_t));
 }
 
-void ps_set_grf_points(double x, int j, Float ymin, Float ymax) 
+
+void ps_set_grf_points(double x, int j, mus_float_t ymin, mus_float_t ymax) 
 {
   xpts[j] = x;
   ypts[j] = ymin;
   ypts1[j] = ymax;
 }
 
-void ps_set_grf_point(double x, int j, Float y) 
+
+void ps_set_grf_point(double x, int j, mus_float_t y) 
 {
   xpts[j] = x;
   ypts[j] = y;
 }
 
-static Float ps_grf_x(axis_info *ap, Float val)
+
+static mus_float_t ps_grf_x(axis_info *ap, mus_float_t val)
 {
   return(ap->x_axis_x0 + bx0 + (val - ap->x0) * ap->x_scale);
 }
 
-static Float ps_grf_y(axis_info *ap, Float val)
+
+static mus_float_t ps_grf_y(axis_info *ap, mus_float_t val)
 {
   return(by0 + ap->height - (ap->y_axis_y0 + (val - ap->y0) * ap->y_scale));
 }
 
-static void ps_draw_lines(axis_info *ap, int j, Float *xpts, Float *ypts)
+
+static void ps_draw_lines(axis_info *ap, int j, mus_float_t *xpts, mus_float_t *ypts)
 {
   int i;
   mus_snprintf(pbuf, PRINT_BUFFER_SIZE, " %.2f %.2f moveto\n", ps_grf_x(ap, xpts[0]), ps_grf_y(ap, ypts[0]));
@@ -171,10 +184,11 @@ static void ps_draw_lines(axis_info *ap, int j, Float *xpts, Float *ypts)
   ps_write(pbuf);
 }
 
-static void ps_draw_dots(axis_info *ap, int j, Float *xpts, Float *ypts, int dot_size)
+
+static void ps_draw_dots(axis_info *ap, int j, mus_float_t *xpts, mus_float_t *ypts, int dot_size)
 {
   int i;
-  Float arc_size;
+  mus_float_t arc_size;
   arc_size = .5 * dot_size; /* radius here, diameter in X */
   for (i = 0; i < j; i++)
     {
@@ -183,7 +197,8 @@ static void ps_draw_dots(axis_info *ap, int j, Float *xpts, Float *ypts, int dot
     }
 }
 
-static void ps_fill_polygons(axis_info *ap, int j, Float *xpts, Float *ypts, Float y0)
+
+static void ps_fill_polygons(axis_info *ap, int j, mus_float_t *xpts, mus_float_t *ypts, mus_float_t y0)
 {
   int i;
   for (i = 1; i < j; i++)
@@ -201,11 +216,13 @@ static void ps_fill_polygons(axis_info *ap, int j, Float *xpts, Float *ypts, Flo
     }
 }
 
-void ps_draw_grf_points(axis_info *ap, int j, Float y0, graph_style_t graph_style, int dot_size) 
+
+void ps_draw_grf_points(axis_info *ap, int j, mus_float_t y0, graph_style_t graph_style, int dot_size) 
 {
   switch (graph_style)
     {
     case GRAPH_LINES:
+    default:
       ps_draw_lines(ap, j, xpts, ypts);
       break;
     case GRAPH_DOTS:
@@ -240,12 +257,14 @@ void ps_draw_grf_points(axis_info *ap, int j, Float y0, graph_style_t graph_styl
     }
 }
 
+
 void ps_draw_both_grf_points(axis_info *ap, int j, graph_style_t graph_style, int dot_size) 
 {
   int i, size8, size4;
   switch (graph_style)
     {
     case GRAPH_LINES:
+    default:
       ps_draw_lines(ap, j, xpts, ypts);
       ps_draw_lines(ap, j, xpts, ypts1);
       break;
@@ -300,22 +319,24 @@ void ps_draw_both_grf_points(axis_info *ap, int j, graph_style_t graph_style, in
     }
 }
 
+
 static int last_color = -1;
 
-void ps_draw_sono_rectangle(axis_info *ap, int color, Float x, Float y, Float width, Float height)
+void ps_draw_sono_rectangle(axis_info *ap, int color, mus_float_t x, mus_float_t y, mus_float_t width, mus_float_t height)
 {
-  unsigned short r, g, b;
+  rgb_t r, g, b;
   if (last_color != color)
     {
       get_current_color(color_map(ss), color, &r, &g, &b);
       last_color = color;
-      mus_snprintf(pbuf, PRINT_BUFFER_SIZE, " %.2f %.2f %.2f RG\n", (float)r / 65535.0, (float)g / 65535.0, (float)b / 65535.0);
+      mus_snprintf(pbuf, PRINT_BUFFER_SIZE, " %.2f %.2f %.2f RG\n", RGB_TO_FLOAT(r), RGB_TO_FLOAT(g), RGB_TO_FLOAT(b));
       ps_write(pbuf);
     }
   mus_snprintf(pbuf, PRINT_BUFFER_SIZE, " %.1f %.1f %.2f %.2f RF\n", ps_grf_x(ap, x) + 2, ps_grf_y(ap, y), width, height);
   ps_write(pbuf);
 }
 
+
 void ps_reset_color(void)
 {
   mus_snprintf(pbuf, PRINT_BUFFER_SIZE, " 0 setgray\n");
@@ -323,6 +344,8 @@ void ps_reset_color(void)
   last_color = -1;
 }
 
+
+#if USE_MOTIF || USE_GTK
 static void ps_set_color(color_t color)
 {
 #if USE_MOTIF
@@ -335,40 +358,75 @@ static void ps_set_color(color_t color)
   tmp_color.pixel = color;
   XQueryColor(dpy, cmap, &tmp_color);
   mus_snprintf(pbuf, PRINT_BUFFER_SIZE, " %.2f %.2f %.2f RG\n",
-	       (float)tmp_color.red / 65535.0,
-	       (float)tmp_color.green / 65535.0,
-	       (float)tmp_color.blue / 65535.0);
+	       RGB_TO_FLOAT(tmp_color.red),
+	       RGB_TO_FLOAT(tmp_color.green),
+	       RGB_TO_FLOAT(tmp_color.blue));
   ps_write(pbuf);
 #else
   #if USE_GTK
   mus_snprintf(pbuf, PRINT_BUFFER_SIZE, " %.2f %.2f %.2f RG\n", 
-	       (float)color->red / 65535.0, 
-	       (float)color->green / 65535.0, 
-	       (float)color->blue / 65535.0);
+	       RGB_TO_FLOAT(color->red), 
+	       RGB_TO_FLOAT(color->green), 
+	       RGB_TO_FLOAT(color->blue));
   ps_write(pbuf);
   #endif
 #endif
   last_color = -1;
 }
+#endif
+
 
 void ps_bg(axis_info *ap, axis_context *ax)
 {
   /* get background color, fill graph, then set foreground for axis */
-  ps_set_color(get_background_color(ap->cp, ax));
+  chan_info *cp;
+  cp = ap->cp;
+#if USE_MOTIF
+  {
+    XGCValues gv;
+    XGetGCValues(MAIN_DISPLAY(ss), ax->gc, GCBackground, &gv);
+    ps_set_color(gv.background);
+  }
+#else
+#if USE_GTK
+  {
+    state_context *sx;
+    sx = ss->sgx;
+    if (cp->cgx->selected) 
+      ps_set_color(sx->selected_graph_color);
+    else ps_set_color(sx->graph_color);
+  }
+#endif
+#endif
   mus_snprintf(pbuf, PRINT_BUFFER_SIZE, " %d %d %d %d RF\n",
 	       ap->graph_x0 + bx0, ap->y_offset + by0, ap->width, ap->height);
   ps_write(pbuf);
-  ps_fg(ap, ax);
+  ps_fg(cp, ax);
 }
 
-void ps_fg(axis_info *ap, axis_context *ax)
+
+void ps_fg(chan_info *cp, axis_context *ax)
 {
   /* set foreground color for subsequent line drawing */
-  ps_set_color(get_foreground_color(ap->cp, ax));
+#if USE_MOTIF
+  ps_set_color(get_foreground_color(ax));
+#else
+#if USE_GTK
+  {
+    state_context *sx;
+    sx = ss->sgx;
+    if (cp->cgx->selected) 
+      ps_set_color(sx->selected_data_color);
+    else ps_set_color(sx->data_color);
+  }
+#endif
+#endif
 }
 
+
 /* the rest are in real coordinates except upsidedown from PS point of view */
-void ps_draw_line (axis_info *ap, int x0, int y0, int x1, int y1) 
+
+void ps_draw_line(axis_info *ap, int x0, int y0, int x1, int y1) 
 {
   int py0, py1, px0, px1;
   px0 = x0 + bx0;
@@ -383,21 +441,23 @@ void ps_draw_line (axis_info *ap, int x0, int y0, int x1, int y1)
   ps_write(pbuf);
 }
 
-void ps_draw_spectro_line(axis_info *ap, int color, Float x0, Float y0, Float x1, Float y1)
+
+void ps_draw_spectro_line(axis_info *ap, int color, mus_float_t x0, mus_float_t y0, mus_float_t x1, mus_float_t y1)
 {
   /* these are in local coords */
-  unsigned short r, g, b;
+  rgb_t r, g, b;
   if (last_color != color)
     {
       get_current_color(color_map(ss), color, &r, &g, &b);
       last_color = color;
-      mus_snprintf(pbuf, PRINT_BUFFER_SIZE, " %.2f %.2f %.2f RG\n", (float)r / 65535.0, (float)g / 65535.0, (float)b / 65535.0);
+      mus_snprintf(pbuf, PRINT_BUFFER_SIZE, " %.2f %.2f %.2f RG\n", RGB_TO_FLOAT(r), RGB_TO_FLOAT(g), RGB_TO_FLOAT(b));
       ps_write(pbuf);
     }
   ps_draw_line(ap, (int)x0, (int)y0, (int)x1, (int)y1);
 }
 
-void ps_fill_rectangle (axis_info *ap, int x0, int y0, int width, int height) 
+
+void ps_fill_rectangle(axis_info *ap, int x0, int y0, int width, int height) 
 {
   int py0, py1, px0, px1;
   px0 = x0 + bx0;
@@ -412,41 +472,50 @@ void ps_fill_rectangle (axis_info *ap, int x0, int y0, int width, int height)
   ps_write(pbuf);
 }
 
-void ps_draw_string (axis_info *ap, int x0, int y0, char *str) 
+
+void ps_draw_string(axis_info *ap, int x0, int y0, const char *str) 
 {
   int px0, py0;
   px0 = x0 + bx0;
   py0 = reflect_y(ap, y0) + by0;
+#if USE_GTK
+  py0 -= 12;
+#endif
   if (px0 > bbx) bbx = px0;
   if (py0 > bby) bby = py0;
   mus_snprintf(pbuf, PRINT_BUFFER_SIZE, " %d %d moveto (%s) show\n", px0, py0, str);
   ps_write(pbuf);
 }
 
+
 void ps_set_number_font(void) 
 {
   mus_snprintf(pbuf, PRINT_BUFFER_SIZE, " /Courier findfont 15 scalefont setfont\n");
   ps_write(pbuf);
 }
 
+
 void ps_set_label_font(void) 
 {
   mus_snprintf(pbuf, PRINT_BUFFER_SIZE, " /Times-Roman findfont 20 scalefont setfont\n");
   ps_write(pbuf);
 }
 
+
 void ps_set_bold_peak_numbers_font(void) 
 {
   mus_snprintf(pbuf, PRINT_BUFFER_SIZE, " /Times-Bold findfont 14 scalefont setfont\n");
   ps_write(pbuf);
 }
 
+
 void ps_set_peak_numbers_font(void) 
 {
   mus_snprintf(pbuf, PRINT_BUFFER_SIZE, " /Times-Roman findfont 14 scalefont setfont\n");
   ps_write(pbuf);
 }
 
+
 void ps_set_tiny_numbers_font(void) 
 {
   mus_snprintf(pbuf, PRINT_BUFFER_SIZE, " /Times-Roman findfont 12 scalefont setfont\n");
@@ -456,7 +525,7 @@ void ps_set_tiny_numbers_font(void)
 
 #define PRINTED_VERTICAL_SPACING 25 
 
-static char *snd_print_or_error(char *output)
+static char *snd_print_or_error(const char *output)
 {
   if ((output) && (*output))
     {
@@ -468,9 +537,9 @@ static char *snd_print_or_error(char *output)
       char *errstr = NULL;
       ccp = current_channel();
       if (ccp == NULL) 
-	return(copy_string(_("nothing to print?")));
+	return(mus_strdup(_("nothing to print?")));
       si = sync_to_chan(ccp);
-      offsets = (int *)CALLOC(si->chans, sizeof(int));
+      offsets = (int *)calloc(si->chans, sizeof(int));
       for (j = 0, i = (si->chans - 1); i >= 0; i--)
 	{
 	  offsets[i] = j;
@@ -497,26 +566,30 @@ static char *snd_print_or_error(char *output)
 	    ps_graph(si->cps[i], 0, offsets[i]);
 	  end_ps_graph();
 	}
-      else errstr = mus_format(_("print %s failed: %s"), output, strerror(errno));
+      else errstr = mus_format(_("print %s failed: %s"), output, snd_io_strerror());
       if (si) si = free_sync_info(si);
-      if (offsets) FREE(offsets);
+      if (offsets) free(offsets);
       return(errstr);
     }
-  else return(copy_string(_("print sound: eps file name needed")));
+  else return(mus_strdup(_("print sound: eps file name needed")));
 }
 
-void snd_print(char *output)
+
+bool snd_print(const char *output)
 {
   char *error;
   error = snd_print_or_error(output);
   if (error)
     {
-      snd_error(error);
-      FREE(error);
+      snd_error_without_format(error);
+      free(error);
+      return(false);
     }
+  return(true);
 }
 
-void region_print(char *output, char* title, chan_info *cp)
+
+void region_print(const char *output, const char *title, chan_info *cp)
 {
   if ((output) && (*output))
     {
@@ -527,12 +600,13 @@ void region_print(char *output, char* title, chan_info *cp)
 	  ps_graph(cp, 0, 0);
 	  end_ps_graph();
 	}
-      else snd_error(_("print region %s failed: %s"), output, strerror(errno));
+      else snd_error(_("print region %s failed: %s"), output, snd_io_strerror());
     }
-  else snd_error(_("print region: eps file name needed"));
+  else snd_error_without_format(_("print region: eps file name needed"));
 }
 
-void print_enved(char *output, int y0)
+
+void print_enved(const char *output, int y0)
 {
   if ((output) && (*output))
     {
@@ -545,26 +619,31 @@ void print_enved(char *output, int y0)
 	  env_redisplay_with_print();
 	  end_ps_graph();
 	}
-      else snd_error(_("print env %s failed: %s"), output, strerror(errno));
+      else snd_error(_("print env %s failed: %s"), output, snd_io_strerror());
     }
-  else snd_error(_("print envelope: eps file name needed"));
+  else snd_error_without_format(_("print envelope: eps file name needed"));
 }
 
+
 static XEN g_graph_to_ps(XEN filename)
 {
-  #define H_graph_to_ps "(" S_graph_to_ps " (filename eps-file)): write the current Snd displays to an EPS file"
+  #define H_graph_to_ps "(" S_graph_to_ps " :optional (filename eps-file)): write the current Snd displays to an EPS file"
+  char *error;
+  const char *file;
+
+  XEN_ASSERT_TYPE(XEN_STRING_IF_BOUND_P(filename), filename, XEN_ONLY_ARG, S_graph_to_ps, "a string (filename)");
 
-  char *error,*file;
   if (XEN_STRING_P(filename))
     file = XEN_TO_C_STRING(filename);
   else file = eps_file(ss);
+
   error = snd_print_or_error(file);
   if (error)
     {
       XEN result;
       result = C_TO_XEN_STRING(error);
-      FREE(error);
-      XEN_ERROR(CANNOT_PRINT,
+      free(error);
+      XEN_ERROR(XEN_ERROR_TYPE("cannot-print"),
 		XEN_LIST_3(C_TO_XEN_STRING(S_graph_to_ps),
 			   C_TO_XEN_STRING(file),
 			   result));
@@ -572,18 +651,130 @@ static XEN g_graph_to_ps(XEN filename)
   return(xen_return_first(C_TO_XEN_STRING(file), filename));
 }
 
+
+/* ---------------- gl -> ps ---------------- */
+
+#if HAVE_GL && MUS_WITH_GL2PS
+
+#include "gl2ps.h"
+
+#define NUM_GL2PS_TYPES 6
+static int gl2ps_types[NUM_GL2PS_TYPES] = {GL2PS_EPS, GL2PS_PS, GL2PS_PDF, GL2PS_TEX, GL2PS_SVG, GL2PS_PGF};
+
+
+static XEN g_gl_graph_to_ps(XEN filename, XEN output_type, XEN snd, XEN chn)
+{
+  #define H_gl_graph_to_ps "(" S_gl_graph_to_ps " :optional filename (type 0) snd chn) produces a postscript output file from \
+OpenGL graphics. type can be 0: eps, 1: ps, 2: pdf, 3: tex, 4: svg, 5: pgf."
+
+  const char *file;
+  FILE *fp;
+  chan_info *cp;
+  int state = GL2PS_OVERFLOW, buffsize = 1024 * 1024, type = 0;
+
+#if HAVE_SETLOCALE
+  char *old_locale = NULL;
+#endif
+
+  XEN_ASSERT_TYPE(XEN_STRING_IF_BOUND_P(filename), filename, XEN_ARG_1, S_gl_graph_to_ps, "a string (filename)");
+  XEN_ASSERT_TYPE(XEN_INTEGER_IF_BOUND_P(output_type), output_type, XEN_ARG_2, S_gl_graph_to_ps, "an integer, 0=eps");
+
+  ASSERT_CHANNEL(S_gl_graph_to_ps, snd, chn, 3);
+  cp = get_cp(snd, chn, S_gl_graph_to_ps);
+  if (!cp) return(XEN_FALSE);
+
+  if (XEN_STRING_P(filename))
+    file = XEN_TO_C_STRING(filename);
+  else file = eps_file(ss);
+
+  if (XEN_INTEGER_P(output_type))
+    type = XEN_TO_C_INT(output_type);
+  if ((type < 0) || (type >= NUM_GL2PS_TYPES))
+    XEN_OUT_OF_RANGE_ERROR(S_gl_graph_to_ps, XEN_ARG_2, output_type, "must be between 0 and 5");
+
+#if HAVE_SETLOCALE
+  old_locale = mus_strdup(setlocale(LC_NUMERIC, "C"));
+#endif
+  
+  fp = fopen(file, "wb");
+
+  while (state == GL2PS_OVERFLOW)
+    {
+      GLint err;
+      buffsize += 1024 * 1024;
+      err = gl2psBeginPage(cp->sound->short_filename, "Snd", NULL, 
+			   gl2ps_types[type],
+			   GL2PS_BSP_SORT,
+			   GL2PS_DRAW_BACKGROUND | GL2PS_USE_CURRENT_VIEWPORT | GL2PS_OCCLUSION_CULL, /* perhaps also GL2PS_NO_BLENDING */
+			   GL_RGBA, 0, NULL, 0, 0, 0, buffsize, fp, file);
+      if (err != GL2PS_SUCCESS) 
+	{
+	  /* if an error occurs, gl2ps itself insists on printing something -- this needs to be fixed! */
+	  state = (int)err;
+	}
+      else
+	{
+	  ss->gl_printing = true;
+	  display_channel_fft_data(cp);
+	  ss->gl_printing = false;
+
+	  state = gl2psEndPage();
+	}
+    }
+  fclose(fp);
+#if HAVE_SETLOCALE
+  setlocale(LC_NUMERIC, old_locale);
+  if (old_locale) free(old_locale);
+#endif
+  
+  return(xen_return_first(C_TO_XEN_STRING(file), filename));
+}
+  
+
+char *gl2ps_version(void);
+char *gl2ps_version(void)
+{
+  char *buf;
+  buf = (char *)calloc(128, sizeof(char));
+  snprintf(buf, 128, "gl2ps %d.%d.%d", GL2PS_MAJOR_VERSION, GL2PS_MINOR_VERSION, GL2PS_PATCH_VERSION);
+  return(buf);
+}
+
+
+void gl2ps_text(const char *msg);
+void gl2ps_text(const char *msg)
+{
+  gl2psText(msg, "Times-Roman", 20);
+}
+
+#else
+
+static XEN g_gl_graph_to_ps(XEN filename_ignore, XEN output_type_ignore, XEN snd_ignore, XEN chn_ignore)
+{
+  #define H_gl_graph_to_ps "gl-graph->ps is a no-op in this version of Snd"
+  return(XEN_FALSE);
+}
+#endif
+
+/* -------------------------------- */
+
+
 static XEN g_eps_file(void) {return(C_TO_XEN_STRING(eps_file(ss)));}
+
 static XEN g_set_eps_file(XEN val) 
 {
   #define H_eps_file "(" S_eps_file "): File:Print and " S_graph_to_ps " file name (snd.eps)"
   XEN_ASSERT_TYPE(XEN_STRING_P(val), val, XEN_ONLY_ARG, S_setB S_eps_file, "a string"); 
-  if (eps_file(ss)) FREE(eps_file(ss));
-  set_eps_file(copy_string(XEN_TO_C_STRING(val))); 
+  if (eps_file(ss)) free(eps_file(ss));
+  set_eps_file(mus_strdup(XEN_TO_C_STRING(val))); 
   return(C_TO_XEN_STRING(eps_file(ss)));
 }
 
+
 #define MAX_EPS_MARGIN 1000.0
+
 static XEN g_eps_left_margin(void) {return(C_TO_XEN_DOUBLE(eps_left_margin(ss)));}
+
 static XEN g_set_eps_left_margin(XEN val) 
 {
   #define H_eps_left_margin "(" S_eps_left_margin "): File:Print and " S_graph_to_ps " left margin"
@@ -592,7 +783,9 @@ static XEN g_set_eps_left_margin(XEN val)
   return(C_TO_XEN_DOUBLE(eps_left_margin(ss)));
 }
 
+
 static XEN g_eps_bottom_margin(void) {return(C_TO_XEN_DOUBLE(eps_bottom_margin(ss)));}
+
 static XEN g_set_eps_bottom_margin(XEN val) 
 {
   #define H_eps_bottom_margin "(" S_eps_bottom_margin "): File:Print and " S_graph_to_ps " bottom margin"
@@ -601,7 +794,9 @@ static XEN g_set_eps_bottom_margin(XEN val)
   return(C_TO_XEN_DOUBLE(eps_bottom_margin(ss)));
 }
 
+
 static XEN g_eps_size(void) {return(C_TO_XEN_DOUBLE(eps_size(ss)));}
+
 static XEN g_set_eps_size(XEN val) 
 {
   #define MAX_EPS_SIZE 1000.0
@@ -611,8 +806,10 @@ static XEN g_set_eps_size(XEN val)
   return(C_TO_XEN_DOUBLE(eps_size(ss)));
 }
 
+
 #ifdef XEN_ARGIFY_1
 XEN_ARGIFY_1(g_graph_to_ps_w, g_graph_to_ps)
+XEN_ARGIFY_4(g_gl_graph_to_ps_w, g_gl_graph_to_ps)
 XEN_NARGIFY_0(g_eps_file_w, g_eps_file)
 XEN_NARGIFY_1(g_set_eps_file_w, g_set_eps_file)
 XEN_NARGIFY_0(g_eps_left_margin_w, g_eps_left_margin)
@@ -623,6 +820,7 @@ XEN_NARGIFY_0(g_eps_bottom_margin_w, g_eps_bottom_margin)
 XEN_NARGIFY_1(g_set_eps_bottom_margin_w, g_set_eps_bottom_margin)
 #else
 #define g_graph_to_ps_w g_graph_to_ps
+#define g_gl_graph_to_ps_w g_gl_graph_to_ps
 #define g_eps_file_w g_eps_file
 #define g_set_eps_file_w g_set_eps_file
 #define g_eps_left_margin_w g_eps_left_margin
@@ -636,6 +834,7 @@ XEN_NARGIFY_1(g_set_eps_bottom_margin_w, g_set_eps_bottom_margin)
 void g_init_print(void)
 {
   XEN_DEFINE_PROCEDURE(S_graph_to_ps, g_graph_to_ps_w, 0, 1, 0, H_graph_to_ps);
+  XEN_DEFINE_PROCEDURE(S_gl_graph_to_ps, g_gl_graph_to_ps_w, 0, 4, 0, H_gl_graph_to_ps);
 
   XEN_DEFINE_PROCEDURE_WITH_SETTER(S_eps_file, g_eps_file_w, H_eps_file,
 				   S_setB S_eps_file, g_set_eps_file_w,  0, 0, 1, 0);
@@ -648,4 +847,8 @@ void g_init_print(void)
 
   XEN_DEFINE_PROCEDURE_WITH_SETTER(S_eps_size, g_eps_size_w, H_eps_size,
 				   S_setB S_eps_size, g_set_eps_size_w,  0, 0, 1, 0);
+
+#if HAVE_GL && MUS_WITH_GL2PS
+  XEN_YES_WE_HAVE("gl2ps");
+#endif
 }
