@@ -3,18 +3,26 @@
 
 #define S_abort                         "abort"
 #define S_add_colormap                  "add-colormap"
+#define S_add_directory_to_view_files_list "add-directory-to-view-files-list"
+#define S_add_file_filter               "add-file-filter"
+#define S_add_file_sorter               "add-file-sorter"
+#define S_add_file_to_view_files_list   "add-file-to-view-files-list"
 #define S_add_mark                      "add-mark"
 #define S_add_player                    "add-player"
 #define S_add_sound_file_extension      "add-sound-file-extension"
+#define S_add_source_file_extension     "add-source-file-extension"
 #define S_add_to_main_menu              "add-to-main-menu"
 #define S_add_to_menu                   "add-to-menu"
 #define S_add_transform                 "add-transform"
-#define S_after_apply_hook              "after-apply-hook"
+#define S_add_watcher                   "add-watcher"
+#define S_after_apply_controls_hook     "after-apply-controls-hook"
 #define S_after_edit_hook               "after-edit-hook"
 #define S_after_graph_hook              "after-graph-hook"
+#define S_after_lisp_graph_hook         "after-lisp-graph-hook"
 #define S_after_open_hook               "after-open-hook"
 #define S_after_save_as_hook            "after-save-as-hook"
 #define S_after_save_state_hook         "after-save-state-hook"
+#define S_after_transform_hook          "after-transform-hook"
 #define S_amp_control                   "amp-control"
 #define S_amp_control_bounds            "amp-control-bounds"
 #define S_apply_controls                "apply-controls"
@@ -25,14 +33,20 @@
 #define S_auto_resize                   "auto-resize"
 #define S_auto_update                   "auto-update"
 #define S_auto_update_interval          "auto-update-interval"
-#define S_autocorrelate                 "autocorrelate"
 #define S_autocorrelation               "autocorrelation"
+#define S_axis_color                    "axis-color"
 #define S_axis_info                     "axis-info"
 #define S_axis_label_font               "axis-label-font"
 #define S_axis_numbers_font             "axis-numbers-font"
+#define S_background_gradient           "background-gradient"
 #define S_bad_header_hook               "bad-header-hook"
 #define S_basic_color                   "basic-color"
+#define S_beats_per_measure             "beats-per-measure"
 #define S_beats_per_minute              "beats-per-minute"
+#define S_before_close_hook             "before-close-hook"
+#define S_before_exit_hook              "before-exit-hook"
+#define S_before_save_as_hook           "before-save-as-hook"
+#define S_before_save_state_hook        "before-save-state-hook"
 #define S_before_transform_hook         "before-transform-hook"
 #define S_bind_key                      "bind-key"  
 #define S_bold_peaks_font               "bold-peaks-font"
@@ -40,11 +54,11 @@
 #define S_c_g                           "c-g?"
 #define S_c_g_x                         "c-g!"
 #define S_cepstrum                      "cepstrum"
-#define S_channel_to_vct                "channel->vct"
 #define S_channel_amp_envs              "channel-amp-envs"
 #define S_channel_data                  "channel-data"
 #define S_channel_properties            "channel-properties"
 #define S_channel_style                 "channel-style"
+#define S_channel_to_vct                "channel->vct"
 #define S_channel_widgets               "channel-widgets"
 #define S_channels                      "channels"
 #define S_channels_combined             "channels-combined"
@@ -52,21 +66,25 @@
 #define S_channels_superimposed         "channels-superimposed"
 #define S_chans                         "chans"
 #define S_clear_listener                "clear-listener"
+#define S_clear_minibuffer              "clear-minibuffer"
+#define S_clip_hook                     "clip-hook"
+#define S_clipping                      "clipping"
 #define S_clm_channel                   "clm-channel"
 #define S_close_hook                    "close-hook"
 #define S_close_sound                   "close-sound"
-#define S_close_sound_file              "close-sound-file"
-#define S_color_to_list                 "color->list"
 #define S_color_cutoff                  "color-cutoff"
-#define S_color_dialog                  "color-dialog"
+#define S_color_orientation_dialog      "color-orientation-dialog"
 #define S_color_hook                    "color-hook"
 #define S_color_inverted                "color-inverted"
 #define S_color_p                       "color?"
 #define S_color_scale                   "color-scale"
+#define S_color_to_list                 "color->list"
 #define S_colormap                      "colormap"
 #define S_colormap_name                 "colormap-name"
+#define S_colormap_p                    "colormap?"
 #define S_colormap_ref                  "colormap-ref"
 #define S_colormap_size                 "colormap-size"
+#define S_colormap_to_integer           "colormap->integer"
 #define S_comment                       "comment"
 #define S_contrast_control              "contrast-control"
 #define S_contrast_control_amp          "contrast-control-amp"
@@ -76,9 +94,7 @@
 #define S_convolve_selection_with       "convolve-selection-with"
 #define S_convolve_with                 "convolve-with"
 #define S_copy_context                  "copy-context"
-#define S_copy_mix                      "copy-mix"
-#define S_copy_sample_reader            "copy-sample-reader"
-#define S_copy_track                    "copy-track"
+#define S_copy_sampler                  "copy-sampler"
 #define S_count_matches                 "count-matches"
 #define S_current_edit_position         "current-edit-position"
 #define S_current_font                  "current-font"
@@ -86,7 +102,6 @@
 #define S_cursor_color                  "cursor-color"
 #define S_cursor_context                "cursor-context"
 #define S_cursor_cross                  "cursor-cross"
-#define S_cursor_follows_play           "cursor-follows-play"
 #define S_cursor_in_middle              "cursor-in-middle"
 #define S_cursor_in_view                "cursor-in-view"
 #define S_cursor_line                   "cursor-line"
@@ -99,30 +114,30 @@
 #define S_cursor_update_interval        "cursor-update-interval"
 #define S_dac_combines_channels         "dac-combines-channels"
 #define S_dac_hook                      "dac-hook"
-#define S_dac_is_running                "dac-is-running"
 #define S_dac_size                      "dac-size"
-#define S_data_clipped                  "data-clipped"
 #define S_data_color                    "data-color"
 #define S_data_format                   "data-format"
 #define S_data_location                 "data-location"
 #define S_data_size                     "data-size"
 #define S_default_output_chans          "default-output-chans"
-#define S_default_output_format         "default-output-format"
+#define S_default_output_data_format    "default-output-data-format"
 #define S_default_output_srate          "default-output-srate"
-#define S_default_output_type           "default-output-type"
+#define S_default_output_header_type    "default-output-header-type"
 #define S_define_envelope               "define-envelope"
 #define S_delete_colormap               "delete-colormap"
+#define S_delete_file_filter            "delete-file-filter"
+#define S_delete_file_sorter            "delete-file-sorter"
 #define S_delete_mark                   "delete-mark"
 #define S_delete_marks                  "delete-marks"
-#define S_delete_mix                    "delete-mix"
 #define S_delete_sample                 "delete-sample"
 #define S_delete_samples                "delete-samples"
 #define S_delete_selection              "delete-selection"
-#define S_delete_track                  "delete-track"
+#define S_delete_transform              "delete-transform"
+#define S_delete_watcher                "delete-watcher"
 #define S_dialog_widgets                "dialog-widgets"
 #define S_display_edits                 "display-edits"
-#define S_doit_button_color             "doit-button-color"
 #define S_doit_again_button_color       "doit-again-button-color"
+#define S_doit_button_color             "doit-button-color"
 #define S_dont_normalize                "dont-normalize"
 #define S_dot_size                      "dot-size"
 #define S_draw_axes                     "draw-axes"
@@ -131,6 +146,7 @@
 #define S_draw_line                     "draw-line"
 #define S_draw_lines                    "draw-lines"
 #define S_draw_mark_hook                "draw-mark-hook"
+#define S_draw_mix_hook                 "draw-mix-hook"
 #define S_draw_string                   "draw-string"
 #define S_drop_hook                     "drop-hook"
 #define S_during_open_hook              "during-open-hook"
@@ -139,9 +155,11 @@
 #define S_edit_hook                     "edit-hook"
 #define S_edit_list_to_function         "edit-list->function"
 #define S_edit_position                 "edit-position"
+#define S_edit_properties               "edit-properties"
 #define S_edit_tree                     "edit-tree"
 #define S_edits                         "edits"
 #define S_env_channel                   "env-channel"
+#define S_env_channel_with_base         "env-channel-with-base"
 #define S_env_selection                 "env-selection"
 #define S_env_sound                     "env-sound"
 #define S_enved_add_point               "enved-add-point"
@@ -163,14 +181,12 @@
 #define S_enved_target                  "enved-target"
 #define S_enved_wave_p                  "enved-wave?"
 #define S_enved_waveform_color          "enved-waveform-color"
-#define S_envelope_base                 "envelope-base"
 #define S_envelope_exponential          "envelope-exponential"
 #define S_envelope_linear               "envelope-linear"
 #define S_eps_bottom_margin             "eps-bottom-margin"
 #define S_eps_file                      "eps-file"
 #define S_eps_left_margin               "eps-left-margin"
 #define S_eps_size                      "eps-size"
-#define S_equalize_panes                "equalize-panes"
 #define S_exit                          "exit"
 #define S_exit_hook                     "exit-hook"
 #define S_expand_control                "expand-control"
@@ -184,7 +200,9 @@
 #define S_fft_log_frequency             "fft-log-frequency"
 #define S_fft_log_magnitude             "fft-log-magnitude"
 #define S_fft_window                    "fft-window"
+#define S_fft_window_alpha              "fft-window-alpha"
 #define S_fft_window_beta               "fft-window-beta"
+#define S_fft_with_phases               "fft-with-phases"
 #define S_file_name                     "file-name"
 #define S_fill_polygon                  "fill-polygon"
 #define S_fill_rectangle                "fill-rectangle"
@@ -198,7 +216,7 @@
 #define S_filter_control_waveform_color "filter-control-waveform-color"
 #define S_filter_selection              "filter-selection"
 #define S_filter_sound                  "filter-sound"
-#define S_find                          "find"
+#define S_find_channel                  "find-channel"
 #define S_find_dialog                   "find-dialog"
 #define S_find_mark                     "find-mark"
 #define S_find_sound                    "find-sound"
@@ -208,11 +226,14 @@
 #define S_forget_region                 "forget-region"
 #define S_fourier_transform             "fourier-transform"
 #define S_frames                        "frames"
-#define S_free_sample_reader            "free-sample-reader"
-#define S_free_track                    "free-track"
+#define S_free_player                   "free-player"
+#define S_free_sampler                  "free-sampler"
+#define S_gl_graph_to_ps                "gl-graph->ps"
 #define S_goto_listener_end             "goto-listener-end"
 #define S_graph                         "graph"
-#define S_graph_to_ps                   "graph->ps"
+#define S_graph_as_sonogram             "graph-as-sonogram"
+#define S_graph_as_spectrogram          "graph-as-spectrogram"
+#define S_graph_as_wavogram             "graph-as-wavogram"
 #define S_graph_color                   "graph-color"
 #define S_graph_cursor                  "graph-cursor"
 #define S_graph_data                    "graph-data"
@@ -222,11 +243,9 @@
 #define S_graph_hook                    "graph-hook"
 #define S_graph_lines                   "graph-lines"
 #define S_graph_lollipops               "graph-lollipops"
-#define S_graph_style                   "graph-style"
 #define S_graph_once                    "graph-once"
-#define S_graph_as_wavogram             "graph-as-wavogram"
-#define S_graph_as_sonogram             "graph-as-sonogram"
-#define S_graph_as_spectrogram          "graph-as-spectrogram"
+#define S_graph_style                   "graph-style"
+#define S_graph_to_ps                   "graph->ps"
 #define S_graphs_horizontal             "graphs-horizontal"
 #define S_grid_density                  "grid-density"
 #define S_haar_transform                "haar-transform"
@@ -245,23 +264,29 @@
 #endif
 #define S_info_dialog                   "info-dialog"
 #define S_initial_graph_hook            "initial-graph-hook"
+#define S_insert_file_dialog            "insert-file-dialog"
 #define S_insert_region                 "insert-region"
 #define S_insert_sample                 "insert-sample"
 #define S_insert_samples                "insert-samples"
 #define S_insert_selection              "insert-selection"
 #define S_insert_silence                "insert-silence"
 #define S_insert_sound                  "insert-sound"
+#define S_integer_to_colormap           "integer->colormap"
+#define S_integer_to_mark               "integer->mark"
+#define S_integer_to_mix                "integer->mix"
+#define S_integer_to_region             "integer->region"
+#define S_integer_to_sound              "integer->sound"
+#define S_integer_to_transform          "integer->transform"
 #define S_just_sounds                   "just-sounds"
-#define S_just_sounds_hook              "just-sounds-hook"
 #define S_key                           "key"
 #define S_key_binding                   "key-binding"
 #define S_key_press_hook                "key-press-hook"
 #define S_keyboard_no_action            "keyboard-no-action"
 #define S_ladspa_dir                    "ladspa-dir"
 #define S_left_sample                   "left-sample"
-#define S_lisp_graph_p                  "lisp-graph?"
 #define S_lisp_graph                    "lisp-graph"
 #define S_lisp_graph_hook               "lisp-graph-hook"
+#define S_lisp_graph_p                  "lisp-graph?"
 #define S_lisp_graph_style              "lisp-graph-style"
 #define S_listener_click_hook           "listener-click-hook"
 #define S_listener_color                "listener-color"
@@ -269,23 +294,18 @@
 #define S_listener_prompt               "listener-prompt"
 #define S_listener_selection            "listener-selection"
 #define S_listener_text_color           "listener-text-color"
-#define S_load_font                     "load-font"
-#define S_lock_track                    "lock-track"
 #define S_log_freq_start                "log-freq-start"
 #define S_main_menu                     "main-menu"
 #define S_main_widgets                  "main-widgets"
 #define S_make_color                    "make-color"
 #define S_make_graph_data               "make-graph-data"
-#define S_make_mix_sample_reader        "make-mix-sample-reader"
+#define S_make_mix_sampler              "make-mix-sampler"
 #define S_make_player                   "make-player"
 #define S_make_region                   "make-region"
-#define S_make_region_sample_reader     "make-region-sample-reader"
-#define S_make_sample_reader            "make-sample-reader"
+#define S_make_region_sampler           "make-region-sampler"
+#define S_make_sampler                  "make-sampler"
 #define S_make_snd_to_sample            "make-snd->sample"
-#define S_make_track                    "make-track"
-#define S_make_track_sample_reader      "make-track-sample-reader"
 #define S_make_variable_graph           "make-variable-graph"
-#define S_make_xen_to_sample            "make-xen->sample"
 #define S_map_chan                      "map-chan"
 #define S_map_channel                   "map-channel"
 #define S_mark_click_hook               "mark-click-hook"
@@ -296,45 +316,48 @@
 #define S_mark_home                     "mark-home"
 #define S_mark_hook                     "mark-hook"
 #define S_mark_name                     "mark-name"
+#define S_mark_properties               "mark-properties"
 #define S_mark_p                        "mark?"
 #define S_mark_sample                   "mark-sample"
 #define S_mark_sync                     "mark-sync"
 #define S_mark_sync_max                 "mark-sync-max"
 #define S_mark_tag_height               "mark-tag-height"
 #define S_mark_tag_width                "mark-tag-width"
+#define S_mark_to_integer               "mark->integer"
 #define S_marks                         "marks"
 #define S_max_regions                   "max-regions"
 #define S_max_transform_peaks           "max-transform-peaks"
+#define S_max_virtual_ptrees            "max-virtual-ptrees"
 #define S_maxamp                        "maxamp"
-#define S_memo_sound                    "memo-sound"
+#define S_maxamp_position               "maxamp-position"
 #define S_menu_widgets                  "menu-widgets"
 #define S_min_dB                        "min-dB"
 #define S_minibuffer_history_length     "minibuffer-history-length"
 #define S_mix                           "mix"
 #define S_mix_amp                       "mix-amp"
 #define S_mix_amp_env                   "mix-amp-env"
-#define S_mix_chans                     "mix-chans"
 #define S_mix_click_hook                "mix-click-hook"
 #define S_mix_color                     "mix-color"
 #define S_mix_dialog_mix                "mix-dialog-mix"
 #define S_mix_drag_hook                 "mix-drag-hook"
 #define S_mix_file_dialog               "mix-file-dialog"
-#define S_mix_frames                    "mix-frames"
+#define S_mix_length                    "mix-length"
 #define S_mix_home                      "mix-home"
-#define S_mix_inverted_p                "mix-inverted?"
-#define S_mix_locked_p                  "mix-locked?"
 #define S_mix_p                         "mix?"
+#define S_mix_name                      "mix-name"
 #define S_mix_position                  "mix-position"
+#define S_mix_properties                "mix-properties"
 #define S_mix_region                    "mix-region"
 #define S_mix_release_hook              "mix-release-hook"
-#define S_mix_sample_reader_p           "mix-sample-reader?"
+#define S_mix_sampler_p                 "mix-sampler?"
 #define S_mix_selection                 "mix-selection"
 #define S_mix_speed                     "mix-speed"
+#define S_mix_sync                      "mix-sync"
+#define S_mix_sync_max                  "mix-sync-max"
 #define S_mix_tag_height                "mix-tag-height"
-#define S_mix_tag_position              "mix-tag-position"
 #define S_mix_tag_width                 "mix-tag-width"
 #define S_mix_tag_y                     "mix-tag-y"
-#define S_mix_track                     "mix-track"
+#define S_mix_to_integer                "mix->integer"
 #define S_mix_vct                       "mix-vct"
 #define S_mix_waveform_height           "mix-waveform-height"
 #define S_mixes                         "mixes"
@@ -353,6 +376,7 @@
 #define S_mus_error_hook                "mus-error-hook"
 #define S_name_click_hook               "name-click-hook"
 #define S_new_sound                     "new-sound"
+#define S_new_sound_dialog              "new-sound-dialog"
 #define S_new_widget_hook               "new-widget-hook"
 #define S_next_sample                   "next-sample"
 #define S_normalize_by_channel          "normalize-by-channel"
@@ -360,40 +384,32 @@
 #define S_normalize_channel             "normalize-channel"
 #define S_normalize_globally            "normalize-globally"
 #define S_open_file_dialog              "open-file-dialog"
+#define S_open_file_dialog_directory    "open-file-dialog-directory"
 #define S_open_hook                     "open-hook"
 #define S_open_raw_sound                "open-raw-sound"
 #define S_open_raw_sound_hook           "open-raw-sound-hook"
 #define S_open_sound                    "open-sound"
-#define S_open_sound_file               "open-sound-file"
 #define S_optimization                  "optimization"
 #define S_optimization_hook             "optimization-hook"
-#define S_orientation_dialog            "orientation-dialog"
 #define S_orientation_hook              "orientation-hook"
 #define S_output_comment_hook           "output-comment-hook"
 #define S_output_name_hook              "output-name-hook"
 #define S_pad_channel                   "pad-channel"
+#define S_pausing                       "pausing"
+#define S_peak_env_hook                 "peak-env-hook"
 #define S_peak_env_info                 "peak-env-info"
 #define S_peaks                         "peaks"
 #define S_peaks_font                    "peaks-font"
 #define S_play                          "play"
-#define S_play_and_wait                 "play-and-wait"
-#define S_play_channel                  "play-channel"
 #define S_play_hook                     "play-hook"
-#define S_play_mix                      "play-mix"
-#define S_play_region                   "play-region"
-#define S_play_selection                "play-selection"
-#define S_play_track                    "play-track"
+#define S_playing                       "playing"
 #define S_player_home                   "player-home"
-#define S_players                       "players"
 #define S_player_p                      "player?"
+#define S_players                       "players"
+#define S_position_color                "position-color"
 #define S_position_to_x                 "position->x"
 #define S_position_to_y                 "position->y"
-#define S_position_color                "position-color"
-#define S_preload_directory             "preload-directory"
-#define S_preload_file                  "preload-file"
-#define S_previous_files_select_hook    "previous-files-select-hook"
-#define S_previous_files_sort           "previous-files-sort"
-#define S_previous_files_sort_procedure "previous-files-sort-procedure"
+#define S_preferences_dialog            "preferences-dialog"
 #define S_previous_sample               "previous-sample"
 #define S_print_dialog                  "print-dialog"
 #define S_print_hook                    "print-hook"
@@ -405,14 +421,12 @@
 #define S_quit_button_color             "quit-button-color"
 #define S_ramp_channel                  "ramp-channel"
 #define S_read_hook                     "read-hook"
+#define S_read_mix_sample               "read-mix-sample"
 #define S_read_only                     "read-only"
 #define S_read_peak_env_info_file       "read-peak-env-info-file"
-#define S_read_sample                   "read-sample"
-#define S_read_mix_sample               "read-mix-sample"
 #define S_read_region_sample            "read-region-sample"
-#define S_read_track_sample             "read-track-sample"
+#define S_read_sample                   "read-sample"
 #define S_recorder_dialog               "recorder-dialog"
-#define S_recorder_file_hook            "recorder-file-hook"
 #if HAVE_RUBY
   #define S_redo                        "redo-edit"
 #else
@@ -421,13 +435,16 @@
 #define S_region_chans                  "region-chans"
 #define S_region_frames                 "region-frames"
 #define S_region_graph_style            "region-graph-style"
+#define S_region_home                   "region-home"
 #define S_region_maxamp                 "region-maxamp"
+#define S_region_maxamp_position        "region-maxamp-position"
 #define S_region_p                      "region?"
 #define S_region_position               "region-position"
 #define S_region_sample                 "region-sample"
-#define S_region_sample_reader_p        "region-sample-reader?"
-#define S_region_to_vct                 "region->vct"
+#define S_region_sampler_p              "region-sampler?"
 #define S_region_srate                  "region-srate"
+#define S_region_to_integer             "region->integer"
+#define S_region_to_vct                 "region->vct"
 #define S_regions                       "regions"
 #define S_remove_from_menu              "remove-from-menu"
 #define S_report_in_minibuffer          "report-in-minibuffer"
@@ -435,7 +452,6 @@
 #define S_reset_controls                "reset-controls"
 #define S_reset_listener_cursor         "reset-listener-cursor"
 #define S_restore_controls              "restore-controls"
-#define S_restore_marks                 "restore-marks"
 #define S_restore_region                "restore-region"
 #define S_reverb_control_decay          "reverb-control-decay"
 #define S_reverb_control_feedback       "reverb-control-feedback"
@@ -452,12 +468,11 @@
 #define S_right_sample                  "right-sample"
 #define S_run                           "run"
 #define S_sample                        "sample"
-#define S_sample_reader_at_end_p        "sample-reader-at-end?"
-#define S_sample_reader_home            "sample-reader-home"
-#define S_sample_reader_position        "sample-reader-position"
-#define S_sample_reader_p               "sample-reader?"
+#define S_sampler_at_end_p              "sampler-at-end?"
+#define S_sampler_home                  "sampler-home"
+#define S_sampler_p                     "sampler?"
+#define S_sampler_position              "sampler-position"
 #define S_samples                       "samples"
-#define S_samples_to_sound_data         "samples->sound-data"
 #define S_sash_color                    "sash-color"
 #define S_save_controls                 "save-controls"
 #define S_save_dir                      "save-dir"
@@ -467,13 +482,14 @@
 #define S_save_listener                 "save-listener"
 #define S_save_macros                   "save-macros"
 #define S_save_marks                    "save-marks"
-#define S_save_options                  "save-options"
+#define S_save_mix                      "save-mix"
 #define S_save_region                   "save-region"
+#define S_save_region_dialog            "save-region-dialog"
 #define S_save_selection                "save-selection"
 #define S_save_selection_dialog         "save-selection-dialog"
 #define S_save_sound                    "save-sound"
-#define S_save_sound_dialog             "save-sound-dialog"
 #define S_save_sound_as                 "save-sound-as"
+#define S_save_sound_dialog             "save-sound-dialog"
 #define S_save_state                    "save-state"
 #define S_save_state_file               "save-state-file"
 #define S_save_state_hook               "save-state-hook"
@@ -496,23 +512,25 @@
 #define S_selected_data_color           "selected-data-color"
 #define S_selected_graph_color          "selected-graph-color"
 #define S_selected_sound                "selected-sound"
-#define S_selection_changed_hook        "selection-changed-hook"
+#define S_selection                     "selection"
 #define S_selection_chans               "selection-chans"
 #define S_selection_color               "selection-color"
 #define S_selection_context             "selection-context"
 #define S_selection_creates_region      "selection-creates-region"
 #define S_selection_frames              "selection-frames"
 #define S_selection_maxamp              "selection-maxamp"
+#define S_selection_maxamp_position     "selection-maxamp-position"
 #define S_selection_member              "selection-member?"
+#define S_selection_to_mix              "selection->mix"
 #define S_selection_p                   "selection?"
 #define S_selection_position            "selection-position"
 #define S_selection_srate               "selection-srate"
-#define S_send_netscape                 "send-netscape"
+#define S_send_mozilla                  "send-mozilla"
 #define S_short_file_name               "short-file-name"
 #define S_show_all_axes                 "show-all-axes"
 #define S_show_all_axes_unlabelled      "show-all-axes-unlabelled"
 #define S_show_axes                     "show-axes"
-#define S_show_backtrace                "show-backtrace"
+#define S_show_bare_x_axis              "show-bare-x-axis"
 #define S_show_controls                 "show-controls"
 #define S_show_grid                     "show-grid"
 #define S_show_indices                  "show-indices"
@@ -531,37 +549,46 @@
 #define S_smooth_channel                "smooth-channel"
 #define S_smooth_selection              "smooth-selection"
 #define S_smooth_sound                  "smooth-sound"
-#define S_snd_apropos                   "snd-apropos"
+#define S_snd_color                     "snd-color"
 #define S_snd_error                     "snd-error"
 #define S_snd_error_hook                "snd-error-hook"
+#define S_snd_font                      "snd-font"
 #define S_snd_gcs                       "snd-gcs"
 #define S_snd_help                      "snd-help"
+#if HAVE_RUBY
+  #define S_snd_opened_sound            "snd-opened-sound"
+#else
+  #define S_snd_opened_sound            "*snd-opened-sound*"
+#endif
 #define S_snd_print                     "snd-print"
 #define S_snd_spectrum                  "snd-spectrum"
 #define S_snd_tempnam                   "snd-tempnam"
+#define S_snd_to_sample                 "snd->sample"
+#define S_snd_to_sample_p               "snd->sample?"
 #define S_snd_url                       "snd-url"
 #define S_snd_urls                      "snd-urls"
 #define S_snd_version                   "snd-version"
 #define S_snd_warning                   "snd-warning"
 #define S_snd_warning_hook              "snd-warning-hook"
-#define S_snd_to_sample                 "snd->sample"
-#define S_snd_to_sample_p               "snd->sample?"
+#define S_sound_file_extensions         "sound-file-extensions"
+#define S_sound_file_p                  "sound-file?"
 #define S_sound_files_in_directory      "sound-files-in-directory"
 #define S_sound_loop_info               "sound-loop-info"
 #define S_sound_p                       "sound?"
 #define S_sound_properties              "sound-properties"
 #define S_sound_widgets                 "sound-widgets"
+#define S_sound_to_integer              "sound->integer"
 #define S_soundfont_info                "soundfont-info"
 #define S_sounds                        "sounds"
-#define S_spectro_cutoff                "spectro-cutoff"
 #define S_spectro_hop                   "spectro-hop"
-#define S_spectro_start                 "spectro-start"
 #define S_spectro_x_angle               "spectro-x-angle"
 #define S_spectro_x_scale               "spectro-x-scale"
 #define S_spectro_y_angle               "spectro-y-angle"
 #define S_spectro_y_scale               "spectro-y-scale"
 #define S_spectro_z_angle               "spectro-z-angle"
 #define S_spectro_z_scale               "spectro-z-scale"
+#define S_spectrum_end                  "spectrum-end"
+#define S_spectrum_start                "spectrum-start"
 #define S_speed_control                 "speed-control"
 #define S_speed_control_as_float        "speed-control-as-float"
 #define S_speed_control_as_ratio        "speed-control-as-ratio"
@@ -586,40 +613,29 @@
 #define S_stop_playing_selection_hook   "stop-playing-selection-hook"
 #define S_swap_channels                 "swap-channels"
 #define S_sync                          "sync"
+#define S_sync_max                      "sync-max"
 #define S_syncd_marks                   "syncd-marks"
 #define S_temp_dir                      "temp-dir"
-#define S_tempo_control_bounds          "tempo-control-bounds"
 #define S_text_focus_color              "text-focus-color"
 #define S_time_graph                    "time-graph"
+#define S_time_graph_hook               "time-graph-hook"
 #define S_time_graph_p                  "time-graph?"
 #define S_time_graph_style              "time-graph-style"
 #define S_time_graph_type               "time-graph-type"
 #define S_tiny_font                     "tiny-font"
-#define S_track                         "track"
-#define S_tracks                        "tracks"
-#define S_track_p                       "track?"
-#define S_track_amp                     "track-amp"
-#define S_track_amp_env                 "track-amp-env"
-#define S_track_chans                   "track-chans"
-#define S_track_color                   "track-color"
-#define S_track_dialog_track            "track-dialog-track"
-#define S_track_frames                  "track-frames"
-#define S_track_position                "track-position"
-#define S_track_sample_reader_p         "track-sample-reader?"
-#define S_track_speed                   "track-speed"
-#define S_track_tempo                   "track-tempo"
-#define S_track_track                   "track-track"
+#define S_tracking_cursor_style         "tracking-cursor-style"
 #define S_transform_dialog              "transform-dialog"
+#define S_transform_frames              "transform-frames"
 #define S_transform_graph               "transform-graph"
 #define S_transform_graph_p             "transform-graph?"
 #define S_transform_graph_style         "transform-graph-style"
 #define S_transform_graph_type          "transform-graph-type"
-#define S_transform_hook                "transform-hook"
 #define S_transform_normalization       "transform-normalization"
+#define S_transform_p                   "transform?"
 #define S_transform_sample              "transform-sample"
-#define S_transform_frames              "transform-frames"
-#define S_transform_to_vct              "transform->vct"
 #define S_transform_size                "transform-size"
+#define S_transform_to_integer          "transform->integer"
+#define S_transform_to_vct              "transform->vct"
 #define S_transform_type                "transform-type"
 #define S_trap_segfault                 "trap-segfault"
 #define S_unbind_key                    "unbind-key"  
@@ -630,18 +646,20 @@
 #define S_update_sound                  "update-sound"
 #define S_update_time_graph             "update-time-graph"
 #define S_update_transform_graph        "update-transform-graph"
-#define S_vct_map                       "vct-map"
+#define S_variable_graph_p              "variable-graph?"
 #define S_vct_to_channel                "vct->channel"
-#define S_vct_to_sound_file             "vct->sound-file"
-#define S_verbose_cursor                "verbose-cursor"
+#define S_view_files_amp                "view-files-amp"
+#define S_view_files_amp_env            "view-files-amp-env"
 #define S_view_files_dialog             "view-files-dialog"
+#define S_view_files_files              "view-files-files"
+#define S_view_files_selected_files     "view-files-selected-files"
+#define S_view_files_select_hook        "view-files-select-hook"
+#define S_view_files_sort               "view-files-sort"
+#define S_view_files_speed              "view-files-speed"
+#define S_view_files_speed_style        "view-files-speed-style"
 #define S_view_mixes_dialog             "view-mixes-dialog"
 #define S_view_regions_dialog           "view-regions-dialog"
 #define S_view_sound                    "view-sound"
-#define S_view_tracks_dialog            "view-tracks-dialog"
-#define S_vu_font                       "vu-font"
-#define S_vu_font_size                  "vu-font-size"
-#define S_vu_size                       "vu-size"
 #define S_walsh_transform               "walsh-transform"
 #define S_wavelet_transform             "wavelet-transform"
 #define S_wavelet_type                  "wavelet-type"
@@ -657,28 +675,31 @@
 #define S_window_x                      "window-x"
 #define S_window_y                      "window-y"
 #define S_with_background_processes     "with-background-processes"
+#define S_with_file_monitor             "with-file-monitor"
 #define S_with_gl                       "with-gl"
 #define S_with_mix_tags                 "with-mix-tags"
 #define S_with_relative_panes           "with-relative-panes"
+#define S_with_tracking_cursor          "with-tracking-cursor"
+#define S_with_verbose_cursor           "with-verbose-cursor"
 #define S_write_peak_env_info_file      "write-peak-env-info-file"
-#define S_xen_to_sample                 "xen->sample"
-#define S_xen_to_sample_p               "xen->sample?"
-#define S_xramp_channel                 "xramp-channel"
-#define S_x_to_position                 "x->position"
+#define S_x_axis_as_clock               "x-axis-as-clock"
 #define S_x_axis_as_percentage          "x-axis-as-percentage"
 #define S_x_axis_in_beats               "x-axis-in-beats"
+#define S_x_axis_in_measures            "x-axis-in-measures"
 #define S_x_axis_in_samples             "x-axis-in-samples"
 #define S_x_axis_in_seconds             "x-axis-in-seconds"
 #define S_x_axis_label                  "x-axis-label"
 #define S_x_axis_style                  "x-axis-style"
 #define S_x_bounds                      "x-bounds"
 #define S_x_position_slider             "x-position-slider"
+#define S_x_to_position                 "x->position"
 #define S_x_zoom_slider                 "x-zoom-slider"
-#define S_y_to_position                 "y->position"
+#define S_xramp_channel                 "xramp-channel"
+#define S_y_axis_label                  "y-axis-label"
 #define S_y_bounds                      "y-bounds"
 #define S_y_position_slider             "y-position-slider"
+#define S_y_to_position                 "y->position"
 #define S_y_zoom_slider                 "y-zoom-slider"
-#define S_yes_or_no_p                   "yes-or-no?"
 #define S_zero_pad                      "zero-pad"
 #define S_zoom_color                    "zoom-color"
 #define S_zoom_focus_active             "zoom-focus-active"
