@@ -1,55 +1,35 @@
 #ifndef SND_X0_H
 #define SND_X0_H
 
-#include <config.h>
 #include <Xm/XmAll.h>
 #include <X11/keysym.h>
+#include <X11/cursorfont.h>
 
 #if HAVE_GL
 #include <GL/gl.h>
-#include <GL/glu.h>
 #include <GL/glx.h>
 #endif
 
-#if (XmVERSION >= 2)
-  #define USE_RENDITIONS 1
-#endif
-
-#if (USE_RENDITIONS)
-  #define xm_font_t XmRenderTable
-  #define XM_FONT_RESOURCE XmNrenderTable
-  #define XM_FONT_FREE XmRenderTableFree
-#else
-  #define xm_font_t XmFontList
-  #define XM_FONT_RESOURCE XmNfontList
-  #define XM_FONT_FREE XmFontListFree
-#endif
-
-#ifdef SGI
-  #define CLOSED_CTRLS_HEIGHT 22
-#else
-  #define CLOSED_CTRLS_HEIGHT 19
-#endif
+#define xm_font_t XmRenderTable
+#define XM_FONT_RESOURCE XmNrenderTable
+#define XM_FONT_FREE XmRenderTableFree
 
 #define LOTSA_PIXELS 10000
-#define XEN_WRAP_WIDGET(Value)       ((Value) ? XEN_LIST_2(C_STRING_TO_XEN_SYMBOL("Widget"), C_TO_XEN_ULONG((unsigned long)Value)) : XEN_FALSE)
-#define XEN_WRAP_WINDOW(Value)       ((Value) ? XEN_LIST_2(C_STRING_TO_XEN_SYMBOL("Window"), C_TO_XEN_ULONG((unsigned long)Value)) : XEN_FALSE)
-#define XEN_WRAP_GC(Value)           XEN_LIST_2(C_STRING_TO_XEN_SYMBOL("GC"), C_TO_XEN_ULONG((unsigned long)Value))
-#define XEN_WRAP_APPCONTEXT(Value)   XEN_LIST_2(C_STRING_TO_XEN_SYMBOL("XtAppContext"), C_TO_XEN_ULONG((unsigned long)Value))
-#define XEN_WRAP_PIXEL(Value)        XEN_LIST_2(C_STRING_TO_XEN_SYMBOL("Pixel"), C_TO_XEN_ULONG((unsigned long)Value))
-#define XEN_UNWRAP_WIDGET(Value)     (XEN_LIST_P(Value) ? XEN_TO_C_ULONG(XEN_CADR(Value)) : 0)
-#define XEN_UNWRAP_GC(Value)         XEN_TO_C_ULONG(XEN_CADR(Value))
-#define XEN_UNWRAP_PIXEL(Value)      XEN_TO_C_ULONG(XEN_CADR(Value))
+#define XEN_WRAP_WIDGET(Value)       ((Value) ? XEN_LIST_2(C_STRING_TO_XEN_SYMBOL("Widget"), XEN_WRAP_C_POINTER(Value)) : XEN_FALSE)
+#define XEN_WRAP_WINDOW(Value)       ((Value) ? XEN_LIST_2(C_STRING_TO_XEN_SYMBOL("Window"), C_TO_XEN_ULONG(Value)) : XEN_FALSE)
+#define XEN_WRAP_PIXEL(Value)        XEN_LIST_2(C_STRING_TO_XEN_SYMBOL("Pixel"), C_TO_XEN_INT((int)Value))
+                                     /* not ulong here! -- messes up the equal? check */
+#define XEN_UNWRAP_WIDGET(Value)     (XEN_LIST_P(Value) ? XEN_UNWRAP_C_POINTER(XEN_CADR(Value)) : 0)
+#define XEN_UNWRAP_PIXEL(Value)      (unsigned long)XEN_TO_C_INT(XEN_CADR(Value))
 #define XEN_WIDGET_P(Value)          (XEN_LIST_P(Value) && (XEN_LIST_LENGTH(Value) >= 2) && (XEN_SYMBOL_P(XEN_CAR(Value))) && \
                                       (strcmp("Widget", XEN_SYMBOL_TO_C_STRING(XEN_CAR(Value))) == 0))
 #define XEN_PIXEL_P(Value)           (XEN_LIST_P(Value) && (XEN_LIST_LENGTH(Value) >= 2) && (XEN_SYMBOL_P(XEN_CAR(Value))) && \
                                       (strcmp("Pixel", XEN_SYMBOL_TO_C_STRING(XEN_CAR(Value))) == 0))
+#define NULL_WIDGET NULL
 
-#define POINT_BUFFER_SIZE 4096
+typedef enum {NOT_ACTIVATABLE, ACTIVATABLE, NOT_ACTIVATABLE_OR_FOCUSED, ACTIVATABLE_BUT_NOT_FOCUSED} text_cr_t;
 
-typedef enum {NOT_ACTIVATABLE, ACTIVATABLE} text_cr_t;
-
-#ifdef SGI
+#ifdef MUS_SGI
   #define LINE_MARGIN 10
   #define CONTROLS_MARGIN 0
 #else
@@ -57,7 +37,7 @@ typedef enum {NOT_ACTIVATABLE, ACTIVATABLE} text_cr_t;
   #define CONTROLS_MARGIN 1
 #endif
 
-#define SCROLLBAR_MAX 1000
+#define SCROLLBAR_MAX 10000
 
 #define BACKGROUND_QUIT 1
 #define BACKGROUND_CONTINUE 0
@@ -65,7 +45,14 @@ typedef enum {NOT_ACTIVATABLE, ACTIVATABLE} text_cr_t;
 /* #define BACKGROUND_ADD(func, data) XtAppAddWorkProc(MAIN_APP(ss), func, (XtPointer)data) */
 #define BACKGROUND_ADD(func, data) add_work_proc(func, (XtPointer)data)
 
-#if DEBUGGING
+#define CALL_TIMEOUT(Func, Wait, Data) XtAppAddTimeOut(MAIN_APP(ss), Wait, Func, (XtPointer)Data)
+#define TIMEOUT_ARGS                   XtPointer context, XtIntervalId *id
+#define TIMEOUT_TYPE                   void
+#define TIMEOUT_RESULT
+#define timeout_result_t               XtIntervalId
+#define TIMEOUT_REMOVE(Id)             XtRemoveTimeOut(Id)
+
+#if MUS_DEBUGGING
   #define ASSERT_WIDGET_TYPE(Cond, Wid) if (!(Cond)) fprintf(stderr, "%s:[%s %d] %s is wrong type", c__FUNCTION__, __FILE__, __LINE__, XtName(Wid))
 #else
   #define ASSERT_WIDGET_TYPE(Cond, Wid)
@@ -73,13 +60,24 @@ typedef enum {NOT_ACTIVATABLE, ACTIVATABLE} text_cr_t;
 
 #define widget_t Widget
 
-#define Cessator XtWorkProcId
-#define Cessate Boolean
-#define Indicium XtPointer
-#define Tempus Time
-#define Locus short
-#define Latus unsigned short
-/* Position/Dimension in X terms */
+#define widget_is_active(Wid) XtIsManaged(Wid)
+#define activate_widget(Wid) XtManageChild(Wid)
+#define deactivate_widget(Wid) XtUnmanageChild(Wid)
+
+#define idle_t XtWorkProcId
+#define idle_func_t Boolean
+#define any_pointer_t XtPointer
+/* can't use "pointer_t" -- Mac already defines that type (CoreServices.h) */
+#define oclock_t Time
+#define color_t Pixel
+#define point_t XPoint
+#define gc_t GC
+
+#define rgb_t unsigned short
+#define RGB_MAX 65535
+#define FLOAT_TO_RGB(Val) (rgb_t)(RGB_MAX * (Val))
+#define RGB_TO_FLOAT(Val) (float)((float)(Val) / (float)RGB_MAX)
+
 
 typedef struct {
   GC gc;
@@ -91,24 +89,38 @@ typedef struct {
 typedef struct {
   /* we need two versions of each GC because the selected channel's colors can be different from the unselected channels' */
   Widget *chan_widgets;
-  Cessator fft_in_progress;
-  Cessator amp_env_in_progress;
-  void *amp_env_state;
+  idle_t fft_in_progress;
+  idle_t peak_env_in_progress;
+  struct env_state *peak_env_state;
   axis_context *ax;
   bool selected;
+  Pixmap fft_pix;
+  unsigned int fft_pix_width, fft_pix_height;
+  int fft_pix_x0, fft_pix_y0;
+  bool fft_pix_ready;
+  mus_float_t fft_pix_cutoff;
+  Cursor current_cursor;
 } chan_context;
 
 typedef struct {
-  Pixmap file_pix;
   Widget *snd_widgets;
+  Widget *progress_widgets;
+  int num_progress_widgets;
   Widget tab;
-  void *flt;
+  struct env_editor *flt;
   Widget dialog;
+  Dimension minibuffer_height;
+  bool minibuffer_watcher;
 } snd_context;
 
+typedef enum {NOT_A_SCANF_WIDGET, SRATE_WIDGET, CHANS_WIDGET, DATA_LOCATION_WIDGET, SAMPLES_WIDGET} scanf_widget_t;
+
 typedef struct {
-  Widget header_list, format_list, srate_text, chans_text, comment_text, location_text, samples_text;
+  Widget header_list, format_list, srate_text, chans_text, comment_text, location_text, samples_text, error_text, dialog, smenu;
   int current_type, current_format, formats, header_pos, format_pos;
+  scanf_widget_t scanf_widget, error_widget;
+  Widget *srates;
+  int num_srates, srates_size;
 } file_data;
 
 typedef struct {
@@ -130,11 +142,14 @@ typedef struct {
   XFontStruct *tiny_fontstruct;
 
   Pixel white, black, red, yellow, green, light_blue, lighter_blue;
-  Pixel data_color, selected_data_color, mark_color, graph_color, selected_graph_color, listener_color, listener_text_color, cursor_color;
-  Pixel basic_color, selection_color, zoom_color, position_color, highlight_color, enved_waveform_color;
+  Pixel data_color, selected_data_color, mark_color, graph_color, selected_graph_color, listener_color, listener_text_color;
+  Pixel basic_color, selection_color, zoom_color, position_color, highlight_color, enved_waveform_color, cursor_color;
   Pixel text_focus_color, filter_control_waveform_color, mix_color, pushed_button_color, sash_color;
   Pixel help_button_color, doit_again_button_color, doit_button_color, quit_button_color, reset_button_color;
-  Pixel selected_grid_color, grid_color;
+  Pixel selected_grid_color, grid_color, axis_color;
+  Pixel orig_data_color, orig_selected_data_color, orig_mark_color, orig_mix_color;
+  Pixel orig_graph_color, orig_selected_graph_color, orig_listener_color, orig_listener_text_color, orig_cursor_color;
+  Pixel orig_basic_color, orig_selection_color, orig_zoom_color, orig_position_color, orig_highlight_color;
 
   GC basic_gc, selected_basic_gc, combined_basic_gc;        
   GC cursor_gc, selected_cursor_gc;      
@@ -145,39 +160,26 @@ typedef struct {
   GC fltenv_basic_gc, fltenv_data_gc;
   Widget listener_pane;
   Widget *dialogs;
-  Cursor graph_cursor, wait_cursor;
-  Widget completion_requestor;
+  int num_dialogs, dialogs_size;
+  Cursor graph_cursor, wait_cursor, bounds_cursor;
+  Widget requestor_dialog;
 #if HAVE_GL
   GLXContext cx;
 #endif
+  XtInputId fam_port;
+  Widget *mw, *pw;
+  bool axis_color_set;
 } state_context;
 
-typedef struct {
-  Widget graph;
-  XPoint *p0, *p1;
-  int lastpj;
-  Pixel color;
-} mix_context;
-
-typedef enum {WITHOUT_PANED_WINDOW, WITH_PANED_WINDOW} dialog_paned_t;
-typedef enum {DONT_PAD_TITLE, PAD_TITLE_ON_RIGHT, PAD_TITLE_ON_LEFT} dialog_pad_t;
-typedef enum {WITHOUT_SORT_BUTTON, WITH_SORT_BUTTON} dialog_sort_t;
-
-typedef struct {
-  Widget ww;
-  Widget list;
-  Widget plw;
-  Widget bydate;
-  Widget bysize;
-  Widget byname;
-  Widget byentry;
-  Widget byproc;
-  Widget panes, toppane;
-} ww_info;
+typedef enum {WITHOUT_CHANNELS_FIELD, WITH_CHANNELS_FIELD, WITH_EXTRACT_CHANNELS_FIELD} dialog_channels_t;
+typedef enum {WITHOUT_SAMPLES_FIELD, WITH_SAMPLES_FIELD} dialog_samples_t;
+typedef enum {WITHOUT_DATA_LOCATION_FIELD, WITH_DATA_LOCATION_FIELD} dialog_data_location_t;
+typedef enum {WITHOUT_HEADER_TYPE_FIELD, WITH_HEADER_TYPE_FIELD} dialog_header_type_t;
+typedef enum {WITHOUT_COMMENT_FIELD, WITH_COMMENT_FIELD} dialog_comment_t;
 
 #define snd_ShiftMask ShiftMask
 #define snd_ControlMask ControlMask
-#ifndef SUN
+#ifndef MUS_SUN
   #define snd_MetaMask Mod1Mask
 #else
   #define snd_MetaMask (Mod1Mask | Mod4Mask)
@@ -189,13 +191,20 @@ typedef struct {
 #define SOUND_PANE_BOX(a) (a->sgx)->soundpanebox
 #define MAIN_APP(a) (a->sgx)->mainapp
 #define MAIN_DISPLAY(a) (a->sgx)->mdpy
-#define PEAK_NUMBERS_FONT(a) (a->sgx)->peaks_fontstruct
-#define BOLD_PEAK_NUMBERS_FONT(a) (a->sgx)->bold_peaks_fontstruct
+#define PEAKS_FONT(a) (a->sgx)->peaks_fontstruct
+#define BOLD_PEAKS_FONT(a) (a->sgx)->bold_peaks_fontstruct
 #define AXIS_NUMBERS_FONT(a) (a->sgx)->axis_numbers_fontstruct
 #define AXIS_LABEL_FONT(a) (a->sgx)->axis_label_fontstruct
-#define TINY_NUMBERS_FONT(a) (a->sgx)->tiny_fontstruct
-#define color_t Pixel
-/* this was unsigned long = Pixel (/usr/X11R6/include/X11/Intrinsic.h) */
+#define TINY_FONT(a) (a->sgx)->tiny_fontstruct
+#define LISTENER_FONT(a) (a->sgx)->listener_fontstruct
+
+#define DEFAULT_GRAPH_CURSOR XC_crosshair
+
+#define DEFAULT_TINY_FONT "6x12"
+#define DEFAULT_PEAKS_FONT "-*-times-medium-r-*-*-14-*-*-*-*-*-*-*"
+#define DEFAULT_BOLD_PEAKS_FONT "-*-times-bold-r-*-*-14-*-*-*-*-*-*-*"
+#define DEFAULT_AXIS_NUMBERS_FONT "9x15"
+#define DEFAULT_AXIS_LABEL_FONT "-*-times-medium-r-*-*-18-*-*-*-*-*-*-*"
 
 #define KEY_TO_NAME(key) XKeysymToString(key)
 /* on the Sun, if key is 0, XKeysymToString segfaults! */
@@ -283,14 +292,8 @@ typedef struct {
 #define snd_K_Right XK_Right	 
 #define snd_K_Down XK_Down		 
 
-#define snd_keypad_PageUp XK_KP_Page_Up
-#define snd_keypad_PageDown XK_KP_Page_Down
 #define snd_keypad_Insert XK_KP_Insert
 #define snd_keypad_Delete XK_KP_Delete
-#define snd_keypad_Left XK_KP_Left
-#define snd_keypad_Up XK_KP_Up
-#define snd_keypad_Right XK_KP_Right
-#define snd_keypad_Down XK_KP_Down
 #define snd_keypad_Multiply XK_KP_Multiply
 #define snd_keypad_Add XK_KP_Add
 #define snd_keypad_Subtract XK_KP_Subtract
