@@ -1,7 +1,7 @@
 #ifndef SND_H
 #define SND_H
 
-#include <config.h>
+#include <mus-config.h>
 
 #include <ctype.h>
 #include <stddef.h>
@@ -26,19 +26,23 @@
 #if HAVE_STRING_H
   #include <string.h>
 #endif
-#if HAVE_VPRINTF
-  #include <stdarg.h>
-#endif
+#include <stdarg.h>
 #include <time.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#if HAVE_LOCALE_H && HAVE_SETLOCALE
+#if HAVE_SETLOCALE
   #include <locale.h>
 #endif
+#if HAVE_FAM_H
+  #include <fam.h>
+#endif
+#if HAVE_PTHREAD_H
+  #include <pthread.h>
+#endif
 
-#include "sndlib.h"
-#include "clm.h"
+#include "_sndlib.h"
 #include "xen.h"
+#include "clm.h"
 #include "sndlib2xen.h"
 #include "vct.h"
 #include "snd-0.h"
@@ -67,9 +71,11 @@
 
 #include "snd-strings.h"
 
-#define SND_DATE "3-Jan-05"
-#define SND_VERSION "7.9"
-#define SND_MAJOR_VERSION "7"
-#define SND_MINOR_VERSION "9"
+#define SND_DATE "30-Nov-09"
+#ifndef SND_VERSION
+#define SND_VERSION "11.1"
+#endif
+#define SND_MAJOR_VERSION "11"
+#define SND_MINOR_VERSION "1"
 
 #endif
