@@ -1,13 +1,14 @@
 /* sndinfo describes sounds */
 
-#include <config.h>
+#include <mus-config.h>
+
 #include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
 #if (defined(HAVE_LIBC_H) && (!defined(HAVE_UNISTD_H)))
   #include <libc.h>
 #else
-  #if (!(defined(_MSC_VER))) && (!(defined(MPW_C)))
+  #if (!(defined(_MSC_VER)))
     #include <unistd.h>
   #endif
   #include <string.h>
@@ -17,60 +18,41 @@
 
 #include "sndlib.h"
 
-#if MACOS
-  #include <console.h>
-#endif
-
-#if DEBUGGING
-void *mem_calloc(size_t len, size_t size, const char *func, const char *file, int line) {return(calloc(len, size));}
-void *mem_malloc(size_t len, const char *func, const char *file, int line) {return(malloc(len));}
-void mem_free(void *ptr, const char *func, const char *file, int line) {free(ptr);}
-void *mem_realloc(void *ptr, size_t size, const char *func, const char *file, int line) {return(realloc(ptr, size));}
-#include <fcntl.h>
-int io_open(const char *pathname, int flags, mode_t mode, const char *func, const char *file, int line) {return(open(pathname, flags, mode));}
-int io_creat(const char *pathname, mode_t mode, const char *func, const char *file, int line) {return(creat(pathname, mode));}
-int io_close(int fd, const char *func, const char *file, int line) {return(close(fd));}
-FILE *io_fopen(const char *path, const char *mode, const char *func, const char *file, int line) {return(fopen(path, mode));}
-int io_fclose(FILE *stream, const char *func, const char *file, int line) {return(fclose(stream));}
-char *copy_string(const char *str);
-char *copy_string(const char *str) {return(strdup(str));}
-#endif
-
 static char *display_maxamps(const char *filename, int chans)
 {
   char *ampstr;
   char fstr[16];
-  int i;
+  int i, len;
   mus_sample_t *vals;
-  off_t *times;
-  ampstr = (char *)CALLOC(chans * 32, sizeof(char));
-  vals = (mus_sample_t *)CALLOC(chans, sizeof(mus_sample_t));
-  times = (off_t *)CALLOC(chans, sizeof(off_t));
-  sprintf(ampstr, "\n  max amp%s: ", (chans > 1) ? "s" : "");
+  mus_long_t *times;
+
+  len = chans * 32;
+  ampstr = (char *)calloc(len, sizeof(char));
+  vals = (mus_sample_t *)calloc(chans, sizeof(mus_sample_t));
+  times = (mus_long_t *)calloc(chans, sizeof(mus_long_t));
+
+  snprintf(ampstr, len, "\n  max amp%s: ", (chans > 1) ? "s" : "");
   mus_sound_maxamps(filename, chans, vals, times);
   for (i = 0; i < chans; i++)
     {
-      sprintf(fstr, "%.3f ", MUS_SAMPLE_TO_FLOAT(vals[i]));
+      snprintf(fstr, 16, "%.3f ", MUS_SAMPLE_TO_FLOAT(vals[i]));
       strcat(ampstr, fstr);
     }
-  FREE(vals);
-  FREE(times);
+  free(vals);
+  free(times);
   return(ampstr);
 }
 
 int main(int argc, char *argv[])
 {
   int chans, srate, format, type, ctr;
-  off_t samples;
+  mus_long_t samples;
   float length = 0.0;
   time_t date;
   int *loops = NULL;
   char *comment, *header_name;
   char *format_info = NULL, *format_name, *ampstr = NULL;
   char timestr[64];
-#if MACOS
-  argc = ccommand(&argv);
-#endif
   if (argc == 1) {printf("usage: sndinfo file\n"); exit(0);}
   mus_sound_initialize();
   for (ctr = 1; ctr < argc; ctr++)
@@ -101,8 +83,8 @@ int main(int argc, char *argv[])
 	      format = mus_sound_original_format(argv[ctr]);
 	      format_name = (char *)mus_header_original_format_name(format, type);
 	      if (format_name)
-		sprintf(format_info, "%d (%s)", format, format_name);
-	      else sprintf(format_info, "%d", format);
+		snprintf(format_info, 64, "%d (%s)", format, format_name);
+	      else snprintf(format_info, 64, "%d", format);
 	    }
 	  fprintf(stdout, "%s:\n  srate: %d\n  chans: %d\n  length: %f",
 		  argv[ctr], srate, chans, length);
@@ -116,10 +98,10 @@ int main(int argc, char *argv[])
 	  fprintf(stdout, "  type: %s\n  format: %s\n  ",
 		  header_name,
 		  format_info);
-#if HAVE_STRFTIME
+
 	  strftime(timestr, 64, "%a %d-%b-%Y %H:%M %Z", localtime(&date));
 	  fprintf(stdout, "written: %s", timestr);
-#endif
+
 	  if ((chans > 0) && (mus_sound_maxamp_exists(argv[ctr])))
 	    {
 	      ampstr = display_maxamps(argv[ctr], chans);
