@@ -1,29 +1,27 @@
 /* sndplay plays sounds */
 
-#include <config.h>
+#include <mus-config.h>
+
 #include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
+
+#include "_sndlib.h"
+
 #if (defined(HAVE_LIBC_H) && (!defined(HAVE_UNISTD_H)))
   #include <libc.h>
 #else
-  #if (!(defined(_MSC_VER))) && (!(defined(MPW_C)))
+  #if (!(defined(_MSC_VER)))
     #include <unistd.h>
   #endif
   #include <string.h>
 #endif
 #include <errno.h>
 
-#include "sndlib.h"
-
-#if MACOS
-  #include <console.h>
-#endif
-
-#if MAC_OSX
+#if MUS_MAC_OSX
   #define BUFFER_SIZE 256
 #else
-#if WIN32
+#if MUS_WINDOZE
   /* this setting from Scott Middleton (actually used 8096) */
   #define BUFFER_SIZE 8192
 #else
@@ -31,7 +29,7 @@
 #endif
 #endif
 
-#if MAC_OSX
+#if MUS_MAC_OSX
   #define OutSample float
   #define MUS_CONVERT(samp) MUS_SAMPLE_TO_FLOAT(samp)
 #else
@@ -49,44 +47,39 @@ static char x_string[2] = {'x','\0'};
   a = atoi(arg);
   arg = strtok(NULL, x_string);
   b = atoi(arg);
-  mus_audio_set_oss_buffers(a, b);
+  mus_oss_set_buffers(a, b);
 #endif
 }
 
+static void set_volume(mus_sample_t **buf, int chans, int length, double volume)
+{ 
+  int n, ch; 
+  for (ch = 0; ch < chans; ch++) 
+    for (n = 0; n < length; n++) 
+      buf[ch][n] *= volume; 
+}
+
 /* special case multicard quad code split out for clarity (it could be folded into the main branches) */
 
-#if DEBUGGING
-void *mem_calloc(size_t len, size_t size, const char *func, const char *file, int line) {return(calloc(len, size));}
-void *mem_malloc(size_t len, const char *func, const char *file, int line) {return(malloc(len));}
-void mem_free(void *ptr, const char *func, const char *file, int line) {free(ptr);}
-void *mem_realloc(void *ptr, size_t size, const char *func, const char *file, int line) {return(realloc(ptr, size));}
-#include <fcntl.h>
-int io_open(const char *pathname, int flags, mode_t mode, const char *func, const char *file, int line) {return(open(pathname, flags, mode));}
-int io_creat(const char *pathname, mode_t mode, const char *func, const char *file, int line) {return(creat(pathname, mode));}
-int io_close(int fd, const char *func, const char *file, int line) {return(close(fd));}
-FILE *io_fopen(const char *path, const char *mode, const char *func, const char *file, int line) {return(fopen(path, mode));}
-int io_fclose(FILE *stream, const char *func, const char *file, int line) {return(fclose(stream));}
-char *copy_string(const char *str);
-char *copy_string(const char *str) {return(strdup(str));}
-#endif
-
-/* 22-Nov-00: moved alsa support to separate block */
-/* 8-Apr-04:  added start/end (seconds-based) args */
+/* 22-Nov-00:  moved alsa support to separate block */
+/* 8-Apr-04:   added start/end (seconds-based) args */
+/* 2-Nov-05:   added -volume arg */
+/* 22-July-08: added -mutable arg */
 
 static int main_not_alsa(int argc, char *argv[])
 {
   int fd, afd, i, j, n, k, chans, srate;
-  off_t frames, m;
+  mus_long_t frames, m;
   mus_sample_t **bufs;
   OutSample *obuf;
-  float val[1];
-  int use_multi_card_code = 0;
+  int use_multi_card_code = 0, use_volume = 0;
   int afd0, afd1, buffer_size = BUFFER_SIZE, curframes, sample_size, out_chans, outbytes;
   mus_sample_t **qbufs;
   short *obuf0, *obuf1;
   char *name = NULL;
-  off_t start = 0, end = 0;
-  double begin_time = 0.0, end_time = 0.0;
+  mus_long_t start = 0, end = 0;
+  double begin_time = 0.0, end_time = 0.0, volume = 1.0;
+  int mutate = 1, include_mutate = 0;
 
   for (i = 1; i < argc; i++)
     {
@@ -123,25 +116,41 @@ static int main_not_alsa(int argc, char *argv[])
 			  end_time = atof(argv[i + 1]);
 			  i++;
 			}
-		      else name = argv[i];
-		    }}}}}
+		      else 
+			{
+			  if (strcmp(argv[i], "-mutable") == 0) 
+			    {
+			      mutate = atoi(argv[i + 1]);
+			      include_mutate = 1;
+			      i++;
+			    }
+			  else 
+			    { 
+			      if (strcmp(argv[i], "-volume") == 0)
+				{ 
+				  volume = atof(argv[i + 1]);
+				  use_volume = 1;
+				  i++; 
+				} 
+			      else name = argv[i];
+			    }}}}}}}
   if (name == NULL) 
     {
-      printf("usage: sndplay file [-start 1.0] [-end 1.0] [-bufsize 256] [-buffers 2x12] [-describe]\n"); 
+      printf("usage: sndplay file [-start 1.0] [-end 1.0] [-bufsize %d] [-buffers 2x12] [-volume 1.0] [-mutable 1] [-describe]\n", BUFFER_SIZE); 
       exit(0);
     }
 
   afd = -1;
   afd0 = -1;
   afd1 = -1;
-  if (!(MUS_HEADER_TYPE_OK(mus_sound_header_type(name))))
+  if (!(mus_header_type_p(mus_sound_header_type(name))))
     {
       fprintf(stderr, "can't play %s (header type: %s?)\n",
 	      name,
 	      mus_header_type_name(mus_header_type()));
       exit(0);
     }
-  if (!(MUS_DATA_FORMAT_OK(mus_sound_data_format(name))))
+  if (!(mus_data_format_p(mus_sound_data_format(name))))
     {
       fprintf(stderr, "can't play %s (data format: %s (%s)?)\n",
 	      name,
@@ -156,6 +165,7 @@ static int main_not_alsa(int argc, char *argv[])
       chans = mus_sound_chans(name);
       if (chans > 2)
 	{
+	  float val[8];
 	  mus_audio_mixer_read(MUS_AUDIO_DEFAULT, MUS_AUDIO_CHANNEL, 0, val);
 	  if (val[0] < chans)
 	    {
@@ -172,29 +182,31 @@ static int main_not_alsa(int argc, char *argv[])
       out_chans = chans;
       srate = mus_sound_srate(name);
       frames = mus_sound_frames(name);
-      sample_size = mus_bytes_per_sample(MUS_COMPATIBLE_FORMAT);
-      start = (off_t)(begin_time * srate);
+      sample_size = mus_bytes_per_sample(MUS_AUDIO_COMPATIBLE_FORMAT);
+      start = (mus_long_t)(begin_time * srate);
       if (start > 0)
-	mus_sound_seek_frame(fd, start);
+	mus_file_seek_frame(fd, start);
       if (end_time > 0.0)
-	end = (off_t)(end_time * srate);
+	end = (mus_long_t)(end_time * srate);
       else end = frames;
       if ((end - start) < frames)
 	frames = end - start;
       if (!use_multi_card_code)
 	{
-	  bufs = (mus_sample_t **)CALLOC(chans, sizeof(mus_sample_t *));
-	  for (i = 0; i < chans; i++) bufs[i] = (mus_sample_t *)CALLOC(buffer_size, sizeof(mus_sample_t));
-	  obuf = (OutSample *)CALLOC(buffer_size * out_chans, sizeof(OutSample));
+	  bufs = (mus_sample_t **)calloc(chans, sizeof(mus_sample_t *));
+	  for (i = 0; i < chans; i++) bufs[i] = (mus_sample_t *)calloc(buffer_size, sizeof(mus_sample_t));
+	  obuf = (OutSample *)calloc(buffer_size * out_chans, sizeof(OutSample));
 	  outbytes = buffer_size * out_chans * sample_size;
 	  for (m = 0; m < frames; m += buffer_size)
 	    {
 	      if ((m + buffer_size) <= frames)
 		curframes = buffer_size;
 	      else curframes = frames - m;
-	      mus_sound_read(fd, 0, curframes - 1, chans, bufs); 
+	      mus_file_read(fd, 0, curframes - 1, chans, bufs); 
 	      /* some systems are happier if we read the file before opening the dac */
-	      /* at this point the data is in separate arrays of ints */
+	      /* at this point the data is in separate arrays of mus_sample_t's */
+	      if (use_volume) 
+		set_volume(bufs, chans, curframes, volume); 
 	      if (chans == 1)
 		{
 		  for (k = 0; k < curframes; k++) 
@@ -219,12 +231,16 @@ static int main_not_alsa(int argc, char *argv[])
 			}
 		    }
 		}
+#if MUS_MAC_OSX
+	      if (include_mutate == 1)
+		mus_audio_output_properties_mutable(mutate);
+#endif
 	      if (afd == -1)
 		{
-#if defined(LINUX) && defined(PPC)
-		  afd = mus_audio_open_output(MUS_AUDIO_DEFAULT, srate, chans, MUS_COMPATIBLE_FORMAT, 0);
+#if defined(MUS_LINUX) && defined(PPC)
+		  afd = mus_audio_open_output(MUS_AUDIO_DEFAULT, srate, chans, MUS_AUDIO_COMPATIBLE_FORMAT, 0);
 #else
-		  afd = mus_audio_open_output(MUS_AUDIO_DEFAULT, srate, out_chans, MUS_COMPATIBLE_FORMAT, outbytes);
+		  afd = mus_audio_open_output(MUS_AUDIO_DEFAULT, srate, out_chans, MUS_AUDIO_COMPATIBLE_FORMAT, outbytes);
 #endif
 		  if (afd == -1) break;
 		}
@@ -233,9 +249,9 @@ static int main_not_alsa(int argc, char *argv[])
 	    }
 	  if (afd != -1) mus_audio_close(afd);
 	  mus_sound_close_input(fd);
-	  for (i = 0; i < chans; i++) FREE(bufs[i]);
-	  FREE(bufs);
-	  FREE(obuf);
+	  for (i = 0; i < chans; i++) free(bufs[i]);
+	  free(bufs);
+	  free(obuf);
 	}
       else
 	{
@@ -255,17 +271,18 @@ static int main_not_alsa(int argc, char *argv[])
 	   */
 	  buffer_size = 256;   /* 128 probably better */
 	  outbytes = buffer_size * 2 * 2; 
-	  qbufs = (mus_sample_t **)CALLOC(chans, sizeof(mus_sample_t *));
-	  for (i = 0; i < chans; i++) qbufs[i] = (mus_sample_t *)CALLOC(buffer_size, sizeof(mus_sample_t));
-	  obuf0 = (short *)CALLOC(buffer_size * 2, sizeof(short));
-	  obuf1 = (short *)CALLOC(buffer_size * 2, sizeof(short));
+	  qbufs = (mus_sample_t **)calloc(chans, sizeof(mus_sample_t *));
+	  for (i = 0; i < chans; i++) qbufs[i] = (mus_sample_t *)calloc(buffer_size, sizeof(mus_sample_t));
+	  obuf0 = (short *)calloc(buffer_size * 2, sizeof(short));
+	  obuf1 = (short *)calloc(buffer_size * 2, sizeof(short));
 	  for (m = 0; m < frames; m += buffer_size)
 	    {
 	      if ((m + buffer_size) <= frames)
 		curframes = buffer_size;
 	      else curframes = frames - m;
-	      mus_sound_read(fd, 0, curframes - 1, chans, qbufs); 
-	      val[0] = 1.0;
+	      mus_file_read(fd, 0, curframes - 1, chans, qbufs); 
+	      if (use_volume) 
+		set_volume(qbufs, chans, curframes, volume); 
 	      for (k = 0, n = 0; k < buffer_size; k++, n += 2) 
 		{
 		  obuf0[n] = MUS_SAMPLE_TO_SHORT(qbufs[0][k]); 
@@ -275,8 +292,8 @@ static int main_not_alsa(int argc, char *argv[])
 		}
 	      if (afd0 == -1)
 		{
-		  afd0 = mus_audio_open_output(MUS_AUDIO_PACK_SYSTEM(0) | MUS_AUDIO_DEFAULT, srate, 2, MUS_COMPATIBLE_FORMAT, outbytes);
-		  afd1 = mus_audio_open_output(MUS_AUDIO_PACK_SYSTEM(1) | MUS_AUDIO_DEFAULT, srate, 2, MUS_COMPATIBLE_FORMAT, outbytes);
+		  afd0 = mus_audio_open_output(MUS_AUDIO_PACK_SYSTEM(0) | MUS_AUDIO_DEFAULT, srate, 2, MUS_AUDIO_COMPATIBLE_FORMAT, outbytes);
+		  afd1 = mus_audio_open_output(MUS_AUDIO_PACK_SYSTEM(1) | MUS_AUDIO_DEFAULT, srate, 2, MUS_AUDIO_COMPATIBLE_FORMAT, outbytes);
 		  if ((afd0 == -1) || (afd1 == -1)) break;
 		}
 	      mus_audio_write(afd0, (char *)obuf0, outbytes);
@@ -285,10 +302,10 @@ static int main_not_alsa(int argc, char *argv[])
 	  mus_audio_close(afd0);
 	  mus_audio_close(afd1);
 	  mus_sound_close_input(fd);
-	  for (i = 0; i < chans; i++) FREE(qbufs[i]);
-	  FREE(qbufs);
-	  FREE(obuf0);
-	  FREE(obuf1);
+	  for (i = 0; i < chans; i++) free(qbufs[i]);
+	  free(qbufs);
+	  free(obuf0);
+	  free(obuf1);
 	}
     }
   return(0);
@@ -307,29 +324,29 @@ static int use_one_device = 1;
 static int main_alsa(int argc, char *argv[])
 {
   int fd, i, chans, srate;
-  off_t frames, ioff;
+  mus_long_t frames, ioff;
   mus_sample_t **read_bufs;
   int afd[MAX_SLOTS];
   short *out_buf[MAX_SLOTS];
   float val[MAX_SLOTS];
-  int ival[MAX_SLOTS];
   int afd0, afd1;
-  char *name;
+  char *name = NULL;
   int base, curframes;
   int allocated;
   int out_devs[MAX_SLOTS];
   int out_chans[MAX_SLOTS];
   int out_format[MAX_SLOTS];
   int out_bytes[MAX_SLOTS];
-  int samples_per_chan;
+  int samples_per_chan = 0;
   int last_device;
   int devices[MAX_SLOTS];
   int available_chans[MAX_SLOTS];
   int min_chans[MAX_SLOTS];
   int max_chans[MAX_SLOTS];
   int alloc_chans;
-  off_t start = 0, end = 0;
-  double begin_time = 0.0, end_time = 0.0;
+  mus_long_t start = 0, end = 0;
+  double begin_time = 0.0, end_time = 0.0, volume = 1.0;
+  int use_volume = 0;
 
   /* -describe => call mus_audio_describe and exit
    * -buffers axb => set OSS fragment numbers 
@@ -362,18 +379,26 @@ static int main_alsa(int argc, char *argv[])
 		      end_time = atof(argv[i + 1]);
 		      i++;
 		    }
-		  else name = argv[i];
-		}}}}
+		  else
+		    {
+		      if (strcmp(argv[i], "-volume") == 0)
+			{ 
+			  volume = atof(argv[i + 1]);
+			  use_volume = 1;
+			  i++; 
+			} 
+		      else name = argv[i];
+		    }}}}}
   afd0 = -1;
   afd1 = -1;
-  if (!(MUS_HEADER_TYPE_OK(mus_sound_header_type(name))))
+  if (!(mus_header_type_p(mus_sound_header_type(name))))
     {
       fprintf(stderr, "can't play %s (header type: %s?)\n",
 	      name,
 	      mus_header_type_name(mus_header_type()));
       exit(0);
     }
-  if (!(MUS_DATA_FORMAT_OK(mus_sound_data_format(name))))
+  if (!(mus_data_format_p(mus_sound_data_format(name))))
     {
       fprintf(stderr, "can't play %s (data format: %s (%s)?)\n",
 	      name,
@@ -498,26 +523,26 @@ static int main_alsa(int argc, char *argv[])
       srate = mus_sound_srate(name);
       frames = mus_sound_frames(name);
       base = 0;
-      start = (off_t)(begin_time * srate);
+      start = (mus_long_t)(begin_time * srate);
       if (start > 0)
-	mus_sound_seek_frame(fd, start);
+	mus_file_seek_frame(fd, start);
       if (end_time > 0.0)
-	end = (off_t)(end_time * srate);
+	end = (mus_long_t)(end_time * srate);
       else end = frames;
       if ((end - start) < frames)
 	frames = end - start;
       /* allocate the list of read buffers, each buffer will hold one channel
 	 of the input soundfile, each sample is going to be mus_sample_t
       */
-      read_bufs = (mus_sample_t **)CALLOC(alloc_chans, sizeof(mus_sample_t *));
+      read_bufs = (mus_sample_t **)calloc(alloc_chans, sizeof(mus_sample_t *));
       for (d = 0; d < allocated; d++)
 	{
 	  int dev = out_devs[d];
 	  for (i = 0; i < out_chans[d]; i++) 
-	    read_bufs[base + i] = (mus_sample_t *)CALLOC(samples_per_chan, sizeof(mus_sample_t));
+	    read_bufs[base + i] = (mus_sample_t *)calloc(samples_per_chan, sizeof(mus_sample_t));
 	  base += out_chans[d];
 	  out_bytes[dev] = samples_per_chan * out_chans[d] * mus_bytes_per_sample(out_format[dev]);
-	  out_buf[dev] = (short *)CALLOC(out_bytes[dev], 1);
+	  out_buf[dev] = (short *)calloc(out_bytes[dev], 1);
 	}
       for (ioff = 0; ioff < frames; ioff += samples_per_chan)
 	{
@@ -538,7 +563,9 @@ static int main_alsa(int argc, char *argv[])
 		  out_bytes[dev] = curframes * out_chans[d] * mus_bytes_per_sample(out_format[dev]);
 		}
 	    }
-	  mus_sound_read(fd, 0, curframes - 1, chans, read_bufs); 
+	  mus_file_read(fd, 0, curframes - 1, chans, read_bufs); 
+	  if (use_volume) 
+	    set_volume(read_bufs, chans, curframes, volume); 
 	  /* some systems are happier if we read the file before opening the dac */
 	  /* at this point the data is in separate arrays of mus_sample_t */
 	  for (d = 0; d < allocated; d++)
@@ -569,12 +596,12 @@ static int main_alsa(int argc, char *argv[])
 	  if (afd[dev] != -1) mus_audio_close(afd[dev]);
 	}
       mus_sound_close_input(fd);
-      for (i = 0; i < alloc_chans; i++) FREE(read_bufs[i]);
-      FREE(read_bufs);
+      for (i = 0; i < alloc_chans; i++) free(read_bufs[i]);
+      free(read_bufs);
       for (d = 0; d < allocated; d++)
 	{
 	  int dev = out_devs[d];
-	  FREE(out_buf[dev]);
+	  free(out_buf[dev]);
 	}
     }
   return(0);
@@ -584,19 +611,15 @@ static int main_alsa(int argc, char *argv[])
 
 int main(int argc, char *argv[])
 {
-#if MACOS
-  argc = ccommand(&argv);
-#endif
-
   if (argc == 1) 
     {
-      printf("usage: sndplay file\n"); 
+      printf("usage: sndplay file [-start 1.0] [-end 1.0] [-bufsize %d] [-buffers 2x12] [-volume 1.0] [-describe]\n", BUFFER_SIZE); 
       exit(0);
     }
   mus_sound_initialize();
  
 #if HAVE_ALSA
-  if (mus_audio_api() == ALSA_API)
+  if (mus_audio_api() == MUS_ALSA_API)
     {
       main_alsa(argc, argv);
       return(0);
