@@ -1,19 +1,6 @@
-#!/usr/local/bin/guile -s
-!#
-
-;;; makegl.scm creates the GL/GLU bindings using gldata.scm, writes gl.c and gl-ruby.c
-
-(use-modules (ice-9 debug))
-(use-modules (ice-9 format))
-(use-modules (ice-9 optargs))
-(use-modules (ice-9 common-list))
-
-(debug-enable 'debug)
-(debug-enable 'backtrace)
-(read-enable 'positions)
+;;; makegl.scm creates the GL/GLU bindings using gldata.scm, writes gl.c
 
 (define gl-file (open-output-file "gl.c"))
-(define gl-ruby-file (open-output-file "gl-ruby.c"))
 
 (define (hey . args)
   (display (apply format #f args) gl-file))
@@ -21,13 +8,6 @@
 (define (heyc arg)
   (display arg gl-file))
 
-(define (say . args)
-  (display (apply format #f args) gl-ruby-file))
-
-(define (say-hey . args)
-  (apply hey args)
-  (apply say args))
-
 (define names '())
 (define types '())
 (define ints '())
@@ -36,61 +16,80 @@
 (define x-types '())
 (define x-funcs '())
 (define x-ints '())
-(define g-types '())
-(define g-funcs '())
-(define g-ints '())
-(define g5-funcs '())
-(define g5-ints '())
+;(define g-types '())
+;(define g-funcs '())
+;(define g-ints '())
+;(define g5-funcs '())
+;(define g5-ints '())
+
+(define in-glu #f)
+
+(define (check-glu name)
+  (if in-glu
+      (if (not (string=? "glu" (substring name 0 3)))
+	  (begin
+	    (set! in-glu #f)
+	    (hey "#endif~%")))
+      (if (string=? "glu" (substring name 0 3))
+	  (begin
+	    (set! in-glu #t)
+	    (hey "#if HAVE_GLU~%")))))
+
+(define (uncheck-glu)
+  (if in-glu
+      (begin
+	(hey "#endif~%")
+	(set! in-glu #f))))
 
 (define (cadr-str data)
   (let ((sp1 -1)
 	(len (string-length data)))
-    (call-with-current-continuation
+    (call-with-exit
      (lambda (return)
-       (do ((i 0 (1+ i)))
+       (do ((i 0 (+ 1 i)))
 	   ((= i len) (substring data sp1))
 	 (if (char=? (string-ref data i) #\space)
 	     (if (= sp1 -1)
 		 (set! sp1 i)
-		 (return (substring data (1+ sp1) i)))))))))
+		 (return (substring data (+ 1 sp1) i)))))))))
 
 (define (caddr-str data)
   (let ((sp1 -1)
 	(sp2 -1)
 	(len (string-length data)))
-    (call-with-current-continuation
+    (call-with-exit
      (lambda (return)
-       (do ((i 0 (1+ i)))
+       (do ((i 0 (+ 1 i)))
 	   ((= i len) (substring data sp2))
 	 (if (char=? (string-ref data i) #\space)
 	     (if (= sp1 -1)
 		 (set! sp1 i)
 		 (if (= sp2 -1)
 		     (set! sp2 i)
-		     (return (substring data (1+ sp2)))))))))))
+		     (return (substring data (+ 1 sp2)))))))))))
 
 (define (car-str data)
   (let ((len (string-length data)))
-    (call-with-current-continuation
+    (call-with-exit
      (lambda (return)
-       (do ((i 0 (1+ i)))
+       (do ((i 0 (+ 1 i)))
 	   ((= i len) data)
 	 (if (char=? (string-ref data i) #\space)
 	     (return (substring data 0 i))))))))
 
 (define (cdr-str data)
   (let ((len (string-length data)))
-    (call-with-current-continuation
+    (call-with-exit
      (lambda (return)
-       (do ((i 0 (1+ i)))
+       (do ((i 0 (+ 1 i)))
 	   ((= i len) data)
 	 (if (char=? (string-ref data i) #\space)
-	     (return (substring data (1+ i)))))))))
+	     (return (substring data (+ 1 i)))))))))
 
 (define (string-upcase name)
   (let* ((len (string-length name))
 	 (str (make-string len)))
-    (do ((i 0 (1+ i)))
+    (do ((i 0 (+ 1 i)))
 	((= i len))
       (string-set! str i (char-upcase (string-ref name i))))
     str))
@@ -112,7 +111,7 @@
     (for-each
      (lambda (arg)
        (if (ref-arg? arg)
-	   (set! ctr (1+ ctr))))
+	   (set! ctr (+ 1 ctr))))
      args)
     ctr))
 
@@ -121,13 +120,13 @@
     (for-each
      (lambda (arg)
        (if (opt-arg? arg)
-	   (set! ctr (1+ ctr))))
+	   (set! ctr (+ 1 ctr))))
      args)
     ctr))
 
 (define (deref-type arg)
   (let ((type (car arg)))
-    (substring type 0 (1- (string-length type)))))
+    (substring type 0 (- (string-length type) 1))))
 
 (define (deref-name arg)
   (let* ((name (cadr arg)))
@@ -135,20 +134,20 @@
 
 (define (derefable type)
   (let ((len (string-length type)))
-    (call-with-current-continuation
+    (call-with-exit
      (lambda (return)
-       (do ((i (1- len) (1- i))
-	    (ctr 0 (1+ ctr)))
+       (do ((i (- len 1) (- i 1))
+	    (ctr 0 (+ 1 ctr)))
 	   ((= i 0) #f)
 	 (if (not (char=? (string-ref type i) #\*))
 	     (return (> ctr 1))))))))
 
 (define (has-stars type)
   (let ((len (string-length type)))
-    (call-with-current-continuation
+    (call-with-exit
      (lambda (return)
-       (do ((i (1- len) (1- i))
-	    (ctr 0 (1+ ctr)))
+       (do ((i (- len 1) (- i 1))
+	    (ctr 0 (+ 1 ctr)))
 	   ((= i 0) #f)
 	 (if (char=? (string-ref type i) #\*)
 	     (return #t)))
@@ -161,36 +160,36 @@
 	  "XVisualInfo"
 	  (let ((len (string-length type))
 		(val (string-copy type)))
-	    (do ((i 0 (1+ i)))
+	    (do ((i 0 (+ 1 i)))
 		((= i len) val)
 	      (if (char=? (string-ref val i) #\*)
 		  (string-set! val i #\_)))))))
 
 (define (no-arg-or-stars name)
   (let ((len (string-length name)))
-    (call-with-current-continuation
+    (call-with-exit
      (lambda (return)
-       (do ((i 0 (1+ i)))
+       (do ((i 0 (+ 1 i)))
 	   ((= i len) name)
 	 (if (or (char=? (string-ref name i) #\()
 		 (char=? (string-ref name i) #\*))
 	     (return (substring name 0 i))))))))
 
-(define* (parse-args args #:optional x)
+(define* (parse-args args :optional x)
   (let ((data '())
 	(sp -1)
 	(type #f)
 	(len (string-length args)))
     (if (string=? args "void")
 	'()
-	(do ((i 0 (1+ i)))
+	(do ((i 0 (+ 1 i)))
 	    ((= i len) (reverse data))
 	  (let ((ch (string-ref args i)))
 	    (if (or (char=? ch #\space)
-		    (= i (1- len)))
+		    (= i (- len 1)))
 		(begin
 		  (if type
-		      (let* ((given-name (substring args (1+ sp) (if (= i (1- len)) (1+ i) i)))
+		      (let* ((given-name (substring args (+ 1 sp) (if (= i (- len 1)) (+ 1 i) i)))
 			     (reftype #f))
 			(if (char=? (string-ref given-name 0) #\@)
 			    (set! data (cons (list type 
@@ -220,8 +219,8 @@
 				(if (not (member type types))
 				    (set! types (cons type types)))))
 			(set! type #f))
-		      (if (> i (1+ sp))
-			  (set! type (substring args (1+ sp) i))))
+		      (if (> i (+ 1 sp))
+			  (set! type (substring args (+ 1 sp) i))))
 		  (set! sp i))))))))
 
 (define (helpify name type args)
@@ -231,7 +230,7 @@
 	 (typed #f)
 	 (help-max 100))
     (hey initial)
-    (do ((i 0 (1+ i)))
+    (do ((i 0 (+ 1 i)))
 	((= i len))
       (let ((ch (string-ref args i)))
 	(if (char=? ch #\space)
@@ -245,13 +244,13 @@
 			(set! line-len 0)))
 		  (set! typed #f))
 		(begin
-		  (set! line-len (1+ line-len))
+		  (set! line-len (+ 1 line-len))
 		  (heyc " ")
 		  (set! typed #t)))
 	    (if (and (not (char=? ch #\@))
 		     (not (char=? ch #\#)))
 		(begin
-		  (set! line-len (1+ line-len))
+		  (set! line-len (+ 1 line-len))
 		  (heyc ch))))))
     (hey ")\"~%")))
 
@@ -303,24 +302,32 @@
 				     (list "Display*" "XVisualInfo*" "int*" "Pixmap" "Font" "GLubyte*"
 					   "GLdouble*" "GLfloat*" "GLvoid*" "GLuint*"
 					   "GLboolean*" "void*" "GLint*" "GLshort*"
+					   "GLsizei" "GLclampd" "GLclampf" "GLbitfield" "GLshort" "GLubyte" "GLbyte"
+					   "unsigned_long"
 					   "void**")))
 			(if (string=? (car typ) "constchar*")
 			    (hey "#define C_TO_XEN_~A(Arg) C_TO_XEN_~A((char *)(Arg))~%" (no-stars (car typ)) (cdr typ))
 			    (hey "#define C_TO_XEN_~A(Arg) C_TO_XEN_~A(Arg)~%" (no-stars (car typ)) (cdr typ))))
-		    (hey "#define XEN_TO_C_~A(Arg) (~A)(XEN_TO_C_~A(Arg))~%" 
-			 (no-stars (car typ)) (car typ) (cdr typ))
-		    (hey "#define XEN_~A_P(Arg) XEN_~A_P(Arg)~%" 
-			 (no-stars (car typ))
-			 (if (string=? (cdr typ) "INT") 
-			     "INTEGER" 
-			     (if (string=? (cdr typ) "DOUBLE")
-				 "NUMBER"
-				 (cdr typ)))))
+		    (if (not (member (car typ)
+				     (list "constchar*")))
+			(hey "#define XEN_TO_C_~A(Arg) (~A)(XEN_TO_C_~A(Arg))~%" 
+			     (no-stars (car typ)) (car typ) (cdr typ)))
+		    (if (not (member (car typ)
+				     (list "constchar*")))
+			(hey "#define XEN_~A_P(Arg) XEN_~A_P(Arg)~%" 
+			     (no-stars (car typ))
+			     (if (string=? (cdr typ) "INT") 
+				 "INTEGER" 
+				 (if (string=? (cdr typ) "DOUBLE")
+				     "NUMBER"
+				     (cdr typ))))))
 		  (begin
 		    (hey "#define XEN_~A_P(Arg) 1~%" (no-stars (car typ)))
 		    (hey "#define XEN_TO_C_~A(Arg) ((gpointer)Arg)~%" (no-stars (car typ)))))))
-	(if (not (or (string=? type "Display*")
-		     (string=? type "XVisualInfo*")))
+
+	(if (not (or (string=? type "Display*")     ; why are these 2 handled specially?
+		     (string=? type "XVisualInfo*")
+		     (string=? type "GLXContext"))) ; Snd g_snd_glx_context (snd-xmain.c) calls this a pointer
 	    (begin
 	      (if (member type glu-1-2) (hey "#ifdef GLU_VERSION_1_2~%"))
 	      (hey "XL_TYPE~A~A(~A, ~A)~%" 
@@ -338,10 +345,13 @@
 		   type)
 	      (if (member type glu-1-2) (hey "#endif~%")))
 	    (if (string=? type "Display*")
-		(hey "XL_TYPE_1(Display, Display*)~%")
-		(hey "XL_TYPE(XVisualInfo, XVisualInfo*)~%"))))))
+		(hey "XL_TYPE_PTR(Display, Display*)~%")
+		(if (string=? type "XVisualInfo*")
+		    (hey "XL_TYPE_PTR(XVisualInfo, XVisualInfo*)~%")
+		    (hey "XL_TYPE_PTR(GLXContext, GLXContext)~%")
+		    ))))))
 
-(define* (CFNC data #:optional spec spec-name)
+(define* (CFNC data :optional spec spec-name)
   (let ((name (cadr-str data))
 	(args (caddr-str data)))
     (if (assoc name names)
@@ -355,14 +365,14 @@
 		(set! funcs (cons (list name type strs args) funcs)))
 	    (set! names (cons (cons name 'fnc) names)))))))
 
-(define* (CINT name #:optional type)
+(define* (CINT name :optional type)
   (if (assoc name names)
       (display (format #f "~A CINT~%" name))
       (begin
 	(set! ints (cons name ints))
 	(set! names (cons (cons name 'int) names)))))
 
-(define* (CFNC-X data #:optional spec spec-name)
+(define* (CFNC-X data :optional spec spec-name)
   (let ((name (cadr-str data))
 	(args (caddr-str data)))
     (if (assoc name names)
@@ -376,60 +386,18 @@
 		(set! x-funcs (cons (list name type strs args) x-funcs)))
 	    (set! names (cons (cons name 'fnc) names)))))))
 
-(define* (CINT-X name #:optional type)
+(define* (CINT-X name :optional type)
   (if (assoc name names)
       (display (format #f "~A CINT-X~%" name))
       (begin
 	(set! x-ints (cons name x-ints))
 	(set! names (cons (cons name 'int) names)))))
 
-(define* (CFNC-G data #:optional spec spec-name)
-  (let ((name (cadr-str data))
-	(args (caddr-str data)))
-    (if (assoc name names)
-	(display (format #f "~A CFNC-G~%" name))
-	(let ((type (car-str data)))
-	  (if (not (member type g-types))
-	      (set! g-types (cons type g-types)))
-	  (let ((strs (parse-args args 'g)))
-	    (if spec
-		(set! g-funcs (cons (list name type strs args spec spec-name) g-funcs))
-		(set! g-funcs (cons (list name type strs args) g-funcs)))
-	    (set! names (cons (cons name 'fnc) names)))))))
-
-(define* (CFNC-G5 data #:optional spec spec-name)
-  (let ((name (cadr-str data))
-	(args (caddr-str data)))
-    (if (assoc name names)
-	(display (format #f "~A CFNC-G5~%" name))
-	(let ((type (car-str data)))
-	  (if (not (member type g-types))
-	      (set! g-types (cons type g-types)))
-	  (let ((strs (parse-args args 'g)))
-	    (if spec
-		(set! g5-funcs (cons (list name type strs args spec spec-name) g5-funcs))
-		(set! g5-funcs (cons (list name type strs args) g5-funcs)))
-	    (set! names (cons (cons name 'fnc) names)))))))
-
-(define* (CINT-G name #:optional type)
-  (if (assoc name names)
-      (display (format #f "~A CINT-G~%" name))
-      (begin
-	(set! g-ints (cons name g-ints))
-	(set! names (cons (cons name 'int) names)))))
-
-(define* (CINT-G5 name #:optional type)
-  (if (assoc name names)
-      (display (format #f "~A CINT-G5~%" name))
-      (begin
-	(set! g5-ints (cons name g5-ints))
-	(set! names (cons (cons name 'int) names)))))
-
 (define (no-arg name)
   (let ((len (string-length name)))
-    (call-with-current-continuation
+    (call-with-exit
      (lambda (return)
-       (do ((i 0 (1+ i)))
+       (do ((i 0 (+ 1 i)))
 	   ((= i len) name)
 	 (if (char=? (string-ref name i) #\()
 	     (return (substring name 0 i))))))))
@@ -438,8 +406,8 @@
 
 (load "gldata.scm")
 
-;;; ---------------------------------------- write output files ----------------------------------------
-(hey "/* gl.c: Guile and Ruby bindings for GL, GLU~%")
+;;; ---------------------------------------- write output file ----------------------------------------
+(hey "/* gl.c: s7, Guile, Ruby, and Forth bindings for GL, GLU~%")
 (hey " *   generated automatically from makegl.scm and gldata.scm~%")
 (hey " *   needs xen.h~%")
 (hey " *~%")
@@ -448,6 +416,12 @@
 (hey " * 'gl is added to *features*~%")
 (hey " *~%")
 (hey " * HISTORY:~%")
+(hey " *     17-Oct-08: removed gtkglext bindings.~%")
+(hey " *     --------~%")
+(hey " *     30-Mar-06: check for glu.h, omit GLU_* if necessary.  Add Forth support.~%")
+(hey " *     --------~%")
+(hey " *     13-Jun-05: merged gl-ruby.c into gl.c.~%")
+(hey " *     --------~%")
 (hey " *     10-Mar:    Gl_Version.~%")
 (hey " *     1-Feb-03:  glGet* funcs now try to handle multiple return values correctly.~%")
 (hey " *     --------~%")
@@ -459,13 +433,14 @@
 (hey " *     20-May-02: initial version.~%")
 (hey " */~%~%")
 
-(hey "#include <config.h>~%~%")
+(hey "#include <mus-config.h>~%~%")
+
+(hey "#if HAVE_EXTENSION_LANGUAGE~%")
 
-(hey "#if USE_GTK~%")
-(hey "  #include <gtk/gtkgl.h>~%")
-(hey "#endif~%")
 (hey "#include <GL/gl.h>~%")
-(hey "#include <GL/glu.h>~%")
+(hey "#if HAVE_GLU~%")
+(hey "  #include <GL/glu.h>~%")
+(hey "#endif~%")
 (hey "#if USE_MOTIF~%")
 (hey "  #include <GL/glx.h>~%")
 (hey "#endif~%")
@@ -477,10 +452,6 @@
 (hey "#else~%")
 (hey "  #include \"xen.h\"~%")
 (hey "#endif~%")
-(hey "#ifndef CALLOC~%")
-(hey "  #define CALLOC(a, b)  calloc((size_t)(a), (size_t)(b))~%")
-(hey "  #define FREE(a)       free(a)~%")
-(hey "#endif~%~%")
 
 (hey "#ifndef unsigned_long~%")
 (hey "  /* for FreeBSD (thanks to Michael Scholz) (can't use ulong here due to collisions elsewhere) */~%")
@@ -488,24 +459,31 @@
 (hey "#endif~%~%")
 
 (hey "/* prefix for all names */~%")
-(hey "#if HAVE_GUILE~%")
+(hey "#if HAVE_SCHEME~%")
 (hey "  #define XL_PRE \"\"~%")
 (hey "  #define XL_POST \"\"~%")
-(hey "#else~%")
+(hey "#endif~%")
+(hey "#if HAVE_RUBY~%")
 (hey "/* for Ruby, XG PRE needs to be uppercase */~%")
 (hey "  #define XL_PRE \"R\"~%")
 (hey "  #define XL_POST \"\"~%")
 (hey "#endif~%")
+(hey "#if HAVE_FORTH~%")
+(hey "  #define XL_PRE \"F\"~%")
+(hey "  #define XL_POST \"\"~%")
+(hey "#endif~%")
 (hey "~%")
 
-(hey "#define WRAP_FOR_XEN(Name, Value) XEN_LIST_2(C_STRING_TO_XEN_SYMBOL(Name), C_TO_XEN_ULONG((unsigned long)Value))~%")
+(hey "#define WRAP_FOR_XEN(Name, Value) XEN_LIST_2(C_STRING_TO_XEN_SYMBOL(Name), XEN_WRAP_C_POINTER(Value))~%")
 (hey "#define WRAP_P(Name, Value) (XEN_LIST_P(Value) && \\~%")
 (hey "                            (XEN_LIST_LENGTH(Value) >= 2) && \\~%")
 (hey "                            (XEN_SYMBOL_P(XEN_CAR(Value))) && \\~%")
 (hey "                            (strcmp(Name, XEN_SYMBOL_TO_C_STRING(XEN_CAR(Value))) == 0))~%")
 (hey "~%")
+
+;;; these have to match the choices in xm.c 
 (hey "#define XL_TYPE(Name, XType) \\~%")
-(hey "  static XEN C_TO_XEN_ ## Name (XType val) {return(WRAP_FOR_XEN(#Name, val));} \\~%")
+(hey "  static XEN C_TO_XEN_ ## Name (XType val) {return(XEN_LIST_2(C_STRING_TO_XEN_SYMBOL(#Name), C_TO_XEN_ULONG(val)));} \\~%")
 (hey "  static XType XEN_TO_C_ ## Name (XEN val) {return((XType)XEN_TO_C_ULONG(XEN_CADR(val)));} \\~%")
 (hey "  static int XEN_ ## Name ## _P(XEN val) {return(WRAP_P(#Name, val));}~%")
 (hey "#define XL_TYPE_1(Name, XType) \\~%")
@@ -514,13 +492,16 @@
 (hey "~%")
 (hey "#define XL_TYPE_PTR(Name, XType) \\~%")
 (hey "  static XEN C_TO_XEN_ ## Name (XType val) {if (val) return(WRAP_FOR_XEN(#Name, val)); return(XEN_FALSE);} \\~%")
-(hey "  static XType XEN_TO_C_ ## Name (XEN val) {if (XEN_FALSE_P(val)) return(NULL); return((XType)XEN_TO_C_ULONG(XEN_CADR(val)));} \\~%")
+(hey "  static XType XEN_TO_C_ ## Name (XEN val) {if (XEN_FALSE_P(val)) return(NULL); return((XType)XEN_UNWRAP_C_POINTER(XEN_CADR(val)));} \\~%")
 (hey "  static int XEN_ ## Name ## _P(XEN val) {return(WRAP_P(#Name, val));} /* if NULL ok, should be explicit */~%")
 (hey "#define XL_TYPE_PTR_1(Name, XType) \\~%")
-(hey "  static XType XEN_TO_C_ ## Name (XEN val) {if (XEN_FALSE_P(val)) return(NULL); return((XType)XEN_TO_C_ULONG(XEN_CADR(val)));} \\~%")
+(hey "  static XType XEN_TO_C_ ## Name (XEN val) {if (XEN_FALSE_P(val)) return(NULL); return((XType)XEN_UNWRAP_C_POINTER(XEN_CADR(val)));} \\~%")
 (hey "  static int XEN_ ## Name ## _P(XEN val) {return(WRAP_P(#Name, val));} /* if NULL ok, should be explicit */~%")
-(hey "#define XL_TYPE_PTR_2(Name, XType) \\~%")
-(hey "  static XEN C_TO_XEN_ ## Name (XType val) {if (val) return(WRAP_FOR_XEN(#Name, val)); return(XEN_FALSE);}~%")
+
+;XL_TYPE_PTR_2 was for "GdkVisual*" "PangoFont*" "GdkColormap*"
+;(hey "#define XL_TYPE_PTR_2(Name, XType) \\~%")
+;(hey "  static XEN C_TO_XEN_ ## Name (XType val) {if (val) return(WRAP_FOR_XEN(#Name, val)); return(XEN_FALSE);}~%")
+
 
 (hey "~%~%/* ---------------------------------------- types ---------------------------------------- */~%~%")
 
@@ -528,9 +509,9 @@
 (for-each type-it (reverse x-types))
 (hey "#endif~%")
 
-(hey "#if USE_GTK~%")
-(for-each type-it (reverse g-types))
-(hey "#endif~%")
+;(hey "#if USE_GTK~%")
+;(for-each type-it (reverse g-types))
+;(hey "#endif~%")
 
 (for-each type-it (reverse types))
 
@@ -617,12 +598,14 @@
        (if (> line-len line-max)
 	   (begin
 	     (hey "~%")
-	     (do ((i 0 (1+ i)))
+	     (do ((i 0 (+ 1 i)))
 		 ((= i arg-start))
 	       (heyc " "))
 	     (set! line-len arg-start))))
 
+     (check-glu name)
      (if (member name glu-1-2) (hey "#ifdef GLU_VERSION_1_2~%"))
+
      (if (and (> (length data) 4)
 	      (eq? (list-ref data 4) 'if))
 	 (hey "#if HAVE_~A~%" (string-upcase (symbol->string (list-ref data 5)))))
@@ -670,7 +653,7 @@
 	      (lambda (arg)
 		(if (not (ref-arg? arg))
 		    (hey "  ~A = XEN_LIST_REF(arglist, ~D);~%" (cadr arg) ctr))
-		(set! ctr (1+ ctr)))
+		(set! ctr (+ 1 ctr)))
 	      args))))
      (if (> (length args) 0)
 	 (let ((ctr 1))
@@ -689,7 +672,7 @@
 				   (no-stars argtype) argname argname ctr name argtype))
 			    (hey "  XEN_ASSERT_TYPE(XEN_~A_P(~A), ~A, ~D, ~S, ~S);~%"
 				 (no-stars argtype) argname argname ctr name argtype))))
-		(set! ctr (1+ ctr))))
+		(set! ctr (+ 1 ctr))))
 	    args)))
      (let ((using-result #f)
 	   (using-loc #f))
@@ -778,108 +761,111 @@
 (for-each handle-func (reverse x-funcs))
 (hey "#endif~%")
 
-(hey "#if USE_GTK~%")
-(for-each handle-func (reverse g-funcs))
-(hey "#ifdef GTKGLEXT_MAJOR_VERSION~%")
-(for-each handle-func (reverse g5-funcs))
-(hey "#endif~%")
-(hey "#endif~%~%")
-
 (for-each handle-func (reverse funcs))
+(uncheck-glu)
 
 
-(say "/* Ruby connection for gl.c */~%~%")
+;;; ---------------- argify linkages
 
+(hey "#ifdef XEN_ARGIFY_1~%")
 (define (argify-func func)
   (let* ((cargs (length (caddr func)))
+	 (name (car func))
 	 (refargs (+ (ref-args (caddr func)) (opt-args (caddr func))))
 	 (args (- cargs refargs))
 	 (if-fnc (and (> (length func) 4)
 		      (eq? (list-ref func 4) 'if))))
-    (if (member (car func) glu-1-2) (say "#ifdef GLU_VERSION_1_2~%"))
+    (check-glu name)
+    (if (member name glu-1-2) (hey "#ifdef GLU_VERSION_1_2~%"))
     (if if-fnc
-	(say "#if HAVE_~A~%" (string-upcase (symbol->string (list-ref func 5)))))
-    (say "XEN_~A(gxg_~A_w, gxg_~A)~%" 
+	(hey "#if HAVE_~A~%" (string-upcase (symbol->string (list-ref func 5)))))
+    (hey "XEN_~A(gxg_~A_w, gxg_~A)~%" 
 	 (if (>= cargs 10) "VARGIFY"
 	     (if (> refargs 0)
 		 (format #f "ARGIFY_~D" cargs)
 		 (format #f "NARGIFY_~D" cargs)))
 	 (car func) (car func))
     (if if-fnc
-	(say "#endif~%"))
-    (if (member (car func) glu-1-2) (say "#endif~%"))))
+	(hey "#endif~%"))
+    (if (member (car func) glu-1-2) (hey "#endif~%"))))
 	 
-(say "#if USE_MOTIF~%")
+(hey "#if USE_MOTIF~%")
 (for-each argify-func (reverse x-funcs))
-(say "#endif~%")
-
-(say "#if USE_GTK~%")
-(for-each argify-func (reverse g-funcs))
-(say "#ifdef GTKGLEXT_MAJOR_VERSION~%")
-(for-each argify-func (reverse g5-funcs))
-(say "#endif~%")
-(say "#endif~%")
+(hey "#endif~%")
 
 (for-each argify-func (reverse funcs))
+(uncheck-glu)
+
+(hey "~%#else~%~%")
+
+(define (unargify-func func)
+  (let* ((cargs (length (caddr func)))
+	 (name (car func))
+	 (refargs (+ (ref-args (caddr func)) (opt-args (caddr func))))
+	 (args (- cargs refargs))
+	 (if-fnc (and (> (length func) 4)
+		      (eq? (list-ref func 4) 'if))))
+
+    (check-glu name)
+    (if (member name glu-1-2) (hey "#ifdef GLU_VERSION_1_2~%"))
+
+    (if if-fnc
+	(hey "#if HAVE_~A~%" (string-upcase (symbol->string (list-ref func 5)))))
+    (hey "#define gxg_~A_w gxg_~A~%" 
+	 (car func) (car func))
+    (if if-fnc
+	(hey "#endif~%"))
+    (if (member (car func) glu-1-2) (hey "#endif~%"))))
+	 
+(hey "#if USE_MOTIF~%")
+(for-each unargify-func (reverse x-funcs))
+(hey "#endif~%")
+
+(for-each unargify-func (reverse funcs))
+(uncheck-glu)
+
+(hey "#endif~%")
 
 
-(hey "#if HAVE_GUILE~%")
-(say-hey "static void define_functions(void)~%")
-(say-hey "{~%")
-(say-hey "  #define GL_DEFINE_PROCEDURE(Name, Value, A1, A2, A3, Help) XEN_DEFINE_PROCEDURE(XL_PRE #Name XL_POST, Value, A1, A2, A3, Help)~%")
+;;; ---------------- procedure linkages
+(hey "static void define_functions(void)~%")
+(hey "{~%")
+(hey "  #define GL_DEFINE_PROCEDURE(Name, Value, A1, A2, A3, Help) XEN_DEFINE_PROCEDURE(XL_PRE #Name XL_POST, Value, A1, A2, A3, Help)~%")
 
 (define (defun func)
   (let* ((cargs (length (caddr func)))
+	 (name (car func))
 	 (refargs (+ (ref-args (caddr func)) (opt-args (caddr func))))
 	 (args (- cargs refargs)))
-    (if (member (car func) glu-1-2) (hey "#ifdef GLU_VERSION_1_2~%"))
-    (hey "  GL_DEFINE_PROCEDURE(~A, gxg_~A, ~D, ~D, ~D, H_~A);~%"
+    (check-glu name)
+    (if (member name glu-1-2) (hey "#ifdef GLU_VERSION_1_2~%"))
+
+    (hey "  GL_DEFINE_PROCEDURE(~A, gxg_~A_w, ~D, ~D, ~D, H_~A);~%"
 		     (car func) (car func) 
 		     (if (>= cargs 10) 0 args)
 		     (if (>= cargs 10) 0 refargs) ; optional ignored
 		     (if (>= cargs 10) 1 0)
 		     (car func))
     (if (member (car func) glu-1-2) (hey "#endif~%"))
-    (if (member (car func) glu-1-2) (say "#ifdef GLU_VERSION_1_2~%"))
-    (say "  GL_DEFINE_PROCEDURE(~A, gxg_~A_w, ~D, ~D, ~D, H_~A);~%"
-		     (car func) (car func) 
-		     (if (>= cargs 10) 0 args)
-		     (if (>= cargs 10) 0 refargs) ; optional ignored
-		     (if (>= cargs 10) 1 0)
-		     (car func))
-    (if (member (car func) glu-1-2) (say "#endif~%"))
     ))
 
-(say-hey "#if USE_MOTIF~%")
+(hey "#if USE_MOTIF~%")
 (for-each defun (reverse x-funcs))
-(say-hey "#endif~%")
-
-(say-hey "#if USE_GTK~%")
-(for-each defun (reverse g-funcs))
-(say-hey "#ifdef GTKGLEXT_MAJOR_VERSION~%")
-(for-each defun (reverse g5-funcs))
-(say-hey "#endif~%")
-(say-hey "#endif~%")
+(hey "#endif~%")
 
 (for-each defun (reverse funcs))
+(uncheck-glu)
 
-(say-hey "}~%~%")
-(hey "#else~%")
-(hey "  #include \"gl-ruby.c\"~%")
-(hey "#endif~%")
+(hey "}~%~%")
 
 
 (hey "/* ---------------------------------------- constants ---------------------------------------- */~%~%")
 (hey "static void define_integers(void)~%")
 (hey "{~%~%")
-(hey "#if HAVE_GUILE~%")
-(hey "#if HAVE_SCM_C_DEFINE~%")
-(hey "  #define DEFINE_INTEGER(Name) scm_c_define(XL_PRE #Name XL_POST, C_TO_XEN_INT(Name))~%")
+(hey "#if HAVE_S7~%")
+(hey "#define DEFINE_INTEGER(Name) s7_define_constant(s7, XL_PRE #Name XL_POST, C_TO_XEN_INT(Name))~%")
 (hey "#else~%")
-(hey "  #define DEFINE_INTEGER(Name) gh_define(XL_PRE #Name XL_POST, C_TO_XEN_INT(Name))~%")
-(hey "#endif~%")
-(hey "#else~%")
-(hey "  #define DEFINE_INTEGER(Name) rb_define_global_const(XL_PRE #Name XL_POST, C_TO_XEN_INT(Name))~%")
+(hey "#define DEFINE_INTEGER(Name) XEN_DEFINE(XL_PRE #Name XL_POST, C_TO_XEN_INT(Name))~%")
 (hey "#endif~%")
 (hey "~%")
 
@@ -890,51 +876,51 @@
  (reverse x-ints))
 (hey "#endif~%")
 
-(hey "#if USE_GTK~%")
 (for-each 
  (lambda (val) 
-   (hey "  DEFINE_INTEGER(~A);~%" val)) 
- (reverse g-ints))
-(hey "#ifdef GTKGLEXT_MAJOR_VERSION~%")
-(for-each 
- (lambda (val) 
-   (hey "  DEFINE_INTEGER(~A);~%" val)) 
- (reverse g5-ints))
-(hey "#endif~%")
-(hey "#endif~%")
 
-(for-each 
- (lambda (val) 
+     (if in-glu
+	 (if (not (string=? "GLU" (substring val 0 3)))
+	     (BEGIN
+	       (set! in-glu #f)
+	       (hey "#endif~%")))
+	 (if (string=? "GLU" (substring val 0 3))
+	     (begin
+	       (set! in-glu #t)
+	       (hey "#if HAVE_GLU~%"))))
+
    (hey "  DEFINE_INTEGER(~A);~%" val)) 
  (reverse ints))
 
+(if in-glu
+    (begin
+      (hey "#endif~%")
+      (set! in-glu #f)))
+
+
 (hey "}~%~%")
 
 (hey "/* -------------------------------- initialization -------------------------------- */~%~%")
 (hey "static bool gl_already_inited = false;~%~%")
-(hey "#if HAVE_GUILE~%")
-(hey " void init_gl(void);~%")
-(hey " void init_gl(void)~%")
-(hey "#else~%")
-(hey " void Init_libgl(void);~%")
-(hey " void Init_libgl(void)~%")
-(hey "#endif~%")
+(hey "void Init_libgl(void);~%")
+(hey "void Init_libgl(void)~%")
 (hey "{~%")
 (hey "  if (!gl_already_inited)~%")
 (hey "    {~%")
 (hey "      define_integers();~%")
 (hey "      define_functions();~%")
 (hey "      XEN_YES_WE_HAVE(\"gl\");~%")
-(hey "#if HAVE_GUILE~%")
-(hey "      XEN_EVAL_C_STRING(\"(define gl-version \\\"~A\\\")\");~%" (strftime "%d-%b-%y" (localtime (current-time))))
-(hey "#endif~%")
-(hey "#if HAVE_RUBY~%")
-(hey "      rb_define_global_const(\"Gl_Version\", C_TO_XEN_STRING(\"~A\"));~%" (strftime "%d-%b-%y" (localtime (current-time))))
-(hey "#endif~%")
+(hey "      XEN_DEFINE(\"gl-version\", C_TO_XEN_STRING(\"~A\"));~%" (strftime "%d-%b-%y" (localtime (current-time))))
 (hey "      gl_already_inited = true;~%")
 (hey "    }~%")
 (hey "}~%")
+(hey "#else~%")
+(hey " void Init_libgl(void);~%")
+(hey " void Init_libgl(void)~%")
+(hey "{~%")
+(hey "}~%")
+(hey "#endif~%")
 
 (close-output-port gl-file)
-(close-output-port gl-ruby-file)
 
+(exit)
