@@ -1,4 +1,4 @@
-#!/usr/local/bin/guile -s
+#!/usr/bin/guile -s
 !#
 
 (use-modules (ice-9 format))
@@ -8,7 +8,7 @@
     "test.scm"
   (lambda ()
     (call-with-input-file 
-	"snd_conffile.scm"
+	"ladspa.scm"
       (lambda (file)
 	(let loop ((line (read-line file 'concat)))
 	  (or (eof-object? line)
@@ -21,4 +21,3 @@
 		(loop (read-line file 'concat)))))))))
 
 
-
