@@ -1,10 +1,13 @@
 (use-modules (ice-9 optargs))
 (provide 'snd-v.scm)
-(if (not (provided? 'snd-ws.scm)) (load-from-path "ws.scm"))
+
+(if (and (not (provided? 'snd-ws.scm)) 
+	 (not (provided? 'sndlib-ws.scm)))
+    (load-from-path "ws.scm"))
 
 ;;; this version of the fm-violin assumes it is running within with-sound (where *output* and *reverb* are defined)
 
-(definstrument (fm-violin startime dur frequency amplitude #:key
+(definstrument (fm-violin startime dur frequency amplitude :key
 	    (fm-index 1.0)
 	    (amp-env '(0 0  25 1  75 1  100 0))
 	    (periodic-vibrato-rate 5.0) 
@@ -28,13 +31,13 @@
 	    (fm1-index #f) 
 	    (fm2-index #f) 
 	    (fm3-index #f)
-	    (degree 0)
+	    (degree #f)
 	    (distance 1.0)
 	    (reverb-amount 0.01)
 	    (base 1.0)
-	    #:allow-other-keys)
+	    :allow-other-keys)
 
-    "(fm-violin startime dur frequency amplitude #:key 
+    "(fm-violin startime dur frequency amplitude :key 
    (fm-index 1.0) (amp-env '(0 0  25 1  75 1  100 0)) 
    (periodic-vibrato-rate 5.0) (random-vibrato-rate 16.0) 
    (periodic-vibrato-amplitude 0.0025) (random-vibrato-amplitude 0.005) 
@@ -45,14 +48,13 @@
    (fm2-env '(0 1  25 .4  75 .6  100 0)) (fm3-rat 4.0) 
    (fm3-env '(0 1  25 .4  75 .6  100 0)) (fm1-rat 1.0) 
    (fm2-rat 3.0) (fm1-index #f) (fm2-index #f) 
-   (fm3-index #f) (degree 0) (distance 1.0) 
+   (fm3-index #f) (degree #f) (distance 1.0) 
    (reverb-amount 0.01) (base 1.0)) 
 This version of the fm-violin assumes it is running within with-sound (where *output* and *reverb* are defined).
   (with-sound () (fm-violin 0 1 440 .1))"
 
-    (let* ((pi 3.141592653589793)
-	   (beg (inexact->exact (floor (* startime (mus-srate)))))
-	   (len (inexact->exact (floor (* dur (mus-srate)))))
+    (let* ((beg (seconds->samples startime))
+	   (len (seconds->samples dur))
 	   (end (+ beg len))
 	   (frq-scl (hz->radians frequency))
 	   (modulate (not (zero? fm-index)))
@@ -70,14 +72,17 @@ This version of the fm-violin assumes it is running within with-sound (where *ou
 			   (= fm3-rat (floor fm3-rat))
 			   (integer? (inexact->exact (/ fm2-rat fm1-rat))) ; might be 2=2 but 1=3 or whatever
 			   (integer? (inexact->exact (/ fm3-rat fm1-rat)))))
-	   (coeffs (and easy-case modulate
-			(partials->polynomial
-			 (list (inexact->exact fm1-rat) index1
-			       (inexact->exact (floor (/ fm2-rat fm1-rat))) index2
-			       (inexact->exact (floor (/ fm3-rat fm1-rat))) index3))))
 	   (norm (or (and easy-case modulate 1.0) index1))
 	   (carrier (make-oscil frequency))
-	   (fmosc1  (and modulate (make-oscil (* fm1-rat frequency))))
+	   (fmosc1  (if modulate 
+			(if easy-case 
+			    (make-polywave (* fm1-rat frequency) 
+					   (list (inexact->exact fm1-rat) index1
+						 (floor (/ fm2-rat fm1-rat)) index2
+						 (floor (/ fm3-rat fm1-rat)) index3)
+					   mus-chebyshev-second-kind)
+			    (make-oscil (* fm1-rat frequency)))
+			#f))
 	   (fmosc2  (and modulate (or easy-case (make-oscil (* fm2-rat frequency)))))
 	   (fmosc3  (and modulate (or easy-case (make-oscil (* fm3-rat frequency)))))
 	   (ampf  (make-env amp-env :scaler amplitude :base base :duration dur))
@@ -96,7 +101,7 @@ This version of the fm-violin assumes it is running within with-sound (where *ou
 	   (amp-noi (if (and (not (= 0.0 amp-noise-amount)) (not (= 0.0 amp-noise-freq)))
 			(make-rand-interp amp-noise-freq amp-noise-amount)
 			#f))
-	   (locs (make-locsig degree distance reverb-amount *output* *reverb* (mus-channels *output*)))
+	   (locs (make-locsig (or degree (random 90.0)) distance reverb-amount))
 	   (vib 0.0) 
 	   (modulation 0.0)
 	   (fuzz 0.0)
@@ -106,7 +111,7 @@ This version of the fm-violin assumes it is running within with-sound (where *ou
       (if (or (not easy-case) ind-noi amp-noi (> noise-amount 0.0) (not modulate))
 	  (run
 	   (lambda ()
-	     (do ((i beg (1+ i)))
+	     (do ((i beg (+ 1 i)))
 		 ((= i end))
 	       (if (not (= 0.0 noise-amount))
 		   (set! fuzz (rand fm-noi)))
@@ -117,7 +122,7 @@ This version of the fm-violin assumes it is running within with-sound (where *ou
 		   (if easy-case
 		       (set! modulation
 			     (* (env indf1) 
-				(polynomial coeffs (oscil fmosc1 vib)))) ;(* vib fm1-rat)??
+				(polywave fmosc1 vib)))
 		       (set! modulation
 			     (+ (* (env indf1) (oscil fmosc1 (+ (* fm1-rat vib) fuzz)))
 				(* (env indf2) (oscil fmosc2 (+ (* fm2-rat vib) fuzz)))
@@ -126,12 +131,12 @@ This version of the fm-violin assumes it is running within with-sound (where *ou
 				 (oscil carrier (+ vib (* ind-fuzz modulation))))))))
 	  (run
 	   (lambda () 
-	     (do ((i beg (1+ i)))
+	     (do ((i beg (+ 1 i)))
 		 ((= i end))
 	       (let* ((vib (+ (env frqf) (triangle-wave pervib) (rand-interp ranvib))))
 		 (locsig locs i (* (env ampf) 
 				   (oscil carrier (+ vib (* (env indf1) 
-							    (polynomial coeffs (oscil fmosc1 vib))))))))))))))
+							    (polywave fmosc1 vib)))))))))))))
 
 
 ; (fm-violin 0 1 440 .1 :fm-index 2.0)
