@@ -6,16 +6,24 @@
 (use-modules (ice-9 common-list) (ice-9 format))
 (provide 'snd-xm-enved.scm)
 
-(if (and (not (provided? 'xm))
-	 (not (provided? 'xg)))
-    (let ((hxm (dlopen "xm.so")))
-      (if (string? hxm)
-	  (snd-error (format #f "xe-enved.scm needs the xm module: ~A" hxm))
-	  (dlinit hxm "init_xm"))))
+
+(if (provided? 'snd-motif)
+    (if (not (provided? 'xm))
+	(let ((hxm (dlopen "xm.so")))
+	  (if (string? hxm)
+	      (snd-error (format #f "xm-enved.scm needs the xm module (either 'make xm' or build Snd with --with-static-xm): ~A" hxm))
+	      (dlinit hxm "Init_libxm"))))
+    (if (provided? 'snd-gtk)
+	(if (not (provided? 'xg))
+	    (let ((hxm (dlopen "xg.so")))
+	      (if (string? hxm)
+		  (snd-error (format #f "xm-enved.scm needs the xg module (either 'make xg' or build Snd with --with-static-xg): ~A" hxm))
+		  (dlinit hxm "Init_libxg"))))))
 
 (define xe-envelope
   (make-procedure-with-setter
    (lambda (drawer)
+     "accessor for the current xm-enved envelope"
      (or (car drawer) 
 	 (let ((ax-inf (list-ref drawer 3)))
 	   (list (list-ref ax-inf 0)
@@ -169,7 +177,6 @@
 	    (set! args (append args (list XmNforeground (data-color)))))
 	(let* ((drawer (XtCreateManagedWidget name xmDrawingAreaWidgetClass parent args))
 	       (gc (car (snd-gcs)))
-	       (egc (list-ref (snd-gcs) 7))
 	       (x0 (car axis-bounds))
 	       (x1 (cadr axis-bounds)) ; too confusing! -- change internally below
 	       (y0 (caddr axis-bounds))
@@ -179,7 +186,7 @@
 			     drawer 
 			     #f  ; axis pixel locs filled in when drawn
 			     (list x0 y0 x1 y1)
-			     (list gc egc) 
+			     (list gc #f)
 			     name)))
 	  (XtAddCallback drawer XmNresizeCallback 
 			 (lambda (w context info) 
@@ -207,7 +214,6 @@
 	  editor))
       (let* ((drawer (gtk_drawing_area_new))
 	     (gc (car (snd-gcs)))
-	     (egc (list-ref (snd-gcs) 7))
 	     (x0 (car axis-bounds))
 	     (x1 (cadr axis-bounds))
 	     (y0 (caddr axis-bounds))
@@ -218,12 +224,23 @@
 			   drawer 
 			   #f  ; axis pixel locs filled in when drawn
 			   (list x0 y0 x1 y1)
-			   (list gc egc) 
+			   (list gc #f)
 			   name)))
+
+	(define white-pixel
+	  (let ((tmp (GdkColor)))
+	    (gdk_color_parse "white" tmp)
+	    (let ((col (gdk_color_copy tmp)))
+	      (gdk_rgb_find_color (gdk_colormap_get_system) col)
+	      col)))
+
 	(gtk_widget_set_events drawer GDK_ALL_EVENTS_MASK)
 	(gtk_box_pack_start (GTK_BOX parent) drawer #t #t 10)
 	(gtk_widget_show drawer)
-	(gdk_window_set_background (.window drawer) (graph-color))
+	(if (not (provided? 'cairo))
+	    (gdk_window_set_background (gtk_widget_get_window drawer) (graph-color))
+	    (gdk_window_set_background (gtk_widget_get_window drawer) white-pixel))
+
 	(gtk_widget_set_size_request drawer -1 200)
 
 	(g_signal_connect_closure_by_id (GPOINTER drawer)
@@ -275,14 +292,14 @@
 	(g_signal_connect_closure_by_id (GPOINTER drawer)
 					(g_signal_lookup "enter_notify_event" (G_OBJECT_TYPE (GTK_OBJECT drawer)))
 					0 (g_cclosure_new (lambda (w e d)
-							    (gdk_window_set_cursor (.window w) arrow-cursor)
+							    (gdk_window_set_cursor (gtk_widget_get_window w) arrow-cursor)
 							    #f)
 							  #f #f)
 					#f)
 	(g_signal_connect_closure_by_id (GPOINTER drawer)
 					(g_signal_lookup "leave_notify_event" (G_OBJECT_TYPE (GTK_OBJECT drawer)))
 					0 (g_cclosure_new (lambda (w e d)
-							    (gdk_window_set_cursor (.window w) old-cursor)
+							    (gdk_window_set_cursor (gtk_widget_get_window w) old-cursor)
 							    #f)
 							  #f #f)
 					#f)
@@ -292,11 +309,10 @@
   (let* ((cur-env (xe-envelope drawer))
 	 (widget (list-ref drawer 1))
 	 (dpy (and (provided? 'snd-motif) (XtDisplay widget)))
-	 (wn (if (provided? 'snd-motif) (XtWindow widget) (.window widget)))
+	 (wn (if (provided? 'snd-motif) (XtWindow widget) (gtk_widget_get_window widget)))
 	 (ax-pix (list-ref drawer 2))
 	 (ax-inf (list-ref drawer 3))
 	 (gc (car (list-ref drawer 4)))
-	 (egc (cadr (list-ref drawer 4)))
 	 (name (list-ref drawer 5))
 	 (len (and (list? cur-env) (length cur-env))))
     (if (and (list? ax-pix)
@@ -357,23 +373,41 @@
 			      (XDrawLine dpy wn gc lx ly cx cy))
 			  (set! lx cx)
 			  (set! ly cy))))
-		    (let ((lx #f)
-			  (ly #f))
+		    (let* ((lx #f)
+			   (ly #f)
+			   (got-cairo (provided? 'cairo))
+			   (cr (and got-cairo (gdk_cairo_create (GDK_DRAWABLE wn)))))
+		      (if got-cairo
+			  (begin
+			    (cairo_set_line_width cr 1.0)
+			    (cairo_set_source_rgb cr 0.0 0.0 0.0)))
 		      (do ((i 0 (+ i 2)))
 			  ((= i len))
 			(let ((cx (xe-grfx drawer (list-ref cur-env i)))
 			      (cy (xe-grfy drawer (list-ref cur-env (+ i 1)))))
-			  (gdk_draw_arc (GDK_DRAWABLE wn) gc #t
-				    (- cx mouse-r)
-				    (- cy mouse-r)
-				    mouse-d mouse-d
-				    0 (* 360 64))
-			  (if lx
-			      (gdk_draw_line (GDK_DRAWABLE wn) gc lx ly cx cy))
+			  (if got-cairo
+			      (begin
+				(cairo_arc cr cx cy mouse-r 0.0 (* 2 pi))
+				(cairo_fill cr)
+				(if lx
+				    (begin
+				      (cairo_move_to cr lx ly)
+				      (cairo_line_to cr cx cy)
+				      (cairo_stroke cr))))
+			      (begin
+				(gdk_draw_arc (GDK_DRAWABLE wn) gc #t
+					      (- cx mouse-r)
+					      (- cy mouse-r)
+					      mouse-d mouse-d
+					      0 (* 360 64))
+				(if lx
+				    (gdk_draw_line (GDK_DRAWABLE wn) gc lx ly cx cy))))
 			  (set! lx cx)
-			  (set! ly cy)))))))))))
+			  (set! ly cy)))
+		      (if got-cairo
+			  (cairo_destroy cr))))))))))
   
-#!
+#|
 (define outer (add-main-pane "hiho" xmFormWidgetClass '()))
 
 (define editor (xe-create-enved "a name" outer 
@@ -382,5 +416,5 @@
 				   XmNbottomAttachment XmATTACH_FORM
 				   XmNrightAttachment  XmATTACH_FORM)
 			     '(0.0 1.0 0.0 1.0)))
-!#
+|#
 
